SET SERVEROUTPUT ON
SET LINESIZE 2000
SET PAGESIZE 60
SPOOL PPSN_2_migration_v11.log

SELECT 'Migration Started: ' || CURRENT_TIMESTAMP FROM DUAL;

SET ECHO ON

REM ###########################################################################
REM #  Description: Create PPS-N 2.0 database objects
REM #
REM #   Usage:
REM #   Input Parameters:
REM #   1) None
REM #   Output:
REM #   1) Terminal
REM #   Internal Script Parameters:
REM #   1) None
REM #
REM ###########################################################################
REM # Change History
REM #
REM # DATE       User Name         DESCRIPTION
REM # ---------- ----------------- -------------------------------------------
REM # 04/29/2014 Duncan Shelley    Created script
REM #
REM ##########################################################################

DECLARE
   -- CONSTANTS

   cc_schema_name   CONSTANT VARCHAR2 (35) := 'PPSNEPL';

   -- VARIABLES

   vc_sql_statement          VARCHAR2 (4000);
   vc_errm                   VARCHAR2 (64);
   vn_code                   NUMBER;
   vn_continue               NUMBER;

   vn_row_count              NUMBER;

   -----------------------------------------------------------------------------
   -- NAME:         check_for_ppsn2
   -- TYPE:         Function
   -- DESCRIPTSION: Check to see if PPS_N 2.0 object already created
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION check_for_ppsn2
      RETURN NUMBER
   IS
   BEGIN
      SELECT COUNT (*)
        INTO vn_row_count
        FROM sys.dba_tables
       WHERE     owner = cc_schema_name
             AND table_name IN ('EPL_NDF_UPDATE_FILE'
                               ,'EPL_STATUS'
                               ,'EPL_IEN_GENERATOR'
                               ,'EPL_NDF_OUTGOING_DIFFERENCES'
                               ,'EPL_VISTA_UPDATE'
                               ,'EPL_VISTA_STATUS'
                               ,'TEMP_EPL_NDCS'
                               ,'EPL_PACKAGE_SIZES'
                               ,'EPL_DDI_SEVERITY');

      IF vn_row_count > 0
      THEN
         RETURN (1);
      END IF;

      SELECT COUNT (*)
        INTO vn_row_count
        FROM sys.dba_views
       WHERE     owner = cc_schema_name
             AND view_name IN ('FDB_GCNSEQNO_PEM_V'
                              ,'FDB_GCNSEQNO_PLBLW_V'
                              ,'FDB_MONOGRAPH_PEM_V'
                              ,'FDB_PLBLWARNINGS_V');

      IF vn_row_count > 0
      THEN
         RETURN (1);
      END IF;

      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         DBMS_OUTPUT.put_line (
            'check_for_ppsn2: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
         RETURN (1);
   END check_for_ppsn2;

   -----------------------------------------------------------------------------
   -- NAME:         create_ppsn_tables
   -- TYPE:         Function
   -- DESCRIPTSION: Create PPS-N 2.0 tables
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION create_ppsn_tables
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  create_ppsn_tables');

      -- Create sequences section -------------------------------------------------
      vc_sql_statement :=
         'CREATE SEQUENCE ppsnepl.update_file_sequence START WITH 1 INCREMENT BY 1 NOCACHE NOCYCLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Create tables section -------------------------------------------------

      -- Table PPSNEPL.EPL_NDF_OUTGOING_DIFFERENCES

      vc_sql_statement :=
            'CREATE TABLE ppsnepl.epl_ndf_outgoing_differences ('
         || ' ndf_outgoing_differences_id   NUMBER (38, 0) NOT NULL '
         || ',new_value                     VARCHAR2 (256)     NULL '
         || ',old_value                     VARCHAR2 (256)     NULL '
         || ',vista_file_number             VARCHAR2 (20) NOT NULL '
         || ',vista_field_number            VARCHAR2 (10) NOT NULL '
         || ',vista_ien                     VARCHAR2 (50) NOT NULL '
         || ',ndc_update_file_fk            NUMBER (38, 0) '
         || ',action_type                   VARCHAR2 (10) NOT NULL '
         || ',created_by                    VARCHAR2 (50) NOT NULL '
         || ',created_dtm                   TIMESTAMP (6) NOT NULL) '
         || 'TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX ppsnepl.pkndfoutgoingdifferences ON ppsnepl.epl_ndf_outgoing_differences (ndf_outgoing_differences_id) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- New constraint
      vc_sql_statement :=
         'ALTER TABLE PPSNEPL.epl_ndf_outgoing_differences ADD CONSTRAINT pkndfoutgoingdifferences PRIMARY KEY (ndf_outgoing_differences_id) USING INDEX ppsnepl.pkndfoutgoingdifferences';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- Table PPSNEPL.EPL_IEN_GENERATOR

      vc_sql_statement :=
            'CREATE TABLE ppsnepl.epl_ien_generator ( '
         || 'CONCEPT             VARCHAR2(50) NOT NULL, '
         || 'LAST_USED_IEN       NUMBER(30 , 0) NOT NULL, '
         || 'CREATED_BY          VARCHAR2(50) NOT NULL, '
         || 'CREATED_DTM         TIMESTAMP NOT NULL, '
         || 'LAST_MODIFIED_BY    VARCHAR2(50), '
         || 'LAST_MODIFIED_DTM   TIMESTAMP) '
         || 'TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE INDEX PPSNEPL.EPL_IEN_PK_IDX ON PPSNEPL.EPL_IEN_GENERATOR (CONCEPT ASC) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- New constraint
      vc_sql_statement :=
         'ALTER TABLE PPSNEPL.EPL_IEN_GENERATOR ADD CONSTRAINT EPL_IEN_PK PRIMARY KEY (CONCEPT) USING INDEX PPSNEPL.EPL_IEN_PK_IDX';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- Table PPSNEPL.EPL_NDF_UPDATE_FILE

      vc_sql_statement :=
            'CREATE TABLE ppsnepl.epl_ndf_update_file( '
         || ' ndf_update_file_id       NUMERIC (30) NOT NULL '
         || ',file_name                VARCHAR (200) '
         || ',comments                 VARCHAR (2000) '
         || ',status_id_fk             NUMERIC (30) NOT NULL '
         || ',prod_transmission_date   TIMESTAMP '
         || ',test_transmission_date   TIMESTAMP '
         || ',status_modified_date     TIMESTAMP '
         || ',file_directory_path      VARCHAR (500) '
         || ',created_by               VARCHAR (50) NOT NULL '
         || ',created_dtm              TIMESTAMP NOT NULL '
         || ',last_modified_by         VARCHAR (50) '
         || ',last_modified_dtm        TIMESTAMP) '
         || 'TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX ppsnepl.pkndfupdatefile ON ppsnepl.epl_ndf_update_file (ndf_update_file_id) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE INDEX ppsnepl.idxstatus_id ON ppsnepl.epl_ndf_update_file (status_id_fk) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_ndf_update_file ADD CONSTRAINT pkndfupdatefile PRIMARY KEY (ndf_update_file_id) USING INDEX ppsnepl.pkndfupdatefile';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- Table PPSNEPL.EPL_STATUS

      vc_sql_statement :=
            'CREATE TABLE ppsnepl.epl_status( '
         || ' status_id           NUMERIC (30) NOT NULL '
         || ',status_name         VARCHAR (200) '
         || ',status_desc         VARCHAR (1000) '
         || ',created_by          VARCHAR (50) NOT NULL '
         || ',created_dtm         TIMESTAMP NOT NULL '
         || ',last_modified_by    VARCHAR (50) '
         || ',last_modified_dtm   TIMESTAMP) '
         || ' TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX ppsnepl.pkstatus ON ppsnepl.epl_status (status_id) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_status ADD CONSTRAINT pkstatus PRIMARY KEY (status_id) USING INDEX ppsnepl.pkstatus';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- Table PPSNEPL.EPL_VISTA_UPDATE

      vc_sql_statement :=
            'CREATE TABLE ppsnepl.epl_vista_update( '
         || ' vista_update_id     NUMERIC (30) NOT NULL '
         || ',file_fk             NUMERIC (30) NOT NULL '
         || ',site                NUMERIC (30) NOT NULL '
         || ',comments            VARCHAR (2000) '
         || ',vista_status_id_fk  NUMERIC (30) NOT NULL '
         || ',transmission_date   TIMESTAMP) '
         || 'TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX ppsnepl.pkvistaupdate ON ppsnepl.epl_vista_update (vista_update_id) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE INDEX ppsnepl.idxvistastatusid ON ppsnepl.epl_vista_update (vista_status_id_fk) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_vista_update ADD CONSTRAINT pkvistaupdate PRIMARY KEY (vista_update_id) USING INDEX ppsnepl.pkvistaupdate';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- Table PPSNEPL.EPL_VISTA_STATUS

      vc_sql_statement :=
            'CREATE TABLE ppsnepl.epl_vista_status( '
         || ' vista_status_id     NUMERIC (30) NOT NULL '
         || ',status_name         VARCHAR (200) '
         || ',status_desc         VARCHAR (1000) '
         || ',created_by          VARCHAR (50) NOT NULL '
         || ',created_dtm         TIMESTAMP NOT NULL '
         || ',last_modified_by    VARCHAR (50) '
         || ',last_modified_dtm   TIMESTAMP) '
         || 'TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX ppsnepl.pkvistastatus ON ppsnepl.epl_vista_status (vista_status_id) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_vista_status ADD CONSTRAINT pkvistastatus PRIMARY KEY (vista_status_id) USING INDEX ppsnepl.pkvistastatus';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- Table PPSNEPL.EPL_DDI_SEVERITY

      vc_sql_statement :=
            'CREATE TABLE ppsnepl.epl_ddi_severity( '
         || ' epl_id              NUMBER (30, 0) NOT NULL '
         || ' ,name                VARCHAR2 (50) NOT NULL '
         || ' ,created_by          VARCHAR2 (50) NOT NULL '
         || ' ,created_dtm         TIMESTAMP (6) NOT NULL '
         || ' ,last_modified_by    VARCHAR2 (50) '
         || ' ,last_modified_dtm   TIMESTAMP (6)) '
         || ' TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX ppsnepl.epl_ddi_severity_pk ON ppsnepl.epl_ddi_severity (epl_id) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_ddi_severity ADD CONSTRAINT epl_ddi_severity_pk PRIMARY KEY (epl_id) USING INDEX ppsnepl.epl_ddi_severity_pk';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- Table PPSNEPL.EPL_DRUG_DRUG_INTERACTION

      vc_sql_statement :=
            'CREATE TABLE ppsnepl.epl_drug_drug_interaction( '
         || ' epl_drug_drug_interaction_id   NUMBER (38) NOT NULL '
         || ' ,interaction_name               VARCHAR2 (200) NOT NULL '
         || '   ,drug_ingredient_1              NUMBER (30, 0) NOT NULL '
         || '   ,drug_ingredient_2              NUMBER (30, 0) NOT NULL '
         || '   ,severity                       NUMBER (30, 0) NOT NULL '
         || '   ,nationally_entered             NUMBER (1, 0) '
         || '   ,total_indexes                  NUMBER (38, 0) '
         || '   ,locally_edited                 NUMBER (1, 0) '
         || '   ,vista_ien                      NUMBER(30 , 0) NOT NULL '
         || '   ,created_by                     VARCHAR2 (50) NOT NULL '
         || '   ,created_dtm                    TIMESTAMP (6) NOT NULL '
         || '   ,inactivation_date              TIMESTAMP (6) '
         || '   ,last_modified_by               VARCHAR2 (50) '
         || '   ,last_modified_dtm              TIMESTAMP (6)) '
         || ' TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_drug_drug_interaction ADD CONSTRAINT uk_epl_ddi_1 UNIQUE (drug_ingredient_1, drug_ingredient_2, severity)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX ppsnepl.epldrugdruginteractionid_pk ON ppsnepl.epl_drug_drug_interaction (epl_drug_drug_interaction_id) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE INDEX ppsnepl.drug_ingredient_1_idx ON ppsnepl.epl_drug_drug_interaction (drug_ingredient_1) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE INDEX ppsnepl.drug_ingredient_2_idx ON ppsnepl.epl_drug_drug_interaction (drug_ingredient_2) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE INDEX ppsnepl.severity_idx ON ppsnepl.epl_drug_drug_interaction (severity) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_drug_drug_interaction ADD CONSTRAINT epldrugdruginteractionid_pk  PRIMARY KEY (epl_drug_drug_interaction_id) USING INDEX ppsnepl.epldrugdruginteractionid_pk';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Create synonyms section -------------------------------------------------

      -- Create relationships section -------------------------------------------------

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_ndf_outgoing_differences ADD (CONSTRAINT fk_pkndfoutgoingfile FOREIGN KEY (ndc_update_file_fk) REFERENCES ppsnepl.epl_ndf_update_file (ndf_update_file_id))';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_ndf_update_file ADD (CONSTRAINT fk_ndfupdatefile_status FOREIGN KEY (status_id_fk) REFERENCES ppsnepl.epl_status (status_id))';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_vista_update ADD (CONSTRAINT fk_vistaupdate_vistastatus FOREIGN KEY (vista_status_id_fk) REFERENCES ppsnepl.epl_vista_status (vista_status_id))';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_drug_drug_interaction ADD CONSTRAINT fk_epl_ddi_epl_ingredients_1 FOREIGN KEY (drug_ingredient_1) REFERENCES ppsnepl.epl_ingredients (epl_id)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_drug_drug_interaction ADD CONSTRAINT fk_epl_ddi_epl_ingredients_2 FOREIGN KEY (drug_ingredient_2) REFERENCES ppsnepl.epl_ingredients (epl_id)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_drug_drug_interaction ADD CONSTRAINT fk_epl_ddi_epl_severity FOREIGN KEY (severity) REFERENCES ppsnepl.epl_ddi_severity (epl_id)';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Grant permissions section -------------------------------------------------

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.epl_ndf_outgoing_differences TO PPSNEPL_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.epl_ien_generator TO PPSNEPL_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.epl_ndf_update_file TO PPSNEPL_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.epl_status TO PPSNEPL_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.epl_vista_update TO PPSNEPL_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.epl_vista_status TO PPSNEPL_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.epl_ddi_severity TO PPSNEPL_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.epl_drug_drug_interaction TO PPSNEPL_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.update_file_sequence TO PPSNEPL_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;


      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  create_ppsn_tables');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'create_ppsn_tables: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'create_ppsn_tables: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'create_ppsn_tables: ' || vc_sql_statement);
         RETURN (1);
   END create_ppsn_tables;

   -----------------------------------------------------------------------------
   -- NAME:         create_ppsn_views
   -- TYPE:         Function
   -- DESCRIPTSION: Create PPS-N 2.0 views
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION create_ppsn_views
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  create_ppsn_views');

      -- Create views section -------------------------------------------------

      -- View PPSNEPL.FDB_GCNSEQNO_PEM_V

      vc_sql_statement :=
         'GRANT SELECT ON fdb_dif.fdb_gcnseqno_pem TO ppsnepl WITH GRANT OPTION';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE VIEW ppsnepl.fdb_gcnseqno_pem_v AS SELECT * FROM fdb_dif.fdb_gcnseqno_pem';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.fdb_gcnseqno_pem_v TO ppsnepl_read_only_role';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- View FDB_GCNSEQNO_PLBLW_V

      vc_sql_statement :=
         'GRANT SELECT ON fdb_dif.fdb_gcnseqno_plblw TO ppsnepl WITH GRANT OPTION';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE VIEW ppsnepl.fdb_gcnseqno_plblw_v AS SELECT * FROM fdb_dif.fdb_gcnseqno_plblw';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.fdb_gcnseqno_plblw_v TO ppsnepl_read_only_role';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- View PPSNEPL.FDB_MONOGRAPH_PEM_V

      vc_sql_statement :=
         'GRANT SELECT ON fdb_dif.fdb_monograph_pem TO ppsnepl WITH GRANT OPTION';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE VIEW ppsnepl.fdb_monograph_pem_v AS SELECT * FROM fdb_dif.fdb_monograph_pem';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.fdb_monograph_pem_v TO ppsnepl_read_only_role';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- View PPSNEPL.FDB_PLBLWARNINGS_V

      vc_sql_statement :=
         'GRANT SELECT ON fdb_dif.fdb_plblwarnings TO ppsnepl WITH GRANT OPTION';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE VIEW ppsnepl.fdb_plblwarnings_v AS SELECT * FROM fdb_dif.fdb_plblwarnings';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.fdb_plblwarnings_v TO ppsnepl_read_only_role';

      EXECUTE IMMEDIATE vc_sql_statement;


      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  create_ppsn_views');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'create_ppsn_views: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'create_ppsn_views: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line ('create_ppsn_views: ' || vc_sql_statement);
         RETURN (1);
   END create_ppsn_views;

   -----------------------------------------------------------------------------
   -- NAME:         insert_ppsn2_data
   -- TYPE:         Function
   -- DESCRIPTSION: INSERT PPS-N 2.0 data into tables
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION insert_ppsn2_data
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  insert_ppsn2_data');

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_status (STATUS_ID,STATUS_NAME,STATUS_DESC,CREATED_BY, CREATED_DTM) '
         || 'VALUES (1,''Initiated'',''File Creation Process Has Been Initiated'',''Initial Load'',sysdate)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_status (STATUS_ID,STATUS_NAME,STATUS_DESC,CREATED_BY, CREATED_DTM) '
         || 'VALUES (2,''Created'',''All changes to drug information in PPS-N have been collected into a file and placed on the file system'',''Initial Load'',sysdate)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_status (STATUS_ID,STATUS_NAME,STATUS_DESC,CREATED_BY, CREATED_DTM) '
         || 'VALUES (3,''Transmitted to Test  FTP'',''Transmission of the Created file to the test FTP location has been completed'',''Initial Load'',sysdate)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_status (STATUS_ID,STATUS_NAME,STATUS_DESC,CREATED_BY, CREATED_DTM) '
         || 'VALUES (4,''Start of National VistA Processing'',''The Test VistA instance has started the update file process'',''Initial Load'',sysdate)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_status (STATUS_ID,STATUS_NAME,STATUS_DESC,CREATED_BY, CREATED_DTM) '
         || 'VALUES (5,''Completion of National VistA Processing'',''The Test VistA instance has processed the update file and is ready for testing. A user may now access the Update approval functionality of PPS-N'',''Initial Load'',sysdate)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_status (STATUS_ID,STATUS_NAME,STATUS_DESC,CREATED_BY, CREATED_DTM) '
         || 'VALUES (6,''Approved by Test'',''A user has marked this update as Approved in the update management area of PPS-N'',''Initial Load'',sysdate)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_status (STATUS_ID,STATUS_NAME,STATUS_DESC,CREATED_BY, CREATED_DTM) '
         || 'VALUES (7,''Rejected by Test'',''A user has marked this update as Rejected in the update management area of PPS-N'',''Initial Load'',sysdate)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_status (STATUS_ID,STATUS_NAME,STATUS_DESC,CREATED_BY, CREATED_DTM) '
         || 'VALUES (8,''Transmitted to Production VistA'',''Transmission of the approved file to the production FTP location has been completed'',''Initial Load'',sysdate)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_status (STATUS_ID,STATUS_NAME,STATUS_DESC,CREATED_BY, CREATED_DTM) '
         || 'VALUES (99,''NDF Update Process Error'',''An error occurred during NDF update file process'',''Initial Load'',sysdate)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ndf_update_file (NDF_UPDATE_FILE_ID,FILE_NAME,COMMENTS,STATUS_ID_FK,PROD_TRANSMISSION_DATE,TEST_TRANSMISSION_DATE,STATUS_MODIFIED_DATE,FILE_DIRECTORY_PATH,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) '
         || ' VALUES (0,''PPS_0PRV_0NEW.DAT'',''Initial Seed Data'',8,SYSDATE,SYSDATE,SYSDATE,''PPS/PROD'',''Developer'',SYSDATE,''Developer'',SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ndf_update_file : '
         || TO_CHAR (SQL%ROWCOUNT));


      COMMIT;

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_DRUG_UNITS'', 10000, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_DOSAGE_FORMS'', 10000, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_INGREDIENTS'', 10000, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_MANUFACTURERS'', 10000, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_NDCS'', 10000, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_PACKAGE_TYPES'', 10000, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_PRODUCTS'', 10000, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_VA_DISPENSE_UNITS'', 10000, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_VA_DRUG_CLASSES'', 10000, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_VA_GEN_NAMES'', 10000, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));
         
/* - 5/10/2017 D.T. JAVA WILL NOW HANDLE THIS OPERATION *******************************************************************

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ien_generator (CONCEPT, LAST_USED_IEN, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (''EPL_DRUG_DRUG_INTERACTION'', 10843, ''DEV'', SYSDATE, ''DEV'', SYSDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;
*/

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ien_generator : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_vista_status (VISTA_STATUS_ID, STATUS_NAME, STATUS_DESC, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (1, ''STARTED'', ''The process has star...'', ''system'', CURRENT_TIMESTAMP, ''system'', CURRENT_TIMESTAMP)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_vista_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_vista_status (VISTA_STATUS_ID, STATUS_NAME, STATUS_DESC, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (2, ''COMPLETED'', ''The process has comp...'', ''system'', CURRENT_TIMESTAMP, ''system'', CURRENT_TIMESTAMP)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_vista_status : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ddi_severity (EPL_ID , NAME, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (1, ''CRITICAL'', ''360'', TIMESTAMP ''2014-05-09 13:02:39'', ''360'', TIMESTAMP ''2014-05-09 13:02:39'')';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ddi_severity : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'INSERT INTO ppsnepl.epl_ddi_severity (EPL_ID , NAME, CREATED_BY, CREATED_DTM, LAST_MODIFIED_BY, LAST_MODIFIED_DTM) '
         || 'VALUES (2, ''SIGNIFICANT'', ''360'', TIMESTAMP ''2014-05-09 13:10:00'', ''360'', TIMESTAMP ''2014-05-09 13:10:00'')';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ppsnepl.epl_ddi_severity : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  insert_ppsn2_data');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'insert_ppsn2_data: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'insert_ppsn2_data: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line ('insert_ppsn2_data: ' || vc_sql_statement);
         RETURN (1);
   END insert_ppsn2_data;

   -----------------------------------------------------------------------------
   -- NAME:         ndc_process_1
   -- TYPE:         Function
   -- DESCRIPTSION: Create epl_package_size table
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION ndc_process_1
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  ndc_process_1');

      -- Create temp table

      vc_sql_statement :=
         'CREATE TABLE ppsnepl.temp_epl_ndcs AS SELECT * FROM ppsnepl.epl_ndcs';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.temp_epl_ndcs ADD (CONSTRAINT temp_epl_ndcs_pk PRIMARY KEY (epl_id))';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Create epl_package_sizes table

      vc_sql_statement :=
         'CREATE TABLE ppsnepl.epl_package_sizes( 
 epl_id                 NUMBER (30, 0) NOT NULL ENABLE
,package_size           NUMBER (13, 6) NOT NULL ENABLE
,ndf_package_size_ien   NUMBER (30, 0)
,created_by             VARCHAR2 (50 BYTE) NOT NULL ENABLE
,created_dtm            TIMESTAMP (6) NOT NULL ENABLE
,last_modified_by       VARCHAR2 (50 BYTE)
,last_modified_dtm      TIMESTAMP (6))
TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX ppsnepl.epl_package_sizes_pki ON ppsnepl.epl_package_sizes (epl_id) TABLESPACE ppsnepl_data';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_package_sizes ADD (CONSTRAINT epl_package_sizes_pk PRIMARY KEY (epl_id) USING INDEX ppsnepl.epl_package_sizes_pki ENABLE VALIDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE SEQUENCE ppsnepl.package_size_sequence MINVALUE 1 MAXVALUE 9999999999999999999999999999 INCREMENT BY 1 START WITH 108 NOCACHE  NOORDER  NOCYCLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ppsnepl.epl_package_sizes TO PPSNEPL_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  ndc_process_1');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'ndc_process_1: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'ndc_process_1: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line ('ndc_process_1: ' || vc_sql_statement);
         RETURN (1);
   END ndc_process_1;

   -----------------------------------------------------------------------------
   -- NAME:         ndc_process_2
   -- TYPE:         Function
   -- DESCRIPTSION: Process ndcs and package size
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION ndc_process_2
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  ndc_process_2');

      vc_sql_statement :=
         'UPDATE ppsnepl.epl_ndcs SET package_size = NULL WHERE 1 = 1';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_ndcs MODIFY package_size NUMBER (30, 0)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_ndcs RENAME COLUMN package_size TO package_size_id_fk';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'UPDATE ppsnepl.epl_ndcs o
      SET o.package_size_id_fk =
          (SELECT MIN (epl_package_sizes.epl_id)
             FROM ppsnepl.epl_package_sizes, ppsnepl.temp_epl_ndcs
            WHERE     epl_package_sizes.package_size =
                         temp_epl_ndcs.package_size
                  AND temp_epl_ndcs.epl_id = o.epl_id)
      WHERE o.epl_id IN (SELECT epl_id FROM ppsnepl.temp_epl_ndcs)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ppsnepl.epl_ndcs : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
         'CREATE INDEX ppsnepl.epl_ndc_pckg_size_fk_idx ON ppsnepl.epl_ndcs (package_size_id_fk)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE ppsnepl.epl_ndcs ADD FOREIGN KEY(package_size_id_fk) REFERENCES ppsnepl.epl_package_sizes (epl_id)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement := 'DROP SEQUENCE ppsnepl.package_size_sequence';

      EXECUTE IMMEDIATE vc_sql_statement;

      COMMIT;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  ndc_process_2');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'ndc_process_2: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'ndc_process_2: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line ('ndc_process_3: ' || vc_sql_statement);
         RETURN (1);
   END ndc_process_2;

   -----------------------------------------------------------------------------
   -- NAME:         insert_epl_national_settings
   -- TYPE:         Function
   -- DESCRIPTSION: Insert data into epl_national_settings table
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
 
   FUNCTION insert_epl_national_settings
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  insert_epl_national_settings');

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) Values (37, ''NDF_UPDATE_RUN_STATE'', ''COMPLETED'', ''DEVELOPER'', SYSDATE, ''NdfUpdateProcess'', SYSDATE)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) Values   (38, ''NDF_UPDATE_LAST_RUN'', NULL, ''DEVELOPER'', TO_TIMESTAMP(''1/31/2014 12:00:00.000000 AM'',''fmMMfm/fmDDfm/YYYY fmHH12fm:MI:SS.FF AM''),''NdfUpdateProcess'', SYSDATE)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (39,''NDF_OUTGOING_FILE_TEXT'',''unmatched from the National Drug File (NDF).  The VA Product |name and CMOP ID corresponding to the unmatched local drug file |name are listed on the indented line beneath each entry.  An |Inactivation Date may be listed for entries when this reason |applies.  Until you rematch these entries to NDF, they will not |transmit to CMOP and drug-drug interaction checks will not check |for these products.  It is critical that you rematch these |products immediately.  You may also need to match a new |orderable item.  Any possible dosages and local possible |dosages for these unmatched products have been deleted. |Therefore, the dosages for each unmatched product should |be reviewed after the rematch or recreated if the product |can not be rematched to a VA Product through the NDF |matching process.'',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (40,''NDF_OUTPUT_FILE_DIRECTORY'',''./tmp/'',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (41,''NDF_OUTPUT_TST_PROTOCOL'',''FTP'',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (42,''NDF_OUTPUT_TST_HOSTNAME'',''IP       '',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (43,''NDF_OUTPUT_TST_PORT'',''21'',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (44,''NDF_OUTPUT_TST_USERNAME'',''USRVISTA'',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (45,''NDF_OUTPUT_TST_PASSWORD'','' '',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (46,''NDF_OUTPUT_TST_DIRECTORY'',''$1$DGA55:[PMIS.MARK]'',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (47,''NDF_OUTPUT_PROD_PROTOCOL'',''FTP'',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (48,''NDF_OUTPUT_PROD_HOSTNAME'',''IP       '',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (49,''NDF_OUTPUT_PROD_PORT'',''21'',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (50,''NDF_OUTPUT_PROD_USERNAME'',''USRVISTA'',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (51,''NDF_OUTPUT_PROD_PASSWORD'','' '',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (52,''NDF_OUTPUT_PROD_DIRECTORY'',''$1$DGA55:[PMIS.MARK]'',null,null,null,null,''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''),''DEVELOPER'',to_timestamp(''21-MAY-14 12.00.00.000000000 AM'',''DD-MON-RR HH.MI.SS.FF AM''))';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (54,''MESSAGE_STRENGTH_TEXT'','' The following Strength(s) have been edited or added. Any product matched to| these products will be unmatched. If site wants to continue to use the| product the site must rematch to local file #50 entries to the listed VA| products.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (55,''MESSAGE_OVERRIDEDOSECHECK_TEXT'','' The override dose form checks field has changed for the following products.|'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (56,''MESSAGE_NATIONALFORMULARY_TEXT'','' The National Formulary Indicator has changed for the following| VA Products.  The National Formulary Indicator will automatically| be changed in your local DRUG file (#50).  Please review the| VISN and Local Formulary designations of these products and| make appropriate changes.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (57,''MESSAGE_INACTIVATEDCMOP_TEXT'','' The following active VA Products are no longer marked for CMOP.| All local drug file entries matched to these VA Products will| be UNMARKED for CMOP.  In order to have these entries dispensed| by CMOP any local DRUG file (#50) entries matched to these| products must be re-matched to another VA product that is actively| marked for CMOP dispensing.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (58,''MESSAGE_ADDEDPRODUCT_TEXT'','' The following VA Products have been added to the National| Drug File.  You may wish to review, then match or unmatch| local drug file entries based on this updated information.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (59,''MESSAGE_SCHEDULECHANGE_TEXT'','' The CS Federal Schedule for the following VA Products has| been added or changed.  You may wish to review the DRUG| file (#50) and make appropriate changes to the DEA, SPECIAL| HDLG field (#3).'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (60,''MESSAGE_POSSIBLEDOSAGE_TEXT'','' The Auto-Create Possible Dosages settings have been edited for the| following VA Products. Please review your local dosages for products| matched to these entries. Edits to your site''''s possible dosages or| local possible dosages may be needed.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (61,''MESSAGE_PRINTNAMECHANGE_TEXT'','' The following VA Print Name(s) have been edited or added. Any product matched| to these products will be unmatched. If site wants to continue to use the| the product the site must rematch to local file #50 entries to the listed VA| product the site must rematch to local file #50 entries to the listed VA| product.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (62,''MESSAGE_DRUGUNITCHANGE_TEXT'','' The following Unit(s) have been edited or added. Any product matched to these| products will be unmatched. If site wants to continue to use the product the| site must rematch to local file #50 entries to the listed VA Product.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (63,''MESSAGE_DISPENSEUNITCHANGE_TEXT'','' The following VA Dispense Units(s) have been edited or added. Any product| matched to these products will be unmatched. If site wants to continue to use| the product the site must rematch to local file #50 entries to the listed VA| product.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (64,''MESSAGE_VACLASSCHANGE_TEXT'','' The following VA Drug Class(es) have been edited or added. The VA Class| for this product will be automatically updated in file #50.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (65,''MESSAGE_CMOPCHANGE_TEXT'','' The following VA Product Identifier(s) have been edited or added. Any product| matched to these products will be unmatched. If site wants to continue to use| the product the site must rematch to local file #50 entries to the listed VA| product.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (66,''MESSAGE_FUTUREINACTIVE_TEXT'','' The following products will be inactivated on the date listed. No alternative| products have been found.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (67,''MESSAGE_SCHEDULECHANGEALL_TEXT'','' The following CS Federal Schedule(s) have been edited or added. Please review| the local DEA special handling field and make edits are required.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (68,''MESSAGE_GENERICNAMECHANGE_TEXT'','' The following VA Generic Name(s) have been edited or added. Any product| matched to these products will be unmatched. If site wants to continue to use| the product the site must rematch to local file #50 entries to the listed VA| product.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (69,''MESSAGE_DOSAGEFORMCHANGE_TEXT'','' The following VA Dosage Form(s) have been edited. Any product matched to| these products will be unmatched. If site wants to continue to use the| product the site must rematch to local file #50 entries to the listed VA| product.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (70,''MESSAGE_INACTIVATEDPRODUCT_TEXT'','' The following product(s) have been inactivated. No alternative products have| been found.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (71,''MESSAGE_REACTIVATEDPRODUCT_TEXT'','' The following VA Product(s) have been reactivated.|'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (72,''MESSAGE2_MEDGUIDE_TEXT'','' The FDA Med Guide for the following products has been changed|'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (73,''MESSAGE2_EXCLUEDEDDDI_TEXT'','' The following products have been flagged for exclusion from drug-drug| interaction checks.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (74,''MESSAGE2_PREVEXCLUEDEDDDI_TEXT'','' The following products, previously flagged for exclusion from drug-drug| interaction checks, have been changed to be included| in drug-drug interaction checks where appropriate.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (75,''MESSAGE2_DDI_TEXT'','' The following interactions in National Drug File (NDF) have been added,| edited or inactivated.  These changes are the result of review and| recommendations from the NDF support group.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'INSERT INTO ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (76,''NATIONAL_SITE_NUMBER'',NULL,442,NULL,NULL,NULL,''DEVELOPER'',SYSDATE,''DEVELOPER'',SYSDATE)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (77,''MESSAGE_INACTIVATEDWSUGG_TEXT'',''The following product(s) have been inactivated. Any file #50 entries matched|to these products will be unmatched. To continue to use the product, the site|must rematch the local file #50 entries to the listed VA Product.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (78,''MESSAGE_PROPOSEDINACTIVATEDWSUGG_TEXT'',''The following product(s) WILL be inactivated in on the date listed. Any file|#50 entries matched to these products will be unmatched once the product is|inactivated. If a site wants to continue to use the product, the site must|rematch the local file #50 entries to the listed VA Product.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'INSERT INTO ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) VALUES (79, ''REPORT_DATA_UPDATE_START'', null, null, null, null, null, ''DEVELOPER'', sysdate, ''DEVELOPER'', sysdate)';

      EXECUTE IMMEDIATE
         'INSERT INTO ppsnepl.epl_national_settings (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) VALUES (80, ''REPORT_DATA_UPDATE_COMPLETE'', null, null, null, null, null, ''DEVELOPER'', sysdate, ''DEVELOPER'', sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.EPL_NATIONAL_SETTINGS (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (81,''MESSAGE_INACTIVATEDREMATCHED_TEXT'',''The following product(s) have been inactivated. Any file|#50 entries matched to these products will be unmatched and rematched to the|local file #50 entries listed.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.EPL_NATIONAL_SETTINGS (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (82,''MESSAGE_OTHERREMATCHED_TEXT'',''The following product(s) have been unmatched. Any file|#50 entries matched to these products will be unmatched and rematched to the|local file #50 entries listed.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.EPL_NATIONAL_SETTINGS (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (83,''MESSAGE_OTHERREMATCHSUGG_TEXT'',''The following product(s) have been unmatched Any file|#50 entries matched to these products will be unmatched. To continue to use|the product, the site must rematch the local file #50 entries to the listed VA Product.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      EXECUTE IMMEDIATE
         'Insert into ppsnepl.EPL_NATIONAL_SETTINGS (ID,KEY_NAME,STRING_VALUE,INTEGER_VALUE,DECIMAL_VALUE,BOOLEAN_VALUE,DATE_VALUE,CREATED_BY,CREATED_DTM,LAST_MODIFIED_BY,LAST_MODIFIED_DTM) values (84,''MESSAGE_OTHERNOREMATCH_TEXT'',''The following product(s) have been unmatched. No alternative products|have been found.'',null,null,null,null,''DEVELOPER'',sysdate,''DEVELOPER'',sysdate)';

      COMMIT;

      EXECUTE IMMEDIATE 'SELECT COUNT (*) FROM ppsnepl.epl_national_settings'
         INTO vn_row_count;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) in ppsnepl.epl_national_settings : '
         || TO_CHAR (vn_row_count));

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  insert_epl_national_settings');

      IF vn_row_count = 0
      THEN
         RETURN (1);
      END IF;

      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'insert_epl_national_settings: Error code '
            || vn_code
            || ': '
            || vc_errm);
         RETURN (1);
   END insert_epl_national_settings;

   -----------------------------------------------------------------------------
   -- NAME:         update_national_setting
   -- TYPE:         Function
   -- DESCRIPTSION: Update the epl_national_settings table
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION update_national_setting
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  update_national_setting');

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following Strength(s) have been edited or added. Any product matched to|these products will be unmatched. If site wants to continue to use the|product the site must rematch to local file #50 entries to the listed VA|products.'' where id =53';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The override dose form checks field has changed for the following products.|'' where id =54';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The National Formulary Indicator has changed for the following|VA Products. The National Formulary Indicator will automatically|be changed in your local DRUG file (#50). Please review the|VISN and Local Formulary designations of these products and|make appropriate changes.'' where id =55';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following active VA Products are no longer marked for CMOP.|All local drug file entries matched to these VA Products will|be UNMARKED for CMOP. In order to have these entries dispensed|by CMOP any local DRUG file (#50) entries matched to these|products must be re-matched to another VA product that is actively|marked for CMOP dispensing.'' where id =56';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following VA Products have been added to the National|Drug File.You may wish to review, then match or unmatch|local drug file entries based on this updated information.'' where id =57';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The CS Federal Schedule for the following VA Products has|been added or changed.You may wish to review the DRUG|file (#50) and make appropriate changes to the DEA, SPECIAL|HDLG field (#3).'' where id =58';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The Auto-Create Possible Dosages settings have been edited for the|following VA Products. Please review your local dosages for products|matched to these entries. Edits to your site''''s possible dosages or|local possible dosages may be needed.'' where id =59';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following VA Print Name(s) have been edited or added. Any product matched|to these products will be unmatched. If site wants to continue to use the|the product the site must rematch to local file #50 entries to the listed VA|product the site must rematch to local file #50 entries to the listed VA|product.'' where id =60';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following Unit(s) have been edited or added. Any product matched to these|products will be unmatched. If site wants to continue to use the product the|site must rematch to local file #50 entries to the listed VA Product.'' where id =61';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following VA Dispense Units(s) have been edited or added. Any product|matched to these products will be unmatched. If site wants to continue to use|the product the site must rematch to local file #50 entries to the listed VA|product.'' where id =62';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following VA Drug Class(es) have been edited or added. The VA Class|for this product will be automatically updated in file #50.'' where id =63';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following VA Product Identifier(s) have been edited or added. Any product|matched to these products will be unmatched. If site wants to continue to use|the product the site must rematch to local file #50 entries to the listed VA|product.'' where id =64';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following products will be inactivated on the date listed. No alternative|products have been found.'' where id =65';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following CS Federal Schedule(s) have been edited or added. Please review|the local DEA special handling field and make edits are required.'' where id =66';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following VA Generic Name(s) have been edited or added. Any product|matched to these products will be unmatched. If site wants to continue to use|the product the site must rematch to local file #50 entries to the listed VA|product.'' where id =67';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following VA Dosage Form(s) have been edited. Any product matched to|these products will be unmatched. If site wants to continue to use the|product the site must rematch to local file #50 entries to the listed VA|product.'' where id =68';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following product(s) have been inactivated. No alternative products have|been found.'' where id =69';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following VA Product(s) have been reactivated.|'' where id =70';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The FDA Med Guide for the following products has been changed|'' where id =71';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following products have been flagged for exclusion from drug-drug|interaction checks.'' where id =72';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following products, previously flagged for exclusion from drug-drug|interaction checks, have been changed to be included|in drug-drug interaction checks where appropriate.'' where id =73';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set STRING_VALUE =''The following interactions in National Drug File (NDF) have been added,|edited or inactivated. These changes are the result of review and|recommendations from the NDF support group.'' where id =74';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set string_value = ''The following VA Print Name(s) have been edited or added. Any product matched|to these products will be unmatched. If site wants to continue to use the|product the site must rematch to local file #50 entries to the listed VA product.'' where id = 60';

      EXECUTE IMMEDIATE
         'UPDATE ppsnepl.epl_national_settings set string_value = ''The following product(s) have been edited Any file|#50 entries matched to these products will be unmatched. To continue to use|the product, the site must rematch the local file #50 entries to the listed VA Product.'' where id = 82';

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  update_national_setting');

      IF vn_row_count = 0
      THEN
         RETURN (1);
      END IF;

      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'update_national_setting: Error code '
            || vn_code
            || ': '
            || vc_errm);
         RETURN (1);
   END update_national_setting;

   -----------------------------------------------------------------------------
   -- NAME:         turn_vista_messaging_off
   -- TYPE:         Function
   -- DESCRIPTSION: Update epl_national_setting table to turn vist_messaging_off
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION turn_vista_messaging_off
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  turn_vista_messaging_off');

      EXECUTE IMMEDIATE
         'update ppsnepl.epl_national_settings set boolean_value = 0 where id = 14';

      EXECUTE IMMEDIATE
         'update ppsnepl.epl_national_settings set string_value = null where id = 15';

      EXECUTE IMMEDIATE
         'update ppsnepl.epl_national_settings set integer_value = 0 where id = 16';

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  turn_vista_messaging_off');

      IF vn_row_count = 0
      THEN
         RETURN (1);
      END IF;

      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'turn_vista_messaging_off: Error code '
            || vn_code
            || ': '
            || vc_errm);
         RETURN (1);
   END turn_vista_messaging_off;
-----------------------------------------------------------------------------
-- MAIN SECTION
-----------------------------------------------------------------------------
BEGIN
   sys.DBMS_OUTPUT.put_line (CHR (10));
   sys.DBMS_OUTPUT.put_line (
      'INFORMATION -   Checking to see if PPS-N 2.0 table(s) already exists.');

   vn_continue := check_for_ppsn2;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -     ONE OR MORE PPS-N 2.0 OBJECTS ALREADY EXISTS. STOPPING MIGRATION.');
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPS-N 2.0 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := create_ppsn_tables;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPS-N 2.0 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := create_ppsn_views;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPS-N 2.0 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := insert_ppsn2_data;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPS-N 2.0 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := ndc_process_1;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPS-N 2.0 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := ndc_process_2;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPS-N 2.0 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;
   
   vn_continue := insert_epl_national_settings;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPS-N 2.0 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := update_national_setting;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPS-N 2.0 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := turn_vista_messaging_off;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPS-N 2.0 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   -- Recompile Schema and Gather Statistics
   DBMS_UTILITY.compile_schema (schema => 'PPSNEPL');
   DBMS_STATS.gather_schema_stats ('PPSNEPL');
EXCEPTION
   WHEN OTHERS
   THEN
      vn_code := SQLCODE;
      vc_errm := SUBSTR (SQLERRM, 1, 64);
      sys.DBMS_OUTPUT.put_line (
         'MAIN: Error code ' || vn_code || ': ' || vc_errm);
END;
/

SELECT 'Migration Stopped: ' || CURRENT_TIMESTAMP FROM DUAL;

SPOOL OFF
EXIT