package gov.va.med.pharmacy.peps.common.vo;

import java.lang.reflect.Field;
import java.util.Collections;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import gov.va.med.pharmacy.peps.common.exception.ValueObjectValidationException;
import gov.va.med.pharmacy.peps.common.vo.validator.ErrorKey;
import gov.va.med.pharmacy.peps.common.vo.validator.Errors;

/**
 * Represents a rematch, which is a reassociation of a local product from a national product to
 * another national product. The rematched national product does not have to be different than
 * the current national product.
 */
public class RematchVo extends ValueObject {
    
    private static final Logger LOG = LogManager.getLogger(RematchVo.class);

  private static final long serialVersionUID = 2L;

  private String eplRematchId;
  private final ProductVo fromProduct;
  private ProductVo toAutomaticProduct;
  private Set<ProductVo> toSuggestedProducts;
  private MatchReason matchReason;
  private NdfUpdateFileVo ndfUpdateFile;

    
    /**
     * Constructs an instance with the required parameters.
     * @param pFromProduct See {@link #getFromProduct()}. Must not be null.
     * @param pMatchReason See {@link #getMatchReason()}. Must not be null.
     */
    protected RematchVo(final ProductVo pFromProduct, MatchReason pMatchReason) {
      fromProduct = Objects.requireNonNull(pFromProduct);
      matchReason = Objects.requireNonNull(pMatchReason);
    }
   
    /**
     * Creates an automatic rematch with the required properties.
     * 
     * @param fromProduct see {@link #getFromProduct()}
     * @param toAutomaticProduct see {@link #getToAutomaticProduct()}. Must not be null.
     * @param matchReason see {@link #getMatchReason()}
     * @return the created object, never null.
     */
    public static RematchVo automaticRematch(final ProductVo fromProduct, final ProductVo toAutomaticProduct,
        final MatchReason matchReason) {
      RematchVo automaticRematch = new RematchVo(fromProduct, matchReason);
      automaticRematch.setToAutomaticProduct(Objects.requireNonNull(toAutomaticProduct));
      return automaticRematch;
    }
    
    /**
     * Creates a suggested rematch with the required properties.
     * 
     * @param fromProduct see {@link #getFromProduct()}
     * @param toSuggestedProduct see {@link #getToSuggestedProducts()}. Must not be null or empty.
     * @param matchReason {@link #getMatchReason()}
     * @return the created object, never null.
     */
    public static RematchVo suggestedRematch(final ProductVo fromProduct, final Set<ProductVo> toSuggestedProduct,
        final MatchReason matchReason) {
      if (toSuggestedProduct.isEmpty()) {
        throw new IllegalArgumentException("toSuggestedProduct must not be empty.");
      }
      RematchVo suggestedRematch = new RematchVo(fromProduct, matchReason);
        suggestedRematch.setToSuggestedProducts(toSuggestedProduct);
      return suggestedRematch;
    }
    

  /**
   * The product to match from. The returned product will be at least minimally populated and may
   * be fully populated.
   * 
   * @return never null
   */
  public ProductVo getFromProduct() {
    return fromProduct;
  }

  /**
   * The automatic product to match to. The returned product will be at least minimally populated
   * and may be fully populated.
   * 
   * @return possibly null
   */
  public ProductVo getToAutomaticProduct() {
    return toAutomaticProduct;
  }

  /**
   * Sets the automatic product and clears out the suggested product set.
   * @param automaticProduct the automatic product to set
   */
  public void setToAutomaticProduct(ProductVo automaticProduct) {
    this.toSuggestedProducts = null;
    this.toAutomaticProduct = automaticProduct;
  }
  

  public Set<ProductVo> getToSuggestedProducts() {
    return toSuggestedProducts;
  }

  /**
   * Sets the suggested product set and clears out the automatic product.
   * @param suggestedProductSet the suggested product set to set
   */
  public void setToSuggestedProducts(Set<ProductVo> suggestedProductSet) {
    this.toAutomaticProduct = null;
    this.toSuggestedProducts = suggestedProductSet;
  }
    
    /**
     * Returns the database key of the rematch. See EplRematchDo.getEplRematchId().
     * @return the database key if the rematch has been saved to the database, or null otherwise
     */
    public String getEplRematchId() {
    
        return eplRematchId;
    }

    
    public void setEplRematchId(String eplRematchId) {
    
        this.eplRematchId = eplRematchId;
    }

    /**
     * Returns the reason for the rematch. Multiple difference modifications can actually trigger a rematch at the same
     * time, but we only store the first one (in {@link MatchReason} order) for now. For example, a {@link
     * MatchReason#INACTIVATION} rematch will override a {@link MatchReason#FUTURE_INACTIVATION} rematch.
     * 
     * @return never null.
     */
    public MatchReason getMatchReason() {
    
        return matchReason;
    }
    
  /**
   * See {@link #getMatchReason()}.
   * 
   * @param pMatchReason must not be null
   */
  public void setMatchReason(MatchReason pMatchReason) {
    matchReason = Objects.requireNonNull(pMatchReason);
  }
  
    /**
     * Returns the update file that contains the rematch.
     * @return the update file VO reference
     */
    public NdfUpdateFileVo getNdfUpdateFile() {
    
        return ndfUpdateFile;
    }

    
    public void setNdfUpdateFile(final NdfUpdateFileVo pNdfUpdateFile) {
    
        ndfUpdateFile = pNdfUpdateFile;
    }
        
  /**
   * Returns whether the rematch is automatic or suggested.
   * 
   * @return refer to description
   */
  public boolean getIsAutomaticRematch() {
    if (toAutomaticProduct != null) {
      return true;
    }
    return false;
  }

  /**
   * Returns the set of products to match to.
   * 
   * @return refer to the description
   */
  public Set<ProductVo> getMatchProducts() {
    if (getIsAutomaticRematch()) {
      return Collections.singleton(toAutomaticProduct);
    }
    return getToSuggestedProducts();
  }
  
  /**
   * Returns a human-readable string identifying the "to" products. E.g., "Suggested rematch to Product A; Product B."
   * @return never null.
   */
  public String getToProductsString() {
      final String delimiter = "; "; // Use semicolon because many products have commas in their names.

      return String.format("%s rematch to %s",
          getIsAutomaticRematch() ? "Automatic" : "Suggested",
          getMatchProducts().stream().map(ProductVo::toShortString).collect(Collectors.joining(delimiter)));
  }
  
  /**
   * <p>Validates that the to products are all active.</p>
   * 
   * <p>
   * Note that this method is not currently called by {@link #validate(UserVo, Environment, int)}. It would match the
   * architecture more to put this code in a separate Validator class, but no time for that now.
   * </p>
   * 
   * @throws ValueObjectValidationException if any of the to products are not active.
   */
  public void validateMatchProducts() throws ValueObjectValidationException {
      final Errors errors = new Errors();
      
      for (final ProductVo toProduct : getMatchProducts()) {
          if (!toProduct.getItemStatus().equals(ItemStatus.ACTIVE)) {
              LOG.debug("Product {} is not active.", toProduct.getId());
              errors.addError(ErrorKey.REMATCH_INACTIVE_PRODUCT, toProduct.getProductLongName());
          }
      }
      
      if (errors.hasErrors()) {
          throw new ValueObjectValidationException(errors);
      }
  }

    @Override
    public <T> T reflectFieldValue(Field field) throws IllegalArgumentException, IllegalAccessException {
        return (T) field.get(this);
    }

    @Override
    public void setAttributeByField(Field field, Object fieldValue) throws IllegalArgumentException, IllegalAccessException {
        field.set(this, fieldValue);
    }

}
