package gov.va.med.pharmacy.peps.common.vo;

import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * SyncProcessorElement is a data container for the NdfFileSyncProcessor classes to use as they are processing fields and
 * gathering the data needed to eventually create an EplNdfOutgoingDifference record.
 */
public class SyncProcessorElement {
    /**
     * Convenience constant which clients may pass to the constructor for the isSingleField argument to indicate that this
     * SyncProcessorElement represents a single (i.e., non-multiple) field in VistA.
     */
    public static final boolean IS_SINGLE_FIELD = true;

    /**
     * Convenience constant which clients may pass to the constructor for the isSingleField argument to indicate that this
     * SyncProcessorElement represents a multiple field in VistA.
     */
    public static final boolean IS_MULTIPLE_FIELD = false;

    /**
     * The VistA file number.
     */
    private String vistaFileNumber;

    /** The vista ien. */
    private String vistaIen;

    /** The new value. */
    private String newValue;

    /** The user name. */
    private String userName;

    /** The field id. */
    private String fieldId;

    /** The old value. */
    private String oldValue;

    /**
     * True if this SyncProcessorElement represents a "single" field in VistA; false if it represents a "multiple" field.
     */
    private boolean isSingleField;

    /**
     * Instantiates a new sync processor element with the parameters used in the various insert methods in the SyncProcessors.
     *
     * @param vistaFileNumber the Vista file number with which the field is associated
     * @param vistaIen the vista ien to the item with which the field is associated
     * @param newValue the new value of the field
     * @param userName the user name making or approving the change
     * @param fieldId the Vista field number
     * @param oldValue the old value used only when modifying a field
     */
    public SyncProcessorElement(String vistaFileNumber, String vistaIen, String newValue, String userName, String fieldId,
        String oldValue) {
        this(vistaIen, newValue, userName, fieldId);
        this.vistaFileNumber = vistaFileNumber;
        this.oldValue = oldValue;
    }

    /**
     * Instantiates a new sync processor element with the parameters used in the various insert methods in the SyncProcessors.
     *
     * @param vistaIen the vista ien to the item with which the field is associated
     * @param newValue the new value of the field
     * @param userName the user name making or approving the change
     * @param fieldId the Vista field number
     */
    public SyncProcessorElement(String vistaIen, String newValue, String userName, String fieldId) {
        super();
        this.vistaIen = vistaIen;
        this.newValue = newValue;
        this.userName = userName;
        this.fieldId = fieldId;
        this.isSingleField = IS_SINGLE_FIELD;
    }

    /**
     * Instantiates a new sync processor element with the parameters used in the various insert methods in the SyncProcessors.
     *
     * @param vistaFileNumber the Vista file number with which the field is associated
     * @param vistaIen the vista ien to the item with which the field is associated
     * @param newValue the new value of the field
     * @param userName the user name making or approving the change
     * @param fieldId the Vista field number
     * @param isSingleField true if this represents an atomic field in VistA; false if this represents one component of a
     *            multiple field in VistA
     */
    public SyncProcessorElement(String vistaFileNumber, String vistaIen, String newValue, String userName, String fieldId,
        boolean isSingleField) {
        this(vistaIen, newValue, userName, fieldId);
        this.vistaFileNumber = vistaFileNumber;
        this.isSingleField = isSingleField;
    }

    /**
     * Gets the vista ien.
     *
     * @return the vistaIen
     */
    public final String getVistaIen() {
        return vistaIen;
    }

    /**
     * Sets the vista ien.
     *
     * @param vistaIen the vistaIen to set
     */
    public final void setVistaIen(String vistaIen) {
        this.vistaIen = vistaIen;
    }

    /**
     * Gets the new value.
     *
     * @return the newValue
     */
    public final String getNewValue() {
        return newValue;
    }

    /**
     * Sets the new value.
     *
     * @param newValue the newValue to set
     */
    public final void setNewValue(String newValue) {
        this.newValue = newValue;
    }

    /**
     * Gets the user name.
     *
     * @return the userName
     */
    public final String getUserName() {
        return userName;
    }

    /**
     * Sets the user name.
     *
     * @param userName the userName to set
     */
    public final void setUserName(String userName) {
        this.userName = userName;
    }

    /**
     * Gets the field id.
     *
     * @return the fieldId
     */
    public final String getFieldId() {
        return fieldId;
    }

    /**
     * Sets the field id.
     *
     * @param fieldId the fieldId to set
     */
    public final void setFieldId(String fieldId) {
        this.fieldId = fieldId;
    }

    /**
     * Gets the old value.
     *
     * @return the oldValue
     */
    public final String getOldValue() {
        return oldValue;
    }

    /**
     * Sets the old value.
     *
     * @param oldValue the oldValue to set
     */
    public final void setOldValue(String oldValue) {
        this.oldValue = oldValue;
    }

    /**
     * Reduce the list of elements to a single-value checksum. Two lists with equal checksums are considered equivalent for the
     * purpose of synchronizing PPS-N changes with VistA.
     * 
     * @param elements to reduce to a checksum
     * @param valueFunction which, given a SyncProcessorElement, returns the value to be used for that element within in the
     *            checksum
     * @return checksum
     */
    public static String checksumReduce(List<SyncProcessorElement> elements,
        Function<SyncProcessorElement, String> valueFunction) {
        String result = elements.stream().map(valueFunction).collect(Collectors.joining("~"));

        /*
         * TODO Do we need to sort the list, so that the checksum is not sensitive to the ordering of the values in a group list
         * data field? As in:
         * 
         *     String result = elements.stream().map(valueFunction).sorted().collect(Collectors.joining("~"));
         * 
         * The sorting may need to be done at the VO collection level, before this method is called.
         */

        if (result.isEmpty()) {
            result = null;
        }

        return result;
    }

    /**
     * Checks whether this element represents a single field in VistA.
     * 
     * @return true if this is a single field; false if this is a multiple field
     */
    public boolean isSingleField() {
        return isSingleField;
    }

    /**
     * Set the isSingleField.
     * 
     * @param pSingleField true if this is a single field; false if this is a
     * multiple field
     */
    public void setSingleField(boolean pSingleField) {
        this.isSingleField = pSingleField;
    }

    /**
     * Get the file number.
     * 
     * @return the file number
     */
    public String getFileNumber() {
        return vistaFileNumber;
    }
}
