package gov.va.med.pharmacy.peps.domain.common.capability.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.CopayTierMigrationDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.SeqNumDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.EplCopayTierDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProdCopayTierAssocDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProductDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplCopayTierDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProdCopayTierAssocDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.ProductConverter;

/**
 * Main implementation of {@link CopayTierMigrationDomainCapability}.
 */
public class CopayTierMigrationDomainCapabilityImpl implements CopayTierMigrationDomainCapability {

  private static final Logger LOG = LogManager.getLogger(CopayTierMigrationDomainCapabilityImpl.class);
  private static String PRODUCT_EPL_ID = "eplProduct.eplId";

  private final ProductConverter productConverter;
  private final EplProductDao eplProductDao;
  private final EplCopayTierDao eplCopayTierDao;
  private final EplProdCopayTierAssocDao eplProdCopayTierAssocDao;
  private final SeqNumDomainCapability seqNumDomainCapability;

  /**
   * Constructs an instance.
   * 
   * @param pProductConverter the {@link ProductConverter} to use
   * @param pEplProductDao the {@link EplProductDao} to use
   * @param pEplCopayTierDao the {@link EplCopayTierDao} to use
   * @param pEplProdCopayTierAssocDao the {@link EplProdCopayTierAssocDo} to use
   * @param pSeqNumDomainCapability the {@link SeqNumDomainCapability} to use
   */
  @Autowired
  public CopayTierMigrationDomainCapabilityImpl(
      final ProductConverter pProductConverter,
      final EplProductDao pEplProductDao,
      final EplCopayTierDao pEplCopayTierDao,
      final EplProdCopayTierAssocDao pEplProdCopayTierAssocDao,
      final SeqNumDomainCapability pSeqNumDomainCapability) {
    productConverter = pProductConverter;
    eplProductDao = pEplProductDao;
    eplCopayTierDao = pEplCopayTierDao;
    eplProdCopayTierAssocDao = pEplProdCopayTierAssocDao;
    seqNumDomainCapability = pSeqNumDomainCapability;
  }

  @Override
  public final EplProductDo retrieveProduct(int vuid) {
    List<EplProductDo> productList = null;
    EplProductDo productDo = null;

    try {
      productList = getEplProductDao().retrieve(EplProductDo.VUID, new Integer(vuid).toString());
      if (productList.size() == 0) {
        LOG.info("NO MATCHING VUID in the database " + vuid);
        return null;
      } else {
        productDo = productList.get(0);
      }

      return productDo;
    } catch (RuntimeException ex) {
      LOG.error("Exception retrieving product by VUID " + vuid, ex);
      throw ex;
    }
  }

  @Override
  public final List<EplProdCopayTierAssocDo> retrieveCopayTierAssocs(EplProductDo eplProductDo) {

    if (eplProductDo.getEplProdCopayTierAssocs() == null) {
      if (getEplProdCopayTierAssocDao() != null) {
        List<EplProdCopayTierAssocDo> copayTierList = getEplProdCopayTierAssocDao().retrieve(PRODUCT_EPL_ID,
            eplProductDo.getEplId());
        return copayTierList;
      }
    }
    return new ArrayList<EplProdCopayTierAssocDo>(eplProductDo.getEplProdCopayTierAssocs());
  }

  @Override
  public final List<EplCopayTierDo> retrieveCopayTiers() {

    List<EplCopayTierDo> copayTierList = null;
    try {
      copayTierList = getEplCopayTierDao().retrieve();
      if (copayTierList.size() == 0) {
        throw new ItemNotFoundException(ItemNotFoundException.ITEM_NOT_FOUND, new Object[] { "No Copay Tier Defined" });
      }
    } catch (Exception ex) {
      LOG.warn("Encountered Exception while attempting to load defined copay tiers.", ex);
      return null;
    }

    return copayTierList;
  }

  @Override
  public void insertCopayTierAssoc(EplProdCopayTierAssocDo data, UserVo user) {

    eplProdCopayTierAssocDao.insert(data, user);

  }

  @Override
  public void updateCopayTierAssoc(EplProdCopayTierAssocDo data, UserVo user) {

    eplProdCopayTierAssocDao.update(data, user);

  }

  @Override
  public void deleteCopayTierAssoc(EplProdCopayTierAssocDo data) {

    eplProdCopayTierAssocDao.delete(data);

  }

  @Override
  public void deleteCopayTierAssocs(String fieldName, Object value) {

    eplProdCopayTierAssocDao.delete(fieldName, value);

  }

  @Override
  public ProductConverter getProductConverter() {
    return productConverter;
  }

  /**
   * Get the EplProductDao.
   */
  private EplProductDao getEplProductDao() {
    return eplProductDao;
  }

  /**
   * Get the EplCopayTierDao.
   */
  private EplCopayTierDao getEplCopayTierDao() {
    return eplCopayTierDao;
  }

  /**
   * Get the EplProdCopayTierAssocDao.
   */
  private EplProdCopayTierAssocDao getEplProdCopayTierAssocDao() {
    return eplProdCopayTierAssocDao;
  }

  @Override
  public SeqNumDomainCapability getSeqNumDomainCapability() {
    return seqNumDomainCapability;
  }
}
