/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.domain.common.capability.impl;


import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.hibernate.Criteria;
import org.hibernate.criterion.Restrictions;

import gov.va.med.pharmacy.peps.common.exception.DuplicateItemException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.utility.PPSConstants;
import gov.va.med.pharmacy.peps.common.vo.DosageFormVo;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.DosageFormDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.DataAccessObject;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDfMedRtDfAssocDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDfNounDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDfUnitDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDispenseUnitsPerDoseDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDosageFormDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplOrderableItemDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplDosageFormDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplDrugUnitDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplOrderableItemDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplVadfOwnerDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.DataFieldsConverter;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.DosageFormConverter;
import gov.va.med.pharmacy.peps.domain.common.utility.querybuilder.HqlBuilder;


/**
 * Perform CRUD operations on dosage forms.
 */
public class DosageFormDomainCapabilityImpl extends
        ManagedDataDomainCapabilityImpl<DosageFormVo, EplDosageFormDo>
        implements DosageFormDomainCapability {

    //private static final org.apache.logging.log4j.Logger LOG = org.apache.logging.log4j.LogManager
    //        .getLogger(DosageFormDomainCapabilityImpl.class);

    /** The dosage form epl id. */
    private static String DOSAGE_FORM_EPL_ID = "eplDosageForm.eplId";

    /** The epl dosage form dao. */
    private EplDosageFormDao eplDosageFormDao;
    
    /** The epl df noun dao. */
    private EplDfNounDao eplDfNounDao;
    
    /** The epl df unit dao. */
    private EplDfUnitDao eplDfUnitDao;
    
    /** The epl orderable item dao. */
    private EplOrderableItemDao eplOrderableItemDao;
    
    /** The epl df med rt df assoc dao. */
    private EplDfMedRtDfAssocDao eplDfMedRtDfAssocDao;
    
    /** The epl dispense units per dose dao. */
    private EplDispenseUnitsPerDoseDao eplDispenseUnitsPerDoseDao;

    /** The data fields converter. */
    private DataFieldsConverter dataFieldsConverter;
    
    /** The dosage form converter. */
    private DosageFormConverter dosageFormConverter;

    /**
     * Insert the given {@link ManagedItemVo} with a duplicate check.
     * 
     * @param managedItem {@link ManagedItemVo}
     * @param user {@link UserVo} performing the action
     * @return {@link ManagedItemVo} inserted with new ID
     * @throws DuplicateItemException if item already exists by uniqueness fields
     */
    @Override
    public DosageFormVo create(DosageFormVo managedItem, UserVo user) throws DuplicateItemException {
        // We generate the IEN immediately upon creating the item in the database. If the item is subsequently deleted
        // before sending it to VistA, then we will create a gap in VistA's IEN sequence. That is OK according to the
        // VistA developers.
        Long id = getSeqNumDomainCapability().generateIen("EPL_DOSAGE_FORMS", user);
        managedItem.setDosageFormIen(id.toString());
        return super.create(managedItem, user);
    }
    
    /* (non-Javadoc)
     * @see gov.va.med.pharmacy.peps.domain.common.capability.impl.ManagedItemDomainCapabilityImpl#deleteItem(
     *      gov.va.med.pharmacy.peps.common.vo.ManagedItemVo)
     */
    @Override
    public void deleteItem(DosageFormVo dosageFormVo) throws ValidationException {
        List<EplDosageFormDo> dosageFormDo = getDataAccessObject().retrieve(EplDrugUnitDo.EPL_ID,
            Long.valueOf(dosageFormVo.getId()));

        if (dosageFormDo.size() == 1) {
            try {
                checkForActiveDependencies(dosageFormVo, null);
            } catch (Exception e) {
                throw new ValidationException(ValidationException.CANNOT_DELETE, dosageFormVo.getDosageFormName(),
                    " because another item is using it.");
            }
            //delete sub-tables first to avoid referential integrity issues.            
            deleteChildRecords(dosageFormVo);                      
                        
            getDataAccessObject().delete(dosageFormDo.get(0));
        }
    }

    /**
     * dosageFormVo cannot be in-activated if is has any active products.
     * 
     * @param dosageFormVo dosageFormVo
     * @param user {@link UserVo} performing the action
     * @throws ValidationException ValidationException
     */
    @Override
    public void checkForActiveDependencies(DosageFormVo dosageFormVo, UserVo user) throws ValidationException {

        StringBuffer query = new StringBuffer();
        query.append(HqlBuilder.create("SELECT  item FROM ").append(EplOrderableItemDo.class)
                .append(PPSConstants.ITEM));
        String whereClause = " WHERE DOSAGE_FORM_ID_FK = " + dosageFormVo.getId() + " AND ITEM_STATUS LIKE 'ACTIVE'";
        query.append(whereClause);

        List<EplOrderableItemDo> ois = eplOrderableItemDao.executeHqlQuery(query.toString());

        if (ois == null || ois.size() == 0) {
            return;
        }

        throw new ValidationException(ValidationException.INACTIVATE_DOSAGEFORM,
                                      ois.size(), ois.get(0).getOiName());

    }

    /**
     * Insert the given DosageFormVo.
     * 
     * @param dosageForm
     *            DosageFormVo
     * @param user
     *            {@link UserVo} performing the action
     * @return DosageFormVo with new ID
     */
    @Override
    public DosageFormVo createWithoutDuplicateCheck(DosageFormVo dosageForm,
            UserVo user) {

        if (dosageForm.getId() == null) {
            dosageForm.setId(getSeqNumDomainCapability().generateId(
                    dosageForm.getEntityType(), user));

            if ((dosageForm.getVaDataFields() != null)
                    && (!(dosageForm.getVaDataFields().isEmpty()))) {
                dosageForm.getVaDataFields().setVaDfOwnerId(
                        getSeqNumDomainCapability().generatedOwnerId(user));
            }
        }

        EplDosageFormDo dosageFormDo = dosageFormConverter.convert(dosageForm);

        eplDosageFormDao.insert(dosageFormDo, user);

        insertChildRecords(dosageForm, user, dosageFormDo);

        return dosageForm;
    }

    /**
     * Insert the child records for the given DosageFormVo.  Child records are:
     * <ul>
     * <li>Dosage Form Noun</li>
     * <li>Dispense Unit per Dose</li>
     * <li>Dosage Form Unit</li>
     * <li>Dosage Form Med Route</li>
     * <li>VA Data Fields</li>
     * <ul>
     * @param dosageForm DosageFormVo
     * @param user the user
     * @param dosageFormDo EplDosageFormDo that was converted from the Vo
     */
    private void insertChildRecords(DosageFormVo dosageForm, UserVo user, EplDosageFormDo dosageFormDo) {
        eplDfNounDao.insert(dosageFormDo.getEplDfNouns(), user);

        eplDfUnitDao.insert(dosageFormDo.getEplDfUnits(), user);

        eplDispenseUnitsPerDoseDao.insert(dosageFormDo.getEplDispenseUnitsPerDoses(), user);

        eplDfMedRtDfAssocDao.insert(dosageFormDo.getEplDfMedRtDfAssocs(), user);

        if ((dosageForm.getVaDataFields() != null) && (!(dosageForm.getVaDataFields().isEmpty()))) {

            // convert dosage form vo's data fields to a set of EplVaDfOwnerDo

            EplVadfOwnerDo owners = dataFieldsConverter.convert(dosageForm.getVaDataFields(), dosageFormDo);

            Set<EplVadfOwnerDo> ownerDOs = new HashSet<EplVadfOwnerDo>();
            ownerDOs.add(owners);

            getDataFieldsDomainCapability().insertEplVaDfOwners(null, null, null, dosageFormDo, ownerDOs, user);
        }
    }

    /**
     * Update the given DosageFormVo. Deletes child records and re-inserts them based on the values on the specified 
     * DosageFormVo.
     * 
     * @param dosageForm
     *            DosageFormVo
     * @param user
     *            {@link UserVo} performing the action
     * @return DosageFormVo
     */
    @Override
    public DosageFormVo update(DosageFormVo dosageForm, UserVo user) {
        EplDosageFormDo dosageFormDo = dosageFormConverter.convert(dosageForm);

        deleteChildRecords(dosageForm);
        
        if (dosageForm.getVaDataFields() != null 
            && !dosageForm.getVaDataFields().isEmpty()) {
            dosageForm.getVaDataFields().setVaDfOwnerId(getSeqNumDomainCapability().generatedOwnerId(user));
        }
        
        insertChildRecords(dosageForm, user, dosageFormDo);

        eplDosageFormDao.update(dosageFormDo, user);

        return dosageForm;

    }

    /**
     * Delete the child records for the given DosageFormVo.  Child records are:
     * <ul>
     * <li>Dosage Form Noun</li>
     * <li>Dispense Unit per Dose</li>
     * <li>Dosage Form Unit</li>
     * <li>Dosage Form Med Route</li>
     * <li>VA Data Fields</li>
     * </ul>
     * @param dosageForm DosageFormVo
     */
    private void deleteChildRecords(DosageFormVo dosageForm) {
        EplDosageFormDo dosageFormDo = dosageFormConverter.convert(dosageForm);

        // delete all existing nouns and re-insert
        eplDfNounDao.delete(DOSAGE_FORM_EPL_ID, dosageFormDo.getEplId());

        // delete all existing dispense units and re-insert
        eplDispenseUnitsPerDoseDao.delete(DOSAGE_FORM_EPL_ID, dosageFormDo.getEplId());

        // deletes all the existing dosage form units and re-insert
        eplDfUnitDao.delete(DOSAGE_FORM_EPL_ID, dosageFormDo.getEplId());

        // deletes all the existing local med routes and re-insert
        eplDfMedRtDfAssocDao.delete(DOSAGE_FORM_EPL_ID, dosageFormDo.getEplId());

        // delete VA Data Fields
        getDataFieldsDomainCapability().delete(dosageForm.getVaDataFields());
    }    
    
    /**
     * Create Hibernate {@link Criteria} that will find an item by its
     * uniqueness fields for the DosageFormDomainCapabilityImpl.
     * 
     * @param item
     *            {@link ManagedItemVo} for which to create uniqueness
     *            {@link Criteria}
     * @return {@link Criteria}
     */
    @Override
    protected Criteria createUniquenessCriteria(DosageFormVo item) {
        Criteria criteria = getDataAccessObject().getCriteria();
        criteria.add(Restrictions.ilike(EplDosageFormDo.DF_NAME,
                item.getDosageFormName()));

        return criteria;
    }

    /**
     * Sets the epl dosage form dao.
     *
     * @param eplDosageFormDao the new epl dosage form dao
     */
    public void setEplDosageFormDao(EplDosageFormDao eplDosageFormDao) {
        this.eplDosageFormDao = eplDosageFormDao;
    }

    /**
     * Get the {@link DataAccessObject} that this capability uses.
     * 
     * @return {@link DataAccessObject}
     */
    @Override
    public DataAccessObject<EplDosageFormDo, Long> getDataAccessObject() {
        return eplDosageFormDao;
    }

    /**
     * Return the {@link Converter} instance for this capability.
     * 
     * @return {@link Converter}
     */
    @Override
    public DosageFormConverter getConverter() {
        return dosageFormConverter;
    }

    /**
     * Sets the epl df noun dao.
     *
     * @param eplDfNounDao the new epl df noun dao
     */
    public void setEplDfNounDao(EplDfNounDao eplDfNounDao) {
        this.eplDfNounDao = eplDfNounDao;
    }

    /**
     * Sets the epl df unit dao.
     *
     * @param eplDfUnitDao the new epl df unit dao
     */
    public void setEplDfUnitDao(EplDfUnitDao eplDfUnitDao) {
        this.eplDfUnitDao = eplDfUnitDao;
    }

    /**
     * Sets the epl dispense units per dose dao.
     *
     * @param eplDispenseUnitsPerDoseDao the new epl dispense units per dose dao
     */
    public void setEplDispenseUnitsPerDoseDao(
            EplDispenseUnitsPerDoseDao eplDispenseUnitsPerDoseDao) {
        this.eplDispenseUnitsPerDoseDao = eplDispenseUnitsPerDoseDao;
    }

    /**
     * Sets the epl df med rt df assoc dao.
     *
     * @param eplDfMedRtDfAssocDao the new epl df med rt df assoc dao
     */
    public void setEplDfMedRtDfAssocDao(
            EplDfMedRtDfAssocDao eplDfMedRtDfAssocDao) {
        this.eplDfMedRtDfAssocDao = eplDfMedRtDfAssocDao;
    }

    /**
     * Sets the data fields converter.
     *
     * @param dataFieldsConverter the new data fields converter
     */
    public void setDataFieldsConverter(DataFieldsConverter dataFieldsConverter) {
        this.dataFieldsConverter = dataFieldsConverter;
    }

    /**
     * Sets the dosage form converter.
     *
     * @param dosageFormConverter the new dosage form converter
     */
    public void setDosageFormConverter(DosageFormConverter dosageFormConverter) {
        this.dosageFormConverter = dosageFormConverter;
    }

    /**
     * Sets the epl orderable item dao.
     *
     * @param eplOrderableItemDao the new epl orderable item dao
     */
    public void setEplOrderableItemDao(EplOrderableItemDao eplOrderableItemDao) {
        this.eplOrderableItemDao = eplOrderableItemDao;
    }

}
