package gov.va.med.pharmacy.peps.domain.common.capability.impl;

import java.util.List;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.exception.ConstraintViolationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate4.SessionFactoryUtils;

import gov.va.med.pharmacy.peps.common.exception.DuplicateRematchException;
import gov.va.med.pharmacy.peps.common.utility.security.UntrustedLogParam;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.UnmatchRematchVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.RematchMgtDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.EplRematchDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplRematchDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.RematchConverter;


/**
 * Main implementation of {@link RematchMgtDomainCapability}.
 */
public class RematchMgtDomainCapabilityImpl implements RematchMgtDomainCapability {
  
    private static final Logger LOG = LogManager.getLogger(RematchMgtDomainCapabilityImpl.class);

    private final EplRematchDao eplRematchDao;
    private final RematchConverter rematchConverter;
    
    /**
     * Constructs an instance with the required dependencies.
     * 
     * @param pEplRematchDao the {@link EplRematchDao} to use.
     * @param pRematchConverter the {@link RematchConverter} to use.
     */
    @Autowired
    public RematchMgtDomainCapabilityImpl(
        final EplRematchDao pEplRematchDao,
        final RematchConverter pRematchConverter) {
      eplRematchDao = pEplRematchDao;
      rematchConverter = pRematchConverter;
    }
    
    @Override
    public RematchVo saveOrUpdate(final RematchVo rematch, final UserVo user) throws DuplicateRematchException {
        EplRematchDo savedDo;
        try {
            if (rematch.getEplRematchId() != null) {
                final Long id = Long.valueOf(rematch.getEplRematchId());
                LOG.debug("Rematch has already been saved (id={}): updating it.", id);
                final EplRematchDo dataObj = eplRematchDao.retrieve(id);
                rematchConverter.setDataObjectPropertiesFrom(dataObj, rematch);
                // We're not actually calling a DAO update method here so we need to call this ourselves.
                eplRematchDao.updateCreatedAndLastModified(dataObj, user);
                // Flush the Session to detect the below ConstraintViolationException.
                eplRematchDao.getCurrentSession().flush();
                savedDo = dataObj;
            } else {
                LOG.debug("Saving new rematch.");
                final EplRematchDo newDo = rematchConverter.convert(rematch);
                savedDo = eplRematchDao.insert(newDo, user);
            }
        // Not using Spring's exception translation so we will just get a Hibernate exception here.
        } catch (final ConstraintViolationException ex) {
            LOG.warn("Violated constraint {}.", ex.getConstraintName());
            if (StringUtils.equals(ex.getConstraintName(), EplRematchDo.UNIQUE_CONSTRAINT_NAME)) {
                throw new DuplicateRematchException();
            } else {
                throw SessionFactoryUtils.convertHibernateAccessException(ex);
            }
        }

        return rematchConverter.convert(savedDo);
    }
    
    @Override
    public void delete(final RematchVo rematch) {
      LOG.debug("Deleting rematch {}.", new UntrustedLogParam(rematch));
      // Use convert minimal because we should only need the primary key here.
      eplRematchDao.delete(rematchConverter.convertMinimal(rematch));
    }
    
    @Override   
    public List<UnmatchRematchVo> findRematchProducts(Long productId) {
        return eplRematchDao.findRematchProducts(productId);
    } 
    
    @Override
    public Optional<RematchVo> getRematchForProduct(long productId) {
      
      Optional<EplRematchDo> retrievedRematch = eplRematchDao.retrieveForProduct(productId);
      if (retrievedRematch.isPresent()) {
        return Optional.of(rematchConverter.convert(retrievedRematch.get()));
      } else {
        return Optional.empty();
      }
    }
}
