package gov.va.med.pharmacy.peps.domain.common.model;

import java.util.Collections;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.ColumnResult;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.SqlResultSetMapping;
import javax.persistence.SqlResultSetMappings;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.annotations.NamedNativeQueries;
import org.hibernate.annotations.NamedNativeQuery;
import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;


/**
 * Data Object version of {@link gov.va.med.pharmacy.peps.common.vo.RematchVo}.
 */
@Entity
@NamedQueries({                
    
    @NamedQuery(
        name = "retrieveByMatch",
        query = "SELECT rematch FROM EplRematchDo rematch " +
            "WHERE rematch.matchType.matchTypeId = :matchTypeId " +
                "AND rematch.ndfUpdateFile is NULL "
        ),
    
    // It is important that this query actually looks for the outgoing difference because the outgoing difference could
    // be deleted without deleting the associated rematch. We would not, of course, want to send the rematch in that case.
    // These conditions will have to be updated as we support more match types. Hopefully a less brittle solution can be
    // found.
    @NamedQuery(
      name = "findOutgoingRematches",
      query =
        "SELECT rematch FROM EplRematchDo AS rematch "
        // Ensure we get at least the old product in the same query. We usually, but not always, want to see both
        // products' information from the query result.
        + "LEFT JOIN FETCH rematch.fromProduct "
        + "WHERE rematch.ndfUpdateFile is NULL "
        + "AND rematch.createdDtm <= :startDate "
        // Now we begin the match-type-specific conditions.
        + "AND rematch.matchType.matchTypeId = 3 "   // Always include manual rematches.
        + "OR (rematch.matchType.matchTypeId = 1 "   // Only include inactivation rematches if we're sending the inactivation.
        +   "AND EXISTS ( "
        +     "FROM EplNdfOutgoingDifferences AS odiff "
        +     "WHERE odiff.ndcUpdateFileFk IS NULL "
        +       "AND odiff.vistaFileNumber = :vistaFileNumber "
        +       "AND odiff.vistaFieldNumber = :vistaFieldNumber "
        +       "AND odiff.vistaIen = rematch.fromProduct.ndfProductIen "
        +       "AND odiff.actionType = 'M'  "
        +       "AND odiff.createdDtm <= :startDate "
        +       "AND odiff.newValue <> ' ' "
        +       "AND odiff.oldValue IS NULL))")
})

@SqlResultSetMappings({
    @SqlResultSetMapping(name = "findRematch",
    columns = { @ColumnResult(name = "matchedProductId"),
                @ColumnResult(name = "matchedProductName"),
                @ColumnResult(name = "matchedCmop"),
                @ColumnResult(name = "matchedIen")
    })
})

@NamedNativeQueries({
    
    @NamedNativeQuery(name = "findRematch",
    query = "select p2.epl_id as matchedProductId, p2.va_product_name as matchedProductName, p2.cmop_id as matchedCmop, p2.ndf_product_ien as matchedIen from ppsnepl.epl_products p1, ppsnepl.epl_products p2 " +
            "where p1.epl_id = :unmatchedProductId " +            
                "and p1.epl_id != p2.epl_id " +
                "and nvl(p1.strength,0) = nvl(p2.strength,0) " +
                "and nvl(p1.drug_unit_id_fk,0) = nvl(p2.drug_unit_id_fk,0)  " +
                "and p1.epl_id_oi_fk = p2.epl_id_oi_fk " +
                "and p2.inactivation_date is null " +
                "and p2.request_status = 'APPROVED' " +
                "and p2.item_status = 'ACTIVE' " +
                "union " +
                "select p1.epl_id as matchedProductId, p1.va_product_name as matchedProductName, p1.cmop_id as matchedCmop, p1.ndf_product_ien as matchedIen from ppsnepl.epl_products p1, ppsnepl.epl_ndcs ndc " +
                "where ndc.EPL_ID_PRODUCT_FK = :unmatchedProductId " +
                "and p1.epl_id != :unmatchedProductId " +
                "and p1.inactivation_date is null " +
                "and p1.request_status = 'APPROVED'  " +
                "and ndc.item_status = 'ACTIVE' " +
                "and ndc.EPL_ID_PRODUCT_FK = p1.epl_id ",
    resultSetMapping="findRematch")
   
} )

@Table(name = "EPL_REMATCH", schema = "PPSNEPL")
public class EplRematchDo extends AnnotatedDataObject implements java.io.Serializable {

  /**
   * Change this when changing the class fields!
   */
  private static final long serialVersionUID = 6L;

  private static final Logger LOG = LogManager.getLogger(EplRematchDo.class);
  
  /**
   * The name of the database constraint that enforces only one rematch per product per update file.
   */
  public static final String UNIQUE_CONSTRAINT_NAME = "PPSNEPL.EPL_REMATCH_U01";

  private Long eplRematchId;
  private EplProductDo fromProduct;
  private EplProductDo toAutoProduct;
  private EplMatchTypeDo matchType;
  private EplNdfUpdateFileDo ndfUpdateFile;
  /**
   * Backing attribute for {@link #getToSuggestedProducts()}. We guarantee that the getter never returns null so this
   * starts off as an empty set instead of null.
   */
  private Set<EplProductDo> toSuggestedProducts = new HashSet<>();

  /**
   * Constructs an instance. (Necessary for Hibernate.)
   */
  public EplRematchDo() {
  }

  /**
   * Minimal constructor useful for constructing test instances. All properties not passed in the constructor will be set to
   * null.
   * 
   * @param pFromProduct See {@link #setFromProduct(EplProductDo)}.
   * @param pToAutoProduct See {@link #setToAutoProduct(EplProductDo)}.
   * @param pMatchType See {@link #setMatchType(EplMatchTypeDo)}.
   */
  public EplRematchDo(
      final EplProductDo pFromProduct, final EplProductDo pToAutoProduct, final EplMatchTypeDo pMatchType) {

    fromProduct = pFromProduct;
    toAutoProduct = pToAutoProduct;
    matchType = pMatchType;
  }

  @Id
  @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "rematchSeqGen")
  @SequenceGenerator(name = "rematchSeqGen", sequenceName = "EPL_REMATCH_SEQ")
  @Column(name = "EPL_REMATCH_ID", unique = true, nullable = false, precision = ID_PRECISION, scale = 0)
  public Long getEplRematchId() {
    return eplRematchId;
  }

  public void setEplRematchId(final Long eplRematchId) {
    this.eplRematchId = eplRematchId;
  }

  /**
   * Returns the product we are matching from. (See {@link
   * gov.va.med.pharmacy.peps.common.vo.RematchVo#getOldProduct()}.)
   * @return the product object
   */
  @ManyToOne( // Using default of no cascade.
    fetch = FetchType.EAGER) // Note: this association could probably be lazy-loaded, but loading even over the VPN
                             // took <200ms so it's probably worth loading initially.
  @JoinColumn(name = "FROM_PRODUCT_ID_FK", nullable = false)
  public EplProductDo getFromProduct() {
    return fromProduct;
  }

  public void setFromProduct(final EplProductDo pFromProduct) {
    fromProduct = pFromProduct;
  }

  /**
   * Returns the product we are automatically matching to.
   * @return Possibly null if this is a suggested rematch.
   */
  @ManyToOne( // Using default of no cascade.
    fetch = FetchType.EAGER) // Note: this association could probably be lazy-loaded, but loading even over the VPN
                             // took <200ms so it's probably worth loading initially.
  @JoinColumn(name = "TO_PRODUCT_AUTO_ID_FK", nullable = true)
  public EplProductDo getToAutoProduct() {
    return toAutoProduct;
  }

  public void setToAutoProduct(final EplProductDo pToAutoProduct) {
    toAutoProduct = pToAutoProduct;
  }
  
  /**
   * Return the suggested rematch products.
   * @return never null but possibly empty.
   */
  @ManyToMany(fetch = FetchType.LAZY)
  @JoinTable(
      schema = "PPSNEPL", name = "EPL_REMATCH_SUGGESTION",
      joinColumns = @JoinColumn(name = "EPL_REMATCH_ID"),
      inverseJoinColumns = @JoinColumn(name = "TO_PRODUCT_ID_FK"))
  public Set<EplProductDo> getToSuggestedProducts() {
    return toSuggestedProducts;
  }
  
  /**
   * Set the suggested rematch products.
   * @param value Must not be null.
   */
  public void setToSuggestedProducts(final Set<EplProductDo> value) {
    toSuggestedProducts = Objects.requireNonNull(value);
  }

  @ManyToOne
  @JoinColumn(name = "MATCH_TYPE_ID_FK")
  public EplMatchTypeDo getMatchType() {
    return matchType;
  }

  public void setMatchType(EplMatchTypeDo matchType) {
    this.matchType = matchType;
  }

  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "NDF_UPDATE_FILE_FK")
  public EplNdfUpdateFileDo getNdfUpdateFile() {
    return ndfUpdateFile;
  }

  public void setNdfUpdateFile(final EplNdfUpdateFileDo ndfUpdateFile) {
    this.ndfUpdateFile = ndfUpdateFile;
  }
  
  /**
   * Returns the "to" product(s), whether automatic or suggested. Note that this will always return a set of 1 product
   * in the case of automatic rematches.
   * 
   * @return never null
   * @see #getToAutoProduct()
   * @see #getToSuggestedProducts()
   */
  @Transient  // Calculated field.
  public Set<EplProductDo> getToProducts() {
    if (getToAutoProduct() != null) {
      LOG.debug("Returning automatic product.");
      return Collections.singleton(getToAutoProduct());
    } else {
      LOG.debug("Returning suggested product(s).");
      return getToSuggestedProducts();
    }
  }

}
