

package gov.va.med.pharmacy.peps.domain.common.utility;


import java.io.RandomAccessFile;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Common functionality for classes that read NDF migration files. These classes are <em>not</em> thread-safe: they must
 * only be used by one thread at a time.
 */
public abstract class NdfFileReader {

    private static final Logger LOG = LogManager.getLogger(NdfFileReader.class);
    
    /** The file path. */
    private RandomAccessFile file;
    private String delimiter = "\\^";
    private final String spaces = "^ ^";
    private final String noSpaces = "^^";

    /**
     * Process file.
     */

    //No longer @Transactional
    public void processFile() {
        if(file ==null){
           throw new IllegalStateException("no file");
        }
        String line = "";
        while (line != null) {
            try {
                line = file.readLine();

                processLine(line);

            } catch (Exception e) {
                LOG.error("Error processing line: " + line, e);
            }
        }

        updateIenGenerator();
    }

    /**
     * Updates the IEN generator based on the last IEN processed in the migration file.
     */
    protected void updateIenGenerator() {
        // No op: overridden in DDIFileReader.
        LOG.debug("No-op implementation.");
    }



    /**
     * Process line.
     *
     * @param line the line
     */
    private void processLine(String line) throws Exception {

        if (line == null) {
            return;
        }
        
        if (LOG.isDebugEnabled()) {
            LOG.debug(line);
        }
        
        while (line != line.replace(noSpaces, spaces)) {
            line = line.replace(noSpaces, spaces);
        }
        String[] tokens = line.split(getDelimiter());
        processLine(tokens);

    }

    /**
     * Process the given line, including updating the database as necessary.
     * 
     * @param tokens tokens from the line
     */
    protected abstract void processLine(String[] tokens) throws Exception;

    /**
     * @param tokens
     * @return
     * @throws ParseException
     */
    protected Date stringToDate(String token) {
        SimpleDateFormat format = new SimpleDateFormat("MMM dd,YYYY");
        Date date;
        try {
            date = format.parse(token);
        } catch (ParseException e) {
            date = new Date();
            e.printStackTrace();
        }
        return date;
    }

    /**
     * Returns true if the given string represents "Yes" in any case, false otherwise.
     * @param string the string to check
     * @return See description.
     */
    protected static boolean stringToBool(String string) {
        return ("YES".equalsIgnoreCase(string));
    }

    /**
     * @return the delimiter
     */
    public String getDelimiter() {
        return delimiter;
    }

    /**
     * @param delimiter the delimiter to set
     */
    public void setDelimiter(String delimiter) {
        this.delimiter = delimiter;
    }

    /**
     * @return the file
     */
    public RandomAccessFile getFile() {
        return file;
    }

    /**
     * @param file the file to set
     */
    public void setFile(RandomAccessFile file) {
        this.file = file;
    }

   
}
