package gov.va.med.pharmacy.peps.domain.common.utility;

import java.text.NumberFormat;
import java.text.ParseException;
import java.text.ParsePosition;
import java.util.Calendar;
import java.util.Locale;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import gov.va.med.pharmacy.peps.common.exception.DomainValidationException;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplIenGeneratorDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplPackageSizeDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplIenGeneratorDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplPackageSizeDo;

public class PackageSizeFileReader extends NdfFileReader {
    
    private final EplPackageSizeDao eplPackageSizeDao;
    /**
     * Return value of {@link #getHighestIenProcessed()} indicating no records have been processed yet. According to 
     * Linda Ellzey, VistA IENs start at 1.
     */
    protected static final long NO_IENS_PROCESSED = 0;
    
    private static final Logger LOG = LogManager.getLogger(PackageSizeFileReader.class);

    private static final int IEN_COLUMN = 0;
    
    private final EplIenGeneratorDao eplIenGeneratorDao;
    private final UserVo user;
    private long highestIenProcessed;
    
    /**
     * Constructs an instance with the required dependencies.
     * @param pEplIenGeneratorDao the {@link EplIenGeneratorDao} to use.
     * @param pUser the user executing the migration
     */    
    public PackageSizeFileReader(
        final EplPackageSizeDao pPackageSizeDao,
        final EplIenGeneratorDao pEplIenGeneratorDao,
        final UserVo pUser) {
        highestIenProcessed = NO_IENS_PROCESSED;
        eplPackageSizeDao = pPackageSizeDao;
        eplIenGeneratorDao = pEplIenGeneratorDao;
        user = pUser;
    }

    private void updateHighestIenProcessed(final long newIen) {
        if (newIen > highestIenProcessed) {
            highestIenProcessed = newIen;
        }
    }
    
    /**
     * Returns the highest IEN processed by this file reader. This IEN may not have been successfully processed: it was
     * just in the file.
     * @return see description
     */
    protected long getHighestIenProcessed() {
        return highestIenProcessed;
    }

    @Override
    protected void processLine(String[] tokens) throws Exception {
        EplPackageSizeDo packageSize = new EplPackageSizeDo();

        packageSize.setCreatedDtm(Calendar.getInstance().getTime());
        final long ien = Long.valueOf(tokens[IEN_COLUMN]);
        packageSize.setNdfPackageSizeIen(ien);
        
        // Update the IEN before validating the data. See method javadoc on getHighestIenProcessed().
        updateHighestIenProcessed(ien);
        
        try {
            Double ps = new Double(parseDecimal(tokens[2]));
            packageSize.setPackageSize(ps);
        } catch (ParseException e) {
            throw new DomainValidationException(e, DomainValidationException.NUMERIC_ERROR, "package size", tokens[2]);
        }
        
        getEplPackageSizeDao().insert(packageSize, user);

    }
    
    private double parseDecimal(String input) throws ParseException {
        NumberFormat numberFormat = NumberFormat.getNumberInstance(Locale.getDefault());
        ParsePosition parsePosition = new ParsePosition(0);
        Number number = numberFormat.parse(input, parsePosition);

        if(parsePosition.getIndex() != input.length()){
          throw new ParseException("Invalid number at", parsePosition.getIndex());
        }

        return number.doubleValue();
      }  

    @Override
    protected void updateIenGenerator() {
        LOG.info("Inserting IEN generator record with last-used IEN {}", getHighestIenProcessed());
        final EplIenGeneratorDo ienGeneratorRecord = new EplIenGeneratorDo(
            EplPackageSizeDo.TABLE_NAME, getHighestIenProcessed(), null, null);
        eplIenGeneratorDao.insert(ienGeneratorRecord, user);
    }
    
    /**
     * @return the eplPackageSizeDao
     */
    public EplPackageSizeDao getEplPackageSizeDao() {
        return eplPackageSizeDao;
    }
    
}
