package gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm;

import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;

import gov.va.med.pharmacy.peps.common.utility.ConfigFileUtility;
import gov.vha.isaac.rest.api1.data.search.RestSearchResultPage;
import gov.vha.isaac.rest.api1.data.sememe.RestDynamicSememeVersion;
import gov.vha.isaac.rest.api1.data.sememe.RestSememeDescriptionVersion;

/**
 * This class handles the process of keeping RxNorm data up to date on the EPL_PRODUCTS table. It uses two web service calls to
 * ISAAC and the product DAO to populate the data into the appropriate columns.
 * 
 * @author DNS   RUSTRA
 *
 */
public class RxNormClient {

    private static final UUID SUPPRESS_UUID = UUID.fromString("3e51fab1-34ec-5346-af99-c62baec1f675");
    private static final UUID NOT_SUPPRESIBBLE_UUID = UUID.fromString("8277b5c7-5720-5eea-b802-8ad3f28c9bf7");

    private static final UUID SAB_UUID = UUID.fromString("bcc300c0-0347-5c11-80cb-9fb7497e44de");
    private static final UUID RXNORM_UUID = UUID.fromString("d9e4f663-497a-5da7-8b4c-2846b86c6971");

    private static final UUID SCD_UUID = UUID.fromString("76de667d-54ca-55a4-ba72-21a7e257c928");
    private static final UUID PSN_UUID = UUID.fromString("6ecd2b47-d1ee-571a-b21c-7abda21c9207");
    private static final UUID SBD_UUID = UUID.fromString("9be227c1-c7e2-582f-b391-13ae1ffb3e7c");
    private static final UUID BPACK_UUID = UUID.fromString("02507be6-9971-5c05-901a-cfcda4eb0eb5");
    private static final UUID GPACK_UUID = UUID.fromString("4bbeba6f-d765-54e0-9c58-6fe0f74d73e2");

    private static final String NOT_SUPPRESSIBLE_STRING = "Not Suppressible";
    private static final int SCD_RANK = 5;
    private static final int PSN_RANK = 4;
    private static final int SBD_RANK = 3;
    private static final int GPACK_RANK = 2;
    private static final int BPACK_RANK = 1;

    private static final String VUID_PARAM = "VUID";
    private static final String UUID_PARAM = "UUID";
    
    private static final org.apache.logging.log4j.Logger LOG = org.apache.logging.log4j.LogManager.getLogger(
        RxNormClient.class);    
    
    /**
     * Calls the ISAAC web service passing a product VUID and returning the Code System Code if the web service contains one for
     * the given VUID. Returns null if no code exists for the VUID.
     *
     * @param vuid String property
     * @param configFile the config file utility
     * @return String
     */
    public String getCode(String vuid, ConfigFileUtility configFile) {
        if (vuid != null) {
            Map<String, String> params = new HashMap<String, String>();
            params.put(VUID_PARAM, vuid);
            RestTemplate restTemplate = new RestTemplate();
            restTemplate.setRequestFactory(new HttpComponentsClientHttpRequestFactory());

            RestSearchResultPage restSearchResultPage = restTemplate.getForObject(configFile.getIsaacCodingConceptURL(),
                RestSearchResultPage.class, params);

            if (restSearchResultPage != null && restSearchResultPage.getResults() != null
                && restSearchResultPage.getResults().length > 0
                && restSearchResultPage.getResults()[0].getReferencedConcept() != null) {

                UUID uuid = restSearchResultPage.getResults()[0].getReferencedConcept().getIdentifiers().getUuids().get(0);
                params.clear();
                params.put(UUID_PARAM, String.valueOf(uuid.toString()));

                RestDynamicSememeVersion restDynamicSememeVersion[] = restTemplate
                    .getForObject(configFile.getIsaacCodingComponentURL(), RestDynamicSememeVersion[].class, params);

                if (restDynamicSememeVersion != null && restDynamicSememeVersion.length > 0
                    && restDynamicSememeVersion[0].getDataColumns() != null
                    && !restDynamicSememeVersion[0].getDataColumns().isEmpty()) {
                    String object = (String) restDynamicSememeVersion[0].getDataColumns().get(0).getData().toString();
                    return object;
                }
            }
        }
        return null;
    }

    /**
     * Calls the ISAAC web service passing a product VUID and returning the related RXNorm information if the web service
     * contains one for the given VUID. Returns null if no record for that product exists or the pieces of data that it actually
     * finds.
     * 
     * @param vuid String property
     * @param configFile the config file utility
     * @return String
     */
    public RxNormData getRxNormData(String vuid, ConfigFileUtility configFile) {
        RxNormData result = null;
        if (vuid != null) {
            Map<String, String> params = new HashMap<String, String>();
            params.put(VUID_PARAM, vuid);
            RestTemplate restTemplate = new RestTemplate();
            restTemplate.setRequestFactory(new HttpComponentsClientHttpRequestFactory());

            RestSearchResultPage restSearchResultPage = restTemplate.getForObject(configFile.getIsaacCodingConceptURL(),
                RestSearchResultPage.class, params);

            // Get the product name.
            if (restSearchResultPage != null && restSearchResultPage.getResults() != null
                && restSearchResultPage.getResults().length > 0
                && restSearchResultPage.getResults()[0].getReferencedConcept() != null) {

                result = new RxNormData();
                result.setProductName(restSearchResultPage.getResults()[0].getReferencedConcept().getDescription());

                UUID uuid = restSearchResultPage.getResults()[0].getReferencedConcept().getIdentifiers().getUuids().get(0);
                params.clear();
                params.put(UUID_PARAM, String.valueOf(uuid.toString()));

                // Get the descriptions to obtain the TTY information.
                RestSememeDescriptionVersion[] descriptions = restTemplate
                    .getForObject(configFile.getIsaacCodingDescriptionsURL(), RestSememeDescriptionVersion[].class, params);

                List<RestSememeDescriptionVersion> potentialDescriptions = getPotentialDescription(descriptions);

                if (!potentialDescriptions.isEmpty()) {
                    Collections.sort(potentialDescriptions, new DescriptionSorter());
                    result.setSuppress(NOT_SUPPRESSIBLE_STRING);
                    result.setTty(typeToString(potentialDescriptions.get(0).getDescriptionExtendedTypeConcept().getUuids()));
                }

            }
        }
                
        if (result == null){
            LOG.debug("No RXNorm Data Found.");
        }
        
        return result;
    }

    /**
     * Get the potential descriptions where we can find the TTY information in the calling method.
     * @param descriptionVersions
     * @return
     */
    private List<RestSememeDescriptionVersion> getPotentialDescription(RestSememeDescriptionVersion[] descriptionVersions) {
        ArrayList<RestSememeDescriptionVersion> potentialDescriptionVersions = new ArrayList<>();
        for (RestSememeDescriptionVersion d : descriptionVersions) {
            boolean sabIsRxNorm = false;
            boolean suppressIsN = false;
            for (RestDynamicSememeVersion ns : d.getNestedSememes()) {
                if (ns.getSememeChronology().getAssemblage().getUuids().contains(SUPPRESS_UUID)
                    && ns.getDataColumns().get(0).getData().toString().equals(NOT_SUPPRESIBBLE_UUID.toString())) {
                    suppressIsN = true;
                }
                if (ns.getSememeChronology().getAssemblage().getUuids().contains(SAB_UUID)
                    && ns.getDataColumns().get(0).getData().toString().equals(RXNORM_UUID.toString())) {
                    sabIsRxNorm = true;
                }
                if (suppressIsN && sabIsRxNorm) {
                    potentialDescriptionVersions.add(d);
                    break;
                }
            }
        }
        return potentialDescriptionVersions;
    }

    private static String typeToString(List<UUID> uuids) {
        if (uuids.contains(SCD_UUID)) {
            return "SCD";
        }
        if (uuids.contains(PSN_UUID)) {
            return "PSN";
        }
        if (uuids.contains(SBD_UUID)) {
            return "SBD";
        }
        if (uuids.contains(GPACK_UUID)) {
            return "GPACK";
        }
        if (uuids.contains(BPACK_UUID)) {
            return "BPACK";
        }
        return null;
    }

    /**
     * This method is used to bypass the SSL certificate checking. TO-DO: We need to work with the ISAAC group to get the
     * correct certificates.
     */
    private static void trustSelfSignedSSL() {
        try {
            SSLContext ctx = SSLContext.getInstance("TLS");
            X509TrustManager tm = new X509TrustManager() {

                @Override
                public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                    // Do Nothing
                }

                @Override
                public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                    // Do Nothing
                }

                @Override
                public X509Certificate[] getAcceptedIssuers() {
                    return null;
                }
            };
            ctx.init(null, new TrustManager[] { (TrustManager) tm }, null);
            SSLContext.setDefault(ctx);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Initializes the class when instantiated by Spring.
     */
    public void init() {
        trustSelfSignedSSL();
    }

    private class DescriptionSorter implements Comparator<RestSememeDescriptionVersion> {

        // Ranking: SCD > PSN > SBD > GPACK > BPACK
        private int getRank(RestSememeDescriptionVersion d) {
            if (d.getDescriptionExtendedTypeConcept().getUuids().contains(RxNormClient.SCD_UUID)) {
                return SCD_RANK;
            }
            if (d.getDescriptionExtendedTypeConcept().getUuids().contains(RxNormClient.PSN_UUID)) {
                return PSN_RANK;
            }

            if (d.getDescriptionExtendedTypeConcept().getUuids().contains(RxNormClient.SBD_UUID)) {
                return SBD_RANK;
            }
            if (d.getDescriptionExtendedTypeConcept().getUuids().contains(RxNormClient.GPACK_UUID)) {
                return GPACK_RANK;
            }
            if (d.getDescriptionExtendedTypeConcept().getUuids().contains(RxNormClient.BPACK_UUID)) {
                return BPACK_RANK;
            }
            return 0;
        }

        @Override
        public int compare(RestSememeDescriptionVersion d1, RestSememeDescriptionVersion d2) {
            return getRank(d2) - getRank(d1);
        }

    }
}
