
package gov.va.med.pharmacy.peps.domain.common.utility.converter;


import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.vo.ComplianceSiteVo;
import gov.va.med.pharmacy.peps.common.vo.VistaSiteVo;
import gov.va.med.pharmacy.peps.domain.common.capability.SiteListDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.model.EplComplianceSiteDo;


/**
 * Convert to/from {@link ComplianceSiteVo} and {@link EplComplianceSiteDo}.
 */
public class ComplianceSiteConverter extends
        Converter<ComplianceSiteVo, EplComplianceSiteDo> {
    
    private SiteListDomainCapability siteListCapability;

    /**
     * Constructor taking fields.
     *
     * @param siteListCapability the SiteListDomainCapability
     */
    @Autowired
    public ComplianceSiteConverter(SiteListDomainCapability siteListCapability) {
        super();
        this.siteListCapability = siteListCapability;
    }

    /**
     * To ComplianceSiteVo from ComplianceSiteDo. 
     *     
     * @param data EplComplianceSiteDo
     * @return ComplianceSiteVo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject(
     *      gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    @Override
    protected ComplianceSiteVo toValueObject(EplComplianceSiteDo data) {
        
        if (data == null){
            return null;
        }
        
        ComplianceSiteVo value = new ComplianceSiteVo();        

        if (data.getStationNumber() != null){
            VistaSiteVo vistaSiteVo = siteListCapability.retrieveByStationNumber(data.getStationNumber());
            if (vistaSiteVo == null){
                vistaSiteVo = new VistaSiteVo(data.getStationNumber(), null);
            }
            value.setVistaSiteVo(vistaSiteVo);            
        }

        value.setIncludeExcludeIndicator(data.getIncludeExcludeIndicator());
        return value;
    }
    
    /**
     * to ComplianceSiteVo from EplComplianceSiteDo.     
     * @param dataList EplComplianceSiteDo
     * @return ComplianceSiteVo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject(
     *      gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    protected List<ComplianceSiteVo> toValueObject(List<EplComplianceSiteDo> dataList) {        
        List<ComplianceSiteVo> list = new  ArrayList<ComplianceSiteVo>();
        if (dataList != null){
            dataList.stream().forEach(data -> list.add(toValueObject(data)));        
        }

        return list;
    }

    /**
     * Fully copies the ComplianceSiteVo to a EplComplianceSiteDo.
     * 
     * @param value EplComplianceSiteDo
     * @return fully populated EplComplianceSiteDo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject
     *      (gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    @Override
    protected EplComplianceSiteDo toDataObject(ComplianceSiteVo value) {
        if (value == null){
            return null;
        }        
        
        EplComplianceSiteDo data = new EplComplianceSiteDo(value.getCreatedBy(), value.getCreatedDate(), value.getModifiedBy(),
            value.getModifiedDate(), false);
        
        data.setIncludeExcludeIndicator(value.getIncludeExcludeIndicator());

        if (value.getVistaSiteVo() != null){
            data.setStationNumber(value.getVistaSiteVo().getStationNumber());
            data.setMinimallyPopulated(true);
        }
        
        return data;
    }
   
    /**
     * Fully copies the ComplianceSiteVo to a EplComplianceSiteDo.
     * 
     * @param valueList EplComplianceSiteDo
     * @return fully populated EplComplianceSiteDo List
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject
     *      (gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    protected List<EplComplianceSiteDo> toDataObject(List<ComplianceSiteVo> valueList) {
        List<EplComplianceSiteDo> list = new ArrayList<EplComplianceSiteDo>();
        if (valueList != null){
            valueList.stream().forEach(value -> list.add(toDataObject(value)));
        }
        return list;
    }
}
