package gov.va.med.pharmacy.peps.domain.common.utility.impl;

import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidationType.LOG_FORGING;
import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator.validateStringInput;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Component;

import gov.va.med.pharmacy.peps.common.utility.EnvironmentUtility;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryVo;
import gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo;
import gov.va.med.pharmacy.peps.common.vo.NotificationVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.domain.common.capability.NotificationDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProductDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability;
import gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm.RxNormClient;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.ProductConverter;
import gov.va.med.pharmacy.peps.service.common.update.NdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.NdfFileSyncProcessorFactory;
import gov.va.med.pharmacy.peps.common.utility.ConfigFileUtility;
/**
 * This class handles the process of keeping RxNorm data up to date on the EPL_PRODUCTS table. It uses RxNormClient to execute
 * two web service calls to ISAAC and the product DAO to populate the data into the appropriate columns.
 * 
 * @author K. Edgecomb
 *
 */
@Component
public class RxNormCapabilityImpl implements RxNormCapability {

    private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(RxNormCapabilityImpl.class);

    private RxNormClient rxNormClient;
    private EplProductDao eplProductDao;
    private ProductConverter productConverter;
    private NdfFileSyncProcessorFactory syncProcessorFactory;
    private NotificationDomainCapability notificationDomainCapability;
    private EnvironmentUtility environmentUtility;

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability#fetchRxNormUpdateProducts()
     */
    @Override
    //@Transactional
    public List<EplProductDo> fetchRxNormUpdateProducts() {
        List<EplProductDo> products = eplProductDao.retrieve();
        return products;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability#populateAndPersist(gov.va.med.pharmacy.peps.common.vo.
     * UserVo, gov.va.med.pharmacy.peps.domain.common.model.EplProductDo, java.util.Date)
     */
    @Override
    public void populateAndPersist(UserVo user, EplProductDo productDo, Date runDate, ConfigFileUtility configFile) {

        LOG.debug("Start RX Norm update for product: " + validateStringInput(productDo.getVaProductName(), LOG_FORGING)
            + validateStringInput(", VUID: ", LOG_FORGING) + productDo.getVuid());

        // Call the ISAAC web service with VUID to get a product if available
        String productCode = rxNormClient.getCode(productDo.getVuid(), configFile);

        LOG.debug("Product Code System Code from ISAAC: " + validateStringInput(productCode, LOG_FORGING));
        // If the returned data is the same as the current product data, don't push to Vista

        List<Difference> differences = getDifferences(productCode, productDo);
        if (!differences.isEmpty()) {

            LOG.debug("Processing differences on Code System info for: "
                + validateStringInput(productDo.getVaProductName(), LOG_FORGING) + validateStringInput(", VUID: ", LOG_FORGING)
                + productDo.getVuid());

            // Saving audit history records
            ProductVo productVo = getProductConverter().toCodingSystemValueObject(productDo);
            productVo.updateFromModDifferences(
                ModDifferenceVo.asModDifferenceList(differences, true, "Coding System Modification"), user,
                getEnvironmentUtility());

            // Save the Product to the PPS-N Database
            productDo.setCodingSystemCode(productCode);
            if (productCode != null) {
                productDo.setCodingSystem(EplProductDo.RX_NORM_CODING_SYSTEM);
            } else {
                productDo.setCodingSystem(null);
            }
            productDo.setCodingSystemLastUpdateDate(runDate);
            eplProductDao.update(productDo, user);

            saveItemAuditHistoryRecords(productVo.getNewAuditHistory(), user);

            // Sending modifications to VISTA
            NdfFileSyncProcessor fileProcessor = getSyncProcessorFactory().getProcessor(productVo);
            // set the request item statUs to approve so the VISTA update gets processed without waiting for pending changes
            productVo.setRequestItemStatus(RequestItemStatus.APPROVED);
            fileProcessor.processModified(productVo, user, differences);
        } else {
            LOG.debug("No differences found in Product Code System information for: "
                + validateStringInput(productDo.getVaProductName(), LOG_FORGING) + validateStringInput(", VUID: ", LOG_FORGING)
                + productDo.getVuid());
        }

        LOG.debug("Finish RX Norm update for product: " + validateStringInput(productDo.getVaProductName(), LOG_FORGING)
            + validateStringInput(", VUID: ", LOG_FORGING) + productDo.getVuid());
    }

    /**
     * Saves the Audit History related records into the Database
     * 
     * @param itemAudit
     * @param user
     * @deprecated should use ItemAuditHistoryUtility instead, but this class must be moved to the Service Layer where
     * it belongs first.
     */
    @Deprecated
    private void saveItemAuditHistoryRecords(ItemAuditHistoryVo itemAudit, UserVo user) {
        if (!(itemAudit == null)) {
            NotificationVo notification = new NotificationVo();
            ArrayList<ItemAuditHistoryVo> itemAudits = new ArrayList<ItemAuditHistoryVo>();
            itemAudits.add(itemAudit);
            notification.setItemAudits(itemAudits);
            notification.determineNotificationType();
            notificationDomainCapability.create(notification, user);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability #getDifferences(java.lang.String,
     * gov.va.med.pharmacy.peps.domain.common.model.EplProductDo)
     */
    @Override
    public List<Difference> getDifferences(String code, EplProductDo product) {

        List<Difference> differences = new ArrayList<Difference>();

        if (different(product.getCodingSystemCode(), product.getCodingSystem(), code)) {
            differences.add(new Difference(FieldKey.PRODUCT_CODING_SYSTEM_CODE, product.getCodingSystemCode(), code));
            if (code == null) {
                differences.add(new Difference(FieldKey.PRODUCT_CODING_SYSTEM, product.getCodingSystem(), null));
            } else {
                differences.add(new Difference(FieldKey.PRODUCT_CODING_SYSTEM, product.getCodingSystem(),
                    EplProductDo.RX_NORM_CODING_SYSTEM));
            }
        }

        return differences;
    }

    /**
     * Tells if a current product code and isaac code are different and if the coding system is different than "RXNorm"
     * 
     * @param productCode
     * @param codingSystem
     * @param isaacCode
     * @return
     */
    private boolean different(String productCode, String codingSystem, String isaacCode) {

        if (codingSystem != null && !codingSystem.equals(EplProductDo.RX_NORM_CODING_SYSTEM))
            return true;

        return !Objects.equals(productCode, isaacCode);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability#getRxNormClient()
     */
    public RxNormClient getRxNormClient() {
        return rxNormClient;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.peps.domain.common.utility.impl.RxNormCapability#setEplProductDao(gov.va.med.pharmacy.peps.domain.
     * common.dao.EplProductDao)
     */
    @Override
    public void setEplProductDao(EplProductDao eplProductDao) {
        this.eplProductDao = eplProductDao;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability#getEplProductDao()
     */
    public EplProductDao getEplProductDao() {
        return eplProductDao;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.peps.domain.common.utility.impl.RxNormCapability#setRxNormClient(gov.va.med.pharmacy.peps.domain.
     * common.utility.client.rxnorm.RxNormClient)
     */
    @Override
    public void setRxNormClient(RxNormClient rxNormClient) {
        this.rxNormClient = rxNormClient;
    }

    /**
     * Gets a productConverter instance.
     * 
     * @return productCoverter
     */
    public ProductConverter getProductConverter() {
        return productConverter;
    }

    /**
     * Sets a productConverter instance.
     * 
     * @param productConverter ProductConverter property
     */
    public void setProductConverter(ProductConverter productConverter) {
        this.productConverter = productConverter;
    }

    /**
     * Gets an ndfFileSyncProcessorFactory instance.
     * 
     * @return syncProcessorFactory
     */
    public NdfFileSyncProcessorFactory getSyncProcessorFactory() {
        return syncProcessorFactory;
    }

    /**
     * Sets an ndfFileSyncProcessFactory instance.
     * 
     * @param syncProcessorFactory NdfFileSyncProcessorFactory property
     */
    public void setSyncProcessorFactory(NdfFileSyncProcessorFactory syncProcessorFactory) {
        this.syncProcessorFactory = syncProcessorFactory;
    }

    /**
     * Gets a notificationDomainCapability instance.
     * 
     * @return notificationDomainCapability NotificationDomainCapability
     */
    public NotificationDomainCapability getNotificationDomainCapability() {
        return notificationDomainCapability;
    }

    /**
     * Sets a notificationDomainCapability instance.
     * 
     * @param notificationDomainCapability NotificationDomainCapability property
     */
    public void setNotificationDomainCapability(NotificationDomainCapability notificationDomainCapability) {
        this.notificationDomainCapability = notificationDomainCapability;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability#getEnvironmentUtility()
     */
    public EnvironmentUtility getEnvironmentUtility() {
        return environmentUtility;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability#setEnvironmentUtility(gov.va.med.pharmacy.peps.common.
     * utility.EnvironmentUtility)
     */
    public void setEnvironmentUtility(EnvironmentUtility environmentUtility) {
        this.environmentUtility = environmentUtility;
    }

}
