package gov.va.med.pharmacy.peps.service.common.capability.impl;

import java.util.Calendar;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.vista.VistaField;
import gov.va.med.pharmacy.peps.common.vo.NdfSyncable;
import gov.va.med.pharmacy.peps.common.vo.SyncProcessorElement;
import gov.va.med.pharmacy.peps.common.vo.diff.GroupDifference;
import gov.va.med.pharmacy.peps.domain.common.dao.EplNdfOutgoingDifferencesDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.service.common.capability.NdfFileSyncCapability;
import gov.va.med.pharmacy.peps.service.common.update.impl.NdfOutgoingDifferenceConsolidator;

/**
 * Contains methods for managing the outgoing records to sync with VistA in an update file.
 */
public class NdfFileSyncCapabilityImpl implements NdfFileSyncCapability {

    private static final org.apache.logging.log4j.Logger LOG = org.apache.logging.log4j.LogManager
        .getLogger(NdfFileSyncCapabilityImpl.class);

    private EplNdfOutgoingDifferencesDao eplNdfOutgoingDifferencesDao;

    private NdfOutgoingDifferenceConsolidator consolidator;

    /**
     * Constructor injecting all dependencies.
     * 
     * @param eplNdfOutgoingDifferencesDao the DAO
     * @param consolidator the object responsible for consolidating updates for the NDF update file
     */
    @Autowired
    public NdfFileSyncCapabilityImpl(
        EplNdfOutgoingDifferencesDao eplNdfOutgoingDifferencesDao, 
        NdfOutgoingDifferenceConsolidator consolidator) {
        this.eplNdfOutgoingDifferencesDao = eplNdfOutgoingDifferencesDao;
        this.consolidator = consolidator;
    }
    
    private boolean isMultipleField(String vistaFieldNumber) {
        return vistaFieldNumber.contains(",");
    }

    @Override
    public void save(EplNdfOutgoingDifferences unsavedRecord) {
        // For now, we branch off for just a subset of fields to limit the scope of implementing this new pattern,
        // but eventually, all fields should be treated the same
        if (!isMultipleField(unsavedRecord.getVistaFieldNumber())) {
            updatePendingQueue(unsavedRecord);
        } else {
            // multiple field (e.g., ingredients, copay tiers, other list fields, etc.)
            eplNdfOutgoingDifferencesDao.save(unsavedRecord);
        }
    }

    /**
     * Update pending queue of outgoing difference records - records that have not been included in a file yet.
     * 
     * @param unsavedRecord the unsaved record
     */
    protected void updatePendingQueue(EplNdfOutgoingDifferences unsavedRecord) {
        List<EplNdfOutgoingDifferences> existingRecords = eplNdfOutgoingDifferencesDao.findPendingRecordsForFileFieldAndIen(
            unsavedRecord.getVistaFileNumber(), unsavedRecord.getVistaFieldNumber(), unsavedRecord.getVistaIen());

        if (existingRecords.isEmpty()) {
            eplNdfOutgoingDifferencesDao.save(unsavedRecord);
        } else {
            consolidatePendingRecords(unsavedRecord, existingRecords);
        }
    }

    /**
     * Consolidate pending records. The result is that there should only ever be one pending record for a given file/field/ien
     * combination.
     *
     * @param unsavedDifference the unsaved difference
     * @param existingRecords the existing records
     */
    private void consolidatePendingRecords(EplNdfOutgoingDifferences unsavedDifference,
        List<EplNdfOutgoingDifferences> existingRecords) {
        if (existingRecords.size() > 1) {
            String msg = "Unsupported Data Situation - multiple pending EplNdfOutgoingDifferences exist for file: "
                + unsavedDifference.getVistaFileNumber() + ", field: " + unsavedDifference.getVistaFieldNumber() + ", IEN: "
                + unsavedDifference.getVistaIen();
            LOG.error(msg);
            throw new IllegalStateException(msg);
        }

        EplNdfOutgoingDifferences existingRecord = existingRecords.get(0);
        // If value was reverted and it is a modify record, then VistA already has the same value as PPS-N.
        if (wasValueReverted(existingRecord, unsavedDifference)
            && EplNdfOutgoingDifferences.MODIFY_ACTION.equals(existingRecord.getActionType())) {
            // change reverts the value back to where it started, we don't need either record
            eplNdfOutgoingDifferencesDao.delete(existingRecord);
        } else {
            // Change does not revert value, or the existing record was an add so update record so latest value gets sent to
            // VistA. Note that we do not save the new difference object in this case.
            existingRecord.setNewValue(unsavedDifference.getNewValue());
            existingRecord.setCreatedBy(unsavedDifference.getCreatedBy());
            existingRecord.setCreatedDtm(unsavedDifference.getCreatedDtm());
            eplNdfOutgoingDifferencesDao.update(existingRecord);
        }

    }

    /**
     * Insert a modified element - making modifications to an item that was sent to VistA in a previous update file.
     * 
     * @param element containing the values for a single outgoing difference record
     */
    protected void insertModifiedElement(SyncProcessorElement element) {
        EplNdfOutgoingDifferences diffObj = new EplNdfOutgoingDifferences();
        diffObj.setVistaIen(element.getVistaIen());
        diffObj.setNewValue(element.getNewValue());
        diffObj.setOldValue(element.getOldValue());
        diffObj.setCreatedBy(element.getUserName());
        diffObj.setVistaFieldNumber(element.getFieldId());
        diffObj.setVistaFileNumber(element.getFileNumber());
        diffObj.setActionType(EplNdfOutgoingDifferences.MODIFY_ACTION); 
        diffObj.setCreatedDtm(Calendar.getInstance().getTime());          
        save(diffObj); 
    }
    
    @Override
    public void processModifiedNdfSyncables(
        GroupDifference difference,  
        String vistaFileNumber,
        String vistaIen,
        String vistaFieldNumberBase,
        String vistaFieldNumber,
        String userValue) {
        // Retrieve all pending outgoing differences, if any, so that the
        // new changes can be reconciled.
        List<EplNdfOutgoingDifferences> pendingDifferences = 
            findPendingRecordsForFileFieldBaseAndIen(
                vistaFileNumber, 
                vistaFieldNumberBase, 
                vistaIen);

        // Delete all pending outgoing differences. We will merge the new changes
        // with the pending and then write the resulting list of outgoing differences.
        removePendingOutgoingDifferencesForMultipleField(
            vistaFileNumber,
            vistaFieldNumberBase, 
            vistaIen);

        // Create SyncProcessorElements from GroupDifference new value
        List<SyncProcessorElement> newElements = 
            NdfSyncable.syncProcessorElementsForNdfSyncables(
                difference.getNewValue(), 
                vistaIen, 
                userValue);
        
        // Generate a checksum for the new values, so that we can compare with the 
        // original checksum.
        String ppsnChecksum = SyncProcessorElement.checksumReduce(
            newElements, 
            element -> element.getNewValue()); 
      
        //Calculate or retrieve checksum representing what VistA already has.
        String vistaChecksum = null;
        
        if(pendingDifferences.isEmpty()) {
            /* Generate checksum based on the oldvalue of the difference. Because PPS-N
             * and VistA are synchronized in this case, the values PPS-N has before 
             * applying the new changes represent what VistA has. So, a checksum based 
             * on PPS-N's old values represents what was most recently sent to VistA. */
            List<SyncProcessorElement> oldElements = 
                NdfSyncable.syncProcessorElementsForNdfSyncables(
                    difference.getOldValue(), 
                    vistaIen, 
                    userValue);            

            // Generate the checksum.
            vistaChecksum = SyncProcessorElement.checksumReduce(
                oldElements, 
                element -> element.getNewValue());
        }
        else {
            /* Retrieve the vistaChecksum stored previously from the existing pending
             * outgoing difference records.*/
            vistaChecksum = pendingDifferences.stream()
                .filter(element -> element.getVistaFieldNumber().contains(".01"))
                .findFirst()
                .get()
                .getOldValue(); 
        }
        
        //Perform the consolidation.
        newElements = consolidator.reconcile(
            newElements, 
            new VistaField(vistaFileNumber, vistaFieldNumber, vistaIen),
            userValue, 
            !pendingDifferences.isEmpty(), 
            vistaChecksum, 
            ppsnChecksum);

        // Store the reconciled outgoing differences.
        for (SyncProcessorElement element : newElements) {
            insertModifiedElement(element);
        }        
    }
    
    /**
     * Determines if value was reverted between two outgoing differences for the same field. The value was reverted if the
     * existing record's old value is equal to the unsaved record's new value. Values comprised of only whitespace, null
     * values and empty strings are all considered equivalent. The whitespace, null and empty string equivalence is needed, 
     * since the existing record from the database may have had an empty string value changed to null per the database vendor,
     * and a single space is used to indicate to VistA that a field has been deleted.
     * 
     * @param existingRecord the existing record for the field
     * @param unsavedRecord the unsaved record for the same field
     * @return true, if the value was reverted; false if the value was not reverted.
     */
    private boolean wasValueReverted(EplNdfOutgoingDifferences existingRecord, EplNdfOutgoingDifferences unsavedRecord) {
        boolean sameValue = StringUtils.equals(
            StringUtils.stripToNull(existingRecord.getOldValue()), 
            StringUtils.stripToNull(unsavedRecord.getNewValue()));
        return sameValue;
    }

    @Override
    public void removePendingOutgoingDifferencesForMultipleField(String vistaFileNumber, String vistaFieldNumberBase,
        String vistaIen) {
        eplNdfOutgoingDifferencesDao.removePendingOutgoingDifferencesForMultipleField(vistaFileNumber, vistaFieldNumberBase,
            vistaIen);

    }

    /**
     * Find pending multiple records for field and ien. The field number specified should be the base field number of the
     * multiple field, without any comma. This will retrieve what VistA terms a "Multiple" field. For example, for product copay
     * tier, passing "45" will find all records with field number "45,.01" "45,.02".
     *
     * @param vistaFileNumber the vista file number
     * @param vistaFieldNumberBase the vista field number base of the VistA Multiple field
     * @param vistaIen the vista ien
     * @return the list
     */
    public List<EplNdfOutgoingDifferences> findPendingRecordsForFileFieldBaseAndIen(String vistaFileNumber,
        String vistaFieldNumberBase, String vistaIen) {
        return eplNdfOutgoingDifferencesDao.findPendingRecordsForFileFieldBaseAndIen(vistaFileNumber, vistaFieldNumberBase,
            vistaIen);
    }
}
