

package gov.va.med.pharmacy.peps.service.common.update.impl;


import java.util.Calendar;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import gov.va.med.pharmacy.peps.common.vista.VistaFile;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.SyncProcessorElement;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.service.common.capability.NdfFileSyncCapability;


/**
 * Abstract Base class of all NdfFileSyncProcessor classes to support queueing information to be included in an update file.
 */
public abstract class AbstractSimpleNdfFileSyncProcessor {
    
    private NdfFileSyncCapability ndfFileSyncCapability;

    private Set<FieldKey> fields;
    private String fileNumber;
    private String fieldAddNumber;

    /**
     * Instantiates a new AbstractSimpleNdfFileSyncProcessor.
     *
     * @param fields the FieldKeys of the fields that, if modified, would trigger the modification logic of this processor
     * @param fileNumber the VistA file number
     * @param fieldAddNumber the VistA field number
     * @param pNdfFileSyncCapability the ndf file sync capability
     */
    public AbstractSimpleNdfFileSyncProcessor(Set<FieldKey> fields, String fileNumber, String fieldAddNumber, 
        NdfFileSyncCapability pNdfFileSyncCapability) {
        this.fields = fields;
        this.fileNumber = fileNumber;
        this.fieldAddNumber = fieldAddNumber;
        this.ndfFileSyncCapability = pNdfFileSyncCapability;

    }
    
    /**
     * Insert new top-level Vista File element.
     *
     * @param vistaIen the vista ien
     * @param newValue the new value
     * @param userName the user name who made or approved the change
     */
    protected void insertNewElement(String vistaIen, String newValue, String userName){
        EplNdfOutgoingDifferences diffObj = new EplNdfOutgoingDifferences();
        diffObj.setVistaIen(vistaIen);
        diffObj.setNewValue(newValue);
        diffObj.setCreatedBy(userName);
        processInsert(diffObj); 
    }
   
    /**
     * Insert new top-level Vista File element.
     *
     * @param element SyncProcessorElement to insert as a new Element
     */
    protected void insertNewElement(SyncProcessorElement element){      
      EplNdfOutgoingDifferences diffObj = new EplNdfOutgoingDifferences();
        diffObj.setVistaIen(element.getVistaIen());
        diffObj.setNewValue(element.getNewValue());
        diffObj.setCreatedBy(element.getUserName());
        processInsert(diffObj); 
    }
    
    /**
     * Insert a modified element - making modifications to an item that was sent to VistA in a previous update file.
     *
     * @param vistaIen the vista ien
     * @param newValue the new value
     * @param userName the user name
     * @param fieldId the field id
     * @param oldValue the old value
     */
    protected void insertModifiedElement(
        String vistaIen, 
        String newValue, 
        String userName, 
        String fieldId, 
        String oldValue){
        
        EplNdfOutgoingDifferences diffObj = new EplNdfOutgoingDifferences();
        diffObj.setVistaIen(vistaIen);
        diffObj.setNewValue(newValue);
        diffObj.setOldValue(oldValue);
        diffObj.setCreatedBy(userName);
        diffObj.setVistaFieldNumber(fieldId);
        processUpdate(diffObj, EplNdfOutgoingDifferences.MODIFY_ACTION); 
    }  
    
    /**
     * Insert new child element, queueing an outgoing difference record.
     *
     * @param vistaIen the vista ien of the main element this field is associated to
     * @param newValue the new value
     * @param userName the user name who made or approved the change.
     * @param fieldId the VistA field id
     * @param action EplNdfOutgoingDifferences.ADD_ACTION or EplNdfOutgoingDifferences.MODIFY_ACTION,
     * indicating whether this is action type "A" or "M"
     */  
    protected void insertNewChildElement(String vistaIen, String newValue, String userName, String fieldId, String action){
        EplNdfOutgoingDifferences diffObj = new EplNdfOutgoingDifferences();
        diffObj.setVistaIen(vistaIen);
        diffObj.setNewValue(newValue);
        diffObj.setCreatedBy(userName);
        diffObj.setVistaFieldNumber(fieldId);
        populateCommonFields(diffObj, action);
        ndfFileSyncCapability.save(diffObj);
    }    

    /**
     * Insert new child element, queueing an outgoing difference record.
     *
     * @param vistaIen the vista ien of the main element this field is associated to
     * @param newValue the new value
     * @param userName the user name who made or approved the change.
     * @param fieldId the VistA field id
     */ 
    protected void insertNewChildElement(String vistaIen, String newValue, String userName, String fieldId) {
        insertNewChildElement(
            vistaIen, 
            newValue, 
            userName, 
            fieldId, 
            EplNdfOutgoingDifferences.ADD_ACTION);
    }    
    
    /**
     * Insert new child element for required fields.  It will queue an outgoing difference record.  If the value is null or 
     * blank (including empty spaces), then a single blank space will be sent. 
     *
     * @param element the SyncProcessorElement with the VistA IEN, New Value, Vista Field Id, and User Name populated
     */    
    protected void insertNewRequiredChildElement(SyncProcessorElement element){
        String requiredNewValue = (StringUtils.isNotBlank(element.getNewValue())) ? element.getNewValue() : VistaFile.BLANK;
        insertNewChildElement(
            element.getVistaIen(), 
            requiredNewValue, 
            element.getUserName(), 
            element.getFieldId(),
            EplNdfOutgoingDifferences.ADD_ACTION);        
    }       
    
    /**
     * Insert new child element for optional fields.  If the value is not null and not blank, it will queue an outgoing 
     * difference record.
     *
     * @param vistaIen the vista ien of the main element this field is associated to
     * @param newValue the new value
     * @param userName the user name who made or approved the change.
     * @param fieldId the VistA field id 
     */
    protected void insertNewOptionalChildElement(String vistaIen, String newValue, String userName, String fieldId){
      if (StringUtils.isNotBlank(newValue)){
        insertNewChildElement(
            vistaIen, 
            newValue, 
            userName, 
            fieldId, 
            EplNdfOutgoingDifferences.ADD_ACTION);
      }

  }     

    /**
     * Inserts an outgoing difference record with {@code EplNdfOutgoingDifferences.ADD_ACTION} for a top-level element.
     *
     * @param diffObj the EplNdfOutgoingDifferences to save
     */
    public void processInsert(EplNdfOutgoingDifferences diffObj) {    
        populateCommonFields(diffObj, EplNdfOutgoingDifferences.ADD_ACTION); 
        diffObj.setVistaFieldNumber(fieldAddNumber);
        ndfFileSyncCapability.save(diffObj);             
    }

    /**
     * Inserts an outgoing difference record with the give action.
     *
     * @param diffObj the EplNdfOutgoingDifferences to save
     * @param action the action type, either {@code EplNdfOutgoingDifferences.MODIFY_ACTION}
     * or {@code EplNdfOutgoingDifferences.ADD_ACTION}
     */    
    public void processUpdate(EplNdfOutgoingDifferences diffObj, String action) {
        populateCommonFields(diffObj, action);
        ndfFileSyncCapability.save(diffObj);        
    }
    
    /**
     * Save the given EplNdfOutgoingDifferences object to the database using
     * action type "M" for modify.
     * 
     * @param diffObj to save
     */
    public void processUpdate(EplNdfOutgoingDifferences diffObj) {
        populateCommonFields(diffObj, EplNdfOutgoingDifferences.MODIFY_ACTION);
        ndfFileSyncCapability.save(diffObj);        
    }
       
    private EplNdfOutgoingDifferences populateCommonFields(EplNdfOutgoingDifferences diffObj, String action){        
        diffObj.setVistaFileNumber(fileNumber);
        diffObj.setActionType(action); 
        diffObj.setCreatedDtm(Calendar.getInstance().getTime());   
        return diffObj;
    }   

    /**
     * Get the NdfFileSyncCapability.
     * 
     * @return NdfFileSyncCapability
     */
    public NdfFileSyncCapability getNdfFileSyncCapability() {
        return ndfFileSyncCapability;
    }
    
    /**
     * Get the fields.
     * 
     * @return fields
     */
    protected Set<FieldKey> getFields() {
        return fields;
    }   

}
