package gov.va.med.pharmacy.peps.service.common.update.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import gov.va.med.pharmacy.peps.common.vista.VistaField;
import gov.va.med.pharmacy.peps.common.vista.VistaFile;
import gov.va.med.pharmacy.peps.common.vo.SyncProcessorElement;

/**
 * Contains operations for reconciling intermediate changes to pending NDF outgoing differences.
 * Intermediate changes are updates in PPS-N which occur in-between successful transmissions of 
 * the NDF update file to VistA.
 */
public class NdfOutgoingDifferenceConsolidator {
    /**
     * Reconcile the PPS-N changes contained in newElements with any pre-existing
     * pending outgoing differences and return a transformed list of SyncProcessorElements
     * containing only the data that should be transmitted to VistA in the next
     * NDF update file. This method applies to group list data fields that are 
     * synchronized with VistA (specifically only those VO classes which implement the 
     * NdfSyncable interface).
     * 
     * The incoming changes contained in newElements may be discarded if they are the
     * same as what VistA most recently received from PPS-N. This is true if and only if
     * vistaChecksum equals ppsnChecksum. In this case, an empty list is returned,
     * indicating that no outgoing differences are necessary because PPS-N and VistA
     * are already synchronized.
     * 
     * If the checksums do not match, then the vistaChecksum is copied into the oldValue 
     * field of the .01 items contained in newElements, so that the same comparison can
     * be made again should further changes be made in PPS-N.
     * 
     * If newElements is empty, a special 'DELETED' item is added and returned, which
     * indicates to VistA that PPS-N has removed all data for this vistaIen and
     * vistaFieldNumber.
     * 
     * @param newElements representing the updated values in PPS-N corresponding to a 
     * particular vistaIen (e.g., product, etc.) and vistaFieldNumber (e.g., copay tier,
     * ingredient, etc.)
     * @param field identifies the particular file in VistA, the top-level entity in VistA, such as
     * the particular product, dosage form, etc., and the particular field within the 
     * top-level entity to which the newElements correspond, such as the effective date on a copay tier or 
     * the strength on an ingredient
     * @param userValue the name of the user creating the outgoing differences
     * @param hasPendingOutgoingDifferences true if and only if PPS-N has pre-existing 
     * pending outgoing differences for this vistaIen and vistaFieldNumber
     * @param vistaChecksum represents the data most recently successfully transmitted to
     * VistA.
     * @param ppsnChecksum represents the updated data in PPS-N in the same format as
     * vistaChecksum
     * @return a list of SyncProcessorElements according to the following decision matrix:
     * <pre>
     * Has Pending | Checksums match | newElements is empty | Return Value
     * ------------------------------------------------------------------------------------------------------------------------
     * true        | true            | true                 | empty list
     * true        | false           | true                 | replace newElements with DELETED element, including vistaChecksum
     * false       | true            | true                 | N/A - Can't happen  
     * false       | false           | true                 | replace newElements with DELETED element, including vistaChecksum
     * true        | true            | false                | empty list  
     * true        | false           | false                | echo newElements back with vistaChecksum in oldValue  
     * false       | true            | false                | N/A - Can't happen  
     * false       | false           | false                | echo newElements back with vistaChecksum in oldValue 
     * </pre>  
     */
    public List<SyncProcessorElement> reconcile(
        List<SyncProcessorElement> newElements,
        final VistaField field, 
        final String userValue,
        final boolean hasPendingOutgoingDifferences, 
        final String vistaChecksum, 
        final String ppsnChecksum) {

        boolean changesReverted = Objects.equals(vistaChecksum, ppsnChecksum);

        if (newElements.isEmpty()) {
            // All group list values have been removed; insert special indicator into
            // outgoing differences to trigger removal in VistA.
            String vistaIenChild = field.getIen() + "," + 1;
            newElements.add(
                new SyncProcessorElement(
                    field.getFileNumber(),
                    vistaIenChild, 
                    VistaFile.ALL_DELETED, 
                    userValue, 
                    field.getFieldNumber(), 
                    vistaChecksum));
        }
        
        /* If PPS-N has new values that are different from VistA, i.e., changes have not
         * been reverted, then carry the existing vistaChecksum forward.*/
        if(!changesReverted) {
            newElements.stream().filter(element -> element.getFieldId().contains(".01"))
                .forEach(element -> element.setOldValue(vistaChecksum));
        }
        else if (hasPendingOutgoingDifferences && changesReverted) {
            /* Changes reverted (checksums match). Since the checksums match, we know that 
             * PPS-N and VistA are already synchronized. There is no need to send the new 
             * changes to VistA. Outgoing differences should already have been deleted before
             * calling this method, so return an empty list.*/
            newElements = new ArrayList<SyncProcessorElement>();
        }

        return newElements;
    }
}
