package gov.va.med.pharmacy.peps.service.common.update.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import gov.va.med.pharmacy.peps.common.vista.VistaField;
import gov.va.med.pharmacy.peps.common.vo.SyncProcessorElement;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.domain.common.model.VistaFieldOutgoingDifference;

/**
 * This class contains operations for reconciling outgoing difference records 
 * contained in a rejected NDF update file with pending difference records in PPS-N.
 */
public class NdfRejectFileConsolidator {
    private static final String MULTIPLE_FIELD_INDICATOR = ".01";

    /**
     * Reconciles originalPendingDiffs from PPS-N's outgoing differences with the
     * originalRejectedDiffs from a rejected NDF update file. Mutually exclusive 
     * differences in both lists are retained unchanged. Differences which exist
     * in both pending and rejected lists are combined into a new record containing
     * the pending new value and the rejected old value.
     * 
     * @param originalPendingDiffs representing edits made in PPS-N which have not
     * been synchronized with VistA yet
     * @param originalRejectedDiffs representing differences contained in a rejected
     * NDF update file
     * @return the reconciled list of differences, suitable for use as the new pending
     * outgoing difference list in PPS-N
     */
    public List<EplNdfOutgoingDifferences> reconcile(
        List<EplNdfOutgoingDifferences> originalPendingDiffs, 
        List<EplNdfOutgoingDifferences> originalRejectedDiffs) {
        
        List<EplNdfOutgoingDifferences> result = new ArrayList<>();
        
        //Transform originalPendingDiffs to VistaFieldOutgoingDifferences for 
        //comparison.
        List<VistaFieldOutgoingDifference> pendingDiffs = 
            asVistaFieldOutgoingDifferences(originalPendingDiffs);
        
        //Transform originalRejectedDiffs to VistaFieldOutgoingDifferences for 
        //comparison.
        List<VistaFieldOutgoingDifference> rejectedDiffs = 
            asVistaFieldOutgoingDifferences(originalRejectedDiffs);
        
        //Add mutually exclusive rejectedDiffs to results.
        result.addAll(
            rejectedDiffs.stream()
                .filter(diff -> !pendingDiffs.contains(diff))
                .map(diff -> diff.getEplNdfOutgoingDifferences())
                .collect(Collectors.toList()));
        
        //Add mutually exclusive pendingDiffs to results.
        result.addAll(
            pendingDiffs.stream()
                .filter(diff -> !rejectedDiffs.contains(diff))
                .map(diff -> diff.getEplNdfOutgoingDifferences())
                .collect(Collectors.toList()));
        
        //Compute intersection.
        List<VistaFieldOutgoingDifference> pendingIntersection = 
            intersection(pendingDiffs, rejectedDiffs);
        List<VistaFieldOutgoingDifference> rejectedIntersection =
            intersection(rejectedDiffs, pendingDiffs);
        
        //Merge and add intersection.
        List<EplNdfOutgoingDifferences> mergedDiffs = 
            merge(pendingIntersection, rejectedIntersection);        
        result.addAll(mergedDiffs);        
        
        //Reset id's so that new pending records may be created.
        result.stream().forEach(diff -> {
            diff.setId(null); 
            diff.setNdcUpdateFileFk(null);
        });
        
        return result;
    }

    private static List<VistaFieldOutgoingDifference> intersection(
        List<VistaFieldOutgoingDifference> leftDiffs,
        List<VistaFieldOutgoingDifference> rightDiffs) {
        List<VistaFieldOutgoingDifference> intersection = new ArrayList<>(leftDiffs);
        intersection.retainAll(rightDiffs); 
        return intersection;
    }

    private static List<EplNdfOutgoingDifferences> merge(
        List<VistaFieldOutgoingDifference> pendingIntersection,
        List<VistaFieldOutgoingDifference> rejectedIntersection) {              
        Map<VistaField, List<EplNdfOutgoingDifferences>> pendingMap = 
            mapDifferencesByVistaField(pendingIntersection); 
        
        Map<VistaField, List<EplNdfOutgoingDifferences>> rejectedMap = 
            mapDifferencesByVistaField(rejectedIntersection);
        
        List<EplNdfOutgoingDifferences> result = new ArrayList<>();
        
        for(VistaField field : pendingMap.keySet()) {
            List<EplNdfOutgoingDifferences> pendingList = pendingMap.get(field);
            List<EplNdfOutgoingDifferences> rejectedList = rejectedMap.get(field);
            boolean reverted = false;
            
            if(pendingList.size() == 1) {
                //Single-value field.
                EplNdfOutgoingDifferences pendingDiff = pendingList.get(0);
                EplNdfOutgoingDifferences rejectedDiff = 
                    rejectedMap.get(field).get(0);
                
                if(Objects.equals(pendingDiff.getNewValue(), rejectedDiff.getOldValue())) {
                    reverted = true;
                }
                else {
                    pendingDiff.setOldValue(rejectedDiff.getOldValue());
                }
            }
            else {
                //Multiple field.
                String vistaChecksum = rejectedList.stream()
                    .filter(diff -> diff.getVistaFieldNumber().contains(MULTIPLE_FIELD_INDICATOR))
                    .findFirst()
                    .get()
                    .getOldValue();
                
                if(multipleFieldHasReverted(pendingList, vistaChecksum)) {
                    reverted = true;
                }
                else {                    
                    pendingList.stream()
                        .filter(diff -> diff.getVistaFieldNumber().contains(MULTIPLE_FIELD_INDICATOR))
                        .forEach(diff -> diff.setOldValue(vistaChecksum));
                }
            }
            
            if(!reverted) {
                result.addAll(pendingList);
            }
        }
        
        return result;
    }

    private static boolean multipleFieldHasReverted(
        List<EplNdfOutgoingDifferences> pendingList, String vistaChecksum) {
        List<SyncProcessorElement> pendingElements = pendingList.stream()
            .map(
                diff -> 
                new SyncProcessorElement(
                    diff.getVistaFileNumber(), 
                    diff.getVistaIen(), 
                    diff.getNewValue(), 
                    null, 
                    diff.getVistaFieldNumber(), 
                    SyncProcessorElement.IS_MULTIPLE_FIELD))
            .collect(Collectors.toList());
        String ppsnChecksum = SyncProcessorElement.checksumReduce(
            pendingElements, 
            element -> element.getNewValue()); 
        return Objects.equals(ppsnChecksum, vistaChecksum);
    }

    private static Map<VistaField, List<EplNdfOutgoingDifferences>> mapDifferencesByVistaField(
        List<VistaFieldOutgoingDifference> differences) {
        Map<VistaField, List<EplNdfOutgoingDifferences>> resultMap = new HashMap<>();
        
        for(VistaFieldOutgoingDifference diff : differences) {
            List<EplNdfOutgoingDifferences> currentList = resultMap.get(diff.getVistaField());
            
            if(currentList == null) {
                currentList = new ArrayList<>();
            }
            
            currentList.add(diff.getEplNdfOutgoingDifferences());
            resultMap.put(diff.getVistaField(), currentList);
        }
        
        return resultMap;
    }

    private static List<VistaFieldOutgoingDifference> asVistaFieldOutgoingDifferences(
        List<EplNdfOutgoingDifferences> diffs) {
        
        List<VistaFieldOutgoingDifference> result = new ArrayList<>();
        
        for(EplNdfOutgoingDifferences diff : diffs) {
            VistaField field = new VistaField(
                diff.getVistaFileNumber(),
                diff.getVistaFieldNumber().split(",")[0],
                diff.getVistaIen().split(",")[0]);
            result.add(new VistaFieldOutgoingDifference(field, diff));
        }
        
        return result;
    }
}
