
package gov.va.med.pharmacy.peps.service.common.update.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;
import gov.va.med.pharmacy.peps.common.utility.NdfProductDataConversionUtility;
import gov.va.med.pharmacy.peps.common.utility.PPSConstants;
import gov.va.med.pharmacy.peps.common.vista.VistaFile;
import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;
import gov.va.med.pharmacy.peps.common.vo.CsFedScheduleVo;
import gov.va.med.pharmacy.peps.common.vo.DispenseUnitVo;
import gov.va.med.pharmacy.peps.common.vo.DosageFormVo;
import gov.va.med.pharmacy.peps.common.vo.DrugClassGroupVo;
import gov.va.med.pharmacy.peps.common.vo.DrugClassVo;
import gov.va.med.pharmacy.peps.common.vo.DrugUnitVo;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorVo;
import gov.va.med.pharmacy.peps.common.vo.GenericNameVo;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.NdfSyncable;
import gov.va.med.pharmacy.peps.common.vo.OrderableItemVo;
import gov.va.med.pharmacy.peps.common.vo.PossibleDosagesAutoCreate;
import gov.va.med.pharmacy.peps.common.vo.ProductPackage;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.common.vo.SyncProcessorElement;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.VuidStatusHistoryVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.common.vo.diff.GroupDifference;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.external.common.utility.AbstractConverter;
import gov.va.med.pharmacy.peps.external.common.vo.outbound.common.ItemAction;
import gov.va.med.pharmacy.peps.service.common.capability.NdfFileSyncCapability;
import gov.va.med.pharmacy.peps.service.common.update.NdfFileSyncProcessor;

/**
 * NdfFileSyncProcessor to add new and modified Product information to the outgoing differences queue to be sent to VistA.
 */
public class VaProductNdfFileSyncProcessor extends AbstractSimpleNdfFileSyncProcessor implements NdfFileSyncProcessor {

  /** Fields that this processor will process. */
  protected static final Set<FieldKey> FIELDS = Collections.unmodifiableSet(new LinkedHashSet<FieldKey>(Arrays
      .asList(new FieldKey[] { 
          FieldKey.VA_PRODUCT_NAME, 
          FieldKey.GENERIC_NAME, 
          FieldKey.DOSAGE_FORM, 
          FieldKey.PRODUCT_STRENGTH,
          FieldKey.PRODUCT_UNIT, 
          FieldKey.ACTIVE_INGREDIENT, 
          FieldKey.NATIONAL_FORMULARY_NAME, 
          FieldKey.VA_PRINT_NAME,
          FieldKey.CMOP_ID, 
          FieldKey.CMOP_DISPENSE, 
          FieldKey.DISPENSE_UNIT,
          FieldKey.GCN_SEQUENCE_NUMBER, 
          FieldKey.DRUG_CLASSES,
          FieldKey.NATIONAL_FORMULARY_INDICATOR, 
          FieldKey.CS_FED_SCHEDULE,
          FieldKey.SINGLE_MULTISOURCE_PRODUCT, 
          FieldKey.INACTIVATION_DATE,
          FieldKey.EXCLUDE_DRG_DRG_INTERACTION_CHECK,
          FieldKey.OVERRIDE_DF_DOSE_CHK_EXCLUSN, 
          FieldKey.CREATE_POSSIBLE_DOSAGE,
          FieldKey.PRODUCT_PACKAGE, 
          FieldKey.FDA_MED_GUIDE, 
          FieldKey.POSSIBLE_DOSAGES_AUTO_CREATE,
          FieldKey.ORDERABLE_ITEM,   //In order to get modifications to Dosage Form caused by changing Orderable Item.
          FieldKey.ITEM_STATUS,
          FieldKey.COPAY_TIER_ASSOC,
          FieldKey.HAZARDOUS_TO_HANDLE, // Hazardous waste fields.
          FieldKey.HAZARDOUS_TO_DISPOSE,
          FieldKey.PRIMARY_EPA,
          FieldKey.WASTE_SORT_CODE,
          FieldKey.DOT_SHIPPING_NAME,
          FieldKey.PRODUCT_TEXT,
          FieldKey.PRODUCT_FORMULARY_DESIGNATOR,
          FieldKey.PRODUCT_CODING_SYSTEM,
          FieldKey.PRODUCT_CODING_SYSTEM_CODE
          })));


  /**
   * Instantiates a new VaProductNdfFileSyncProcessor.
   *
   * @param pNdfFileSyncCapability the ndf file sync capability
   * @see AbstractSimpleNdfFileSyncProcessor#AbstractSimpleNdfFileSyncProcessor(Set, String, String)
   */
  public VaProductNdfFileSyncProcessor(NdfFileSyncCapability pNdfFileSyncCapability) {
    super(FIELDS, VistaProductFile.FILE_NUMBER, VistaProductFile.NEW_PRODUCT_FIELD_NUMBER, pNdfFileSyncCapability);
  }

    /**
     * For approved new products, send all VistA required field values or a blank single space. For optional fields, only send
     * if they have a non-null, non-empty value.
     * 
   * @see gov.va.med.pharmacy.peps.service.common.update.NdfFileSyncProcessor#
   * processNew(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, gov.va.med.pharmacy.peps.common.vo.UserVo)
   */
  @Override
  public void processNew(ManagedItemVo managedItem, UserVo user) {

    ProductVo productVo = (ProductVo) managedItem;

    if (RequestItemStatus.APPROVED.equals(productVo.getRequestItemStatus()) && productVo.getValue() != null) {
      String vistaIen = String.valueOf(productVo.getNdfProductIen());
      String userName = user.getUsername();
      
      processNewVistaRequiredFields(productVo, vistaIen, userName);     
      processNewVistAOptionalFields(productVo, vistaIen, userName);
    }

  }

  /**
   * Process all the VistA required fields for a new Product.
   *
   * @param productVo the product vo
   * @param vistaIen the vista ien
   * @param userName the user name
   */
  protected void processNewVistaRequiredFields(ProductVo productVo, String vistaIen, String userName) {
    //This method was purposefully designed to be able to create tests that would catch when new fields were added.
    List<SyncProcessorElement> elementsToSendList = new ArrayList<SyncProcessorElement>();
    String newValue;
    
    //The main new element field to denote a new Product
    newValue = productVo.getVaProductName();
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.NEW_PRODUCT_FIELD_NUMBER);    

    // Master Entry For VUID
    newValue = NdfProductDataConversionUtility.convertBooleanDigit(productVo.isMasterEntryForVuid()); 
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.MASTERVUID_FIELD_NUMBER);
        
    newValue = NdfProductDataConversionUtility.getStringValue(productVo.getVuid());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.VUID_FIELD_NUMBER);

    // Effective Date Time Records
    // Note: PPS-N does not update the VUID status history when the product is inactivated.
    // Instead, VistA inactivates the VUID based on product inactivation alone.
    if (productVo.getEffectiveDates() != null) {
      int pairValue = 0;
      
      for (VuidStatusHistoryVo effectiveDate : productVo.getEffectiveDates()) {
        pairValue = pairValue + 1;
        String vistaIenChild = vistaIen + "," + pairValue;
        newValue = MumpsConverter.convertDate(effectiveDate.getEffectiveDateTime());
        elementsToSendList.add(
            new SyncProcessorElement(
                VistaProductFile.FILE_NUMBER,
                vistaIenChild, 
                newValue, 
                userName, 
                VistaProductFile.CHILD_FIELD_NUMBER01, 
                SyncProcessorElement.IS_MULTIPLE_FIELD));

        newValue = effectiveDate.getItemStatus().isActive() ? "1" : "0";   
        elementsToSendList.add(
            new SyncProcessorElement(
                VistaProductFile.FILE_NUMBER,
                vistaIenChild, 
                newValue, 
                userName, 
                VistaProductFile.CHILD_FIELD_NUMBER02, 
                SyncProcessorElement.IS_MULTIPLE_FIELD));
      }
    }      
    
    newValue = NdfProductDataConversionUtility.convertDosageForm(productVo.getDosageForm());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.DOSAGEFORM_FIELD_NUMBER);

    newValue = NdfProductDataConversionUtility.getStringValue(productVo.getProductStrength());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.STRENGTH_FIELD_NUMBER);

    newValue = NdfProductDataConversionUtility.convertDrugUnit(productVo.getProductUnit());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.PRODUCTUNIT_FIELD_NUMBER);

    newValue = NdfProductDataConversionUtility.convertGenericName(productVo.getGenericName());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.GENERIC_IEN_FIELD_NUMBER);

    processNewDrugClassPrimaryRequired(elementsToSendList, productVo.getDrugClasses(), vistaIen, userName);

    // Active Ingredients
    elementsToSendList.addAll(
        NdfSyncable.syncProcessorElementsForNdfSyncables(
            productVo.getActiveIngredients(), vistaIen, userName));

    newValue = NdfProductDataConversionUtility.convertBooleanDigit(productVo.isOverrideDfDoseChkExclusn());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.OVERRIDEDF_FIELD_NUMBER);
    
    for (SyncProcessorElement element : elementsToSendList) {     
        if (VistaProductFile.NEW_PRODUCT_FIELD_NUMBER.equals(element.getFieldId())){
            insertNewElement(element);
        } else {
            String requiredNewValue = (StringUtils.isNotBlank(element.getNewValue())) ? element.getNewValue() : VistaFile.BLANK;
            
            insertNewChildElement(
                element.getVistaIen(), 
                requiredNewValue, 
                element.getUserName(), 
                element.getFieldId(),
                EplNdfOutgoingDifferences.ADD_ACTION); 
        }
    }
    
  }

  private void addElementToSend(List<SyncProcessorElement> elementsToSendList, String vistaIen, String newValue, 
      String userName, String fieldId){
    elementsToSendList.add(new SyncProcessorElement(vistaIen, newValue, userName, fieldId));
  }
  
  /**
     * Process Drug Class Codes for a new product. Sends the primary drug class in field 15 and any secondary drug classes in
     * the multiple field 16.
   *
   * @param elementsToInsertMap the elements to insert map
   * @param drugClasses the va product vo
   * @param vistaIen the vista ien
   * @param userName the user name
   */
  private void processNewDrugClassPrimaryRequired(List<SyncProcessorElement> elementsToSendList, 
      Collection<DrugClassGroupVo> drugClasses,String vistaIen, String userName) {
    // TODO: Primary drug class is Vista required, should we send blank or throw an exception if no primary drug classes? 
    for (DrugClassGroupVo drugClass : drugClasses) {
      if (drugClass.getPrimary()) {
        addElementToSend(elementsToSendList, vistaIen, getDrugClassIen(drugClass), userName, 
            VistaProductFile.PRIMARYDRUGCLASS_FIELD_NUMBER);
      }
    }
  }

  /**
     * Process Drug Class Codes for a new product. Sends the primary drug class in field 15 and any secondary drug classes in
     * the multiple field 16.
   *
   * @param elementsToInsertMap the elements to insert map
   * @param drugClasses the va product vo
   * @param vistaIen the vista ien
   * @param userName the user name
   */
  private void processNewDrugClassesSecondaryOptional(List<SyncProcessorElement> elementsToSendList, 
      Collection<DrugClassGroupVo> drugClasses,String vistaIen, String userName) {
    for (DrugClassGroupVo drugClass : drugClasses) {
      if (!drugClass.getPrimary()) {
        addElementToSend(elementsToSendList, vistaIen + "," + getDrugClassIen(drugClass), getDrugClassIen(drugClass), 
            userName, VistaProductFile.SECONDARYDRUGCLASS_FIELD_NUMBER);
      }
    }
  }  
  
    @Override
    public void processModified(ManagedItemVo managedItem, UserVo user, Collection<Difference> differences) {

        ProductVo vaProductVo = (ProductVo) managedItem;

        Map<FieldKey, Difference> setDifference = Difference.toDifferenceMap(differences);

        addDifferences(setDifference);

        boolean hasDifference = AbstractConverter.hasNewOrModifiedFields(super.getFields(), setDifference, ItemAction.MODIFY);

        if (hasDifference) {
            if (RequestItemStatus.APPROVED.equals(vaProductVo.getRequestItemStatus()) && vaProductVo.getValue() != null) {
                String vistaIen;
                String userValue;
                // IEN
                vistaIen = vaProductVo.getNdfProductIen().toString();

                userValue = user.getUsername();

                for (FieldKey key : super.getFields()) {
                    if (setDifference.containsKey(key)) {
                        if (AbstractConverter.fieldWasModified(key, differences)) {
                            if (FieldKey.ITEM_STATUS.equals(key)) {
                                modifyElement(key, setDifference.get(FieldKey.INACTIVATION_DATE), userValue, vistaIen,
                                    vaProductVo);
                            } else if (FieldKey.DRUG_CLASSES.equals(key)) {
                                processModifiedProductDrugClasses(vaProductVo, setDifference, vistaIen, userValue, key);
                            } else if (FieldKey.COPAY_TIER_ASSOC.equals(key)) {
                                getNdfFileSyncCapability().processModifiedNdfSyncables(
                                    (GroupDifference) setDifference.get(key), 
                                    VistaProductFile.FILE_NUMBER,
                                    vistaIen,
                                    VistaProductFile.COPAY_TIER_FIELD_NUMBER_BASE, 
                                    VistaProductFile.COPAY_TIER_FIELD_NUMBER,
                                    userValue);
                            } else if (FieldKey.ACTIVE_INGREDIENT.equals(key)) {
                                getNdfFileSyncCapability().processModifiedNdfSyncables(
                                    (GroupDifference) setDifference.get(key), 
                                    VistaProductFile.FILE_NUMBER,
                                    vistaIen,
                                    VistaProductFile.INGREDIENT_FIELD_NUMBER_BASE, 
                                    VistaProductFile.INGREDIENT_FIELD_NUMBER,
                                    userValue);
                            } else {
                                modifyElement(key, setDifference.get(key), userValue, vistaIen, vaProductVo);
                            }
                        }
                    }
                }
            }
        }

    }

  /**
   * Secondary Drug Classes - if a drug class was changed from primary to secondary, it is a new secondary with no old value.
   * Likewise, if a drug class is changed from a secondary to primary, it is removed from secondary and added as a new primary.
   * 
   * @param vaProductVo
   * @param setDifference
   * @param vistaIen
   * @param userValue
   * @param key
   */
  private void processModifiedProductDrugClasses(ProductVo vaProductVo, Map<FieldKey, Difference> setDifference,
      String vistaIen, String userValue, FieldKey key) {
    // addDifferences already found primary drug class changes and stored them in setDifference under different key
    if (setDifference.get(FieldKey.PRIMARY_DRUG_CLASS) != null) {
      modifyElement(key, setDifference.get(FieldKey.PRIMARY_DRUG_CLASS), userValue, vistaIen, vaProductVo);
    }

    // secondary drug classes
    @SuppressWarnings("unchecked")
    Collection<DrugClassGroupVo> oldList = findSecondaryDrugClasses(
        (Collection<DrugClassGroupVo>) AbstractConverter.getOldValue(FieldKey.DRUG_CLASSES, setDifference));
    @SuppressWarnings("unchecked")
    Collection<DrugClassGroupVo> newList = findSecondaryDrugClasses(
        (Collection<DrugClassGroupVo>) AbstractConverter.getNewValue(FieldKey.DRUG_CLASSES, setDifference));

    for (DrugClassGroupVo oldDrugClassGroupVo : oldList) {
      boolean foundMatch = false;
      for (DrugClassGroupVo newDrugClassGroupVo : newList) {
        if (newDrugClassGroupVo.equals(oldDrugClassGroupVo)) {
          // unchanged record, no need to send anything
          newList.remove(oldDrugClassGroupVo);
          foundMatch = true;
          break;
        } else if (oldDrugClassGroupVo.getDrugClass().getVuid().equals(newDrugClassGroupVo.getDrugClass().getVuid())) {
          // changed
          insertModifiedElement(vistaIen + "," + getDrugClassIen(newDrugClassGroupVo), getDrugClassIen(newDrugClassGroupVo),
              userValue, VistaProductFile.SECONDARYDRUGCLASS_FIELD_NUMBER, getDrugClassIen(oldDrugClassGroupVo));
          newList.remove(oldDrugClassGroupVo);
          foundMatch = true;
          break;
        }
      }
      if (!foundMatch) {
        // old didn't find a new match - send remove
        insertModifiedElement(vistaIen + "," + getDrugClassIen(oldDrugClassGroupVo), null, userValue,
            VistaProductFile.SECONDARYDRUGCLASS_FIELD_NUMBER, getDrugClassIen(oldDrugClassGroupVo));
      }
    }
    // process any remaining new records - send new
    for (DrugClassGroupVo newDrugClassGroupVo : newList) {
      insertModifiedElement(vistaIen + "," + getDrugClassIen(newDrugClassGroupVo), getDrugClassIen(newDrugClassGroupVo),
          userValue, VistaProductFile.SECONDARYDRUGCLASS_FIELD_NUMBER, null);
    }
  }

  /**
   * @param drugClassGroupVo
   * @return
   */
  private String getDrugClassIen(DrugClassGroupVo drugClassGroupVo) {
    return (drugClassGroupVo.getDrugClass().getDrugClassIen() == null) ? VistaFile.BLANK
        : drugClassGroupVo.getDrugClass().getDrugClassIen();
  }

  /**
   * Find the secondary drug class list.
   * 
   * @param group drug class group
   * @return primary drug class if found, otherwise an empty drug class
   */
  private static Collection<DrugClassGroupVo> findSecondaryDrugClasses(Collection<DrugClassGroupVo> group) {
    if (group != null) {
      // copy the list
      List<DrugClassGroupVo> secondaryDrugClassList = new ArrayList<DrugClassGroupVo>(group);
      // loop through the original list and remove the primary from the copy
      for (DrugClassGroupVo drugClass : group) {
        if (drugClass.getPrimary()) {
          secondaryDrugClassList.remove(drugClass);
        }
      }
      return secondaryDrugClassList;
    }

    return new ArrayList<DrugClassGroupVo>();
  }

  /**
   * Process all the VistA optional fields for a new Product.
   *
   * @param productVo the product vo
   * @param vistaIen the vista ien
   * @param userName the user name
   */
  protected void processNewVistAOptionalFields(ProductVo productVo, String vistaIen, String userName) {
    //This method was purposefully designed to be able to create tests that would catch when new fields were added.
    List<SyncProcessorElement> elementsToSendList = new ArrayList<SyncProcessorElement>();
    String newValue;    
    
    processNewDrugClassesSecondaryOptional(elementsToSendList, productVo.getDrugClasses(), vistaIen, userName);
    
    newValue = NdfProductDataConversionUtility.getStringValue(productVo.getNationalFormularyName());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.NATIONALFORMULARY_FIELD_NUMBER);

    newValue = NdfProductDataConversionUtility.getStringValue(productVo.getVaPrintName());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.VAPRINTNAME_FIELD_NUMBER);

    newValue = NdfProductDataConversionUtility.getStringValue(productVo.getCmopId());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.VAPRODUCTIDENTIFIER_FIELD_NUMBER);

    newValue = NdfProductDataConversionUtility.convertBooleanDigit(productVo.getCmopDispense());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.CMOPINDICATOR_FIELD_NUMBER);

    newValue = NdfProductDataConversionUtility.convertDispenseUnit(productVo.getDispenseUnit());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.DISPENSEUNIT_FIELD_NUMBER);

    newValue = NdfProductDataConversionUtility.getStringValue(productVo.getGcnSequenceNumber());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.GCNSEQUENCE_FIELD_NUMBER);

    newValue = NdfProductDataConversionUtility.convertBooleanDigit(productVo.getNationalFormularyIndicator());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.NATFORMULARYIND_FIELD_NUMBER);

    if (productVo.getCsFedSchedule() != null) {
      newValue = NdfProductDataConversionUtility.convertFedSchedule(productVo.getCsFedSchedule().getValue());
      addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.CSFEDSCH_FIELD_NUMBER);
    }

    // Single/Multisource Product

    if (productVo.getSingleMultiSourceProduct() != null) {
      if (productVo.getSingleMultiSourceProduct().getValue().equals(PPSConstants.MULTISOURCE_MULTI)) {
        addElementToSend(elementsToSendList, vistaIen, "M", userName, VistaProductFile.SOURCE_FIELD_NUMBER);
      } else if (productVo.getSingleMultiSourceProduct().getValue().equals(PPSConstants.MULTISOURCE_BOTH)) {
        addElementToSend(elementsToSendList, vistaIen, "M", userName, VistaProductFile.SOURCE_FIELD_NUMBER);
      } else if (productVo.getSingleMultiSourceProduct().getValue().equals(PPSConstants.MULTISOURCE_SINGLE)) {
        addElementToSend(elementsToSendList, vistaIen, "S", userName, VistaProductFile.SOURCE_FIELD_NUMBER);
      }
    }

    newValue = NdfProductDataConversionUtility.convertNewBooleanBlank(productVo.isExcludeDrgDrgInteractionCheck());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.EXCLUDEDRUGINT_FIELD_NUMBER);

    // Create Default Possible Dosage
    if (productVo.getCreatePossibleDosage()) {
      addElementToSend(elementsToSendList, vistaIen, "Y", userName, VistaProductFile.DEFAULTPOSDOSAGE_FIELD_NUMBER);
    } else {
      addElementToSend(elementsToSendList, vistaIen, "N", userName, VistaProductFile.DEFAULTPOSDOSAGE_FIELD_NUMBER);
      // Possible Dosages To Create
      newValue = NdfProductDataConversionUtility.convertDosageAutoCreate(productVo.getPossibleDosagesAutoCreate());
      addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.POSSIBLEDOSAGE_FIELD_NUMBER);
    }

    // Product Package
    newValue = NdfProductDataConversionUtility.convertProductPackageValue(productVo.getProductPackage());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.PRODUCTPACK_FIELD_NUMBER);

    elementsToSendList.addAll(
        NdfSyncable.syncProcessorElementsForNdfSyncables(
            productVo.getCopayTierAssocs(), vistaIen, userName));

    newValue = NdfProductDataConversionUtility.getStringValue(productVo.getFdaMedGuide());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.FDAMEDGUIDE_FIELD_NUMBER);

    newValue = NdfProductDataConversionUtility.getStringValue(productVo.getServiceCode());
    addElementToSend(elementsToSendList, vistaIen, newValue, userName, VistaProductFile.SERVICECODE_FIELD_NUMBER);

    // New Hazardous waste fields for add.
    addElementToSend(elementsToSendList, vistaIen,
        NdfProductDataConversionUtility.convertBooleanDigit(productVo.getHazardousToHandle()), userName,
        VistaProductFile.HAZTOHAN_FIELD_NUMBER);

    addElementToSend(elementsToSendList, vistaIen,
        NdfProductDataConversionUtility.convertBooleanDigit(productVo.getHazardousToDispose()), userName,
        VistaProductFile.HAZTODIS_FIELD_NUMBER);

    addElementToSend(elementsToSendList, vistaIen, NdfProductDataConversionUtility.getStringValue(
        productVo.getPrimaryEpa()), userName, VistaProductFile.PRIMARY_EPA_CODE_FIELD_NUMBER);

    addElementToSend(elementsToSendList, vistaIen, NdfProductDataConversionUtility.getStringValue(productVo.getWasteSortCode()),
        userName, VistaProductFile.WASTE_SORT_CODE_FIELD_NUMBER);

    addElementToSend(elementsToSendList, vistaIen, NdfProductDataConversionUtility.getStringValue(
        productVo.getDotShippingName()), userName, VistaProductFile.DOT_SHIPPING_CODE_FIELD_NUMBER);

    // Formulary Designator
    addElementToSend(elementsToSendList, vistaIen,
        NdfProductDataConversionUtility.convertFormularyDesignator(productVo.getProductFormularyDesignator()), userName,
        VistaProductFile.FORMULARY_DESIGNATOR_FIELD_NUMBER);

    // Product Text
    String productText = NdfProductDataConversionUtility.getStringValue(productVo.getProductText());
    addElementToSend(elementsToSendList, vistaIen, productText, userName, VistaProductFile.PRODUCT_TEXT_FIELD_NUMBER);
    
    for (SyncProcessorElement element : elementsToSendList) {
      if (VistaProductFile.EXCLUDEDRUGINT_FIELD_NUMBER.equals(element.getFieldId())){
          //special case - we need to send the blank, so we treat this differently than other optional fields.
          insertNewChildElement(
              element.getVistaIen(), 
              element.getNewValue(), 
              element.getUserName(), 
              element.getFieldId(), 
              EplNdfOutgoingDifferences.ADD_ACTION);
      } else {
        if (StringUtils.isNotBlank(element.getNewValue())) {
            insertNewChildElement(
                element.getVistaIen(), 
                element.getNewValue(), 
                element.getUserName(),
                element.getFieldId(), 
                EplNdfOutgoingDifferences.ADD_ACTION);
        }
      }
    }  
    
  }
   
  /**
   * Processes a modified Product field, adding it to the outgoing differences.
   *
   * @param key FieldKey of the field that was modified
   * @param diff Difference object containing the old and new values of the field
   * @param userValue String containing the user who has modified the product
   * @param vistaIEN String containing the the vista ien of the Product
   * @param vaProductVo ProductVo containing the field that was modified
   */
  protected void modifyElement(FieldKey key, Difference diff, String userValue, String vistaIEN, ProductVo vaProductVo) {
    // The complexity and length of this method could be reduced by consolidating some of the following cases where
    // the conversion utility methods used are the same, and the only real difference is mapping a key to a Vista Field Number
    if (FieldKey.PRODUCT_STRENGTH.equals(key)) {

      String oldStrength = NdfProductDataConversionUtility.getOldValueString(diff);
      String newStrength = NdfProductDataConversionUtility.getNewValueString(diff);
      insertModifiedElement(vistaIEN, newStrength, userValue, VistaProductFile.STRENGTH_FIELD_NUMBER, oldStrength);

    } else if (FieldKey.FDA_MED_GUIDE.equals(key)) {

      String oldMedGuide = NdfProductDataConversionUtility.getOldValueString(diff);
      String newMedGuide = NdfProductDataConversionUtility.getNewValueString(diff);
      insertModifiedElement(vistaIEN, newMedGuide, userValue, VistaProductFile.FDAMEDGUIDE_FIELD_NUMBER, oldMedGuide);

    } else if (FieldKey.DISPENSE_UNIT.equals(key)) {

      String newDispenseUnitIen = NdfProductDataConversionUtility.convertDispenseUnit((DispenseUnitVo) diff.getNewValue());
      String oldDispenseUnitIen = NdfProductDataConversionUtility.convertDispenseUnit((DispenseUnitVo) diff.getOldValue());
      insertModifiedElement(vistaIEN, newDispenseUnitIen, userValue, VistaProductFile.DISPENSEUNIT_FIELD_NUMBER,
          oldDispenseUnitIen);

    } else if (FieldKey.CMOP_ID.equals(key)) {
      //CMOP Id is not directly editable, but if VA Print Name is changed, the CMOP Id will be changed.
      String oldCmopId = NdfProductDataConversionUtility.getOldValueString(diff);
      String newCmopId = NdfProductDataConversionUtility.getNewValueString(diff);
      insertModifiedElement(vistaIEN, newCmopId, userValue, VistaProductFile.VAPRODUCTIDENTIFIER_FIELD_NUMBER, oldCmopId);

    } else if (FieldKey.PRODUCT_UNIT.equals(key)) {

      String oldDrugUnitIen = NdfProductDataConversionUtility.convertDrugUnit((DrugUnitVo) diff.getOldValue());
      String newDrugUnitIen = NdfProductDataConversionUtility.convertDrugUnit((DrugUnitVo) diff.getNewValue());
      insertModifiedElement(vistaIEN, newDrugUnitIen, userValue, VistaProductFile.PRODUCTUNIT_FIELD_NUMBER, oldDrugUnitIen);

    } else if (FieldKey.CREATE_POSSIBLE_DOSAGE.equals(key)) {
      //This field is different from the other checkboxes, as VistA expects Y=checked, N=unchecked 
      String oldPossibleDosage = NdfProductDataConversionUtility.convertYesNoValue((Boolean) diff.getOldValue());
      String newPossibleDosage = NdfProductDataConversionUtility.convertYesNoValue((Boolean) diff.getNewValue());
      insertModifiedElement(vistaIEN, newPossibleDosage, userValue, VistaProductFile.DEFAULTPOSDOSAGE_FIELD_NUMBER,
          oldPossibleDosage);

    } else if (FieldKey.PRODUCT_PACKAGE.equals(key)) {

      String oldProductPackageValue = NdfProductDataConversionUtility.convertProductPackageValue(
          (ProductPackage) diff.getOldValue());
      String newProductPackageValue = NdfProductDataConversionUtility.convertProductPackageValue(
          (ProductPackage) diff.getNewValue());
      insertModifiedElement(vistaIEN, newProductPackageValue, userValue, VistaProductFile.PRODUCTPACK_FIELD_NUMBER,
          oldProductPackageValue);      
      
    } else if (FieldKey.POSSIBLE_DOSAGES_AUTO_CREATE.equals(key)) {
      
      String oldDosageAutoCreateValue = NdfProductDataConversionUtility.convertDosageAutoCreate(
          (PossibleDosagesAutoCreate) diff.getOldValue());
      String newDosageAutoCreateValue = NdfProductDataConversionUtility.convertDosageAutoCreate(
          (PossibleDosagesAutoCreate) diff.getNewValue());
      insertModifiedElement(vistaIEN, newDosageAutoCreateValue, userValue, VistaProductFile.POSSIBLEDOSAGE_FIELD_NUMBER,
            oldDosageAutoCreateValue);      

    } else if (FieldKey.VA_PRINT_NAME.equals(key)) {

      String oldVaPrintName = NdfProductDataConversionUtility.getOldValueString(diff);
      String newVaPrintName = NdfProductDataConversionUtility.getNewValueString(diff);
      insertModifiedElement(vistaIEN, newVaPrintName, userValue, VistaProductFile.VAPRINTNAME_FIELD_NUMBER, oldVaPrintName);

    } else if (FieldKey.NATIONAL_FORMULARY_NAME.equals(key)) {

      String oldNatFormularyName = NdfProductDataConversionUtility.getOldValueString(diff);
      String newNatFormularyName = NdfProductDataConversionUtility.getNewValueString(diff);
      insertModifiedElement(vistaIEN, newNatFormularyName, userValue, VistaProductFile.NATIONALFORMULARY_FIELD_NUMBER,
          oldNatFormularyName);

    } else if (FieldKey.GENERIC_NAME.equals(key)) {

      String oldGenericNameIen = NdfProductDataConversionUtility.convertGenericName((GenericNameVo) diff.getOldValue());
      String newGenericNameIen = NdfProductDataConversionUtility.convertGenericName((GenericNameVo) diff.getNewValue());
      insertModifiedElement(vistaIEN, newGenericNameIen, userValue, VistaProductFile.GENERIC_IEN_FIELD_NUMBER,
          oldGenericNameIen);

    } else if (FieldKey.CMOP_DISPENSE.equals(key)) {

      String oldCMOPIndicator = NdfProductDataConversionUtility.convertBooleanDigit(diff.getOldValue());
      String newCMOPIndicator = NdfProductDataConversionUtility.convertBooleanDigit(diff.getNewValue());
      insertModifiedElement(vistaIEN, newCMOPIndicator, userValue, VistaProductFile.CMOPINDICATOR_FIELD_NUMBER,
          oldCMOPIndicator);

    } else if (FieldKey.NATIONAL_FORMULARY_INDICATOR.equals(key)) {

      String oldNatFormularyInd = NdfProductDataConversionUtility.convertBooleanDigit(diff.getOldValue());
      String newNatFormularyInd = NdfProductDataConversionUtility.convertBooleanDigit(diff.getNewValue());
      insertModifiedElement(vistaIEN, newNatFormularyInd, userValue, VistaProductFile.NATFORMULARYIND_FIELD_NUMBER,
          oldNatFormularyInd);

    } else if (FieldKey.EXCLUDE_DRG_DRG_INTERACTION_CHECK.equals(key)) {
      //This field is different from the other checkboxes, as VistA expects 1=checked, space = unchecked 
      String oldExcDrgInteractionChk = NdfProductDataConversionUtility.convertOldBooleanBlank(diff);
      String newExcDrgInteractionChk = NdfProductDataConversionUtility.convertNewBooleanBlank(diff);
      insertModifiedElement(vistaIEN, newExcDrgInteractionChk, userValue, VistaProductFile.EXCLUDEDRUGINT_FIELD_NUMBER,
          oldExcDrgInteractionChk);

    } else if (FieldKey.OVERRIDE_DF_DOSE_CHK_EXCLUSN.equals(key)) {

      String oldOverrideDoseChkExcl = NdfProductDataConversionUtility.convertBooleanDigit(diff.getOldValue());
      String newOverrideDoseChkExcl = NdfProductDataConversionUtility.convertBooleanDigit(diff.getNewValue());
      insertModifiedElement(vistaIEN, newOverrideDoseChkExcl, userValue, VistaProductFile.OVERRIDEDF_FIELD_NUMBER,
          oldOverrideDoseChkExcl);

    } else if (FieldKey.CS_FED_SCHEDULE.equals(key)) {
      CsFedScheduleVo fedSchedule = (CsFedScheduleVo) diff.getNewValue();
      CsFedScheduleVo oldFedSchedule = (CsFedScheduleVo) diff.getOldValue();
      processModifiedFedSchedule(fedSchedule, userValue, vistaIEN, oldFedSchedule);

    } else if (FieldKey.DRUG_CLASSES.equals(key)) {

      DrugClassVo drugClass = (DrugClassVo) diff.getNewValue();
      String newDrugClassIen = NdfProductDataConversionUtility.convertNewDrugClass(diff, vaProductVo);
      String oldDrugClassIen = NdfProductDataConversionUtility.convertOldDrugClass(diff);
      if (drugClass != null) {
        insertModifiedElement(vistaIEN, newDrugClassIen, userValue, VistaProductFile.PRIMARYDRUGCLASS_FIELD_NUMBER,
            oldDrugClassIen);
      }

    } else if (FieldKey.ITEM_STATUS.equals(key)) {

      String priorInactivatedDate = convertPriorInactivationDate(diff);
      String inactivatedDate = convertInactivationDate(diff, vaProductVo);
      insertModifiedElement(vistaIEN, inactivatedDate, userValue, VistaProductFile.STATUS_FIELD_NUMBER, priorInactivatedDate);

    } else if (FieldKey.ORDERABLE_ITEM.equals(key)) {

      String oldOIIEN = NdfProductDataConversionUtility.convertOrderableItem((OrderableItemVo) diff.getOldValue());
      String newOIIEN = NdfProductDataConversionUtility.convertOrderableItem((OrderableItemVo) diff.getNewValue());
      insertModifiedElement(vistaIEN, newOIIEN, userValue, VistaProductFile.DOSAGEFORM_FIELD_NUMBER, oldOIIEN);

    } else if (FieldKey.DOSAGE_FORM.equals(key)) {

      String oldDosageFormIen = NdfProductDataConversionUtility.convertDosageForm((DosageFormVo) diff.getOldValue());
      String newDosageFormIen = NdfProductDataConversionUtility.convertDosageForm((DosageFormVo) diff.getNewValue());
      insertModifiedElement(vistaIEN, newDosageFormIen, userValue, VistaProductFile.DOSAGEFORM_FIELD_NUMBER, oldDosageFormIen);
      
    } else if (FieldKey.HAZARDOUS_TO_HANDLE.equals(key)) {

      String oldHazToHandle = NdfProductDataConversionUtility.convertBooleanDigit(diff.getOldValue());
      String newHazToHandle = NdfProductDataConversionUtility.convertBooleanDigit(diff.getNewValue());
      insertModifiedElement(vistaIEN, newHazToHandle, userValue, VistaProductFile.HAZTOHAN_FIELD_NUMBER, oldHazToHandle);

    } else if (FieldKey.HAZARDOUS_TO_DISPOSE.equals(key)) {

      String oldHazToDispose = NdfProductDataConversionUtility.convertBooleanDigit(diff.getOldValue());
      String newHazToDispose = NdfProductDataConversionUtility.convertBooleanDigit(diff.getNewValue());
      insertModifiedElement(vistaIEN, newHazToDispose, userValue, VistaProductFile.HAZTODIS_FIELD_NUMBER, oldHazToDispose);

    } else if (FieldKey.PRIMARY_EPA.equals(key)) {

      String oldPrimaryEPA = NdfProductDataConversionUtility.getOldValueString(diff);
      String newPrimaryEPA = NdfProductDataConversionUtility.getNewValueString(diff);
      insertModifiedElement(vistaIEN, newPrimaryEPA, userValue, VistaProductFile.PRIMARY_EPA_CODE_FIELD_NUMBER, oldPrimaryEPA);

    } else if (FieldKey.WASTE_SORT_CODE.equals(key)) {

      String oldWasteSortCode = NdfProductDataConversionUtility.getOldValueString(diff);
      String newWasteSortCode = NdfProductDataConversionUtility.getNewValueString(diff);
      insertModifiedElement(vistaIEN, newWasteSortCode, userValue, VistaProductFile.WASTE_SORT_CODE_FIELD_NUMBER,
          oldWasteSortCode);

    } else if (FieldKey.DOT_SHIPPING_NAME.equals(key)) {

      String oldDOTShippingName = NdfProductDataConversionUtility.getOldValueString(diff);
      String newDOTShippingName = NdfProductDataConversionUtility.getNewValueString(diff);
      insertModifiedElement(vistaIEN, newDOTShippingName, userValue, VistaProductFile.DOT_SHIPPING_CODE_FIELD_NUMBER,
          oldDOTShippingName);

    } else if (FieldKey.PRODUCT_TEXT.equals(key)) {
      
      String oldValue = NdfProductDataConversionUtility.getOldValueString(diff);
      String newValue = NdfProductDataConversionUtility.getNewValueString(diff);
      insertModifiedElement(vistaIEN, newValue, userValue, VistaProductFile.PRODUCT_TEXT_FIELD_NUMBER, oldValue);
      
    } else if (FieldKey.PRODUCT_FORMULARY_DESIGNATOR.equals(key)) {
      
      String oldValue = NdfProductDataConversionUtility.convertFormularyDesignator((FormularyDesignatorVo)diff.getOldValue());
      String newValue = NdfProductDataConversionUtility.convertFormularyDesignator((FormularyDesignatorVo)diff.getNewValue());
      insertModifiedElement(vistaIEN, newValue, userValue, VistaProductFile.FORMULARY_DESIGNATOR_FIELD_NUMBER, oldValue);
      
    } else if (FieldKey.PRODUCT_CODING_SYSTEM.equals(key)) {

        String oldValue = NdfProductDataConversionUtility.getOldValueString(diff);
        String newValue = NdfProductDataConversionUtility.getNewValueString(diff);
        String codingSystemIEN = vistaIEN + ",1";
        if (newValue.trim().isEmpty()) { // No RXNorm found, send a delete notification
            newValue = VistaFile.ALL_DELETED;
        }
        insertModifiedElement(codingSystemIEN, newValue, userValue, 
            VistaProductFile.PRODUCT_CODING_SYSTEM_NUMBER, oldValue);

    } else if (FieldKey.PRODUCT_CODING_SYSTEM_CODE.equals(key)) {

        String oldValue = NdfProductDataConversionUtility.getOldValueString(diff);
        String newValue = NdfProductDataConversionUtility.getNewValueString(diff);
        String codingSystemCodeIEN = vistaIEN + ",1,1";
        if (newValue.trim().isEmpty()) { // No RXNorm found, send a delete notification
            newValue = VistaFile.ALL_DELETED;
        }
        insertModifiedElement(codingSystemCodeIEN, newValue, userValue,
            VistaProductFile.PRODUCT_CODING_SYSTEM_CODE_NUMBER, oldValue);

    }

  }


  private void processModifiedFedSchedule(CsFedScheduleVo csFedSchedule, String userValue, String vistaIen, 
      CsFedScheduleVo oldCsFedSchedule) {
    String newCsFwdScheduleValue = null;
    String oldCsFwdScheduleValue = null;
      if (csFedSchedule == null) {
        newCsFwdScheduleValue = VistaFile.BLANK;
      } else {
        newCsFwdScheduleValue = NdfProductDataConversionUtility.convertFedSchedule(csFedSchedule.getValue());
      }
      if (oldCsFedSchedule == null) {
        oldCsFwdScheduleValue = VistaFile.BLANK;
      } else {
        oldCsFwdScheduleValue = NdfProductDataConversionUtility.convertFedSchedule(oldCsFedSchedule.getValue());
      }
      insertModifiedElement(vistaIen, newCsFwdScheduleValue, userValue, VistaProductFile.CSFEDSCH_FIELD_NUMBER,
          oldCsFwdScheduleValue);
  }

  private String convertPriorInactivationDate(Difference diff) {

    return diff != null ? MumpsConverter.convertDate((Date) diff.getOldValue()) : null;

  }

  private String convertInactivationDate(Difference diff, ProductVo vaProductVo) {

    return diff == null ? MumpsConverter.convertDate(vaProductVo.getInactivationDate()) : VistaFile.BLANK;

  }

  /**
   * Find the primary drug class. Same logic as {@link ProductVo#getPrimaryDrugClass()}.
   * 
   * @param group drug class group
   * @return primary drug class if found, otherwise an empty drug class
   */
  private static DrugClassVo findPrimaryDrugClass(Collection<DrugClassGroupVo> group) {
    if (group != null) {
      for (DrugClassGroupVo drugClass : group) {
        if (drugClass.getPrimary()) {
          return drugClass.getDrugClass();
        }
      }
    }

    return new DrugClassVo();
  }

  private void addDifferences(Map<FieldKey, Difference> setDifference) {

    if (ItemStatus.INACTIVE.equals(AbstractConverter.getOldValue(FieldKey.ITEM_STATUS, setDifference))
        && ItemStatus.ACTIVE.equals(AbstractConverter.getNewValue(FieldKey.ITEM_STATUS, setDifference))) { // reactivate

      setDifference.put(FieldKey.INACTIVATION_DATE, new Difference(FieldKey.INACTIVATION_DATE, new Date(), null));
    }

    if (AbstractConverter.hasNewValue(FieldKey.DRUG_CLASSES, setDifference)) { // check for primary drug class modification
      DrugClassVo newPrimaryClass = findPrimaryDrugClass(
          (Collection<DrugClassGroupVo>) AbstractConverter.getNewValue(FieldKey.DRUG_CLASSES, setDifference));

      DrugClassVo oldPrimaryClass = findPrimaryDrugClass(
          (Collection<DrugClassGroupVo>) AbstractConverter.getOldValue(FieldKey.DRUG_CLASSES, setDifference));

      if (!newPrimaryClass.equals(oldPrimaryClass)) {
        setDifference.put(FieldKey.PRIMARY_DRUG_CLASS,
            new Difference(FieldKey.PRIMARY_DRUG_CLASS, oldPrimaryClass, newPrimaryClass));
      }
    }

  }
}
