package gov.va.med.pharmacy.peps.domain.common.dao;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;
import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;
import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.domain.common.model.EplMatchTypeDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplRematchDo;
import gov.va.med.pharmacy.peps.domain.common.utility.test.AbstractDomainIT;
import gov.va.med.pharmacy.peps.domain.common.utility.test.EplNdfOutgoingDifferencesGenerator;

/**
 * Automated integration tests for {@link EplRematchDao}.
 */
public class EplRematchDaoIT extends AbstractDomainIT {
  
  private static final Logger LOG = LogManager.getLogger(EplRematchDaoIT.class);
  
  /**
   * The EPL ID of "CIPROFLOXACIN 10MG/ML INJ IV". Some of these tests assume this product exists.
   */
  private static final long CIPRO_10MG_INJ_PRODUCT_ID = 9991350429193351L;
  
  /**
   * The EPL ID of "CIPROFLOXACIN 2MG/ML/DEXTROSE 5% INJ". Some of these tests assume this product exists.
   */
  private static final long CIPRO_2MG_INJ_PRODUCT_ID = 9991350429335118L;
  
  /**
   * The EPL ID of "CIPROFLOXACIN 500MG TAB,SA". Some of these tests assume this product exists.
   */
  private static final long CIPRO_500MG_TAB_PRODUCT_ID = 9991350430911346L;
  
  // Using field-based autowiring only in automated tests.
  @Autowired
  private EplRematchDao eplRematchDao;

  @Autowired
  private EplProductDao eplProductDao;
  
  @Autowired
  private EplNdfOutgoingDifferencesDao eplNdfOutgoingDifferencesDao;
  
  /**
   * Simply verifies that {@link EplRematchDao#findRematchProducts(Long)} returns a non-empty List. The actual
   * returned results are not verified since they are quite dependent on database data.
   */
  @Test
  @Transactional(readOnly = true)
  public void testFindRematchProducts() {
    
    // This product must exist and have at least one default possible rematch for this test to work.
    final long productWithPossibleRematches = CIPRO_10MG_INJ_PRODUCT_ID;

    /* Behavior & Verification */
    final List<?> returnedList = eplRematchDao.findRematchProducts(productWithPossibleRematches);
    LOG.debug("Returned list is: {}", returnedList);
    assertNotNull("should return a List", returnedList);
    assertNotEquals("should return a non-empty List", Collections.emptyList(), returnedList);
  }
  
  @Test
  @Transactional
  public void testRetrieveForProduct() {
    
    /* Setup */
    
    // Assumption: the old product does not already have an outgoing rematch.
    final EplRematchDo createdRematch = createSampleInactivationRematch();
    final EplProductDo fromProduct = createdRematch.getFromProduct();
    final EplProductDo toAutoProduct = createdRematch.getToAutoProduct();
    
    /* Behavior & Verification */
    
    // Use Matchers here?
    assertThat("should not return a rematch before saving it",
        eplRematchDao.retrieveForProduct(fromProduct.getEplId()), equalTo(Optional.empty()));
    
    LOG.debug("Saving test rematch.");
    eplRematchDao.insert(createdRematch, new UserGenerator().getNationalManagerTwo());
    final Optional<EplRematchDo> returnedRematch = eplRematchDao.retrieveForProduct(fromProduct.getEplId());
    LOG.debug("Returned rematch was: {}", returnedRematch);

    // Can't just compare to an expected Optional here because we don't actually know the ID of the saved rematch.
    assertThat("should return a rematch after saving it", returnedRematch.isPresent(), equalTo(true));
    assertThat("returned rematch should be for old product", returnedRematch.get().getFromProduct(), equalTo(fromProduct));
    assertThat("returned rematch should be to correct product",
        returnedRematch.get().getToAutoProduct(), equalTo(toAutoProduct));
    assertThat("returned rematch should not have suggested products",
        returnedRematch.get().getToSuggestedProducts(), equalTo(Collections.emptySet()));
        
  }

    @Test
    @Transactional(readOnly = true)
    public void testFindUnsentRematches() {

        /* Setup */
        final UserGenerator userGenerator = new UserGenerator();

        // Save off the original list because the database may already have unsent rematches.
        final List<EplRematchDo> initialList = eplRematchDao.findUnsentRematches();
        fixDates(initialList);
        final HashSet<EplRematchDo> expectedRematches = new HashSet<>(initialList);

        // Create a couple rematches in the database.
        final EplRematchDo inactivationRematch = createSampleInactivationRematch();
        // insert() below will clear the session for some reason. Call hashCode() now to avoid a LazyInitializationException
        // later.
        inactivationRematch.hashCode();
        eplRematchDao.insert(inactivationRematch, userGenerator.getNationalManagerTwo());
        expectedRematches.add(inactivationRematch);

        /* Behavior & Verification */
        
        final List<EplRematchDo> returnedList = eplRematchDao.findUnsentRematches();
        fixDates(returnedList);
        
        assertThat(
            "returned rematches should now contain the inactivation rematch",
            new HashSet<>(returnedList),
            equalTo(expectedRematches));
    }    

  /**
   * Verifies that {@link EplNdfOutgoingDifferencesDaoImpl#findOutgoingRematches(Date)} returns only rematches that
   * have the associated outgoing difference to VistA.
   */
  @Test
  @Transactional(readOnly = true)
  public void testFindOutgoingRematches() {
    
    /* Setup */
    final UserGenerator userGenerator = new UserGenerator();
    
    // Save off the original list because the database may already have outgoing rematches.
    final List<EplRematchDo> initialList = eplRematchDao.findOutgoingRematches(new Date());
    fixDates(initialList);
    final HashSet<EplRematchDo> expectedRematches = new HashSet<>(initialList);
    
    // Create a couple rematches in the database.
    final EplRematchDo inactivationRematch = createSampleInactivationRematch();
    // insert() below will clear the session for some reason. Call hashCode() now to avoid a LazyInitializationException
    // later.
    inactivationRematch.hashCode();
    eplRematchDao.insert(inactivationRematch, userGenerator.getNationalManagerTwo());
    final EplRematchDo manualRematch = createSampleManualRematch();
    manualRematch.hashCode();
    eplRematchDao.insert(manualRematch, userGenerator.getNationalManagerOne());
    
    /* Behavior & Verification */
    
    final List<EplRematchDo> listAfterSavingRematch = eplRematchDao.findOutgoingRematches(new Date());
    fixDates(listAfterSavingRematch);
    expectedRematches.add(manualRematch);
    assertThat(
        "returned rematches should now contain the manual rematch",
        new HashSet<>(listAfterSavingRematch),
        equalTo(expectedRematches));
    
    // Now add the outgoing difference and ensure the list contains the rematch now.
    final EplNdfOutgoingDifferences inactivationDiff = EplNdfOutgoingDifferencesGenerator.createDiff(
        VistaProductFile.FILE_NUMBER, VistaProductFile.STATUS_FIELD_NUMBER,
        inactivationRematch.getFromProduct().getNdfProductIen().toString(),
        EplNdfOutgoingDifferences.MODIFY_ACTION);
    // The difference must be from no inactivation date to an inactivation date.
    inactivationDiff.setOldValue(null);
    inactivationDiff.setNewValue("3170201");
    eplNdfOutgoingDifferencesDao.save(inactivationDiff);
    final List<EplRematchDo> listAfterSavingDiff = eplRematchDao.findOutgoingRematches(new Date());
    fixDates(listAfterSavingDiff);
    expectedRematches.add(inactivationRematch);
    assertThat(
        "returned rematches should now contain the inactivation rematch",
        new HashSet<>(listAfterSavingDiff),
        equalTo(expectedRematches));
  }

  /* Helpers */

  private EplRematchDo createSampleInactivationRematch() {
    return new EplRematchDo(
        eplProductDao.retrieve(CIPRO_10MG_INJ_PRODUCT_ID),
        eplProductDao.retrieve(CIPRO_2MG_INJ_PRODUCT_ID),
        createMatchType(MatchReason.INACTIVATION));
  }
  
  private EplRematchDo createSampleManualRematch() {
      return new EplRematchDo(
          eplProductDao.retrieve(CIPRO_500MG_TAB_PRODUCT_ID),
          eplProductDao.retrieve(CIPRO_2MG_INJ_PRODUCT_ID),
          createMatchType(MatchReason.MANUAL_REMATCH));
  }
  
  private EplMatchTypeDo createMatchType(final MatchReason reason) {
      return (EplMatchTypeDo)eplRematchDao.getCurrentSession().load(EplMatchTypeDo.class, reason.getId());
      // We just need the ID. Use obviously-wrong values 
//      return new EplMatchTypeDo(reason.getId(), "", "");
  }
}
