package gov.va.med.pharmacy.peps.service.common.update.impl;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.not;
import static org.junit.Assert.assertThat;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.test.generator.CopayTierAssocVoGenerator;
import gov.va.med.pharmacy.peps.common.vista.VistaField;
import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;
import gov.va.med.pharmacy.peps.common.vo.CopayTierAssocVo;
import gov.va.med.pharmacy.peps.common.vo.CopayTierVo;
import gov.va.med.pharmacy.peps.common.vo.NdfSyncable;
import gov.va.med.pharmacy.peps.common.vo.SyncProcessorElement;

/**
 * Tests for the NdfOutgoingDifferenceConsolidator.
 */
public class NdfOutgoingDifferenceConsolidatorTest {
    private static final int EXPECTED_NUMBER_OF_ELEMENTS = 3;
    
    private String vistaIen;
    private String userValue;
    private List<CopayTierAssocVo> oldValue;
    private NdfOutgoingDifferenceConsolidator consolidator;
    private List<SyncProcessorElement> newElements;
    private Function<SyncProcessorElement, String> valueFunction;

    /**
     * Setup before each test.
     */
    @Before
    public void setUp() {
        vistaIen = "123";
        userValue = "TESTUSER";
        oldValue = new CopayTierAssocVoGenerator().getList();
        consolidator = new NdfOutgoingDifferenceConsolidator();
        newElements = new ArrayList<SyncProcessorElement>();
        valueFunction = element -> element.getNewValue();
    }

    /**
     * Verifies an empty list is returned when there are pending outgoing
     * differences and PPS-N has reverted those changes by deleting all
     * records.
     */
    @Test
    public void shouldReturnEmptyListWhenHasPendingAndIsRevertedAndAllDeleted() {
        final boolean HAS_PENDING_OUTGOING_DIFFERENCES = true;
        
        //All deleted...New value list is empty.
        oldValue = new ArrayList<CopayTierAssocVo>();
        List<CopayTierAssocVo> newValue = oldValue;
        
        // Create SyncProcessorElements from new value.
        newElements = NdfSyncable.syncProcessorElementsForNdfSyncables(
            newValue, 
            vistaIen, 
            userValue);
        
        //Reverted...Checksums should be equal.
        String ppsnChecksum = SyncProcessorElement.checksumReduce(
            newElements, 
            valueFunction);
        String vistaChecksum = ChecksumCalculator.forCopayTierAssocVos(oldValue);
        assertThat(vistaChecksum, equalTo(ppsnChecksum));
        
        newElements = 
            consolidator.reconcile(
                newElements, 
                new VistaField(
                    VistaProductFile.FILE_NUMBER, 
                    VistaProductFile.COPAY_TIER_FIELD_NUMBER,
                    vistaIen),
                userValue, 
                HAS_PENDING_OUTGOING_DIFFERENCES, 
                vistaChecksum, 
                ppsnChecksum);
        
        assertThat(newElements.size(), equalTo(0));
    }
    
    /**
     * Verifies that a "deleted" record is returned when there are no pending
     * outgoing differences and PPS-N deletes all records, but VistA currently
     * has records on file.
     */
    @Test
    public void shouldReturnDeletedWhenNoPendingAndNotRevertedAndAllDeleted() {
        String vistaChecksum = ChecksumCalculator.forCopayTierAssocVos(oldValue);
        final boolean HAS_PENDING_OUTGOING_DIFFERENCES = false;
        
        String ppsnChecksum = SyncProcessorElement.checksumReduce(
            newElements, 
            valueFunction);
                
        newElements = 
            consolidator.reconcile(
                newElements, 
                new VistaField(
                    VistaProductFile.FILE_NUMBER, 
                    VistaProductFile.COPAY_TIER_FIELD_NUMBER,
                    vistaIen),
                userValue, 
                HAS_PENDING_OUTGOING_DIFFERENCES, 
                vistaChecksum, 
                ppsnChecksum);
        
        assertThat(newElements.size(), equalTo(1));
        assertThat(newElements.get(0).getNewValue(), equalTo("DELETED"));
        assertThat(vistaChecksum, equalTo(newElements.get(0).getOldValue()));
    }
    
    /**
     * Verifies that a "deleted" record is returned when there are pending
     * outgoing differences, and PPS-N has deleted all records, but VistA has
     * records on file.
     */
    @Test
    public void shouldReturnDeletedWhenHasPendingAndNotRevertedAndAllDeleted() {
        String vistaChecksum = ChecksumCalculator.forCopayTierAssocVos(oldValue);
        final boolean HAS_PENDING_OUTGOING_DIFFERENCES = true;
        
        String ppsnChecksum = SyncProcessorElement.checksumReduce(
            newElements, 
            valueFunction);
                
        newElements = 
            consolidator.reconcile(
                newElements, 
                new VistaField(
                    VistaProductFile.FILE_NUMBER, 
                    VistaProductFile.COPAY_TIER_FIELD_NUMBER,
                    vistaIen),
                userValue, 
                HAS_PENDING_OUTGOING_DIFFERENCES, 
                vistaChecksum, 
                ppsnChecksum);
        
        assertThat(newElements.size(), equalTo(1));
        assertThat(newElements.get(0).getNewValue(), equalTo("DELETED"));
        assertThat(vistaChecksum, equalTo(newElements.get(0).getOldValue()));
    }
    
    /**
     * Verifies that an empty list is returned when there are pending outgoing
     * differences, PPS-N has new values, but the new values match what VistA
     * already has.
     */
    @Test
    public void shouldReturnEmptyListWhenHasPendingAndIsRevertedAndHasNewValues() {
        final boolean HAS_PENDING_OUTGOING_DIFFERENCES = true;
        
        List<CopayTierAssocVo> newValue = oldValue;        
        
        // Has new values...newElements non-empty.
        newElements = NdfSyncable.syncProcessorElementsForNdfSyncables(
            newValue, 
            vistaIen, 
            userValue);
        
        //Reverted...Checksums should be equal.
        String ppsnChecksum = SyncProcessorElement.checksumReduce(
            newElements, 
            valueFunction);
        String vistaChecksum = ChecksumCalculator.forCopayTierAssocVos(oldValue);
        assertThat(vistaChecksum, equalTo(ppsnChecksum));
        
        newElements = 
            consolidator.reconcile(
                newElements, 
                new VistaField(
                    VistaProductFile.FILE_NUMBER, 
                    VistaProductFile.COPAY_TIER_FIELD_NUMBER,
                    vistaIen),
                userValue, 
                HAS_PENDING_OUTGOING_DIFFERENCES, 
                vistaChecksum, 
                ppsnChecksum);
        
        assertThat(newElements.size(), equalTo(0));
    }
    
    /**
     * Verifies that new records are returned when there are pending outgoing
     * differences, PPS-N has new values, and VistA does not already have the
     * new values.
     * @throws ParseException if there is a problem parsing the hard-coded
     * dates
     */
    @Test
    public void shouldReturnNewValuesWhenHasPendingAndNotRevertedAndHasNewValues() throws ParseException {
        final boolean HAS_PENDING_OUTGOING_DIFFERENCES = true;
        
        List<CopayTierAssocVo> newValue = new ArrayList<>(); 
        CopayTierAssocVo copayAssoc = new CopayTierAssocVo();
        
        SimpleDateFormat df = new SimpleDateFormat("mm/dd/yyyy");
        copayAssoc.setEffectiveDate(df.parse("01/01/2000"));
        copayAssoc.setEndDate(df.parse("01/01/2001"));
        
        CopayTierVo copayTier = new CopayTierVo();
        copayTier.setTierLevel("999");
        
        copayAssoc.setCopayTier(copayTier);
        newValue.add(copayAssoc);
        
        // Has new values...newElements non-empty.
        newElements = NdfSyncable.syncProcessorElementsForNdfSyncables(
            newValue, 
            vistaIen, 
            userValue);
        
        //Not reverted...Checksums should be unequal.
        String ppsnChecksum = SyncProcessorElement.checksumReduce(
            newElements, 
            valueFunction);
        String vistaChecksum = ChecksumCalculator.forCopayTierAssocVos(oldValue);
        assertThat(vistaChecksum, not(equalTo(ppsnChecksum)));
        
        newElements = 
            consolidator.reconcile(
                newElements, 
                new VistaField(
                    VistaProductFile.FILE_NUMBER, 
                    VistaProductFile.COPAY_TIER_FIELD_NUMBER,
                    vistaIen),
                userValue, 
                HAS_PENDING_OUTGOING_DIFFERENCES, 
                vistaChecksum, 
                ppsnChecksum);
        
        assertThat(newElements.size(), equalTo(EXPECTED_NUMBER_OF_ELEMENTS));
        assertThat(newElements.get(0).getOldValue(), equalTo(vistaChecksum));
        assertThat(newElements.get(0).getNewValue(), equalTo("999"));
        assertThat(newElements.get(1).getNewValue(), equalTo("3000101"));
        assertThat(newElements.get(2).getNewValue(), equalTo("3010101"));
    }
    
    /**
     * Verifies new records are returned when there are no pending outgoing
     * differences, PPS-N has new values, and VistA does not have the new 
     * values already.
     * @throws ParseException if there is a problem parsing the hard-coded
     * dates.
     */
    @Test
    public void shouldReturnNewValuesWhenNoPendingAndNotRevertedAndHasNewValues() throws ParseException {
        final boolean HAS_PENDING_OUTGOING_DIFFERENCES = false;
        
        List<CopayTierAssocVo> newValue = new ArrayList<>(); 
        CopayTierAssocVo copayAssoc = new CopayTierAssocVo();
        
        SimpleDateFormat df = new SimpleDateFormat("mm/dd/yyyy");
        copayAssoc.setEffectiveDate(df.parse("01/01/2000"));
        copayAssoc.setEndDate(df.parse("01/01/2001"));
        
        CopayTierVo copayTier = new CopayTierVo();
        copayTier.setTierLevel("999");
        
        copayAssoc.setCopayTier(copayTier);
        newValue.add(copayAssoc);
        
        // Has new values...newElements non-empty.
        newElements = NdfSyncable.syncProcessorElementsForNdfSyncables(
            newValue, 
            vistaIen, 
            userValue);
        
        //Not reverted...Checksums should be unequal.
        String ppsnChecksum = SyncProcessorElement.checksumReduce(
            newElements, 
            valueFunction);
        String vistaChecksum = ChecksumCalculator.forCopayTierAssocVos(oldValue);
        assertThat(vistaChecksum, not(equalTo(ppsnChecksum)));
        
        newElements = 
            consolidator.reconcile(
                newElements, 
                new VistaField(
                    VistaProductFile.FILE_NUMBER, 
                    VistaProductFile.COPAY_TIER_FIELD_NUMBER,
                    vistaIen),
                userValue, 
                HAS_PENDING_OUTGOING_DIFFERENCES, 
                vistaChecksum, 
                ppsnChecksum);
        
        assertThat(newElements.size(), equalTo(EXPECTED_NUMBER_OF_ELEMENTS));
        assertThat(newElements.get(0).getOldValue(), equalTo(vistaChecksum));
        assertThat(newElements.get(0).getNewValue(), equalTo("999"));
        assertThat(newElements.get(1).getNewValue(), equalTo("3000101"));
        assertThat(newElements.get(2).getNewValue(), equalTo("3010101"));
    }
}
