package gov.va.med.pharmacy.peps.service.common.update.impl;

import static gov.va.med.pharmacy.peps.service.common.update.impl.NdfFileSyncProcessorTestHelper.expectedAddDiff;
import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;

/**
 * Tests for the NdfRejectFileConsolidator class.
 */
public class NdfRejectFileConsolidatorTest {    
    private NdfRejectFileConsolidator consolidator;

    /**
     * Initialization performed before each test method.
     * 
     * @throws Exception if an error occurs
     */
    @Before
    public void setUp() throws Exception {
        consolidator = new NdfRejectFileConsolidator();
    }
    
    /**
     * The consolidator should return an empty list as the result if both
     * the pending and rejected lists it receives are empty.
     */
    @Test
    public void shouldReturnEmptyListForEmptyLists() {
        List<EplNdfOutgoingDifferences> result = consolidator.reconcile(
            new ArrayList<>(), 
            new ArrayList<>());
        assertThat(result.size(), equalTo(0));
    }
    
    /**
     * The consolidator should retain differences which only exist in the pending
     * list.
     */
    @Test
    public void shouldPropagateMutuallyExclusivePendingDiffs() {
        List<EplNdfOutgoingDifferences> originalPendingDiffs = Collections.unmodifiableList(
            Stream.of(
                expectedAddDiff("Ingredient IEN", "14,.01", "1,1"),
                expectedAddDiff("Ingredient Strength", "14,1", "1,1")
            ).collect(Collectors.toList()));
        
        List<EplNdfOutgoingDifferences> originalRejectedDiffs = new ArrayList<>();
        
        List<EplNdfOutgoingDifferences> mergedQueue = consolidator.reconcile(
            originalPendingDiffs, originalRejectedDiffs);
        
        assertTrue(mergedQueue.containsAll(originalPendingDiffs));
    }
    
    /**
     * The consolidator should retain differences which only exist in the rejected
     * list.
     */
    @Test
    public void shouldPropagateMutuallyExclusiveRejectedDiffs() {
        List<EplNdfOutgoingDifferences> originalPendingDiffs = new ArrayList<>();
        
        List<EplNdfOutgoingDifferences> originalRejectedDiffs = Collections.unmodifiableList(
            Stream.of(
                expectedAddDiff("Copay Tier", "45,.01", "1,1"),
                expectedAddDiff("Copay Tier Effective Date", "45,1", "1,1"),
                expectedAddDiff("Copay Tier End Date", "45,2", "1,1")
            ).collect(Collectors.toList()));

        List<EplNdfOutgoingDifferences> mergedQueue = consolidator.reconcile(
            originalPendingDiffs, originalRejectedDiffs);
        
        assertTrue(mergedQueue.containsAll(originalRejectedDiffs));
    }
    
    /**
     * When the consolidator finds differences which exist in both the pending and rejected
     * lists, it should merge the differences, retaining the new value from the pending list
     * and the old value from the rejected list.
     */
    @Test
    public void shouldMergePendingWithRejectedDiffs() {
        List<EplNdfOutgoingDifferences> originalPendingDiffs = Collections.unmodifiableList(
            Stream.of(
                expectedAddDiff("Pending Copay Tier", "45,.01", "1,1"),
                expectedAddDiff("Pending Copay Tier Effective Date", "45,1", "1,1"),
                expectedAddDiff("Pending Copay Tier End Date", "45,2", "1,1")
            ).collect(Collectors.toList()));
        
        List<EplNdfOutgoingDifferences> originalRejectedDiffs = Collections.unmodifiableList(
            Stream.of(
                expectedAddDiff("Rejected Copay Tier", "45,.01", "1,1", "Original Checksum"),
                expectedAddDiff("Rejected Copay Tier Effective Date", "45,1", "1,1", null),
                expectedAddDiff("Rejected Copay Tier End Date", "45,2", "1,1", null)
            ).collect(Collectors.toList()));
        
        List<EplNdfOutgoingDifferences> mergedQueue = consolidator.reconcile(
            originalPendingDiffs, 
            originalRejectedDiffs);
        
        assertTrue(mergedQueue.containsAll(originalPendingDiffs));
        assertThat(mergedQueue.get(0).getOldValue(), equalTo("Original Checksum"));
        assertThat(mergedQueue.get(1).getOldValue(), equalTo(null));
        assertThat(mergedQueue.get(2).getOldValue(), equalTo(null));
    }
    
    /**
     * If there are pending differences where the new value is equal to the old value
     * in the associated rejected differences, the consolidator should omit these
     * records from the result. In this case, PPS-N has reverted to the value which 
     * VistA already has, so the system does not need to write a pending outgoing 
     * difference.
     */
    @Test
    public void shouldOmitRevertedDiffs() {
        List<EplNdfOutgoingDifferences> originalPendingDiffs = Collections.unmodifiableList(
            Stream.of(
                expectedAddDiff("Reverted Copay Tier", "45,.01", "1,1"),
                expectedAddDiff("Reverted Copay Tier Effective Date", "45,1", "1,1"),
                expectedAddDiff("Reverted Copay Tier End Date", "45,2", "1,1")
            ).collect(Collectors.toList()));
        
        List<EplNdfOutgoingDifferences> originalRejectedDiffs = Collections.unmodifiableList(
            Stream.of(
                expectedAddDiff(
                    "Rejected Copay Tier", 
                    "45,.01", 
                    "1,1", 
                    "Reverted Copay Tier~Reverted Copay Tier Effective Date~"
                    + "Reverted Copay Tier End Date"),
                expectedAddDiff("Rejected Copay Tier Effective Date", "45,1", "1,1", null),
                expectedAddDiff("Rejected Copay Tier End Date", "45,2", "1,1", null)
            ).collect(Collectors.toList()));
        
        List<EplNdfOutgoingDifferences> mergedQueue = consolidator.reconcile(
            originalPendingDiffs, 
            originalRejectedDiffs);
        
        assertThat(mergedQueue.size(), equalTo(0));
    }
}
