package gov.va.med.pharmacy.peps.service.common.capability;

import gov.va.med.pharmacy.peps.common.exception.DuplicateRematchException;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryVo;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;

import java.util.Collection;
import java.util.Optional;

/**
 * Business operations pertaining to rematches.
 * 
 * @see RematchVo
 */
public interface RematchCapability {
    
    /**
     * The name used of the rematch "field". Products don't actually have a field for rematches, but this name is used for
     * {@link gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryDetailsVo#setColumnName(String) audit history column names} and
     * perhaps other places.
     */
    String FIELD_NAME = "Rematch";

    /**
     * Returns a bean that contains data necessary for creating or updating a rematch.
     * @param productId the fromProduct product Id for the rematch to prepare for.
     * 
     * @return never null.
     * 
     */
    public RematchPreperationBean prepareForRematch(Long productId);

  /**
   * Returns the existing rematch for the given item and differences, if any.
   * @param item the item 
   * @param differences the differences that triggered the rematch
   * @return never null
   */
  public Optional<RematchVo> getExistingRematchForDifferences(
      final ManagedItemVo item, final Collection<Difference> differences);
  
  /**
   * Updates rematches as necessary when the given item has been inactivated per a proposed inactivation.
   * @param item the updated item, now inactive
   * @param user the user updating the item (probably a batch job user)
   */
  public void updateUponProposedInactivation(final ManagedItemVo item, UserVo user);
  
  /**
   * <p>
   * Saves or updates the given rematch for the given changes to the item. This may involve, for example, deleting the
   * rematch if the changes no longer trigger one. {@link #saveOrUpdateCommittedRematch(ManagedItemVo, Collection,
   * Optional, UserVo)} should be called instead if actually committing the changes to the item.
   * </p>
   * 
   * <p>
   * This method may add details to the given audit history.
   * </p>
   * 
   * @param committedDiffs changes to the item
   * @param rematch the rematch associated with the changes. May be updated or deleted.
   * @param itemAuditHistory the audit history to which details will be added. Must not be null.
   * @param user the user performing changes
   * @return the saved rematch, or an empty Optional if the rematch was deleted.
   * @throws DuplicateRematchException if trying to create a rematch for a product when an unsent one already exists
   */
  public Optional<RematchVo> saveOrUpdateUncommittedRematch(
      final Collection<Difference> committedDiffs, final RematchVo rematch, ItemAuditHistoryVo itemAuditHistory,
      final UserVo user) throws DuplicateRematchException;

  /**
   * <p>
   * Like {@link #saveOrUpdateUncommittedRematch(Collection, Optional, ItemAuditHistoryVo, UserVo)} but also removes any
   * pre-existing rematch if the changes reverse it. This one should be called when actually committing the changes to
   * the item.
   * </p>
   * 
   * <p>
   * This method may add details to {@link ManagedItemVo#getNewAuditHistory()}.
   * </p>
   * 
   * @param item the modified item. Should have modifications applied already.
   * @param committedDiffs changes being committed to the item
   * @param optionalModRematch represents the rematch to save, update, or delete. (See {@link ModRematch#getOptionalRematch()}.)
   * @param user the user performing changes
   * @return the saved rematch, or an empty Optional if the rematch was deleted.
   * @throws DuplicateRematchException if trying to create a rematch for a product when an unsent one already exists
   */
  public Optional<RematchVo> saveOrUpdateCommittedRematch(
      final ManagedItemVo item, final Collection<Difference> committedDiffs, Optional<ModRematch> optionalModRematch,
      final UserVo user) throws DuplicateRematchException;
  
  /**
   * Deletes the given rematch from the database.
   * @param item the rematch to delete
   */
  public void delete(RematchVo item);

}
