package gov.va.med.pharmacy.peps.service.common.capability.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;

import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.logging.log4j.LogManager;

import gov.va.med.pharmacy.peps.common.exception.DuplicateRematchException;
import gov.va.med.pharmacy.peps.common.utility.EnvironmentUtility;
import gov.va.med.pharmacy.peps.common.utility.security.UntrustedLogParam;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryDetailsVo;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryVo;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.domain.common.capability.RematchMgtDomainCapability;
import gov.va.med.pharmacy.peps.service.common.capability.ModRematch;
import gov.va.med.pharmacy.peps.service.common.capability.RematchCapability;
import gov.va.med.pharmacy.peps.service.common.capability.RematchPreperationBean;

/**
 * Main implementation of {@link RematchCapability}.
 */
public class RematchCapabilityImpl implements RematchCapability{
    
    /**
     * The modification reason used for manual rematches.
     */
    protected static final String MANUAL_MODIFICATION_REASON = "Manually-triggered rematch";

    private static final Logger LOG = LogManager.getLogger(RematchCapabilityImpl.class);

    private final RematchMgtDomainCapability rematchMgtDomainCapability;
    private final EnvironmentUtility environmentUtility;

    /**
     * Constructs an instance with the required dependencies.
     * 
     * @param pRematchMgtDomainCapability the {@link RematchMgtDomainCapability} to use. Must not be null.
     * @param pEnvironmentUtility the {@link EnvironmentUtility} to use. Must not be null.
     */
    @Autowired
    public RematchCapabilityImpl(
        final RematchMgtDomainCapability pRematchMgtDomainCapability, final EnvironmentUtility pEnvironmentUtility) {
        rematchMgtDomainCapability = Objects.requireNonNull(pRematchMgtDomainCapability);
        environmentUtility = Objects.requireNonNull(pEnvironmentUtility);
    }
    
    @Override
    public RematchPreperationBean prepareForRematch(Long productId) {
        Optional<RematchVo> rematchForProduct = rematchMgtDomainCapability.getRematchForProduct(productId);
        return new RematchPreperationBean(
            rematchForProduct.isPresent(),
            rematchMgtDomainCapability.findRematchProducts(productId));
    }
    
    
    @Override
    public Optional<RematchVo> getExistingRematchForDifferences(
        final ManagedItemVo item, final Collection<Difference> differences) {

        final Set<MatchReason> matchReasons = item.determineRematchTriggers(differences);

        if (!matchReasons.isEmpty()) { // Optimization: don't pull the rematch if we don't trigger one at all.
            final long itemId = Long.parseLong(item.getId());
            final Optional<RematchVo> existingRematch = rematchMgtDomainCapability.getRematchForProduct(itemId);
            if (existingRematch.isPresent() && matchReasons.contains(existingRematch.get().getMatchReason())) {
                return existingRematch;
            }
        }

        // Either there was no existing rematch or these diffs didn't trigger it.
        return Optional.empty();
    }
    
    @Override
    public void updateUponProposedInactivation(final ManagedItemVo item, UserVo user) {
        if (item instanceof ProductVo) {
            final long itemId = Long.parseLong(item.getId());
            final Optional<RematchVo> existingRematch = rematchMgtDomainCapability.getRematchForProduct(itemId);

            if (existingRematch.isPresent() && existingRematch.get().getMatchReason() == MatchReason.FUTURE_INACTIVATION) {
                final MatchReason newReason = MatchReason.INACTIVATION;
                LOG.debug("Updating match reason to {} to associate it with the generated request.", MatchReason.INACTIVATION);
                existingRematch.get().setMatchReason(newReason);
                try {
                    rematchMgtDomainCapability.saveOrUpdate(existingRematch.get(), user);
                } catch (final DuplicateRematchException e) {
                    // We are only updating the match reason here: this should not trigger a DuplicateRematchException.
                    throw new RuntimeException("Unexpected exception!", e);
                }
            }
        }
    }

    @Override
    public Optional<RematchVo> saveOrUpdateUncommittedRematch(
        final Collection<Difference> committedDiffs,
        final RematchVo rematch,
        ItemAuditHistoryVo itemAuditHistory,
        final UserVo user)
            throws DuplicateRematchException {

        // Only update the match reason, etc. if it's not a manual rematch.
        if (!rematch.getMatchReason().equals(MatchReason.MANUAL_REMATCH)) {
            final Set<MatchReason> newTriggers = rematch.getFromProduct().determineRematchTriggers(committedDiffs);

            if (newTriggers.isEmpty()) {
                LOG.debug("The approved changes no longer trigger a rematch: dropping the rematch.");
                if (rematch.getEplRematchId() != null) {
                    // The rematch has previously been saved to the database: delete it.
                    rematchMgtDomainCapability.delete(rematch);
                }
                return Optional.empty();

            } else {
                final MatchReason newReason = newTriggers.iterator().next();
                if (newReason != rematch.getMatchReason()) {
                    LOG.debug("The approved changes have changed the rematch reason: updating from {} to {}.",
                        rematch.getMatchReason(), newReason);
                    rematch.setMatchReason(newReason);
                }
                // We want to save or update the rematch: flow down to the below return statement.
            }
        }

        return Optional.of(rematchMgtDomainCapability.saveOrUpdate(rematch, user));
    }

    @Override
    public Optional<RematchVo> saveOrUpdateCommittedRematch(
        final ManagedItemVo item, final Collection<Difference> committedDiffs, Optional<ModRematch> optionalModRematch,
        final UserVo user) throws DuplicateRematchException {

        LOG.debug("Updating rematches for changes {}.", new UntrustedLogParam(committedDiffs));
        
        deleteReversedRematch(item, committedDiffs);
        
        // Don't do anything else if the caller did not ask to create/update/delete a rematch.
        if (!optionalModRematch.isPresent()) {
            return Optional.empty();
        }
        
        // Now we know we have a ModRematch. Extract out some objects for convenient use below.
        final ModRematch modRematch = optionalModRematch.get();
        final Optional<RematchVo> optionalRematch = modRematch.getOptionalRematch();
        
        final Optional<RematchVo> deletedRematch = deletePreviousRematchWhenAppropriate(item, optionalRematch);
        
        final ItemAuditHistoryVo newAuditHistory = ObjectUtils.defaultIfNull(
            item.getNewAuditHistory(), item.createModificationAuditHistory(user, environmentUtility));
        
        // Actually save the rematch if one given.
        final Optional<RematchVo> savedRematch = optionalRematch.isPresent()
            ? saveOrUpdateUncommittedRematch(committedDiffs, optionalRematch.get(), newAuditHistory, user)
            : Optional.empty();

        // For now, just add audit history for manual rematches. There is a User Story (162889) to do this for all
        // rematches. This if block may need to be moved elsewhere if we add audit history for other rematches.
        if (deletedRematch.isPresent()
            || (savedRematch.isPresent() && savedRematch.get().getMatchReason().equals(MatchReason.MANUAL_REMATCH))) {
            LOG.debug("Adding manual rematch audit history.");
            newAuditHistory.addDetail(new ItemAuditHistoryDetailsVo(
                newAuditHistory.getEventCategory(),
                modRematch.getReason(),
                FIELD_NAME,
                deletedRematch.isPresent() ? deletedRematch.get().getToProductsString() : Difference.NO_VALUE,
                savedRematch.isPresent() ? savedRematch.get().getToProductsString() : Difference.NO_VALUE));
        }
        
        // If we created the audit history, set it on the item.
        if (item.getNewAuditHistory() == null && !newAuditHistory.getDetails().isEmpty()) {
            item.setNewAuditHistory(newAuditHistory);
        }
        
        return savedRematch;
    }
    
    /**
     * Deletes the pre-existing rematch if the rematch is empty or if the match reason is manual.
     * @return the deleted rematch, if any
     */
    private Optional<RematchVo> deletePreviousRematchWhenAppropriate(
        ManagedItemVo item, Optional<RematchVo> optionalRematch) {
        if (!optionalRematch.isPresent() || optionalRematch.get().getMatchReason().equals(MatchReason.MANUAL_REMATCH)) {
            Optional<RematchVo> previousRematch = rematchMgtDomainCapability.getRematchForProduct(
                Long.parseLong(item.getId()));
            if (previousRematch.isPresent()) {
                LOG.debug("Deleting the previous rematch for the given product.");
                delete(previousRematch.get());
                return previousRematch;
            }
        }
        
        return Optional.empty();
    }

    /**
     * Deletes an unsent rematch for the given item that the given diffs are reversing. E.g., if there is an unsent rematch for
     * product inactivation and the given diffs are re-activating the product, then the rematch will be deleted.
     * 
     * @param item the affected item. May or may not have the changes applied.
     * @param committedDiffs the {@link Difference}s being applied to to the item
     */
    private void deleteReversedRematch(final ManagedItemVo item, final Collection<Difference> committedDiffs) {
        final ArrayList<Difference> reversedDiffs = new ArrayList<>(committedDiffs.size());
        for (final Difference committedDiff : committedDiffs) {
            reversedDiffs.add(new Difference(
                committedDiff.getFieldKey(), committedDiff.getNewValue(), committedDiff.getOldValue()));
        }

        // If we are reversing the differences that created an existing rematch, delete it.
        final Optional<RematchVo> existingReversedRematch = getExistingRematchForDifferences(item, reversedDiffs);
        if (existingReversedRematch.isPresent()) {
            rematchMgtDomainCapability.delete(existingReversedRematch.get());
            // Log this at INFO level because there's no audit history for this operation. The log will provide us some
            // history in case of trouble.
            LOG.info(
                "Deleted pre-existing rematch for item {} since the committed changes reverse it.",
                new UntrustedLogParam(item.getId()));
        }
    }

    @Override
    public void delete(RematchVo rematch) {
        rematchMgtDomainCapability.delete(rematch);    
    }

}
