package gov.va.med.pharmacy.peps.service.common.migration;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.validation.constraints.NotNull;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.POIXMLException;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.DataFormatter;
import org.apache.poi.ss.usermodel.Row;

import gov.va.med.pharmacy.peps.common.vo.MigrationRow;
import gov.va.med.pharmacy.peps.service.common.session.impl.MigrationImportFormatException;

/**
 * Reads MigrationRows from Excel spreadsheets.
 *
 * @param <R> the generic type of MigrationRow that is processed
 */
public abstract class AbstractMigrationImportReader<R extends MigrationRow> {

    /**
     * Converts all rows of the first sheet of the given Excel workbook.
     * 
     * @param is the stream providing the Excel workbook. Must be closed by the caller.
     * @return a List containing one {@link R} for each data row in the spreadsheet, or an empty list if
     *         there are no data rows.
     * @throws IOException if the given InputStream cannot be opened.
     * @throws POIXMLException if the given InputStream is not a spreadsheet file (.xlsx only).
     * @throws MigrationImportFormatException if there is an issue with the format of the row.
     */
    public abstract List<R> getRows(InputStream is)
        throws IOException, POIXMLException, MigrationImportFormatException;
    
    
    /**
     * Gets the cell value, converting blanks to null.  Uses DataFormatter to get the value according to type in the cell.
     *
     * @param cell the cell
     * @return the cell value
     */
    protected String getCellValue(Cell cell) {
        DataFormatter formatter = new DataFormatter(Locale.getDefault());
        
        String value = (cell == null) ? null : formatter.formatCellValue(cell);
        //convert blank to null so we compare with the product from the DB.
        if (StringUtils.isEmpty(value)){
            value = null;
        }
        return value;
    }

    /**
     * Discard header row.  Ensures that the first column of the header has the expected column header value, otherwise, a 
     * MigrationImportFormatException is thrown.
     *
     * @param iterator the iterator
     * @param firstHeaderColNumber the first header column number ex. 0 is the first column in a POI row. 
     * @param expectedFirstHeaderColValue the expected first header column value ex. "VUID"
     * @return the iterator of Excel Rows that is positioned at the beginning of the file.
     * @throws MigrationImportFormatException if the specified column header value does not match the specified expected value
     */
    protected Iterator<Row> discardHeaderRow(Iterator<Row> iterator, int firstHeaderColNumber, 
        @NotNull String expectedFirstHeaderColValue) throws MigrationImportFormatException {
        if (iterator.hasNext()) {
            Row headerRow = iterator.next();
            String firstColumnHeader = getCellValue(headerRow.getCell(firstHeaderColNumber));

            if (!expectedFirstHeaderColValue.equalsIgnoreCase(firstColumnHeader)) {
                throw new MigrationImportFormatException(
                    "Please ensure the first row of the import file contains the column headers. "
                        + "Expected first column header to be " + expectedFirstHeaderColValue
                        + ", but found " + firstColumnHeader + " instead.");
            }

            iterator.remove();
        }

        return iterator;
    }
}
