package gov.va.med.pharmacy.peps.service.common.migration;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.poi.POIXMLException;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorImportRow;
import gov.va.med.pharmacy.peps.service.common.session.impl.FormularyDesignatorImportFormatException;

/**
 * Reads {@link FormularyDesignatorImportRow}s from Excel spreadsheets.
 */
public class FormularyDesignatorImportReader {
    /**
     * The index of the workbook sheet containing the formulary designator data.
     */
    private static final int SHEET_INDEX = 0;

    /**
     * Converts all rows of the first sheet of the given Excel workbook.
     * 
     * @param is the stream providing the Excel workbook. Must be closed by the caller.
     * @return a List containing one {@link FormularyDesignatorImportRow} for each data row in 
     *         the spreadsheet, or an empty list if there are no data rows.
     * @throws IOException if the given InputStream cannot be opened.
     * @throws POIXMLException if the given InputStream is not a spreadsheet file (.xlsx only).
     * @throws FormularyDesignatorImportFormatException if the EPL ID column is not formatted as
     *         text in the Excel spreadsheet, if the first row is not a header row, or if the 
     *         columns are in the wrong order.
     */
    public List<FormularyDesignatorImportRow> getRows(InputStream is)
        throws IOException, POIXMLException, FormularyDesignatorImportFormatException {
        List<FormularyDesignatorImportRow> rows = 
                new ArrayList<FormularyDesignatorImportRow>();  
        FormularyDesignatorImportFormatException importFormatException = null;
        XSSFWorkbook workbook = null;
       
        try {
            workbook = new XSSFWorkbook(is);
            XSSFSheet sheet = workbook.getSheetAt(SHEET_INDEX);
            Iterator<Row> iterator = sheet.iterator();
            iterator = discardHeaderRow(iterator);

            while (iterator.hasNext()) {
                Row xlsxRow = iterator.next();
                String eplIdString = 
                    getCellValue(xlsxRow.getCell(FormularyDesignatorMigrationConstants.EPL_ID_COLUMN));                         
                Long eplId = eplIdString == null 
                        ? null 
                        : Long.valueOf(eplIdString);                
                String oiName = getCellValue(xlsxRow.getCell(FormularyDesignatorMigrationConstants.OI_NAME_COLUMN));
                String formularyDesignator = 
                    getCellValue(xlsxRow.getCell(FormularyDesignatorMigrationConstants.FORMULARY_DESIGNATOR_COLUMN));
                String oiText = getCellValue(xlsxRow.getCell(FormularyDesignatorMigrationConstants.OI_TEXT_COLUMN));
                
                FormularyDesignatorImportRow resultRow = new FormularyDesignatorImportRow(
                        eplId,
                        oiName,
                        formularyDesignator,
                        oiText);
                
                rows.add(resultRow);
            }
        } catch (IllegalArgumentException e) {
            importFormatException = new FormularyDesignatorImportFormatException(e.getMessage());
        } catch (IllegalStateException e) {
            importFormatException = new FormularyDesignatorImportFormatException(
                    "Please ensure the EPL ID column is formatted as text in Excel.");
        }
        finally {
            if(workbook != null) {
                workbook.close();
            }
        }
        
        if(importFormatException != null) {
            throw importFormatException;
        }
        
        return rows;
    }

    private String getCellValue(Cell cell) {
        return cell == null 
            ? null 
            : cell.getStringCellValue();
    }

    private Iterator<Row> discardHeaderRow(Iterator<Row> iterator) throws FormularyDesignatorImportFormatException {
        if(iterator.hasNext()) {
            Row headerRow = iterator.next();
            String eplIdColumnHeader = 
                getCellValue(headerRow.getCell(FormularyDesignatorMigrationConstants.EPL_ID_COLUMN)); 
            
            if(!FormularyDesignatorMigrationConstants.EPL_ID_COLUMN_HEADER.equalsIgnoreCase(eplIdColumnHeader)) {
                throw new FormularyDesignatorImportFormatException(
                        "Please ensure the first row of the import file contains the column headers. "
                        + "Expected first column header to be " + FormularyDesignatorMigrationConstants.EPL_ID_COLUMN_HEADER 
                        + ", but found " + eplIdColumnHeader + " instead.");
            }
            
            iterator.remove();
        }
        
        return iterator;
    }
}
