package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorImportRow;
import gov.va.med.pharmacy.peps.domain.common.dao.EplFormularyDesignatorDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplOrderableItemDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplFormularyDesignatorDo;


/**
 * This class validates that a row of data from the Formulary
 * Designator import spreadsheet complies with business rules.
 */
public class FormularyDesignatorImportRowValidator {

    private static final int MAX_ORDERABLE_ITEM_TEXT_LENGTH = 255;
    
    private EplFormularyDesignatorDao fdDao;
    private EplOrderableItemDao oiDao;

    /**
     * The constructor.
     * 
     * @param fdDao DAO for working with formulary designator data
     * @param oiDao DAO for working with orderable item data 
     */
    @Autowired
    public FormularyDesignatorImportRowValidator(EplFormularyDesignatorDao fdDao, EplOrderableItemDao oiDao) {
        this.fdDao = fdDao;
        this.oiDao = oiDao;
    }
    
    /**
     * Ensures the data in the given row complies with business rule validation.
     * 
     * @param row to validate
     * @return the same row, updated to contain any validation errors that occurred
     */
    public FormularyDesignatorImportRow validate(FormularyDesignatorImportRow row) {
        if(row.getOrderableItemText().length() > MAX_ORDERABLE_ITEM_TEXT_LENGTH) {
            row.addError("OI Text exceeds character limit of " + MAX_ORDERABLE_ITEM_TEXT_LENGTH + ".");
        }
        
        if(row.getEplId() == null) {
            row.addError("EPL ID may not be blank.");
        }
        else if(!eplIdExists(row.getEplId())) {
            row.addError("No matching EPL ID.");
        }
        
        if("".equals(row.getFormularyDesignatorName())) {
            row.addError("Formulary Designator may not be blank");
        }
        else if(!formularyDesignatorNameExists(row.getFormularyDesignatorName())) {
            row.addError("No matching formulary designator.");
        }
        
        return row;
    }
    
    private boolean formularyDesignatorNameExists(String formularyDesignatorName) {
        List<EplFormularyDesignatorDo> fdList = 
                fdDao.retrieve(EplFormularyDesignatorDo.NAME, formularyDesignatorName);
        return fdList.size() > 0;
    }

    private boolean eplIdExists(Long eplId) {
        return oiDao.exists(eplId);
    }
}
