package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.io.File;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.pharmacy.peps.common.utility.PpsConfiguration;
import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorImportRow;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.service.common.migration.FormularyDesignatorMigrationResultFileWriter;
import gov.va.med.pharmacy.peps.service.common.migration.MigrationStatus;
import gov.va.med.pharmacy.peps.service.common.session.FormularyDesignatorMigrationService;

/**
 * Main implementation of {@link FormularyDesignatorMigrationService}.
 */
public class FormularyDesignatorMigrationServiceImpl implements FormularyDesignatorMigrationService {

    private static final String RESULTS_FILE_NAME = 
            "MigrationFormularyDesignatorResults.xlsx";
  
    private PpsConfiguration config;

    private FormularyDesignatorImportRowValidator validator;

    private FormularyDesignatorMigrator migrator;
    
    private MigrationStatus status;

    private FormularyDesignatorMigrationResultFileWriter writer;

    private UserVo user;
    
    /**
     * The constructor.
     * 
     * @param config containing application configuration settings
     * @param validator for checking migration business rules
     * @param migrator for processing rows
     * @param writer for creating the result file
     */
    @Autowired
    public FormularyDesignatorMigrationServiceImpl(
            PpsConfiguration config,
            FormularyDesignatorImportRowValidator validator,
            FormularyDesignatorMigrator migrator,
            FormularyDesignatorMigrationResultFileWriter writer) {
        this.config = config;
        this.validator = validator;
        this.migrator = migrator;
        this.writer = writer;
        this.status = new MigrationStatus();
        this.user = null;
    }
    
    @Override
    @Transactional
    public List<FormularyDesignatorImportRow> process(List<FormularyDesignatorImportRow> rows) {
        int rowCounter = 0;
        
        for(FormularyDesignatorImportRow row : rows) {
            row = validator.validate(row);
            
            if(row.isValid()) {
                row = migrator.migrate(row, user);
            }
            
            synchronized(this) {
                status.setProgressCount(++rowCounter);
            }
        }
        
        return rows;
    }

    @Override
    public synchronized void initialize(int rowCount, UserVo migrationUser) 
            throws FormularyDesignatorMigrationInProgressException {
        if(MigrationStatus.INPROGRESS.equals(status.getStatus())) {
            throw new FormularyDesignatorMigrationInProgressException();
        }
        
        status.setProgressCount(0);
        status.setTotalCount(rowCount);
        status.setStatus(MigrationStatus.INPROGRESS);
        this.user = migrationUser;
    }
    
    @Override
    public synchronized MigrationStatus getStatus() {
        return status;
    }
  
    @Override
    public File getResultFile() {
        return new File(config.getPersistentDirectory(), RESULTS_FILE_NAME);
    }

    @Override
    public void writeResultFile(List<FormularyDesignatorImportRow> rows) {
        writer.write(rows, getResultFile());
        
        synchronized(this) {
            status.setStatus(MigrationStatus.DONE);
        }
    }
}
