package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorImportRow;
import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorVo;
import gov.va.med.pharmacy.peps.common.vo.OrderableItemVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplFormularyDesignatorDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplFormularyDesignatorDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.FormularyDesignatorConverter;
import gov.va.med.pharmacy.peps.service.common.capability.ManagedItemCapability;

/**
 * This class processes the formulary designator migration
 * for a single input row from the import spreadsheet.
 */
public class FormularyDesignatorMigrator {
    private ManagedItemCapability managedItemCapability;
    private EplFormularyDesignatorDao formularyDesignatorDao;
    private FormularyDesignatorConverter formularyDesignatorConverter;

    /**
     * The constructor.
     * 
     * @param managedItemCapability for interacting with managed items
     * @param formularyDesignatorDao for performing formulary designator database
     * operations
     * @param formularyDesignatorConverter for converting between DO and VO objects
     */
    @Autowired
    public FormularyDesignatorMigrator(
            ManagedItemCapability managedItemCapability, 
            EplFormularyDesignatorDao formularyDesignatorDao,
            FormularyDesignatorConverter formularyDesignatorConverter) {
        this.managedItemCapability = managedItemCapability;
        this.formularyDesignatorDao = formularyDesignatorDao;
        this.formularyDesignatorConverter = formularyDesignatorConverter;
    }
    
    /**
     * Migrates the given row as the given user.
     * 
     * @param row to migrate
     * @param user performing the migration
     * @return the same row updated to contain any errors which occurred during
     * processing.
     */
    public FormularyDesignatorImportRow migrate(
            FormularyDesignatorImportRow row, 
            UserVo user) {
        try {
            OrderableItemVo originalOi = (OrderableItemVo) managedItemCapability.retrieve(
                    row.getEplId().toString(), 
                    EntityType.ORDERABLE_ITEM);
            OrderableItemVo modifiedOi = originalOi.copy();
            
            /*
             * If orderable item text is empty, then the migration should
             * ignore that value. It should not overwrite the existing orderable
             * item and product text with an empty value.
             */
            if(!row.getOrderableItemText().isEmpty()) {
                modifiedOi.setOiText(row.getOrderableItemText());
            }
            
            List<EplFormularyDesignatorDo>  fdDoList = formularyDesignatorDao.retrieve(
                    EplFormularyDesignatorDo.NAME, 
                    row.getFormularyDesignatorName());
            FormularyDesignatorVo fdVo = 
                    formularyDesignatorConverter.convert(fdDoList.get(0));
            modifiedOi.setOiFormularyDesignator(fdVo);
            
            //Save modifications to OI and propagate to child products.
            managedItemCapability.commitFormularyDesignatorMigrationModifications(
                    originalOi, 
                    modifiedOi,
                    user);
        } catch (ItemNotFoundException e) {
            row.addError(e.getMessage());
        } catch (ValidationException e) {
            row.addError(e.getMessage());
        }
        
        return row;
    }
}
