package gov.va.med.pharmacy.peps.service.common.session.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;

import gov.va.med.pharmacy.peps.common.vo.HazardousWasteImportRow;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProductDao;

/**
 * This class validates that a row of data from the Hazardous Waste import spreadsheet complies with business rules.
 */
public class HazardousWasteImportRowValidator extends AbstractMigrationImportRowValidator<HazardousWasteImportRow>{

    private static final int MAX_PRIMARY_EPA_LENGTH = 50;
    private static final int MAX_WASTE_SORT_CODE_LENGTH = 50;
    private static final int MAX_DOT_SHIPPING_NAME_LENGTH = 200;

    private EplProductDao productDao;

    /**
     * The constructor.
     *
     * @param pProductDao the product dao
     */
    @Autowired
    public HazardousWasteImportRowValidator(EplProductDao pProductDao) {
        this.productDao = pProductDao;
    }

    /**
     * Ensures the data in the given row complies with business rule validation.
     * 
     * @param row to validate
     * @return the same row, updated to contain any validation errors that occurred
     */
    @Override
    public HazardousWasteImportRow validate(HazardousWasteImportRow row) {
        if (row.getPrimaryEPA() != null && row.getPrimaryEPA().length() > MAX_PRIMARY_EPA_LENGTH) {
            row.addError("Primary EPA exceeds character limit of " + MAX_PRIMARY_EPA_LENGTH + ".");
        }
        if (row.getWasteSortCode() != null && row.getWasteSortCode().length() > MAX_WASTE_SORT_CODE_LENGTH) {
            row.addError("Waste Sort Code exceeds character limit of " + MAX_WASTE_SORT_CODE_LENGTH + ".");
        }
        if (row.getDotShippingName() != null && row.getDotShippingName().length() > MAX_DOT_SHIPPING_NAME_LENGTH) {
            row.addError("DOT Shipping Name exceeds character limit of " + MAX_DOT_SHIPPING_NAME_LENGTH + ".");
        }
        if (row.getEplId() == null) {
            row.addError("EPL ID may not be blank.");
        } else if (!eplIdExists(row.getEplId())) {
            row.addError("No matching EPL ID.");
        }
        validateDisposeFlagAndSubfields(row);
        
        return row;
    }

    /**
     * Validates that the Hazardous to Dispose flag is set to 'Y' if any of the subfields are populated (Primary EPA, 
     * Waste Sort Code, and DOT Shipping Name).
     * @param row HazardousWasteImportRow to validate
     */
    private void validateDisposeFlagAndSubfields(HazardousWasteImportRow row) {
        if (! "Y".equals(row.getHaz2DisposeFlagStr())){
            if (!StringUtils.isEmpty(row.getPrimaryEPA()) 
                || !StringUtils.isEmpty(row.getWasteSortCode()) 
                || !StringUtils.isEmpty(row.getDotShippingName())){
                row.addError("Hazardous to Dispose Flag is not 'Y', but sub-fields are populated");
            }
        }
    }

    private boolean eplIdExists(Long eplId) {
        return productDao.exists(eplId);
    }
}
