package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.util.Objects;

import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.HazardousWasteImportRow;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.service.common.capability.ManagedItemCapability;

/**
 * This class processes the hazardous waste migration for a single input row from the import spreadsheet.
 */
public class HazardousWasteMigrator extends AbstractMigrator<HazardousWasteImportRow>{
    private ManagedItemCapability managedItemCapability;

    /**
     * The constructor.
     * 
     * @param managedItemCapability for interacting with managed items
     */
    @Autowired
    public HazardousWasteMigrator(ManagedItemCapability managedItemCapability) {
        this.managedItemCapability = managedItemCapability;
    }

    /**
     * Migrates the given row as the given user.
     * 
     * @param row to migrate
     * @param user performing the migration
     * @return the same row updated to contain any errors which occurred during processing.
     */
    @Override
    public HazardousWasteImportRow migrate(HazardousWasteImportRow row, UserVo user) {
        try {
            ProductVo originalProduct = (ProductVo) managedItemCapability.retrieve(row.getEplId().toString(),
                EntityType.PRODUCT);
            //pre-check if any fields are different before making the copy
            if (Objects.equals(originalProduct.getHazardousToDispose(), row.getHaz2DisposeFlagStr())
                && Objects.equals(originalProduct.getHazardousToHandle(), row.getHaz2HandleFlagStr())
                && Objects.equals(originalProduct.getPrimaryEpa(), row.getPrimaryEPA())
                && Objects.equals(originalProduct.getWasteSortCode(), row.getWasteSortCode())
                && Objects.equals(originalProduct.getDotShippingName(), row.getDotShippingName())){
                //product already looks like what we want, so nothing to do
                return row;
            }
              
            //This may belong in the validator, but put it here to leverage the retrieve.
            if (!Objects.equals(originalProduct.getVaProductName(), row.getVaProductName())){
                row.addError("VA Product Name in Database does not match VA Product Name in input file.");
                return row;
            }
            
            ProductVo modifiedProduct = originalProduct.copy();
            modifiedProduct.setHazardousToHandle(toBoolean(row.getHaz2HandleFlagStr()));
            modifiedProduct.setHazardousToDispose(toBoolean(row.getHaz2DisposeFlagStr()));
            modifiedProduct.setPrimaryEpa(row.getPrimaryEPA());
            modifiedProduct.setWasteSortCode(row.getWasteSortCode());
            modifiedProduct.setDotShippingName(row.getDotShippingName());           
            
            // Save modifications to product, queue for update file, and create item audit history
            managedItemCapability.commitHazardousWasteMigrationModifications(originalProduct, modifiedProduct, user);
        } catch (ValidationException e) {
            row.addError(e.getMessage());
        } 

        return row;
    }
}
