package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import javax.transaction.Transactional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDdiSeverityDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDrugDrugInteractionDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplPackageSizeDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplIenGeneratorDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplIngredientDao;
import gov.va.med.pharmacy.peps.domain.common.utility.DDIFileReader;
import gov.va.med.pharmacy.peps.domain.common.utility.PackageSizeFileReader;
import gov.va.med.pharmacy.peps.service.common.session.Ppsn3MigrationService;

/**
 * Primary implementation of {@link Ppsn3MigrationService}.
 */
public class Ppsn3MigrationServiceImpl implements Ppsn3MigrationService {
    
    private static final Logger LOG = LogManager.getLogger(Ppsn3MigrationServiceImpl.class);
    
    private final EplIngredientDao eplIngredientDao;
    private final EplDdiSeverityDao eplDdiSeverityDao;
    private final EplDrugDrugInteractionDao drugDrugInteractionDao;
    private final EplPackageSizeDao packageSizeDao;
    private final EplIenGeneratorDao eplIenGeneratorDao;
    
    /**
     * Constructs an instance with the required dependencies.
     * @param pEplIngredientDao the {@link EplIngredientDao} to use.
     * @param pEplDdiSeverityDao the {@link EplDdiSeverityDao} to use.
     * @param pDrugDrugInteractionDao the {@link EplDrugDrugInteractionDao} to use.
     * @param pEplIenGeneratorDao the {@link EplIenGeneratorDao} to use.
     */
    @Autowired
    public Ppsn3MigrationServiceImpl(
        final EplIngredientDao pEplIngredientDao,
        final EplDdiSeverityDao pEplDdiSeverityDao,
        final EplDrugDrugInteractionDao pDrugDrugInteractionDao,
        final EplIenGeneratorDao pEplIenGeneratorDao,
        final EplPackageSizeDao pPackageSizeDao) {
        eplIngredientDao = pEplIngredientDao;
        eplDdiSeverityDao = pEplDdiSeverityDao;
        drugDrugInteractionDao = pDrugDrugInteractionDao;
        eplIenGeneratorDao = pEplIenGeneratorDao;
        packageSizeDao = pPackageSizeDao;
    }

    @Transactional
    @Override
    public void migrateDdis(UserVo user, File importFile) {
        try (final RandomAccessFile rFile = new RandomAccessFile(importFile, "rw")) {
            // This class is not thread-safe so construct an instance for every operation.
            final DDIFileReader fileReader = new DDIFileReader(
                eplIngredientDao, eplDdiSeverityDao, drugDrugInteractionDao, eplIenGeneratorDao, user);
            fileReader.setFile(rFile);
            fileReader.processFile();
        } catch (FileNotFoundException e) {
            throw new IllegalArgumentException("Unable to open given file for reading.", e);
        } catch (final IOException ex) {
            // This block will only be hit if unable to close the file.
            LOG.warn("Unable to close import file. Ignoring error for now.", ex);
        }
    }

    @Transactional
    @Override
    public void migratePackageSizes(UserVo user, File importFile) {
        try (final RandomAccessFile rFile = new RandomAccessFile(importFile, "rw")) {
            // This class is not thread-safe so construct an instance for every operation.
            final PackageSizeFileReader fileReader = new PackageSizeFileReader(packageSizeDao, 
                eplIenGeneratorDao, user);
            fileReader.setFile(rFile);
            fileReader.processFile();
        } catch (FileNotFoundException e) {
            throw new IllegalArgumentException("Unable to open given file for reading.", e);
        } catch (final IOException ex) {
            // This block will only be hit if unable to close the file.
            LOG.warn("Unable to close import file. Ignoring error for now.", ex);
        }
    }    
}
