package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.util.Date;
import java.util.List;

import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionException;

import gov.va.med.pharmacy.peps.common.exception.CsvFileException;
import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.ReportCopayTierChangeVo;
import gov.va.med.pharmacy.peps.service.common.reports.ReportProcessStatus;
import gov.va.med.pharmacy.peps.service.common.session.AbstractReportExportProcess;
import gov.va.med.pharmacy.peps.service.common.session.ReportCopayTierChangeProcess;
import gov.va.med.pharmacy.peps.service.common.utility.ReportCopayTierChangeCsvFile;


/**
 * This class retrieves the data for the 'Products with Changes to Copay Tier Data'
 * report and creates a caret separated value file for download.
 */
@Service
public class ReportCopayTierChangeProcessImpl extends AbstractReportExportProcess implements ReportCopayTierChangeProcess {
    /**
     * Log4j logger.
     */
	private static final org.apache.logging.log4j.Logger LOG = org.apache.logging.log4j.LogManager.getLogger(
			ReportCopayTierChangeProcessImpl.class);
    
    /**
     * The report will return products where the associated copay tier information has
     * changed on or after startDate. If startDate is null, all dates will be returned. 
     */
    private Date startDate;
    
    /**
     * Numeric flag indicating whether active products should be
     * included in the report. A value of 1 will include active products;
     * a value of 0 will exclude active products.
     */
    private int includeActiveProducts;
    
    /**
     * Numeric flag indicating whether inactive products should be
     * included in the report. A value of 1 will include inactive products;
     * a value of 0 will exclude inactive products.
     */
    private int includeInactiveProducts;
    
    /**
     * The value used to separate multiple drug class codes for the same
     * product.
     */
    private String drugClassDelimiter;
    
    /**
     * Constructor.
     */
    public ReportCopayTierChangeProcessImpl() {
        super();
        includeActiveProducts = 1;
        includeInactiveProducts = 1;
        drugClassDelimiter = "|";
    }
    
    @Override    
    public final void doExport() throws InterruptedException {
        LOG.debug("Exporting Product with Copay Tier Changes Report");

        boolean errorOccurred = false;
        this.getExportState().setReportProcessStatus(ReportProcessStatus.RUNNING);    

        try {
            //Set start time
            setDateValue(NationalSetting.REPORT_COPAY_TIER_CHANGE_START, getDateTime());
            
            ReportCopayTierChangeCsvFile csv = new ReportCopayTierChangeCsvFile();

            try {
                List<ReportCopayTierChangeVo> list = getReportDomainCapability().getCopayTierChanges(
                    getStartDate(),
                    getIncludeActiveProducts(),
                    getIncludeInactiveProducts(),
                    getDrugClassDelimiter());
                
                getExportState().setRecordTotal(list.size());
                csv.createFile(list, getExportState());
            } catch (CsvFileException e) {
                LOG.error(ERROR, e);
                errorOccurred = true;
            }
        } catch (TransactionException e) {
            LOG.error("FAILED TO retrieve List. Error Message is " + e.getMessage(), e);
            errorOccurred = true;            
        }
        finally {
            if(!errorOccurred) {
                //Set stop time
                setDateValue(NationalSetting.REPORT_COPAY_TIER_CHANGE_COMPLETE, getDateTime());                
                this.getExportState().setReportProcessStatus(ReportProcessStatus.COMPLETED);
            }
            else {
                this.stopProcess();
            }
        }
    }

    @Override
    public final Date getStartDate() {
        return startDate;
    }

    @Override
    public final void setStartDate(Date startDate) {
        this.startDate = startDate;
    }

    @Override
    public final String getDrugClassDelimiter() {
        return drugClassDelimiter;
    }

    @Override
    public final void setDrugClassDelimiter(String drugClassDelimiter) {
        this.drugClassDelimiter = drugClassDelimiter;
    }

    @Override
    public final void setIncludeActiveProducts(boolean includeActiveProducts) {
        if(includeActiveProducts) {
            setIncludeActiveProducts(1);
        }
        else {
            setIncludeActiveProducts(0);
        }
    }

    /**
     * @return the includeActiveProducts
     */
    private int getIncludeActiveProducts() {
      return includeActiveProducts;
    }

    /**
     * @param includeActiveProducts the includeActiveProducts to set
     */
    private void setIncludeActiveProducts(int includeActiveProducts) {
      this.includeActiveProducts = includeActiveProducts;
    }


    @Override
    public final void setIncludeInactiveProducts(boolean includeInactiveProducts) {
        if(includeInactiveProducts) {
            setIncludeInactiveProducts(1);
        }
        else {
            setIncludeInactiveProducts(0);
        }
    }

    /**
     * @return the includeInactiveProducts
     */
    public int getIncludeInactiveProducts() {
      return includeInactiveProducts;
    }

    /**
     * @param includeInactiveProducts the includeInactiveProducts to set
     */
    private void setIncludeInactiveProducts(int includeInactiveProducts) {
      this.includeInactiveProducts = includeInactiveProducts;
    }

}
