/**
 * Source file created in 2008 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Comparator;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;

import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;

import gov.va.med.pharmacy.peps.common.exception.CsvFileException;
import gov.va.med.pharmacy.peps.common.vo.ComplianceSiteVo;
import gov.va.med.pharmacy.peps.common.vo.DataUpdateFileVo;
import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.ReportDataUpdateVo;
import gov.va.med.pharmacy.peps.common.vo.UpdateSiteVo;
import gov.va.med.pharmacy.peps.common.vo.VistaSiteVo;
import gov.va.med.pharmacy.peps.common.vo.validator.ErrorKey;
import gov.va.med.pharmacy.peps.domain.common.capability.ComplianceSiteDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.NationalSettingDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.ReportDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.SiteListDomainCapability;
import gov.va.med.pharmacy.peps.service.common.session.AbstractReportExportProcess;
import gov.va.med.pharmacy.peps.service.common.session.DomainService;
import gov.va.med.pharmacy.peps.service.common.session.ReportDataUpdateProcess;
import gov.va.med.pharmacy.peps.service.common.utility.ReportDataUpdateCsvFile;


/** Generates the Data Update Compliance report in a background process. */
@Service
public class ReportDataUpdateProcessImpl 
    extends AbstractReportExportProcess implements ReportDataUpdateProcess {

    private static final int MILLISECONDS_PER_DAY = 24 * 60 * 60 * 1000;
    private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(ReportDataUpdateProcessImpl.class);
    private static final int DAYS_IN_COMPLIANCE = 7;
    
    private ComplianceSiteDomainCapability complianceSiteDomainCapability;
    private SiteListDomainCapability siteListDomainCapability;
    
    /**
 * Instantiates a new ReportDataUpdateProcessImpl.
 *
 * @param complianceSiteDomainCapability the compliance site domain capability
 * @param siteListDomainCapability the site list domain capability
 * @param reportDomainCapability the report domain capability
 * @param domainService the domain service
 * @param transactionManager the transaction manager
 * @param nationalSettingDomainCapability the national setting domain capability
 */
    @Autowired
    public ReportDataUpdateProcessImpl(
        ComplianceSiteDomainCapability complianceSiteDomainCapability,
        SiteListDomainCapability siteListDomainCapability,
        ReportDomainCapability reportDomainCapability,
        DomainService domainService,
        PlatformTransactionManager transactionManager,
        NationalSettingDomainCapability nationalSettingDomainCapability
        ) {
        super();
        this.complianceSiteDomainCapability = complianceSiteDomainCapability;
        this.siteListDomainCapability = siteListDomainCapability;
        setReportDomainCapability(reportDomainCapability);
        setDomainService(domainService);
        setTransactionManager(transactionManager);
        setNationalSettingDomainCapability(nationalSettingDomainCapability);
    }

    /** 
     * Generates Data Update Compliance Report.
     * @throws InterruptedException 
     */
    @Override
    public final void doExport() throws InterruptedException {
        LOG.debug("Exporting Data Update Compliance Report");
        
        try {
            //Set start time
            setDateValue(NationalSetting.REPORT_DATA_UPDATE_START, getDateTime());

            ReportDataUpdateCsvFile reportDataUpdateCsvFile = new ReportDataUpdateCsvFile();

            try {
                reportDataUpdateCsvFile.createFile();
            } catch (CsvFileException e) {
                LOG.error("Could not create Data Update Compliance file for export.", e);
                getExportState().getLastRunErrors().addError(ErrorKey.REPORT_COULD_NOT_CREATE_FILE);
            }
            
            ReportDataUpdateVo dataUpdateReport = new ReportDataUpdateVo();
            DataUpdateFileVo updateFile = getReportDomainCapability().getLatestUpdateFileInfo(); 
            dataUpdateReport.setFilename(updateFile.getFileName());
            dataUpdateReport.setReleaseDt(updateFile.getReleaseDt());
            Calendar cal = new GregorianCalendar();
            cal.setTime(updateFile.getReleaseDt());
            cal.add(Calendar.DAY_OF_MONTH, DAYS_IN_COMPLIANCE);
            Date complianceDt = cal.getTime();
            dataUpdateReport.setComplianceDt(complianceDt);                                    
            
            Set<VistaSiteVo> sitesForReport = new TreeSet<>();
            try {
                sitesForReport = determineSitesForReport();
            } catch (Exception ex){
                LOG.error("Could not read sites list from text file", ex);
                getExportState().getLastRunErrors().addError(ErrorKey.REPORT_ERROR_ENCOUNTERED);
            }
            
            //get sites that ARE compliant
            List<UpdateSiteVo> sitesInCompliance = getReportDomainCapability().getSitesInCompliance(
                updateFile.getId(), complianceDt);    
            List<UpdateSiteVo> sitesOutOfCompliance = new ArrayList<>(); 
            
            //loop and determine whether to add the site to the in compliance or out of compliance list - put into UpdateSiteVo
            for (VistaSiteVo siteForReport : sitesForReport) {
                UpdateSiteVo updateSiteVo = null;
                List<UpdateSiteVo> inSiteMatches = sitesInCompliance.stream()
                    .filter(inSite -> Objects.equals(String.valueOf(inSite.getSiteNbr()), siteForReport.getStationNumber()))
                    .collect(Collectors.toList());
                if (inSiteMatches.isEmpty()){
                    //out of compliance
                    UpdateSiteVo siteVo = getReportDomainCapability().getSiteOutOfComplianceInfo(
                        Long.valueOf(siteForReport.getStationNumber()));
                    updateSiteVo = new UpdateSiteVo(Long.valueOf(siteForReport.getStationNumber()), siteForReport.getName());
                    updateSiteVo.setFileName(siteVo == null ? "" : siteVo.getFileName());
                    updateSiteVo.setInstallDt(siteVo == null ? null : siteVo.getInstallDt());
                    updateSiteVo.setDaysOutOfCompliance(siteVo == null ? 0 : daysBetween(siteVo.getInstallDt(), complianceDt));
                    sitesOutOfCompliance.add(updateSiteVo);
                }
                else{
                    //in compliance
                    updateSiteVo = inSiteMatches.get(0);
                    updateSiteVo.setSiteNbr(Long.valueOf(siteForReport.getStationNumber()));
                    updateSiteVo.setSiteName(siteForReport.getName());
                    updateSiteVo.setDaysOfInstall(daysBetween(updateSiteVo.getInstallDt(), updateFile.getReleaseDt()));
                    updateSiteVo.setSiteName(siteForReport.getName());                    
                }
            }
            
            dataUpdateReport.setSitesInCompliance(sitesInCompliance);
            dataUpdateReport.setSitesOutOfCompliance(sitesOutOfCompliance);
            
            // print report
            reportDataUpdateCsvFile.printDataUpdateComplianceReport(dataUpdateReport);
            
            reportDataUpdateCsvFile.closeExport();

            //Set end time
            setDateValue(NationalSetting.REPORT_DATA_UPDATE_COMPLETE, getDateTime());

        } 
        catch (CsvFileException e) {
            LOG.error("FAILED TO generate Data Update Compliance Report. Error Message is ", e);
            getExportState().getLastRunErrors().addError(ErrorKey.REPORT_ERROR_ENCOUNTERED);
            this.stopProcess();
        }

    }

    /**
     * Determines the complete list of sites that should show on the report, based on the automatically filtered list, plus
     * the inclusions and minus the exclusions specified in the Compliance Site Table.
     * @return Set<VistaSiteVo> containing the list of sites for the report, sorted by station name. 
     */
    protected Set<VistaSiteVo> determineSitesForReport() throws Exception{
        //get sites that should be shown on the compliance report
        Collection<VistaSiteVo> filteredList = siteListDomainCapability.retrieveFiltered();
        //add inclusions
        Collection<ComplianceSiteVo> includeComplianceSites = complianceSiteDomainCapability.
            retrieveByIncludeExcludeIndicator(ComplianceSiteVo.INCLUDE);
        
        Collection<VistaSiteVo> includeVistaSiteVos = includeComplianceSites.stream()
            .map(site -> site.getVistaSiteVo()).collect(Collectors.toList());
        //remove exclusion
        Collection<ComplianceSiteVo> excludeComplianceSites = complianceSiteDomainCapability.
            retrieveByIncludeExcludeIndicator(ComplianceSiteVo.EXCLUDE);
        Collection<VistaSiteVo> excludeVistaSiteVos = excludeComplianceSites.stream()
            .map(site -> site.getVistaSiteVo()).collect(Collectors.toList());
        
        Comparator<VistaSiteVo> byStationNumber = (VistaSiteVo o1,
            VistaSiteVo o2) -> o1.getName().compareTo(o2.getName());
        
        Set<VistaSiteVo> sitesForReport = new TreeSet<>(byStationNumber);
        
        sitesForReport.addAll(filteredList);
        sitesForReport.addAll(includeVistaSiteVos);
        sitesForReport.removeAll(excludeVistaSiteVos);
        return sitesForReport;
    }
    
    private int daysBetween(Date d1, Date d2) {
        if ( d1 == null ) {
            LOG.warn("The d1 parameter passed to daysBetween() is null.");
            return 0;
        }
        if ( d2 == null ) {
            LOG.warn("The d2 parameter passed to daysBetween() is null.");
            return 0;
        }
        
        long diff = d2.getTime() - d1.getTime();
        return (int)(Math.abs(diff / MILLISECONDS_PER_DAY));
    }
}
