/**
 * Source file created in 2011 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.service.common.session.impl;


import org.apache.logging.log4j.Logger; 
import org.springframework.core.task.SimpleAsyncTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.pharmacy.peps.common.vo.ReportType;
import gov.va.med.pharmacy.peps.common.vo.ReportVo;
import gov.va.med.pharmacy.peps.service.common.reports.ReportExportState;
import gov.va.med.pharmacy.peps.service.common.reports.ReportObserver;
import gov.va.med.pharmacy.peps.service.common.session.ReportCopayTierChangeProcess;
import gov.va.med.pharmacy.peps.service.common.scheduler.SchedulerCommands;
import gov.va.med.pharmacy.peps.service.common.session.ReportExportManager;
import gov.va.med.pharmacy.peps.service.common.session.ReportExportProcess;
import gov.va.med.pharmacy.peps.service.common.session.ReportIngredientProcess;
import gov.va.med.pharmacy.peps.service.common.session.ReportNdfProcess;
import gov.va.med.pharmacy.peps.service.common.session.ReportWarningProcess;
import gov.va.med.pharmacy.peps.service.common.session.ReportDataUpdateProcess;
import gov.va.med.pharmacy.peps.service.common.utility.CsvFile;

/**
 * ReportExportManagerImpl's brief summary
 * 
 * Details of ReportExportManagerImpl's operations, special dependencies
 * or protocols developers shall know about when using the class.
 *
 */
@Service
public class ReportExportManagerImpl implements ReportExportManager, ReportObserver {

    private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(ReportExportManagerImpl.class);
    private static final int DELAY = 2000; // norm 2000

    private ReportExportState exportState;
    private ReportExportState ndfState;
    private ReportExportState ingredientState;
    private ReportExportState warningState;
    private ReportExportState copayTierChangeState;
    private ReportExportState dataUpdateState;
    private SimpleAsyncTaskExecutor reportExportExecutor;
    private ReportNdfProcess reportNdfProcess;
    private ReportIngredientProcess reportIngredientProcess;
    private ReportWarningProcess reportWarningProcess;
    /**
     * Represents the thread which does the work of generating the
     * 'Products with Changes to Copay Tier Data' report.
     */
    private ReportCopayTierChangeProcess reportCopayTierChangeProcess;    
    private ReportDataUpdateProcess reportDataUpdateProcess;

    @Override
    @Transactional
    public final ReportExportState startProcess(ReportVo reportVo) {
        ReportType reportType = reportVo.getReportType();
        command(SchedulerCommands.START, reportVo);
        return getStatus(reportType);
    }

    @Override
    @Transactional(readOnly=true)
    public final ReportExportState getStatus(ReportType reportType) {
        switch (reportType) {
            case NDC_LIST_PRINT_TEMPLATE:
                return ndfState;
            case LIST_INGREDIENTS:
                return ingredientState;
            case PRINT_PRODUCTS_WARNING_LABELS:
                return warningState;
            case PRODUCTS_WITH_CHANGES_TO_COPAY_TIER_DATA:
                return copayTierChangeState;
            case DATA_UPDATE_COMPLIANCE_REPORT:
                return dataUpdateState;
            default:
                return exportState;
        }
    }

    @Override
    @Transactional
    public ReportExportState stopProcess() {
        command(SchedulerCommands.STOP, null);

        return exportState;
    }

    /**
     * Execute pCommand for the report type specified by reportVo.getReportType().
     *
     * @param sCommand SchedulerCommands
     * @param reportVo contains the type of report to export as well as any 
     * criteria for the report.
     */
    private void command(SchedulerCommands sCommand, ReportVo reportVo) {
        ReportType reportType = null;
        
        if(reportVo != null) {
            reportType = reportVo.getReportType();
        }

        switch (sCommand) {
            case START:
                processStartCommand(reportType, reportVo);    
                break;                
            case STOP:
                processStopCommand(reportType);                
                break;
            default:
                break;
        }
    }

    /**
     * Start processing a report.
     * 
     * @param reportType the type of report to start
     * @param reportVo containing parameters for the report 
     */
    private void processStartCommand(ReportType reportType, ReportVo reportVo) {
        switch (reportType) {
            case NDC_LIST_PRINT_TEMPLATE:
                this.reportExportExecutor.execute(reportNdfProcess);
                sleepThread(DELAY); // delay to allow for the state to be

                if (reportNdfProcess != null) {
                    ndfState = reportNdfProcess.getExportState();
                    reportNdfProcess.registerObserver(this);
                }

                break;
            case LIST_INGREDIENTS:
                this.reportExportExecutor.execute(reportIngredientProcess);
                sleepThread(DELAY); // delay to allow for the state to be

                if (reportIngredientProcess != null) {
                    ingredientState = reportIngredientProcess.getExportState();
                    reportIngredientProcess.registerObserver(this);
                }

                break;
            case PRINT_PRODUCTS_WARNING_LABELS:
                this.reportExportExecutor.execute(reportWarningProcess);
                sleepThread(DELAY); // delay to allow for the state to be

                if (reportWarningProcess != null) {
                    warningState = reportWarningProcess.getExportState();
                    reportWarningProcess.registerObserver(this);
                }

                break;
            case PRODUCTS_WITH_CHANGES_TO_COPAY_TIER_DATA:
                reportCopayTierChangeProcess.setStartDate(reportVo.getStartDate());
                reportCopayTierChangeProcess.setDrugClassDelimiter(CsvFile.FIELD_MULTIPLE_SEPARATOR);
                
                if(!reportVo.getIncludeActiveProducts() && !reportVo.getIncludeInactiveProducts()) {
                    //If neither active nor inactive is checked, include all statuses.
                    reportCopayTierChangeProcess.setIncludeActiveProducts(true);
                    reportCopayTierChangeProcess.setIncludeInactiveProducts(true);
                }
                else {
                    reportCopayTierChangeProcess.setIncludeActiveProducts(reportVo.getIncludeActiveProducts());
                    reportCopayTierChangeProcess.setIncludeInactiveProducts(reportVo.getIncludeInactiveProducts());
                }
                
                this.reportExportExecutor.execute(reportCopayTierChangeProcess);
                sleepThread(DELAY);
                
                if(reportCopayTierChangeProcess != null) {
                    copayTierChangeState = reportCopayTierChangeProcess.getExportState();
                    reportCopayTierChangeProcess.registerObserver(this);
                }                        
                
                break;
            case DATA_UPDATE_COMPLIANCE_REPORT:
                this.reportExportExecutor.execute(reportDataUpdateProcess);
                sleepThread(DELAY); // delay to allow for the state to be

                if (reportDataUpdateProcess != null) {
                    dataUpdateState = reportDataUpdateProcess.getExportState();
                    reportDataUpdateProcess.registerObserver(this);
                }

                break;
            default:
                //Default case should not be possible because only "download only" reports
                //can be started.
                throw new DownloadOnlyReportException(
                    "Attempted to start report type which is not 'download only': " + reportType);
        }
    }
    
    /**
     * Stop processing a report.
     * 
     * @param reportType the type of report to stop
     */
    private void processStopCommand(ReportType reportType) {
        switch (reportType) {
            case NDC_LIST_PRINT_TEMPLATE:
                reportNdfProcess.stopProcess();
                break;
            case LIST_INGREDIENTS:
                reportIngredientProcess.stopProcess();
                break;
            case PRINT_PRODUCTS_WARNING_LABELS:
                reportWarningProcess.stopProcess();
                break;
            case PRODUCTS_WITH_CHANGES_TO_COPAY_TIER_DATA:
                reportCopayTierChangeProcess.stopProcess();
                break;
            case DATA_UPDATE_COMPLIANCE_REPORT:
                reportDataUpdateProcess.stopProcess();
                break;
            default:
                //Default case should not be possible because only "download only" reports
                //can be stopped.
                throw new DownloadOnlyReportException(
                    "Attempted to stop report type which is not 'download only': " + reportType);
        }
    }

    /**
     * sets the thread to sleep
     * @param count count
     */
    private void sleepThread(long count) {
        try {
            
            // set the thread to sleep for count milliseconds.
            Thread.sleep(count);
        } catch (InterruptedException x) {
            x.getStackTrace();
        }
    }

    /**
     * Observers update method.
     * @param pExportState export state 
     */
    @Override
    @Transactional
    public void update(ReportExportState pExportState) {
        LOG.debug("ReportExportManagerImpl==>update(), observer ");

        if (pExportState != null) {
            this.setReportExportState(pExportState);
            LOG.debug("update().isRunning: " + pExportState.isRunning());
            LOG.debug("update().isExportComplete: " + pExportState.isExportComplete());
        }

    }

    /**
     * description.
     * 
     * @return the reportExportState
     */
    public ReportExportState getReportExportState() {
        return exportState;
    }

    /**
     * description.
     * 
     * @param reportExportState the reportExportState to set
     */
    public void setReportExportState(ReportExportState reportExportState) {
        this.exportState = reportExportState;
    }

    /**
     * description.
     * 
     * @return the reportExportExecutor
     */
    public SimpleAsyncTaskExecutor getReportExportExecutor() {
        return reportExportExecutor;
    }

    /**
     * description.
     * 
     * @param reportExportExecutor the reportExportExecutor to set
     */
    public void setReportExportExecutor(SimpleAsyncTaskExecutor reportExportExecutor) {
        this.reportExportExecutor = reportExportExecutor;
    }

    /**
     * setReportNdfProcess.
     * @param reportNdfProcess the reportNdfProcess to set
     */
    public void setReportNdfProcess(ReportNdfProcess reportNdfProcess) {
        this.reportNdfProcess = reportNdfProcess;
    }

    /**
     * getReportNdfProcess.
     * @return the reportNdfProcess
     */
    public ReportNdfProcess getReportNdfProcess() {
        return reportNdfProcess;
    }

    /**
     * setReportIngredientProcess.
     * @param reportIngredientProcess the reportIngredientProcess to set
     */
    public void setReportIngredientProcess(ReportIngredientProcess reportIngredientProcess) {
        this.reportIngredientProcess = reportIngredientProcess;
    }

    /**
     * getReportIngredientProcess.
     * @return the reportIngredientProcess
     */
    public ReportIngredientProcess getReportIngredientProcess() {
        return reportIngredientProcess;
    }

    /**
     * setReportWarningProcess.
     * @param reportWarningProcess the reportWarningProcess to set
     */
    public void setReportWarningProcess(ReportWarningProcess reportWarningProcess) {
        this.reportWarningProcess = reportWarningProcess;
    }

    /**
     * getReportWarningProcess.
     * @return the reportWarningProcess
     */
    public ReportWarningProcess getReportWarningProcess() {
        return reportWarningProcess;
    }

    
    public final ReportExportProcess getReportCopayTierChangeProcess() {
        return reportCopayTierChangeProcess;
    }

    
    public final void setReportCopayTierChangeProcess(ReportCopayTierChangeProcess reportCopayTierChangeProcess) {
        this.reportCopayTierChangeProcess = reportCopayTierChangeProcess;
    }

    /**
     * getReportDataUpdateProcess.
     * @return the reportDataUpdateProcess
     */
    public ReportDataUpdateProcess getReportDataUpdateProcess() {
        return reportDataUpdateProcess;
    }

    /**
     * setReportDataUpdateProcess.
     * @param reportDataUpdateProcess the reportDataUpdateProcess to set
     */ 
    public void setReportDataUpdateProcess(ReportDataUpdateProcess reportDataUpdateProcess) {
        this.reportDataUpdateProcess = reportDataUpdateProcess;
    }

}
