/**
 * Source file created in 2008 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.service.common.session.impl;


import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidationType.LOG_FORGING;
import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator.validateLongInput;
import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator.validateStringInput;

import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionException;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import gov.va.med.pharmacy.peps.common.exception.CsvFileException;
import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.ReportProductVo;
import gov.va.med.pharmacy.peps.service.common.session.AbstractReportExportProcess;
import gov.va.med.pharmacy.peps.service.common.session.ReportWarningProcess;
import gov.va.med.pharmacy.peps.service.common.utility.ReportWarningLabelCsvFile;


/** Generates the Products and Warning Labels report in a background process. */
@Service
public class ReportWarningProcessImpl 
    extends AbstractReportExportProcess implements ReportWarningProcess {

    private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(ReportWarningProcessImpl.class);

    /**
     * The method is the ReportWarningProcessImpl method to Generates Warning Label Export.
     * @throws InterruptedException 
     */
    public void doExport() throws InterruptedException {     
        DefaultTransactionDefinition def = null;
        TransactionStatus status = null;        

        try {
            //Set start time
            setDateValue(NationalSetting.REPORT_WARNING_START, getDateTime());

            ReportWarningLabelCsvFile reportWarningLabelCsvFile = new ReportWarningLabelCsvFile();

            try {
                reportWarningLabelCsvFile.createFile();
            } catch (CsvFileException e) {
                /*
                 * TODO: There is no indication to the user that an exception has occurred.
                 */
                LOG.error("ReportWarningProcessImpl for export: " + e);
            }

            def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
            def.setTimeout(TIMEOUT);
            status = getTransactionManager().getTransaction(def);
            List<ReportProductVo> list = getReportDomainCapability().getProductWarningLabelData();
            LOG.debug("ReportWarningProcessImpl Warning Label list size: " + list.size());
            getExportState().setRecordTotal(list.size());
            getTransactionManager().commit(status);
            
            // Notify observer
            notifyObservers();

            List<ReportProductVo> actionList = new ArrayList<ReportProductVo>(BATCH_SIZE);
            int i = 0;

            for (ReportProductVo vo : list) {
                LOG.info("Loading: " + validateLongInput(new Long(i).toString()) + " of " 
                        + validateLongInput(new Long(list.size()).toString()) + " which is Product " 
                        + validateStringInput(vo.getVaProductName(), LOG_FORGING) + ":" 
                        + validateStringInput(vo.getGcnSeqNo(), LOG_FORGING));
                actionList.add(vo);
                i++;

                if (actionList.size() >= BATCH_SIZE) {
                    processBatch(reportWarningLabelCsvFile, actionList, i);
                }                
            }

            LOG.debug("Action List Size at print remaining: " + actionList.size());

            //Write any records where the remaining total is less than the
            //batch size.
            if (actionList.size() > 0) {
                LOG.debug("Entered Print Remaining: " + actionList.size());
                def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
                def.setTimeout(TIMEOUT);
                status = getTransactionManager().getTransaction(def);

                processBatch(reportWarningLabelCsvFile, actionList, i);
            }

            reportWarningLabelCsvFile.closeExport();

        } catch (TransactionException e) {
            LOG.error("FAILED TO retrieve List. Error Message is " + e.getMessage());
            this.stopProcess();
        }

        //Set end time for the Report Warning Complete label when it finishes
        setDateValue(NationalSetting.REPORT_WARNING_COMPLETE, getDateTime());

    }

    private void processBatch(ReportWarningLabelCsvFile reportWarningLabelCsvFile, List<ReportProductVo> actionList, int i) {
        DefaultTransactionDefinition def;
        TransactionStatus status;
        for (ReportProductVo rpVo : actionList) {
            def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
            def.setTimeout(TIMEOUT);
            status = getTransactionManager().getTransaction(def);
            rpVo = getDomainService().getDrugReferenceCapability().retrieveWarningLabels(rpVo);
            getTransactionManager().commit(status);
        }

        try {
            reportWarningLabelCsvFile.printWarningLabelReport(actionList);
        } catch (CsvFileException e) {
            /*
             * TODO: There is no indication to the user that an exception has occurred.
             * This exception would occur if there was an error writing to the file
             * on disk. The report might be incomplete or missing records, and the user
             * wouldn't know.
             */
            LOG.error("Cound not write to Warning Label file for export: " + e);
        }

        actionList.clear();
        
        // Update record count                    
        getExportState().setRecordCounter(i);
        notifyObservers();
    }
}
