package gov.va.med.pharmacy.peps.service.national.updatefile;

import static gov.va.med.pharmacy.peps.service.national.updatefile.UpdateFileReportFormatter.EMPTYLINE;
import static gov.va.med.pharmacy.peps.service.national.updatefile.UpdateFileReportFormatter.formatProductReference;
import static gov.va.med.pharmacy.peps.service.national.updatefile.UpdateFileReportFormatter.doubleIndent;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplRematchDo;
import gov.va.med.pharmacy.peps.domain.common.model.dto.ProductChangeDto;

/**
 * Generates sections of the Data Update for NDF report dealing with rematches.
 */
public class RematchReportGenerator {
    
    private static final Logger LOG = LogManager.getLogger(RematchReportGenerator.class);

    /**
     * The string {@value} for generating rematch reports.
     */
    public static final String USE = "  USE";
    
    private final UpdateFileReportGenerator reportGenerator;
    private final List<EplRematchDo> outgoingRematches;
    private final Map<Long, EplRematchDo> productToRematchMap;
    private final List<String> inactivatedProductsReport;
    private final List<String> otherRematchesReport;
    
    /**
     * Constructs an instance.
     * 
     * @param pUpdateFileReportGenerator The {@link UpdateFileReportGenerator} to use.
     * @param pOutgoingRematches The rematches being sent in the update file.
     * @param otherChanges All outgoing changes that may trigger a rematch (other than inactivation changes).
     * @param inactivatedProducts The inactivated products that are being sent in the update file. 
     */
    public RematchReportGenerator(
        final UpdateFileReportGenerator pUpdateFileReportGenerator,
        final List<EplRematchDo> pOutgoingRematches,
        final Set<ProductChangeDto> otherChanges,
        final List<ProductChangeDto> inactivatedProducts) {
        reportGenerator = pUpdateFileReportGenerator;
        outgoingRematches = pOutgoingRematches;

        LOG.debug("There are {} outoing rematches. Generating product-to-rematch map...", outgoingRematches.size());
        productToRematchMap = outgoingRematches.stream()
            .collect(Collectors.toMap(rematch -> rematch.getFromProduct().getEplId(), Function.identity()));
        
        // Used products is necessary as it used to determine which rematches have associated changes, doing so allows
        // us to determine which have no associated change and are therefore rematch only.
        final Set<Long> usedProducts = new HashSet<>();
        inactivatedProductsReport = generateInactivatedProductsReport(inactivatedProducts, usedProducts);
        otherRematchesReport = generateOtherEditReport(otherChanges, usedProducts);
    }

    /**
     * Generates the inactivated products report, both with and without rematches.
     * 
     * @param inactiveProducts lists the inactivated products
     * @return the report content, with one string per line
     */
    private List<String> generateInactivatedProductsReport(
        final List<ProductChangeDto> inactiveProducts,
        final Set<Long> usedProducts) {
        
        LOG.debug("Generating inactivated rematches report for {} products.", inactiveProducts.size());
        
        /*-
         * The below code builds the reports in two stages:
         * 
         * 1) We iterate through all of the product changes and build the content of each section (changes with
         *    rematches and changes without rematches.)
         * 
         * 2) We then add the content sections with their appropriate headers to the report itself.
         */
        
        /* Stage 1 */
        
        final ArrayList<String> withAutomaticRematchContent = new ArrayList<>();
        final ArrayList<String> withSuggestedRematchContent = new ArrayList<>();
        final ArrayList<String> withoutRematchContent = new ArrayList<>();
        for (final ProductChangeDto product : inactiveProducts) {
            // This list will contain the report content for this particular product whether it has a rematch or not.
            final ArrayList<String> productContent = new ArrayList<>();
            // This variable will be set below to point to the correct content section. We will use it later to append
            // productContent to the correct content section.
            final ArrayList<String> targetContent;
        
            // This part is the same whether with a rematch or without.
            productContent.add(formatProductReference(product));
            usedProducts.add(product.getEplId());
        
            // This part depends on whether there is a rematch or not.
            final long productId = product.getEplId().longValue();
            final EplRematchDo productRematch = productToRematchMap.get(productId);
            if (productRematch != null) {
                // With rematch.
                productContent.add(doubleIndent(USE));
                for (final EplProductDo toProduct : productRematch.getToProducts()) {
                    productContent.add(formatProductReference(toProduct));
                }
        
                if (productRematch.getToAutoProduct() != null) {
                    LOG.debug("Product {} has an automatic rematch.", productId);
                    targetContent = withAutomaticRematchContent;
                } else {
                    LOG.debug("Product {} has a suggested rematch.", productId);
                    targetContent = withSuggestedRematchContent;
                }
            } else {
                // Without rematch.
                LOG.debug("Product {} does not have a rematch.", productId);
                targetContent = withoutRematchContent;
            }
        
            // This part is the same either way again.
            productContent.add(EMPTYLINE);
        
            // Add the whole product content to the correct content list.
            targetContent.addAll(productContent);
        }
        
        /* Stage 2 */
        
        return formatWithHeaders(
            ReportMessageCode.INACTIVATEDPRODUCT,
            ReportMessageCode.INACTIVATEDREMATCHED,
            ReportMessageCode.INACTIVATEDWSUGG,
            withAutomaticRematchContent,
            withSuggestedRematchContent,
            withoutRematchContent);
    }
    
    private List<String> generateOtherEditReport(
        final Collection<ProductChangeDto> productChanges,
        final Set<Long> usedProducts) {
        
        LOG.debug("Generating other rematches report for {} products.", productChanges.size());
        
        final TreeSet<EplRematchDo> rematchToInclude = new TreeSet<>(new ReportRematchComparator());
        
        /*-
         * The below code builds the reports in three stages:
         * 
         * 1) We iterate through all of the product changes and add those changes without associated rematches to the
         * appropriate without rematch content section.
         * 
         * 2) We iterate through all of the rematches and add those not already in a report to the appropriate with rematch
         * content section.
         * 
         * 3) We then add the content sections with their appropriate headers to the report itself.
         */
        
        final ArrayList<String> withAutomaticRematchContent = new ArrayList<>();
        final ArrayList<String> withSuggestedRematchContent = new ArrayList<>();
        final ArrayList<String> withoutRematchContent = new ArrayList<>();
        
        for (final ProductChangeDto product : productChanges) {
            
            final ArrayList<String> productContent = new ArrayList<>();
            final long productId = product.getEplId().longValue();
            final EplRematchDo productRematch = productToRematchMap.get(productId);
            
            if (productRematch == null) {
                // Without rematch.
                LOG.debug("Product {} does not have a rematch.", productId);
                productContent.add(formatProductReference(product));
                usedProducts.add(product.getEplId());
                productContent.add(EMPTYLINE);
                // Add the whole product content to the correct content list.
                withoutRematchContent.addAll(productContent);
            }
            else {
                rematchToInclude.add(productRematch);
            }
        }
        
        for (final EplRematchDo rematch : outgoingRematches) {
            if (!usedProducts.contains(rematch.getFromProduct().getEplId())) {
                rematchToInclude.add(rematch);
            }
            
        }
        
        LOG.debug("Adding the {} remaining rematches to the report.", rematchToInclude.size());
        
        for (final EplRematchDo rematch : rematchToInclude) {
            if (rematch.getToAutoProduct() != null) {
                withAutomaticRematchContent.add(formatProductReference(rematch.getFromProduct()));
                withAutomaticRematchContent.add(doubleIndent(USE));
                withAutomaticRematchContent.add(formatProductReference(rematch.getToAutoProduct()));
                withAutomaticRematchContent.add(EMPTYLINE);
            } else {
                withSuggestedRematchContent.add(formatProductReference(rematch.getFromProduct()));
                withSuggestedRematchContent.add(doubleIndent(USE));
                for (EplProductDo toSuggProduct : rematch.getToSuggestedProducts()) {
                    withSuggestedRematchContent.add(formatProductReference(toSuggProduct));
                }
                withSuggestedRematchContent.add(EMPTYLINE);
            }
        }
        
        return formatWithHeaders(
            ReportMessageCode.OTHERNOREMATCH,
            ReportMessageCode.OTHERREMATCHED,
            ReportMessageCode.OTHERREMATCHSUGG,
            withAutomaticRematchContent,
            withSuggestedRematchContent,
            withoutRematchContent);
    }

    private List<String> formatWithHeaders(final ReportMessageCode noRematchHeader, final ReportMessageCode autoRematchHeader,
        final ReportMessageCode suggestedRematchHeader, final ArrayList<String> withAutomaticRematchContent,
        final ArrayList<String> withSuggestedRematchContent, final ArrayList<String> withoutRematchContent) {
        final List<String> result = new ArrayList<String>();

        // Changes with automatic rematches.
        if (!withAutomaticRematchContent.isEmpty()) {
          result.addAll(reportGenerator.formatHeaderTextWithPadding(autoRematchHeader));
          result.addAll(withAutomaticRematchContent);
        }

        // Changes with suggested rematches.
        if (!withSuggestedRematchContent.isEmpty()) {
          result.addAll(reportGenerator.formatHeaderTextWithPadding(suggestedRematchHeader));
          result.addAll(withSuggestedRematchContent);
        }

        // Changes without rematches.
        if (!withoutRematchContent.isEmpty()) {
          result.addAll(reportGenerator.formatHeaderTextWithPadding(noRematchHeader));
          result.addAll(withoutRematchContent);
        }

        return result;
    }

    /**
     * Returns the "other rematch-triggering changes" report, e.g. for VA Print Name changes. Includes the sections
     * for both changes with and without rematches.
     * 
     * @return the report content, with one string per line
     */
    public List<String> otherRematchesReport() {
        return otherRematchesReport;
    }

    /**
     * Returns the inactivated products report, both with and without rematches.
     * @return the report content, with one string per line.
     */
    public List<String> inactivatedProductsReport() {
        return inactivatedProductsReport;
    }
}
