package gov.va.med.pharmacy.peps.service.national.updatefile;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.Objects;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.context.NoSuchMessageException;

import gov.va.med.pharmacy.peps.common.utility.security.UntrustedLogParam;

/**
 * <p>Contains some common functionality useful for generating the reports included in the NDF Update File. At time of
 * writing, those reports are the "Data Update for NDF" Report and "UPDATED INTERACTIONS AND FDA MED GUIDE" report.</p>
 * 
 * <p>Eventually more functionality should probably move here from {@link
 * gov.va.med.pharmacy.peps.service.common.capability.impl.MessageDataCapabilityImpl}.</p>
 */
public class UpdateFileReportGenerator {
  
  private static final Logger LOG = LogManager.getLogger(UpdateFileReportGenerator.class);

  private final MessageSource messageSource;
  
  @Autowired
  public UpdateFileReportGenerator(final MessageSource pMessageSource) {
    messageSource = Objects.requireNonNull(pMessageSource);

    LOG.debug("Using {} as the MessageSource.", new UntrustedLogParam(pMessageSource));
  }

  /**
   * Returns the same as {@link #formatHeaderText(ReportMessageCode)} except wraps the text in a blank line before and
   * after.
   * 
   * @param messageCode specifies the message to look up
   * @return One string per line. Never null or empty.
   */
  public List<String> formatHeaderTextWithPadding(final ReportMessageCode messageCode) {
    List<String> result = new ArrayList<String>();
    result.add("");
    result.addAll(formatHeaderText(messageCode));
    result.add("");
    return result;
  }

  /**
   * <p>Looks up the message in the configured {@link MessageSource}, breaks into separate lines as necessary, and returns
   * it.</p>
   * 
   * <p>Note the vertical bar delimited text is probably UI implementation creeping inappropriately into the backend as
   * the responsibility to format the text (line size) rightly rests on a presentation layer component.</p>
   * 
   * @param messageCode specifies the message to look up
   * @return One string per line. Never null or empty.
   * @throws NoSuchMessageException if the specified message does not exist
   */
  public List<String> formatHeaderText(final ReportMessageCode messageCode) throws NoSuchMessageException {
    // Only supporting the default locale for now.
    final Locale locale = Locale.getDefault();
    final String messageText = messageSource.getMessage(messageCode.getCode(), null, locale);
    return Arrays.asList(messageText.split("\\|"));
  }
}
