package gov.va.med.pharmacy.peps.external.common.utility.test;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertThat;

import java.io.File;

import org.apache.commons.io.FileUtils;
import org.apache.commons.vfs2.FileObject;
import org.apache.commons.vfs2.FileSystemException;
import org.apache.commons.vfs2.impl.StandardFileSystemManager;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Test;

import gov.va.med.pharmacy.peps.service.common.utility.FileInfo;
import gov.va.med.pharmacy.peps.service.common.utility.TransferFile;
import junit.framework.TestCase;
/**
 * The Class TransferFileIT.
 */
public class TransferFileIT extends TestCase {

    /** The Constant LOG. */
    private static final Logger LOG = LogManager.getLogger(TransferFileIT.class);

    /**
     * Test local file to remote file transfer and also getSizeFor.
     * @throws Exception 
     */
    @Test
    public final void testLocalToRemote() throws Exception {
        File tempFile = null; 
        FileInfo fileInfo = null;
        try{
          tempFile = File.createTempFile("LocalToRemote", ".txt");
          String randomText = "Some text here";
          FileUtils.writeStringToFile(tempFile, randomText);
          
          fileInfo = createFileInfo(tempFile.getName(), "pending");
  
          String inputFileDirectory = tempFile.getParentFile().getAbsolutePath() + File.separator;
  
          boolean fileTransmitted = TransferFile.moveTo(inputFileDirectory, fileInfo);
          assertThat(TransferFile.getSizeFor(fileInfo), equalTo(String.valueOf(randomText.length())));
          LOG.info(fileTransmitted);
          assertTrue(fileTransmitted);
        }
        finally{
          // cleanup test files
          try{
            if (tempFile!= null){
              tempFile.delete();
            }
          }
          finally{
            deleteRemoteTestFile(fileInfo);
          }
        }
    }

    /**
     * Test local file to remote file transfer and also getSizeFor.
     * @throws Exception 
     */
    @Test
    public final void testLocalToRemoteWhenLocalDoesNotExist() throws Exception {
        File tempFile = null; 
        FileInfo fileInfo = null;
        try{
          tempFile = File.createTempFile("LocalToRemote", ".txt");
          String randomText = "Some text here";
          FileUtils.writeStringToFile(tempFile, randomText);
          
          fileInfo = createFileInfo(tempFile.getName(), "pending");
  
          String inputFileDirectory = tempFile.getParentFile().getAbsolutePath() + "/invalidgarbage";
  
          boolean fileTransmitted = TransferFile.moveTo(inputFileDirectory, fileInfo);
          assertFalse(fileTransmitted);
        }
        finally{
          // cleanup test files
          try{
            if (tempFile!= null){
              tempFile.delete();
            }
          }
          finally{
            deleteRemoteTestFile(fileInfo);
          }
        }
    }    
    
    /**
     * Test remote file to remote file transfer.
     * @throws Exception 
     */
    @Test
    public final void testRemoteToRemote() throws Exception {
        String fileName = "RemoteToRemote.txt";
        FileInfo fileInfoPending = createFileInfo(fileName, "pending");
        // create the initial file to be used during the move
        FileObject testFile = createRemoteTestFile(fileInfoPending);
        LOG.info(testFile.getName());
        FileInfo fileInfoApproved = createFileInfo(fileName, "approved");

        boolean fileTransmitted = TransferFile.moveTo(fileInfoPending, fileInfoApproved);

        // cleanup test file
        deleteRemoteTestFile(fileInfoApproved);
        assertTrue(fileTransmitted);
    }

    /**
     * Creates a FileInfo object when given fileName and folder of the file.
     *
     * @param fileName
     *            the file name
     * @param folder
     *            the folder
     * @return the file info
     * @throws Exception if the password is not entered for the SFTP server
     */
    private FileInfo createFileInfo(String fileName, String folder) throws Exception {
        FileInfo fileInfo = new FileInfo();

        fileInfo.setHostName("DNS           .aac.DNS   ");//FQDN
        fileInfo.setPort("22");
        fileInfo.setUserName("PW     "); //username
        fileInfo.setAuthorization("PW     "); //fill in pwd here
        if (fileInfo.getAuthorization().equals("")){
          throw new Exception("You must fill in the SFTP server password in createFileInfo(). See the Developer KEEPASS file.");
        }
          
        fileInfo.setDirectoryPath("/home/PW     /pps-n/dev/localhost/" + folder + "/"); //an existing folder for the test
        fileInfo.setFileName(fileName);

        return fileInfo;
    }

    /**
     * Creates the remote test file.
     *
     * @param fileInfo
     *            the file info
     * @return the file object
     */
    public final FileObject createRemoteTestFile(FileInfo fileInfo) {

        StandardFileSystemManager remoteFsm = new StandardFileSystemManager();
        FileObject remoteFile = null;
        try {

            remoteFsm.init();
            if (fileInfo != null) {
                remoteFile = remoteFsm.resolveFile(fileInfo.getEncodedFilePath(), TransferFile.createFileSystemOptions());
                remoteFile.createFile();
            }

        } catch (FileSystemException e) {
            LOG.error("Exception creating test file", e);

        } finally {
            if (remoteFsm != null) {
                remoteFsm.close();
            }
        }
        return remoteFile;
    }

    /**
     * Delete remote test file for cleanup of tests.
     *
     * @param fileInfo
     *            the file info
     * @return the file object
     */
    public final FileObject deleteRemoteTestFile(FileInfo fileInfo) {

        StandardFileSystemManager remoteFsm = new StandardFileSystemManager();
        FileObject remoteFile = null;
        try {

            remoteFsm.init();
            if (fileInfo != null) {
                remoteFile =
                        remoteFsm.resolveFile(fileInfo.getEncodedURIDirectory() + File.separator + fileInfo.getFileName(),
                                TransferFile.createFileSystemOptions());
                remoteFile.delete();
            }

        } catch (FileSystemException e) {
            LOG.error("Exception deleting remote test file ");

        } finally {
            if (remoteFsm != null) {
                remoteFsm.close();
            }
        }
        return remoteFile;
    }
   
}
