package gov.va.med.pharmacy.peps.service.common.capability.impl;

import static org.easymock.EasyMock.*;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Path;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.easymock.EasyMockSupport;
import org.junit.Test;
import org.springframework.core.env.Environment;
import org.springframework.mock.env.MockEnvironment;

import gov.va.med.pharmacy.peps.common.email.EmailService;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;
import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.common.vo.StatusVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.NdfUpdateFileMgtDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.utility.test.NoOpTransactionManager;
import gov.va.med.pharmacy.peps.service.common.capability.NdfUpdateProcessCapability;
import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdateProcessFile;

/**
 * Automated tests for {@link NdfUpdateCapabilityImpl}.
 */
public class NdfUpdateCapabilityImplTest extends EasyMockSupport {
  
  private static final Logger LOG = LogManager.getLogger(NdfUpdateCapabilityImplTest.class);
  /**
   * Tests that {@link NdfUpdateCapabilityImpl#createOnDemandFileAsync(UserVo)} throws the proper exception if there
   * is already a file going through the process.
   */
  public final void testCreateOnDemandFileAsync_NonFinalStatus() {
    
    /* Setup */
    final NdfUpdateCapabilityImpl updateCapability = new NdfUpdateCapabilityImpl();
    final UserVo user = new UserGenerator().getNationalManagerTwo();

    // NdfUpdateFileMgtDomainCapability
    final NdfUpdateFileMgtDomainCapability mockUpdateFileMgtCapability = mock(NdfUpdateFileMgtDomainCapability.class);
    updateCapability.setNdfUpdateFileMgtDomainCapability(mockUpdateFileMgtCapability);
    
    final NdfUpdateFileVo currentUpdateFile = new NdfUpdateFileVo();
    currentUpdateFile.setStatus(new StatusVo(StatusVo.INITIATED));

    /* Expected method calls */
    
    expect(mockUpdateFileMgtCapability.retrieveCurrentStatus()).andReturn(currentUpdateFile);
    
    /* Behavior & Verification */
    
    replayAll();
    
    try {
      updateCapability.createOnDemandFileAsync(user);
      fail("Expected exception to be thrown.");
    } catch (final ValidationException ex) {
      assertEquals(ex.getKey(), equalTo(ValidationException.NON_FINAL_STATE_FILE_IN_PROCESS));
    }
    
    verifyAll();
  }
  
  /**
   * Tests that {@link NdfUpdateCapabilityImpl#createScheduledFile(UserVo)} does not try to generate a file if one is already in the
   * process.
   */
  public final void testCreateScheduledFile_NonFinal() throws IOException {
    /* Setup */
    final NdfUpdateCapabilityImpl updateCapability = new NdfUpdateCapabilityImpl();
    final UserVo user = new UserGenerator().getNationalManagerOne();
    
    final NdfUpdateFileVo currentUpdateFile = new NdfUpdateFileVo();
    currentUpdateFile.setFileName("PPS_60PRV_61NEW.DAT");
    currentUpdateFile.setStatus(new StatusVo(StatusVo.READY_FOR_TESTING));

    // NdfUpdateFileMgtDomainCapability
    final NdfUpdateFileMgtDomainCapability mockUpdateFileMgtCapability = mock(NdfUpdateFileMgtDomainCapability.class);
    updateCapability.setNdfUpdateFileMgtDomainCapability(mockUpdateFileMgtCapability);
    
    /* Expected method calls */
    
    expect(mockUpdateFileMgtCapability.retrieveCurrentStatus()).andReturn(currentUpdateFile);
    
    replayAll();
    
    /* Behavior & Verification */
    
    updateCapability.createScheduledFile(user);
    
    verifyAll();
  }

  /**
   * Tests that {@link
   * NdfUpdateCapabilityImpl#transmitUpdateFileToTest(NdfUpdateFileVo, gov.va.med.pharmacy.peps.common.vo.UserVo)}
   * does not throw an exception when the transmission succeeds but sending the email fails.
   */
  @Test
  public final void transmitUpdateFileToTest_EmailError() throws Exception {
    
    /* Setup */
    final NdfUpdateCapabilityImpl updateCapability = new NdfUpdateCapabilityImpl();
    final String mockFileSize = "1000";
    final UserVo user = new UserGenerator().getNationalManagerOne();
    
    // NdfUpdateFileVo. Just set the minimal properties.
    final NdfUpdateFileVo updateFileVo = new NdfUpdateFileVo();
    updateFileVo.setFileName("PPS_0PRV_1NEW.DAT");

    // NdfUpdateProcessFile
    final NdfUpdateProcessFile mockFile = mock(NdfUpdateProcessFile.class);
    updateCapability.setNdfUpdateProcessFile(mockFile);

    // EmailService
    final EmailService mockEmailService = mock(EmailService.class);
    updateCapability.setEmailService(mockEmailService);
    
    // Environment
    updateCapability.setEnvironment(makeMockEnvironment());
    
    // NdfUpdateFileMgtDomainCapability
    final NdfUpdateFileMgtDomainCapability mockUpdateFileMgtCapability = mock(NdfUpdateFileMgtDomainCapability.class);
    updateCapability.setNdfUpdateFileMgtDomainCapability(mockUpdateFileMgtCapability);
    
    // PlatformTransactionManager
    updateCapability.setTransactionManager(new NoOpTransactionManager());
    
    // Expected dependency method calls.
    mockFile.transmitFileToTest(updateFileVo.getFileName());
    expect(mockFile.getPendingFileSize(updateFileVo.getFileName())).andReturn(mockFileSize);
    mockEmailService.sendEmailMutlipleTimes(anyObject());
    expectLastCall().andThrow(new RuntimeException("exception from mock EmailService for testing"));
    
    replayAll();
    
    /* Behavior & Verification */
    
    // We do not want this to throw an exception.
    updateCapability.transmitUpdateFileToTest(updateFileVo, user);
    
    verifyAll();
    
    assertThat(updateFileVo.getStatus().getStatusId(), equalTo(StatusVo.READY_FOR_TESTING));
    
    // Verify that updateCapability set a comment on the file.
    assertFalse(
        "comment should be set",
        StringUtils.isEmpty(updateFileVo.getComments()));
  }
  
  /**
   * Tests that {@link NdfUpdateCapabilityImpl#generateAndTransmitFile(NdfUpdateFileVo, UserVo)} properly throws an
   * IOException when {@link NdfUpdateProcessCapability} does.
   */
  @Test
  public void testGenerateAndTransmitFile_IOException() throws Exception {

    /* Setup */
    final NdfUpdateCapabilityImpl updateCapability = new NdfUpdateCapabilityImpl();
    final UserVo user = new UserGenerator().getNationalManagerOne();
    final String expectedFileName = "EXPECTED_FILE_NAME.DAT";
    final Path expectedPath = FileSystems.getDefault().getPath("test", expectedFileName);

    // NdfUpdateFileVo
    final NdfUpdateFileVo updateFileVo = new NdfUpdateFileVo();
    updateFileVo.setFileId("11");
    updateFileVo.setStatus(new StatusVo(StatusVo.INITIATED));

    // NdfUpdateProcessFile
    final NdfUpdateProcessFile mockFile = mock(NdfUpdateProcessFile.class);
    updateCapability.setNdfUpdateProcessFile(mockFile);

    // EmailService
    final EmailService mockEmailService = mock(EmailService.class);
    updateCapability.setEmailService(mockEmailService);
    
    // Environment
    updateCapability.setEnvironment(makeMockEnvironment());

    // NdfUpdateFileMgtDomainCapability
    final NdfUpdateFileMgtDomainCapability mockUpdateFileMgtCapability = mock(NdfUpdateFileMgtDomainCapability.class);
    updateCapability.setNdfUpdateFileMgtDomainCapability(mockUpdateFileMgtCapability);
    
    // NdfUpdateProcessCapability
    final NdfUpdateProcessCapability mockUpdateProcessCapability = mock(NdfUpdateProcessCapability.class);
    updateCapability.setNdfUpdateProcessCapability(mockUpdateProcessCapability);
    
    // PlatformTransactionManager
    updateCapability.setTransactionManager(new NoOpTransactionManager());
    
    // Expected dependency method calls.
    expect(mockFile.createFile(updateFileVo.getFileId())).andReturn(expectedPath);
    expect(mockFile.getPath()).andReturn(expectedPath);
    mockUpdateFileMgtCapability.update(updateFileVo, user);
    mockUpdateProcessCapability.addDataToFile(updateFileVo, mockFile);
    // I'd really like to test the process all the way through, but too much more to mock right now.
    expectLastCall().andThrow(new IOException("IOException thrown by mock"));
    
    replayAll();

    /* Behavior & Verification */
    
    try {
      updateCapability.generateAndTransmitFile(updateFileVo, user);
      fail("Expected an IOException here.");
    } catch (IOException ex) {
      LOG.debug("Caught exception as expected.");
    }

    verifyAll();
    
    assertThat(updateFileVo.getFileName(), equalTo(expectedFileName));
    assertThat(updateFileVo.getDirectoryPath(), equalTo(expectedPath.toString()));
  }

  /**
   * Creates a mock {@link Environment} that provides useful properties.
   * @return
   */
  private MockEnvironment makeMockEnvironment() {
    final MockEnvironment mockEnvironment = new MockEnvironment();
    mockEnvironment.setProperty(NdfUpdateCapabilityImpl.FILE_READY_FOR_TESTING_TO_EMAIL_PROPERTY, "fake_email_address");
    return mockEnvironment;
  }

}
