package gov.va.med.pharmacy.peps.service.common.capability.impl;

import static org.easymock.EasyMock.expect;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import org.easymock.EasyMockSupport;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.exception.DuplicateRematchException;
import gov.va.med.pharmacy.peps.common.utility.impl.test.EnvironmentUtilityStub;
import gov.va.med.pharmacy.peps.common.utility.test.generator.ProductGenerator;
import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.IngredientVo;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryDetailsVo;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryVo;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.common.vo.OrderableItemVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.common.vo.diff.test.DifferenceGenerator;
import gov.va.med.pharmacy.peps.domain.common.capability.RematchMgtDomainCapability;
import gov.va.med.pharmacy.peps.service.common.capability.ModRematch;
import gov.va.med.pharmacy.peps.service.common.capability.RematchCapability;

/**
 * Unit tests for {@link RematchCapabilityImpl}.
 */
public class RematchCapabilityImplTest extends EasyMockSupport {
  
  /**
   * Just for convenience since many tests use this.
   */
  private final ProductGenerator productGenerator = new ProductGenerator();
  
  /**
   * Just for convenience since many tests use this.
   */
  private final UserGenerator userGenerator = new UserGenerator();

  private final EnvironmentUtilityStub environmentUtility = new EnvironmentUtilityStub();
  
  /**
   * Tests {@link
   * RematchCapabilityImpl#getExistingRematchForDifferences(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo,
   * Collection)}.
   */
  @Test
  public void testGetExistingRematchForDifferences() {

    /* Setup */

    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);
    
    // Create the existing rematch.
    final RematchVo existingRematch = createExistingRematch(MatchReason.FUTURE_INACTIVATION);
    final ProductVo testProduct = existingRematch.getFromProduct();

    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    final Collection<Difference> inactivationDiffs = Arrays.asList(
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.ACTIVE, ItemStatus.INACTIVE));
    final Collection<Difference> futureInactivationDiffs = Arrays.asList(
        DifferenceGenerator.makeDifferenceForDataField(FieldKey.PROPOSED_INACTIVATION_DATE, null, new Date()));

    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.of(existingRematch)).times(2); // Combining two tests in one here.
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.getExistingRematchForDifferences(testProduct, inactivationDiffs),
        equalTo(Optional.empty()));
    
    assertThat(
        objUnderTest.getExistingRematchForDifferences(testProduct, futureInactivationDiffs),
        equalTo(Optional.of(existingRematch)));
    
    verifyAll();

  }
  
  /**
   * Tests calling {@link RematchCapabilityImpl#updateUponProposedInactivation(
   * gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, UserVo)} for a non-product.
   */
  @Test
  public void testUpdateUponProposedInactivation_NonProduct() {
    /* Setup */

    final UserVo updatingUser = userGenerator.getNationalManagerOne();

    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);

    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

    // No expected method calls
    
    replayAll();
    
    /* Behavior & verification */
    
    objUnderTest.updateUponProposedInactivation(new OrderableItemVo(), updatingUser);
    objUnderTest.updateUponProposedInactivation(new IngredientVo(), updatingUser);
    
    verifyAll();
  }

  @Test
  public void testUpdateUponProposedInactivation_NoExisting() {
    /* Setup */

    final UserVo updatingUser = userGenerator.getNationalManagerTwo();
    final ProductVo testProduct = productGenerator.getFirst();

    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);

    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.empty());
    
    replayAll();
    
    /* Behavior & verification */
    
    objUnderTest.updateUponProposedInactivation(testProduct, updatingUser);
    
    verifyAll();
  }

  @Test
  public void testUpdateUponProposedInactivation_ExistingFutureInact() throws DuplicateRematchException {
    /* Setup */

    final UserVo updatingUser = userGenerator.getNationalManagerOne();
    final RematchVo existingRematch = createExistingRematch(MatchReason.FUTURE_INACTIVATION);
    final ProductVo testProduct = existingRematch.getFromProduct();

    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);

    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.of(existingRematch));
    expect(mockRematchDomainCapability.saveOrUpdate(existingRematch, updatingUser)).andReturn(existingRematch);
    
    replayAll();
    
    /* Behavior & verification */
    
    objUnderTest.updateUponProposedInactivation(testProduct, updatingUser);
    
    verifyAll();

    assertThat("should have changed match reason", existingRematch.getMatchReason(), equalTo(MatchReason.INACTIVATION));
  }

  @Test
  public void testUpdateUponProposedInactivation_ExistingOther() {
    /* Setup */

    final MatchReason initialMatchReason = MatchReason.MANUAL_REMATCH;
    final RematchVo existingRematch = createExistingRematch(initialMatchReason);
    final ProductVo testProduct = existingRematch.getFromProduct();

    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);

    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.of(existingRematch));
    
    replayAll();
    
    /* Behavior & verification */
    
    objUnderTest.updateUponProposedInactivation(testProduct, userGenerator.getNationalManagerOne());
    
    verifyAll();
    
    assertThat("should not have changed match reason", existingRematch.getMatchReason(), equalTo(initialMatchReason));
  }

    /**
     * Tests that
     * {@link RematchCapabilityImpl#saveOrUpdateUncommittedRematch(Collection, RematchVo, ItemAuditHistoryVo, UserVo)} deletes
     * the rematch if it is obsolete.
     */
  @Test
  public void testSaveOrUpdateUncommittedRematch_DeleteObsolete() throws DuplicateRematchException {
    
    /* Setup */
    
    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);
    
    final List<ProductVo> testProducts = productGenerator.getList();
    final ProductVo fromProduct = testProducts.get(0);
    final ProductVo toProduct = testProducts.get(1);
    final UserVo testUser = userGenerator.getNationalManagerTwo();
    final ItemAuditHistoryVo auditHistory = new ItemAuditHistoryVo();

    final Collection<Difference> differences = Arrays.asList(
        // Red herring difference.
        new Difference(FieldKey.HERBAL, Boolean.FALSE, Boolean.TRUE));
    final RematchVo testRematch = RematchVo.automaticRematch(fromProduct, toProduct, MatchReason.INACTIVATION);
    testRematch.setEplRematchId(null);  // Unsaved.
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // No expected method calls, should just return an empty optional.
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.saveOrUpdateUncommittedRematch(differences, testRematch, auditHistory, testUser),
        equalTo(Optional.empty()));
    
    assertThat(
        new ArrayList<>(auditHistory.getDetails()),  // wrap in a List for equals()
        equalTo(Collections.emptyList()));
    
    verifyAll();
  }

  /**
   * Tests that {@link
   * RematchCapabilityImpl#saveOrUpdateCommittedRematch(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
   * Optional, UserVo)} does not delete an existing rematch if it shouldn't.
   */
  @Test
  public void testSaveOrUpdateCommittedRematch_PreserveExisting() throws DuplicateRematchException {

    /* Setup */

    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);
    
    final ProductVo testProduct = productGenerator.getFirst();
    final UserVo testUser = userGenerator.getNationalManagerTwo();
    
    final Collection<Difference> differences = Arrays.asList(
        // The item has a rematch and we are inactivating it. This shouldn't actually happen in product, but it's a good
        // edge case for this test.
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.ACTIVE, ItemStatus.INACTIVE));
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // No expected method calls: should _not_ call delete.
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.saveOrUpdateCommittedRematch(testProduct, differences, Optional.empty(), testUser),
        equalTo(Optional.empty()));
    
    verifyAll();
  }

  
  /**
   * Tests that {@link
   * RematchCapabilityImpl#saveOrUpdateCommittedRematch(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
   * Optional, UserVo)} deletes the existing rematch if we are reversing the trigger.
   */
  @Test
  public void testSaveOrUpdateCommittedRematch_DeleteReversed() throws DuplicateRematchException {

    /* Setup */

    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = mock(RematchMgtDomainCapability.class);
    
    // Create the existing rematch.
    final RematchVo existingRematch = createExistingRematch(MatchReason.INACTIVATION);
    final UserVo testUser = userGenerator.getNationalManagerTwo();
    final ProductVo testProduct = existingRematch.getFromProduct();

    final Collection<Difference> differences = Arrays.asList(
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.INACTIVE, ItemStatus.ACTIVE));
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // Expected method calls
    mockRematchDomainCapability.delete(existingRematch);
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.of(existingRematch));
    
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.saveOrUpdateCommittedRematch(testProduct, differences, Optional.empty(), testUser),
        equalTo(Optional.empty()));
    
    verifyAll();
  }
  
  /**
   * Tests that {@link
   * RematchCapabilityImpl#saveOrUpdateCommittedRematch(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
   * Optional, UserVo)} works properly if we are reversing the trigger and there is no existing rematch.
   */
  @Test
  public void testSaveOrUpdateCommittedRematch_DeleteReversedNonExisting() throws DuplicateRematchException {

    /* Setup */
    
    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);
    
    final ProductVo testProduct = productGenerator.getFirst();
    final UserVo testUser = userGenerator.getNationalManagerOne();

    final Collection<Difference> differences = Arrays.asList(
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.INACTIVE, ItemStatus.ACTIVE));
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.empty());
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.saveOrUpdateCommittedRematch(testProduct, differences, Optional.empty(), testUser),
        equalTo(Optional.empty()));
    
    verifyAll();
  }

  /**
   * Tests that {@link
   * RematchCapabilityImpl#saveOrUpdateCommittedRematch(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
   * Optional, UserVo)} deletes the rematch if it is obsolete.
   */
  @Test
  public void testSaveOrUpdateCommittedRematch_DeleteObsolete() throws DuplicateRematchException {
    
    /* Setup */
    
    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);
    
    final List<ProductVo> testProducts = productGenerator.getList();
    final ProductVo fromProduct = testProducts.get(0);
    final ProductVo toProduct = testProducts.get(1);
    final UserVo testUser = userGenerator.getNationalManagerTwo();

    final Collection<Difference> differences = Arrays.asList(
        // Red herring difference.
        new Difference(FieldKey.HERBAL, Boolean.FALSE, Boolean.TRUE));
    final RematchVo testRematch = RematchVo.automaticRematch(fromProduct, toProduct, MatchReason.INACTIVATION);
    testRematch.setEplRematchId("1999");
    final ModRematch modRematch = new ModRematch(Optional.of(testRematch),
        "ignored reason");  // The reason is currently ignored for inactivation rematches.
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // Expected method calls
    mockRematchDomainCapability.delete(testRematch);
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.saveOrUpdateCommittedRematch(fromProduct, differences, Optional.of(modRematch), testUser),
        equalTo(Optional.empty()));

    assertThat("should not add audit history", fromProduct.getNewAuditHistory(), nullValue());
    
    verifyAll();
  }
  
  /**
   * Tests that {@link
   * RematchCapabilityImpl#saveOrUpdateCommittedRematch(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
   * Optional, UserVo)} does not delete the rematch if it is not obsolete.
   */
  @Test
  public void testSaveOrUpdateCommittedRematch_KeepCurrent() throws DuplicateRematchException {
    
    /* Setup */

    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);
    
    final List<ProductVo> testProducts = productGenerator.getList();
    final ProductVo fromProduct = testProducts.get(0);
    final ProductVo toProduct = testProducts.get(1);
    final UserVo testUser = userGenerator.getNationalManagerTwo();
    final Collection<Difference> differences = Arrays.asList(
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.ACTIVE, ItemStatus.INACTIVE));
    final RematchVo testRematch = RematchVo.automaticRematch(fromProduct, toProduct, MatchReason.INACTIVATION);
    final ModRematch modRematch = new ModRematch(Optional.of(testRematch),
        "ignored reason");  // The reason is currently ignored for inactivation rematches.
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // Expected method calls.
    expect(mockRematchDomainCapability.saveOrUpdate(testRematch, testUser)).andReturn(testRematch);
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.saveOrUpdateCommittedRematch(fromProduct, differences, Optional.of(modRematch), testUser),
        equalTo(Optional.of(testRematch)));
    
    assertThat("should not add audit history", fromProduct.getNewAuditHistory(), nullValue());
    
    verifyAll();
  }
  
  /**
   * Tests that {@link RematchCapabilityImpl#updateUponCommittingItemChanges(RequestVo,
   * gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, java.util.Collection)} updates the rematch if the trigger reason
   * changed.
   */
  @Test
  public void testSaveOrUpdateCommittedRematch_UpdateCurrent() throws DuplicateRematchException {
    
    /* Setup */

    // Mock the RematchMgtDomainCapability.
    final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);
    
    final List<ProductVo> testProducts = productGenerator.getList();
    final ProductVo fromProduct = testProducts.get(0);
    final ProductVo toProduct = testProducts.get(1);
    final UserVo testUser = userGenerator.getNationalManagerOne();
    final Collection<Difference> differences = Arrays.asList(
        DifferenceGenerator.makeDifferenceForDataField(FieldKey.PROPOSED_INACTIVATION_DATE, null, new Date()),
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.ACTIVE, ItemStatus.INACTIVE));
    final RematchVo testRematch = RematchVo.automaticRematch(fromProduct, toProduct, MatchReason.FUTURE_INACTIVATION);
    final RematchVo expectedRematch = RematchVo.automaticRematch(fromProduct, toProduct, MatchReason.INACTIVATION);
    final ModRematch modRematch = new ModRematch(Optional.of(testRematch),
        "ignored reason");  // The reason is currently ignored for inactivation rematches.
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // Expected method calls.
    expect(mockRematchDomainCapability.saveOrUpdate(expectedRematch, testUser)).andReturn(expectedRematch);
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.saveOrUpdateCommittedRematch(fromProduct, differences, Optional.of(modRematch), testUser),
        equalTo(Optional.of(expectedRematch)));
    
    verifyAll();
  }
  
    /**
     * Tests that {@link
     * RematchCapabilityImpl#saveOrUpdateCommittedRematch(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
     * Optional, UserVo)} does not delete a manual rematch.
     */
    @Test
    public void testSaveOrUpdateCommittedRematch_KeepManual() throws DuplicateRematchException {

        // Mock the RematchMgtDomainCapability.
        final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);

        final String providedReason = "manual rematch reason";
        final List<ProductVo> testProducts = productGenerator.getList();
        final ProductVo fromProduct = testProducts.get(0);
        final ProductVo toProduct = testProducts.get(1);
        final UserVo testUser = userGenerator.getNationalManagerTwo();
        final Collection<Difference> differences = Collections.emptyList(); // No differences.
        final RematchVo testRematch = RematchVo.automaticRematch(fromProduct, toProduct, MatchReason.MANUAL_REMATCH);
        final ModRematch modRematch = new ModRematch(Optional.of(testRematch), providedReason);
        
        final ItemAuditHistoryVo expectedHistory = expectedManualRematchHistory(
            testUser, fromProduct, providedReason, Difference.NO_VALUE, testRematch.getToProductsString());

        final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

        // Expected method calls.
        expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testRematch.getFromProduct().getId())))
            .andReturn(Optional.empty());
        expect(mockRematchDomainCapability.saveOrUpdate(testRematch, testUser)).andReturn(testRematch);

        replayAll();

        /* Behavior & verification */

        assertThat(
            objUnderTest.saveOrUpdateCommittedRematch(fromProduct, differences, Optional.of(modRematch), testUser),
            equalTo(Optional.of(testRematch)));

        assertThat(
            "should add correct audit history",
            fromProduct.getNewAuditHistory(),
            equalTo(expectedHistory));

        verifyAll();
    }
    
    /**
    * Tests that {@link
    * RematchCapabilityImpl#saveOrUpdateCommittedRematch(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
    * Optional, UserVo)} overwrites a previous rematch for it's fromProduct if it exists.
    */
    @Test
    public void testSaveOrUpdateCommittedRematch_OverwritePrevious() throws DuplicateRematchException {
        
        // Mock the RematchMgtDomainCapability.
        final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);
        
        final String providedReason = "overwrite rematch reason";
        final List<ProductVo> testProducts = productGenerator.getList();
        final ProductVo fromProduct = testProducts.get(0);
        final ProductVo toProductPrevious = testProducts.get(1);
        final ProductVo toProductManual = testProducts.get(2);
        final UserVo testUser = userGenerator.getNationalManagerTwo();
        final Collection<Difference> differences = Collections.emptyList(); // No differences.
        final RematchVo previousRematch = RematchVo.automaticRematch(fromProduct, toProductPrevious, MatchReason.INACTIVATION);
        final RematchVo manualRematch = RematchVo.automaticRematch(fromProduct, toProductManual, MatchReason.MANUAL_REMATCH);
        final ModRematch modRematch = new ModRematch(Optional.of(manualRematch), providedReason);
        
        final ItemAuditHistoryVo expectedHistory = expectedManualRematchHistory(
            testUser, fromProduct, providedReason, previousRematch.getToProductsString(), manualRematch.getToProductsString());

        final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

        // Expected method calls.
        expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(manualRematch.getFromProduct().getId())))
            .andReturn(Optional.of(previousRematch));
        mockRematchDomainCapability.delete(previousRematch);
        expect(mockRematchDomainCapability.saveOrUpdate(manualRematch, testUser)).andReturn(manualRematch);

        replayAll();

        /* Behavior & verification */

        assertThat(
            objUnderTest.saveOrUpdateCommittedRematch(fromProduct, differences, Optional.of(modRematch), testUser),
            equalTo(Optional.of(manualRematch)));
        
        assertThat(
            "should add correct audit history",
            fromProduct.getNewAuditHistory(),
            equalTo(expectedHistory));

        verifyAll();
        
    }

    /**
     * Tests that {@link RematchCapabilityImpl#saveOrUpdateCommittedRematch(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo,
     * Collection, Optional, UserVo)} deletes an existing rematch if requested.
     */
    @Test
    public void testSaveOrUpdateCommittedRematch_DeleteExisting() throws DuplicateRematchException {

        /* Setup */

        // Mock the RematchMgtDomainCapability.
        final RematchMgtDomainCapability mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);

        final String providedReason = "delete reason";
        final List<ProductVo> testProducts = productGenerator.getList();
        final ProductVo fromProduct = testProducts.get(0);
        final ProductVo toProductPrevious = testProducts.get(1);
        final RematchVo existingRematch = RematchVo.automaticRematch(fromProduct, toProductPrevious, MatchReason.INACTIVATION);
        final UserVo testUser = userGenerator.getNationalManagerTwo();
        final ModRematch modRematch = new ModRematch(Optional.empty(), providedReason);

        final ItemAuditHistoryVo expectedHistory = expectedManualRematchHistory(
            testUser, fromProduct, providedReason, existingRematch.getToProductsString(), Difference.NO_VALUE);

        final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

        // Expected method calls.
        expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(fromProduct.getId())))
            .andReturn(Optional.of(existingRematch));
        mockRematchDomainCapability.delete(existingRematch);

        replayAll();

        /* Behavior & verification */

        assertThat(
            objUnderTest.saveOrUpdateCommittedRematch(fromProduct, Collections.emptyList(), Optional.of(modRematch), testUser),
            equalTo(Optional.empty()));
        
        assertThat(
            "should add correct audit history",
            fromProduct.getNewAuditHistory(),
            equalTo(expectedHistory));
    }

    /**
     * Returns the expected {@link ItemAuditHistoryVo} for a manual rematch.
     * @return never null
     */
    private ItemAuditHistoryVo expectedManualRematchHistory(
        final UserVo testUser, final ProductVo product, final String reason, final String oldValue, final String newValue) {
        final ItemAuditHistoryVo expectedAuditHistory =
            product.createModificationAuditHistory(testUser, environmentUtility);
        expectedAuditHistory.addDetail(new ItemAuditHistoryDetailsVo(
            expectedAuditHistory.getEventCategory(),
            reason,
            RematchCapability.FIELD_NAME,
            oldValue,
            newValue));
        return expectedAuditHistory;
    }
    
  private RematchVo createExistingRematch(final MatchReason matchReason) {
    
    // Create the existing rematch.
    final List<ProductVo> testProducts = productGenerator.getList();
    final ProductVo fromProduct = testProducts.get(0);
    final ProductVo toProduct = testProducts.get(1);
    final RematchVo existingRematch = RematchVo.automaticRematch(fromProduct, toProduct, MatchReason.INACTIVATION);
    existingRematch.setMatchReason(matchReason);
    existingRematch.setEplRematchId("1244");
    
    return existingRematch;
  }

}
