package gov.va.med.pharmacy.peps.service.common.scheduler;

import static org.easymock.EasyMock.*;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.time.LocalDate;
import java.util.Date;

import org.easymock.EasyMockSupport;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.DateTimeUtility;
import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;
import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.NationalSettingVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.NationalSettingDomainCapability;

/**
 * Unit tests for {@link ProcessStatusUpdater}.
 */
public class ProcessStatusUpdaterTest extends EasyMockSupport {

  /**
   * Tests {@link ProcessStatusUpdater#updateStatus(NationalSetting, ProcessStatus, UserVo)}.
   */
  @Test
  public void testUpdateStatus() {
    /* Setup */
    final NationalSetting testSetting = NationalSetting.FDB_ADD_RUN_STATE;
    final ProcessStatus testProcessStatus = ProcessStatus.RUNNING;
    final UserVo user = new UserGenerator().getNationalManagerTwo();
    
    // NationalSettingDomainCapability
    final NationalSettingDomainCapability nationalSettingDc = mock(NationalSettingDomainCapability.class);
    final NationalSettingVo statusSettingVo = prepareStatusExpectations(nationalSettingDc, testSetting, user);

    final ProcessStatusUpdater objUnderTest = new ProcessStatusUpdater(nationalSettingDc);
    
    replayAll();
    
    /* Behavior */
    objUnderTest.updateStatus(testSetting, testProcessStatus, user);
    
    /* Verification */

    verifyAll();
    
    assertThat(statusSettingVo.getStringValue(), equalTo(testProcessStatus.toString()));
  }
  
  /**
   * Tests {@link ProcessStatusUpdater#startJob(NationalSetting, NationalSetting, UserVo)}.
   */
  @Test
  public void testStartJob() {
    /* Setup */
    final NationalSetting stateSetting = NationalSetting.FSS_RUN_STATE;
    final NationalSetting lastRunSetting = NationalSetting.FSS_UPDATE_LAST_RUN;
    final UserVo user = new UserGenerator().getNationalManagerTwo();
    final Date lastRunDate = Date.from(DateTimeUtility.toInstant(LocalDate.of(2017, 2, 9)));
    
    // NationalSettingDomainCapability
    final NationalSettingDomainCapability nationalSettingDc = mock(NationalSettingDomainCapability.class);

    final ProcessStatusUpdater objUnderTest = new ProcessStatusUpdater(nationalSettingDc);
    
    // Expected method calls.
    final NationalSettingVo lastRunSettingVo = new NationalSettingVo();
    lastRunSettingVo.setKeyName(lastRunSetting.toString());
    lastRunSettingVo.setDateValue(lastRunDate);
    expect(nationalSettingDc.retrieve(lastRunSetting.toString())).andReturn(lastRunSettingVo);
    final NationalSettingVo statusSettingVo = prepareStatusExpectations(nationalSettingDc, stateSetting, user);
    
    replayAll();
    
    /* Behavior */
    objUnderTest.startJob(stateSetting, lastRunSetting, user);
    
    /* Verification */

    verifyAll();
    
    assertThat(statusSettingVo.getStringValue(), equalTo(ProcessStatus.RUNNING.toString()));
  }

  /**
   * Tests {@link ProcessStatusUpdater#updateStatusAndRuntime(NationalSetting, ProcessStatus, NationalSetting, Date,
   * UserVo)}.
   */
  @Test
  public void testUpdateStatusAndRuntime_SpecifyDate() {
    /* Setup */
    final NationalSetting stateSetting = NationalSetting.FSS_RUN_STATE;
    final NationalSetting lastRunSetting = NationalSetting.FSS_UPDATE_LAST_RUN;
    final ProcessStatus testProcessStatus = ProcessStatus.ERROR;
    final Date newLastRunTime = Date.from(DateTimeUtility.toInstant(LocalDate.of(2017, 2, 3)));
    final UserVo user = new UserGenerator().getNationalManagerTwo();

    // NationalSettingDomainCapability
    final NationalSettingDomainCapability nationalSettingDc = mock(NationalSettingDomainCapability.class);
    final NationalSettingVo statusSettingVo = prepareStatusExpectations(nationalSettingDc, stateSetting, user);
    final NationalSettingVo lastRunSettingVo = prepareLastRunExpectations(nationalSettingDc, lastRunSetting, user);
    // Save off the initial date for later checking.
    final Date initialDate = lastRunSettingVo.getDateValue();

    final ProcessStatusUpdater objUnderTest = new ProcessStatusUpdater(nationalSettingDc);
    replayAll();
    
    /* Behavior */
    final Date returnedDate = objUnderTest.updateStatusAndRuntime(
        stateSetting, testProcessStatus, lastRunSetting, newLastRunTime, user);

    /* Verification */

    verifyAll();

    assertThat("check return value", returnedDate, equalTo(initialDate));
    assertThat("check new status", statusSettingVo.getStringValue(), equalTo(testProcessStatus.toString()));
    assertThat("check new last run time", lastRunSettingVo.getDateValue(), equalTo(newLastRunTime));
  }

  /**
   * Tests {@link ProcessStatusUpdater#updateStatusAndRuntime(NationalSetting, ProcessStatus, NationalSetting, UserVo).
   */
  @Test
  public void testUpdateStatusAndRuntime_CurrentDate() {
    /* Setup */
    final NationalSetting stateSetting = NationalSetting.FSS_RUN_STATE;
    final NationalSetting lastRunSetting = NationalSetting.FSS_UPDATE_LAST_RUN;
    final ProcessStatus testProcessStatus = ProcessStatus.ERROR;
    final UserVo user = new UserGenerator().getNationalManagerTwo();

    // NationalSettingDomainCapability
    final NationalSettingDomainCapability nationalSettingDc = mock(NationalSettingDomainCapability.class);
    final NationalSettingVo statusSettingVo = prepareStatusExpectations(nationalSettingDc, stateSetting, user);
    final NationalSettingVo lastRunSettingVo = prepareLastRunExpectations(nationalSettingDc, lastRunSetting, user);
    // Save off the initial date for later checking.
    final Date initialDate = lastRunSettingVo.getDateValue();

    final ProcessStatusUpdater objUnderTest = new ProcessStatusUpdater(nationalSettingDc);
    replayAll();
    
    /* Behavior */
    final Date rightBeforeCallingMethod = new Date();
    final Date returnedDate = objUnderTest.updateStatusAndRuntime(stateSetting, testProcessStatus, lastRunSetting, user);

    /* Verification */

    verifyAll();

    assertThat(returnedDate, equalTo(initialDate));

    assertThat(statusSettingVo.getStringValue(), equalTo(testProcessStatus.toString()));

    final Date newDate = lastRunSettingVo.getDateValue();
    assertTrue(
        "new date should be after test started",
        newDate.equals(rightBeforeCallingMethod) || newDate.after(rightBeforeCallingMethod));
  }

  private NationalSettingVo prepareStatusExpectations(
      final NationalSettingDomainCapability nationalSettingDc, final NationalSetting testSetting, final UserVo user) {
    final NationalSettingVo statusSetting = new NationalSettingVo();
    statusSetting.setKeyName(testSetting.toString());
    statusSetting.setStringValue(ProcessStatus.COMPLETED.toString());

    // Expected method calls.
    expect(nationalSettingDc.retrieve(statusSetting.getKeyName())).andReturn(statusSetting);
    expect(nationalSettingDc.update(statusSetting, user)).andReturn(statusSetting);
    return statusSetting;
  }

  private NationalSettingVo prepareLastRunExpectations(
      final NationalSettingDomainCapability nationalSettingDc, final NationalSetting testSetting, final UserVo user) {
    final Date initialDate = Date.from(DateTimeUtility.toInstant(LocalDate.of(2016, 3, 9)));
    final NationalSettingVo lastRunSetting = new NationalSettingVo();
    lastRunSetting.setKeyName(testSetting.toString());
    lastRunSetting.setDateValue(initialDate);
    
    // Expected method calls.
    expect(nationalSettingDc.retrieve(lastRunSetting.getKeyName())).andReturn(lastRunSetting);
    expect(nationalSettingDc.update(lastRunSetting, user)).andReturn(lastRunSetting);

    return lastRunSetting;
  }

}
