package gov.va.med.pharmacy.peps.service.common.session.impl;

import static org.easymock.EasyMock.anyObject;
import static org.easymock.EasyMock.anyString;
import static org.easymock.EasyMock.expect;
import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertThat;

import java.io.File;
import java.time.Instant;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.easymock.EasyMockSupport;
import org.hamcrest.CoreMatchers;
import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.ConfigFileUtility;
import gov.va.med.pharmacy.peps.common.utility.DateTimeUtility;
import gov.va.med.pharmacy.peps.common.utility.EnvironmentUtility;
import gov.va.med.pharmacy.peps.common.utility.PpsConfiguration;
import gov.va.med.pharmacy.peps.common.utility.test.generator.CopayTierGenerator;
import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;
import gov.va.med.pharmacy.peps.common.vo.CopayTierVo;
import gov.va.med.pharmacy.peps.common.vo.MigrationCopayTierVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.CopayTierMigrationDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.ItemAuditHistoryDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.EplVuidStatusHistoryDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplCopayTierDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.CopayTierConverter;
import gov.va.med.pharmacy.peps.service.common.migration.MigrationStatus;
import gov.va.med.pharmacy.peps.service.common.update.NdfFileSyncProcessorFactory;

/**
 * Unit tests for {@link CopayTierMigrationServiceImpl}.
 */
public class CopayTierMigrationServiceImplTest extends EasyMockSupport {
  
  /**
   * The directory containing test resources.
   */
  private static final File RESOURCE_DIRECTORY = new File("src/test/resources/copay_tier_migration");

  /* Dependencies - declare here so both setUp() and tests can use them. */
  
  private CopayTierMigrationDomainCapability mockCtmDc;
  private ItemAuditHistoryDomainCapability mockItemAuditHistoryDc;
  private EplVuidStatusHistoryDao mockVshDao;
  private PpsConfiguration ppsConfiguration;
  private NdfFileSyncProcessorFactory mockSyncProcessorFactory;
  private EnvironmentUtility mockEnvironment;
  
  /**
   * Set up each test.
   */
  @Before
  public void setUp() {
    // Construct the mocks here for use in tests.
    mockCtmDc = mock(CopayTierMigrationDomainCapability.class);
    mockItemAuditHistoryDc = mock(ItemAuditHistoryDomainCapability.class);
    mockVshDao = mock(EplVuidStatusHistoryDao.class);
    ppsConfiguration = new ConfigFileUtility();
    mockSyncProcessorFactory = mock(NdfFileSyncProcessorFactory.class);
    mockEnvironment = mock(EnvironmentUtility.class);
  }

  /**
   * Tests for the default {@link MigrationStatus} values when no import has been run.
   */
  @Test
  public void testInitialMigrationStatusFresh() {
    
    final CopayTierMigrationServiceImpl objUnderTest = new CopayTierMigrationServiceImpl(
        mockCtmDc, mockItemAuditHistoryDc, mockVshDao, ppsConfiguration, mockSyncProcessorFactory, mockEnvironment);
    
    final MigrationStatus actualStatus = objUnderTest.getProgress();
    assertThat(actualStatus.getStatus(), CoreMatchers.equalTo(MigrationStatus.INITIAL));
    assertThat(actualStatus.getProgressCount(), CoreMatchers.equalTo(0));
    assertThat(actualStatus.getTotalCount(), CoreMatchers.equalTo(0));
  }
  
  /**
   * Tests for the loaded {@link MigrationStatus} when an import has been run before.
   */
  @Test
  public void testInitialMigrationStatusLoaded() {
    
    /* Setup */
    final int rowsInResultFile = 44;
    
    // Mock the PpsConfiguration so we can override the persistent directory.
    ppsConfiguration = mock(PpsConfiguration.class);
    expect(ppsConfiguration.getPersistentDirectory()).andReturn(RESOURCE_DIRECTORY);
    
    /* Behavior & Verification */
    
    replayAll();
    
    final CopayTierMigrationServiceImpl objUnderTest = new CopayTierMigrationServiceImpl(
        mockCtmDc, mockItemAuditHistoryDc, mockVshDao, ppsConfiguration, mockSyncProcessorFactory, mockEnvironment);

    final MigrationStatus actualStatus = objUnderTest.getProgress();
    assertThat(actualStatus.getStatus(), CoreMatchers.equalTo(MigrationStatus.DONE));
    assertThat(actualStatus.getProgressCount(), CoreMatchers.equalTo(rowsInResultFile));
    assertThat(actualStatus.getTotalCount(), CoreMatchers.equalTo(rowsInResultFile));
  }
  
  private List<EplCopayTierDo> convertToDo(final Collection<CopayTierVo> vos) {
    final ArrayList<EplCopayTierDo> dos = new ArrayList<>(vos.size());
    final CopayTierConverter converter = new CopayTierConverter();
    for (final CopayTierVo vo : vos) {
      dos.add(converter.convert(vo));
    }
    return dos;
  }
  
  private List<EplCopayTierDo> makeCopayTierList() {
    final List<CopayTierVo> copayTierVos = new CopayTierGenerator().generate();
    return convertToDo(copayTierVos);
  }
  
  /**
   * Tests {@link CopayTierMigrationServiceImpl#processCopayTierImport(List, UserVo)} when the product has validation
   * errors.
   */
  @Test
  public void testProcessCopayTierImportValidationError() {
    
    /* Setup */
    // We always need a future effective date, so generate an Instant in the future.
    final Instant twoMonthsFromNow = DateTimeUtility.toInstant(LocalDate.now().plusMonths(2));
    final MigrationCopayTierVo rowForProductWithNoTiers = new MigrationCopayTierVo();
    final List<EplCopayTierDo> copayTierList = makeCopayTierList();
    final int VUID = 1111;
    rowForProductWithNoTiers.setVUID(VUID);
    rowForProductWithNoTiers.setVaProductName("Product with No Tiers");
    rowForProductWithNoTiers.setCopayTierLevel(copayTierList.get(1).getTierLevel());
    rowForProductWithNoTiers.setEffectiveDate(Date.from(twoMonthsFromNow));
    
    final EplProductDo productWithNoTiers = new EplProductDo();
    productWithNoTiers.setVaProductName(rowForProductWithNoTiers.getVaProductName());
    productWithNoTiers.setMasterEntryForVuid("1");
    productWithNoTiers.setOverrideDfDoseChkExclusn("1");
    // Note that we are not setting the generic name, primary drug class code, or dosage form.
    productWithNoTiers.setEplProdCopayTierAssocs(Collections.emptySet());
    
    // Set up mocks
    expect(mockCtmDc.retrieveCopayTiers()).andReturn(copayTierList);
    expect(mockCtmDc.retrieveProduct(rowForProductWithNoTiers.getVUID())).andReturn(productWithNoTiers);
    // This is an ugly mock to match an ugly direct HQL query from the service.
    expect(mockVshDao.executeHqlQuery(anyString(), anyObject())).andReturn(Arrays.asList(1L));
    
    final CopayTierMigrationServiceImpl objUnderTest = new CopayTierMigrationServiceImpl(
        mockCtmDc, mockItemAuditHistoryDc, mockVshDao, ppsConfiguration, mockSyncProcessorFactory, mockEnvironment);
    
    final UserVo user = new UserGenerator().getMigrator();

    replayAll();
    
    /* Behavior */

    objUnderTest.processCopayTierImport(Arrays.asList(rowForProductWithNoTiers), user);
    
    /* Verification */
    
    // Note: relying on processCopayTierImport() modifying the original MigrationCopayTierVo here
    assertThat(rowForProductWithNoTiers.isError(), equalTo(true));
    assertThat(rowForProductWithNoTiers.getErrorMessage().isEmpty(), equalTo(false));
  }
  
  // Note: It would certainly be nice to test a complete import here, but I (David Tombs) ran out of time to implement
  // such a test. Mainly setting up the needed ProductConverter was too involved.

}
