package gov.va.med.pharmacy.peps.service.common.session.impl;

import static org.easymock.EasyMock.anyObject;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.List;

import org.easymock.EasyMockSupport;
import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.PpsConfiguration;
import gov.va.med.pharmacy.peps.common.vo.HazardousWasteImportRow;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.service.common.migration.HazardousWasteMigrationResultFileWriter;
import gov.va.med.pharmacy.peps.service.common.migration.MigrationStatus;

/**
 * Unit tests for HazardousWasteMigrationServiceImpl.
 */
public class HazardousWasteMigrationServiceImplTest extends EasyMockSupport {
    private static final int SAMPLE_TOTAL_ROWS = 50;

    private PpsConfiguration mockConfig;
    private HazardousWasteImportRowValidator mockValidator;
    private HazardousWasteMigrationServiceImpl service;
    private List<HazardousWasteImportRow> rows;
    private HazardousWasteMigrator mockMigrator;
    private HazardousWasteMigrationResultFileWriter mockWriter;
    private UserVo mockUser;

    /**
     * Set up the mocks and other state for each test.
     * 
     * @throws Exception if a setup error occurs.
     */
    @Before
    public void setUp() throws Exception {
        mockConfig = mock(PpsConfiguration.class);
        mockValidator = mock(HazardousWasteImportRowValidator.class);
        mockMigrator = mock(HazardousWasteMigrator.class);
        mockWriter = mock(HazardousWasteMigrationResultFileWriter.class);
        mockUser = mock(UserVo.class);
        service = new HazardousWasteMigrationServiceImpl(mockConfig, mockValidator, mockMigrator, mockWriter);
        rows = new ArrayList<>();
    }

    /**
     * If no import rows are submitted, processing should complete without doing anything.
     */
    @Test
    public void shouldDoNothingWithEmptyList() {
        // Succeeds if no mocks are invoked.
        service.process(rows);
    }

    /**
     * The migration logic should validate each row that is submitted.
     * 
     * @throws MigrationInProgressException if more than one migration is in progress at the same time; should not
     *             be thrown by this test.
     */
    @Test
    public void shouldValidateEachRow() throws MigrationInProgressException {
        HazardousWasteImportRow row1 = new HazardousWasteImportRow("vuid", 123L, "cmopId", 456L, "vaProductName", "genericName",
            "scheduleName", "dosageFormName", "haz2DisposeFlagStr", "haz2HandleFlagStr", "primaryEpa", "wasteSortCode",
            "dotShippingName");
        HazardousWasteImportRow row2 = new HazardousWasteImportRow("vuid", 123L, "cmopId", 456L, "vaProductName", "genericName",
            "scheduleName", "dosageFormName", "haz2DisposeFlagStr", "haz2HandleFlagStr", "primaryEpa", "wasteSortCode",
            "dotShippingName");

        rows.add(row1);
        rows.add(row2);
        expect(mockValidator.validate(row1)).andReturn(row1);
        expect(mockValidator.validate(row2)).andReturn(row2);
        expect(mockMigrator.migrate(row1, mockUser)).andReturn(row1);
        expect(mockMigrator.migrate(row2, mockUser)).andReturn(row2);
        replayAll();

        service.initialize(2, mockUser);
        service.process(rows);
        verifyAll();
    }

    /**
     * Check that all rows which pass validation are migrated, and that rows which do not pass validation are not migrated.
     * 
     * @throws MigrationInProgressException if more than one migration is in progress at the same time; should not
     *             be thrown by this test.
     */
    @Test
    public void shouldMigrateValidRows() throws MigrationInProgressException {
        HazardousWasteImportRow row1 = new HazardousWasteImportRow("vuid", 123L, "cmopId", 456L, "vaProductName", "genericName",
            "scheduleName", "dosageFormName", "haz2DisposeFlagStr", "haz2HandleFlagStr", "primaryEpa", "wasteSortCode",
            "dotShippingName");
        HazardousWasteImportRow row2 = new HazardousWasteImportRow("vuid", 123L, "cmopId", 456L, "vaProductName", "genericName",
            "scheduleName", "dosageFormName", "haz2DisposeFlagStr", "haz2HandleFlagStr", "primaryEpa", "wasteSortCode",
            "dotShippingName");
        row2.addError("Row 2 has a validation error.");
        rows.add(row1);
        rows.add(row2);

        expect(mockValidator.validate(row1)).andReturn(row1);
        expect(mockValidator.validate(row2)).andReturn(row2);
        expect(mockMigrator.migrate(row1, mockUser)).andReturn(row1);
        replayAll();

        service.initialize(2, mockUser);
        service.process(rows);
        verifyAll();
    }

    /**
     * Check that the migration status is properly initialized.
     * 
     * @throws MigrationInProgressException if more than one migration is in progress at the same time; should not
     *             be thrown by this test.
     */
    @Test
    public void shouldInitializeStatus() throws MigrationInProgressException {
        MigrationStatus status = service.getStatus();
        assertEquals(0, status.getTotalCount());
        assertEquals(0, status.getProgressCount());
        assertEquals(MigrationStatus.INITIAL, status.getStatus());

        service.initialize(SAMPLE_TOTAL_ROWS, mockUser);
        status = service.getStatus();
        assertEquals(SAMPLE_TOTAL_ROWS, status.getTotalCount());
        assertEquals(0, status.getProgressCount());
        assertEquals(MigrationStatus.INPROGRESS, status.getStatus());
    }

    /**
     * Check that the migration status is updated as the migration progresses.
     * 
     * @throws MigrationInProgressException if more than one migration is in progress at the same time; should not
     *             be thrown by this test.
     */
    @Test
    public void shouldUpdateStatusWhileMigrating() throws MigrationInProgressException {
        HazardousWasteImportRow row1 = new HazardousWasteImportRow("vuid", 123L, "cmopId", 456L, "vaProductName", "genericName",
            "scheduleName", "dosageFormName", "haz2DisposeFlagStr", "haz2HandleFlagStr", "primaryEpa", "wasteSortCode",
            "dotShippingName");
        HazardousWasteImportRow row2 = new HazardousWasteImportRow("vuid", 123L, "cmopId", 456L, "vaProductName", "genericName",
            "scheduleName", "dosageFormName", "haz2DisposeFlagStr", "haz2HandleFlagStr", "primaryEpa", "wasteSortCode",
            "dotShippingName");
        row2.addError("Row 2 has a validation error.");
        rows.add(row1);
        rows.add(row2);

        expect(mockValidator.validate(row1)).andReturn(row1);
        expect(mockValidator.validate(row2)).andReturn(row2);
        expect(mockMigrator.migrate(row1, mockUser)).andReturn(row1);
        expect(mockConfig.getPersistentDirectory()).andReturn(null);
        mockWriter.write(anyObject(), anyObject(),anyObject());
        expectLastCall();
        replayAll();

        service.initialize(2, mockUser);
        List<HazardousWasteImportRow> resultRows = service.process(rows);
        service.writeResultFile(resultRows, "Hazardous Waste Data");
        verifyAll();

        MigrationStatus status = service.getStatus();
        assertEquals(2, status.getTotalCount());
        assertEquals(2, status.getProgressCount());
        assertEquals(MigrationStatus.DONE, status.getStatus());
    }
}
