package gov.va.med.pharmacy.peps.service.national.updatefile;

import static gov.va.med.pharmacy.peps.service.national.updatefile.UpdateFileReportFormatter.doubleIndent;
import static gov.va.med.pharmacy.peps.service.national.updatefile.UpdateFileReportFormatter.formatProductReference;
import static org.hamcrest.CoreMatchers.equalTo;
import static org.easymock.EasyMock.*;
import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.collections4.CollectionUtils;
import org.easymock.EasyMockSupport;
import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.domain.common.model.EplMatchTypeDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplRematchDo;
import gov.va.med.pharmacy.peps.domain.common.model.dto.ProductChangeDto;

/**
 * Automated unit tests for {@link RematchReportGenerator}.
 */
public class RematchReportGeneratorTest extends EasyMockSupport {

    private static final EplMatchTypeDo INACTIVATION_MATCH_TYPE = 
        new EplMatchTypeDo(MatchReason.INACTIVATION.getId(), "inactivation", "inactivation");
    /**
     * Mock stored as a member variable for convenient creation in {@link #setup()}.
     */
    private UpdateFileReportGenerator mockUpdateFileReportGenerator;
    
    /**
     * Instantiates all the mock objects for convenience.
     */
    @Before
    public void setup() {
        mockUpdateFileReportGenerator = mock(UpdateFileReportGenerator.class);
    }

    /**
     * Test all rematches in the same test in order to verify that rematches are placed in the proper sections.
     * eg rematches with associated product inactivations should not appear in the rematch only report.
     */
    @Test
    public void testRematchReport() {
        /* Setup */
        final String noInactivationRematchHeader = "Inactivated No Rematch Header";
        final String autoInactivationRematchHeader = "Inactivated Automatic Rematch Header";
        final String suggInactivationRematchHeader = "Inactivated Suggested Rematch Header";
        final String noRematchHeader = "Changed No Rematch Header";
        final String autoRematchHeader = "Changed Automatic Rematch Header";
        final String suggRematchHeader = "Changed Suggested Rematch Header";
        
        
        // Build the rematches
        final EplRematchDo autoRematchOnly = buildAutoRematchMutator(1l);
        final EplRematchDo suggRematchOnly = buildSuggestedRematchMutator(2l);
        final EplRematchDo autoRematchInactivation = buildAutoRematchMutator(3l);
        final EplRematchDo suggRematchInactivation = buildSuggestedRematchMutator(4l);
        final EplRematchDo autoRematchOther = buildAutoRematchMutator(5l);
        final EplRematchDo suggRematchOther = buildSuggestedRematchMutator(6l);
        
        final List<EplRematchDo> outgoingRematches = Arrays.asList(
            autoRematchOnly, suggRematchOnly,
            autoRematchInactivation, suggRematchInactivation,
            autoRematchOther, suggRematchOther);
        
        final EplProductDo fromRematchOnlyAutoProduct = autoRematchOnly.getFromProduct();
        final EplProductDo fromRematchOnlySuggProduct = suggRematchOnly.getFromProduct();
        
        final EplProductDo fromInactivationSuggProduct = suggRematchInactivation.getFromProduct();
        final EplProductDo fromInactivationAutoProduct = autoRematchInactivation.getFromProduct();
        final List<ProductChangeDto> inactivatedProducts = Arrays.asList(
            new ProductChangeDto(1L, "Product A", "CmopA", "3160302", null),
            new ProductChangeDto(fromInactivationSuggProduct.getEplId(), fromInactivationSuggProduct.getVaProductName(),
                fromInactivationSuggProduct.getCmopId(), "3170204", null),
            new ProductChangeDto(fromInactivationAutoProduct.getEplId(), fromInactivationAutoProduct.getVaProductName(),
                fromInactivationAutoProduct.getCmopId(), "3161225", null));
        
        final EplProductDo fromOtherSuggProduct = suggRematchOther.getFromProduct();
        final EplProductDo fromOtherAutoProduct = autoRematchOther.getFromProduct();
        final ProductChangeDto noRematchChange =
            new ProductChangeDto(1L, "No Rematch Product", "NoRemId", "FLUCONAZ", "BACLOFEN");
        final Set<ProductChangeDto> changedProducts = new HashSet<>(Arrays.asList(
            noRematchChange,
            new ProductChangeDto(fromOtherAutoProduct.getEplId(), fromOtherAutoProduct.getVaProductName(),
                fromOtherAutoProduct.getCmopId(), "6", "7"),
            new ProductChangeDto(fromOtherSuggProduct.getEplId(), fromOtherSuggProduct.getVaProductName(),
                fromOtherSuggProduct.getCmopId(), "NEVER", "ALWAYS")));
        
        final List<String> expectedInactivationReport = Arrays.asList(
            autoInactivationRematchHeader,
            formatProductReference(fromInactivationAutoProduct),
            doubleIndent(RematchReportGenerator.USE),
            formatProductReference(autoRematchInactivation.getToAutoProduct()),
            "",
            suggInactivationRematchHeader,
            formatProductReference(fromInactivationSuggProduct),
            doubleIndent(RematchReportGenerator.USE),
            formatProductReference(CollectionUtils.get(suggRematchInactivation.getToSuggestedProducts(), 0)),
            "",
            noInactivationRematchHeader,
            formatProductReference(inactivatedProducts.get(0)),
            "");
        
        final List<String> expectedOtherReport = Arrays.asList(
            autoRematchHeader,
            formatProductReference(fromRematchOnlyAutoProduct),
            doubleIndent(RematchReportGenerator.USE),
            formatProductReference(autoRematchOnly.getToAutoProduct()),
            "",
            formatProductReference(fromOtherAutoProduct),
            doubleIndent(RematchReportGenerator.USE),
            formatProductReference(autoRematchOther.getToAutoProduct()),
            "",
            suggRematchHeader,
            formatProductReference(fromRematchOnlySuggProduct),
            doubleIndent(RematchReportGenerator.USE),
            formatProductReference(CollectionUtils.get(suggRematchOnly.getToSuggestedProducts(), 0)),
            "",
            formatProductReference(fromOtherSuggProduct),
            doubleIndent(RematchReportGenerator.USE),
            formatProductReference(CollectionUtils.get(suggRematchOther.getToSuggestedProducts(), 0)),
            "",
            noRematchHeader,
            formatProductReference(noRematchChange),
            "");
        
        // Expected method calls.
        expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.OTHERREMATCHED))
            .andReturn(Arrays.asList(autoRematchHeader));
        expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.OTHERREMATCHSUGG))
            .andReturn(Arrays.asList(suggRematchHeader));
        expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.OTHERNOREMATCH))
            .andReturn(Arrays.asList(noRematchHeader));
        expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.INACTIVATEDREMATCHED))
            .andReturn(Arrays.asList(autoInactivationRematchHeader));
        expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.INACTIVATEDWSUGG))
            .andReturn(Arrays.asList(suggInactivationRematchHeader));
        expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.INACTIVATEDPRODUCT))
            .andReturn(Arrays.asList(noInactivationRematchHeader));
        
        /* Behavior & Verification */
        replayAll();
        
        final RematchReportGenerator objUnderTest = new RematchReportGenerator(
            mockUpdateFileReportGenerator, outgoingRematches, changedProducts, inactivatedProducts);
        
        assertThat("Inactivation Rematch",
            objUnderTest.inactivatedProductsReport(),
            equalTo(expectedInactivationReport));
        
        assertThat("Other Rematch",
            objUnderTest.otherRematchesReport(),
            equalTo(expectedOtherReport));
        
        verifyAll();
        
    }
    
    /**
     * Builds a sample automatic (@link EplRematchDo} for the above tests using number as mutator temporarily.
     * @return a new instance every time
     */
    
    private EplRematchDo buildAutoRematchMutator(Long mutator) {
        final EplProductDo fromAutoProduct = new EplProductDo();
        fromAutoProduct.setEplId(100L + mutator);
        fromAutoProduct.setVaProductName("Auto Rematch From Product Name" + mutator);
        fromAutoProduct.setCmopId("AutoCmop" + mutator);
        final EplProductDo toAutoProduct = new EplProductDo();
        toAutoProduct.setEplId(200L + mutator);
        toAutoProduct.setVaProductName("Auto Rematch To Product Name" + mutator);
        toAutoProduct.setCmopId("AutoCmop1" + mutator);

        final EplRematchDo rematch = new EplRematchDo();
        rematch.setEplRematchId(fromAutoProduct.getEplId());
        rematch.setMatchType(INACTIVATION_MATCH_TYPE);
        rematch.setFromProduct(fromAutoProduct);
        rematch.setToAutoProduct(toAutoProduct);
        return rematch;
    }
    
    /**
     * Builds a sample suggested {@link EplRematchDo} for the above tests using number as mutator temporarily.
     * @return a new instance every time
     */
    private EplRematchDo buildSuggestedRematchMutator(Long mutator) {
        final EplProductDo fromSuggProduct = new EplProductDo();
        fromSuggProduct.setEplId(300L + mutator);
        fromSuggProduct.setVaProductName("Suggested Rematch From Product Name" + mutator);
        fromSuggProduct.setCmopId("SuggCmop" + mutator);
        final EplProductDo toSuggProduct1 = new EplProductDo();
        toSuggProduct1.setVaProductName("Suggested Rematch To Product 1" + mutator);
        toSuggProduct1.setCmopId("SuggCmop1" + mutator);

        final EplRematchDo suggRematch = new EplRematchDo();
        suggRematch.setEplRematchId(fromSuggProduct.getEplId());
        suggRematch.setMatchType(INACTIVATION_MATCH_TYPE);
        suggRematch.setFromProduct(fromSuggProduct);
        suggRematch.setToSuggestedProducts(new HashSet<>(Arrays.asList(toSuggProduct1)));
        return suggRematch;
    }

}
