package gov.va.med.pharmacy.peps.presentation.common.auth;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.exception.AuthenticationException;
import gov.va.med.pharmacy.peps.common.utility.security.UntrustedLogParam;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.presentation.common.context.UserContext;
import gov.va.med.pharmacy.peps.presentation.common.controller.UserController;
import gov.va.med.pharmacy.peps.presentation.common.spring.interceptor.PepsSpringInterceptor;
import gov.va.med.pharmacy.peps.service.common.session.UserService;

/**
 * Bypass version of LoginSpringInterceptor that just sets the currently authenticated user to a user-supplied value.
 * This class should <strong>never</strong> make it into an official build!
 */
public class LoginSpringInterceptor extends PepsSpringInterceptor {
    
    /**
     * The HTTP Request parameter to specify which VAUID to login as.
     */
    private static final String VAUID_PARAMETER = "vauid";

    private static final Logger LOG = LogManager.getLogger(LoginSpringInterceptor.class);
    
    private final UserService userService;
    
    /**
     * Constructs an instance with the required dependencies.
     * @param pUserService The {@link UserService} to use.
     */
    @Autowired
    public LoginSpringInterceptor(final UserService pUserService) {
        userService = pUserService;
    }
    
    /**
     * If the user needs login, check the request for {@link #VAUID_PARAMETER} and set the current user to the one with
     * the specified VAUID.
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
        throws IOException, AuthenticationException {
        
        LOG.debug("Request URI is: {}", new UntrustedLogParam(request.getRequestURI()));
        
        if (needsLogin(request)) {
            
            LOG.warn("Attempting bypass authentication.");
            
            final String passedVauidString = request.getParameter(VAUID_PARAMETER);
            if (StringUtils.isEmpty(passedVauidString)) {
                response.getWriter().write(
                    "No VAUID parameter passed. Please add ?" + VAUID_PARAMETER + "=[vauid] at the end of the URL.");
                return false;
            }
            final long passedVauid = Long.parseLong(passedVauidString);

            final UserVo user = toUser(passedVauid);
            
            final HttpSession session = request.getSession();
            UserContext userContext = getUserContext(session);
            userContext.setUser(user);
            
            //Set the user attribute on the session so the controller can use it
            setUserContext(request, session, userContext);
            session.setAttribute(PpsAuthentication.USER_ATTRIBUTE, user);
            
            // Warn the user that we're using bypass authentication.
            response.getWriter().print(
                    "WARNING: Using bypass authentication! Username " + user.getUsername() + ". Refresh the page to proceed.");
            return false;
        }

        if (request.getSession().getAttribute(PpsAuthentication.USER_ATTRIBUTE) == null) {
            LOG.debug("No user redirecting to index page");
            response.sendRedirect(request.getContextPath() + UserController.INITIAL_LOGIN_URL);
            return false;
        }

        return true;
    }
    
    private boolean needsLogin(final HttpServletRequest request) {
        return request.getSession().getAttribute(PpsAuthentication.USER_ATTRIBUTE) == null
                && request.getRequestURI().endsWith(UserController.INITIAL_LOGIN_URL);
    }

    /**
     * Loads the user with the given VAUID and userName from the database.
     */
    private UserVo toUser(long vauid) throws AuthenticationException {

        UserVo user = userService.retrieve(vauid);

        if (user == null) {
            throw new UnsupportedOperationException(LoginSpringInterceptor.class + " does not support creating new users.");
        } else {
            LOG.debug("User Exists :: userName = {}, VAUID = {}", user.getUsername(), user.getVauid());
            
            addRole(user, Role.PSS_PPSN_VIEWER);

            if (user.getRolePnmApvdFlag()) {
                addRole(user, Role.PSS_PPSN_MANAGER);
            }

            if (user.getRolePnsApvdFlag()) {
                addRole(user, Role.PSS_PPSN_SUPERVISOR);
            }

            if (user.getRolePsrApvdFlag()) {
                addRole(user, Role.PSS_PPSN_SECOND_APPROVER);
            }

            if (user.getRoleMigApvdFlag()) {
                addRole(user, Role.PSS_PPSN_MIGRATOR);
            }
        }

        return user;
    }

    /**
     * Adds the role to the user and logs it.
     */
    private void addRole(final UserVo user, final Role role) {
        LOG.debug("Adding role {} to user.", role);
        user.addRole(role);
    }
}
