package gov.va.med.pharmacy.peps.presentation.common.auth;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidationType;
import gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator;

/**
 * Foundational constants and methods related to authentication in PPS.
 */
public final class PpsAuthentication {
    
    private static final Logger LOG = LogManager.getLogger(PpsAuthentication.class);

    /**
     * The session attribute key for the UserVo. This session attribute seems redundant with {@link
     * UserContext#getUser()} and should probably be removed.
     */
    public static final String USER_ATTRIBUTE = "user";
    
    /**
     * <p>
     * Returns the URL to which the user should be directed upon logout from PPS. The returned URL is provided by the SSOi
     * agent, but this method validates that the returned URL is an HTTPS URL in the DNS    domain.
     * </p>
     * 
     * <p>
     * The validation is done to prevent Open Redirect security holes. Allowing anything in DNS    is pretty broad, but
     * appropriate as we highly trust the SSOi headers. Only the SSOi agent can set them, and we trust the agent to authenticate
     * the user. This basic DNS    validation simply provides some defense-in-depth.
     * </p>
     * 
     * @param servletRequest the HTTP Servlet request. Necessary to read headers set by the SSOi agent.
     * @return the URL as a string. Possibly null or empty if no target URL could be determined.
     */
    public static String getLogoutTargetUrl(final HttpServletRequest servletRequest) {
        // Try to send the user to the SSOi Landing page first.
        try {
            final String ssoiLandingUrl = getValidatedUrlFromSsoi(servletRequest, SsoiHeader.SSOI_LANDING_URL);
            if (StringUtils.isNotEmpty(ssoiLandingUrl)) {
                return ssoiLandingUrl;
            }
        } catch (final Exception ex) {
            LOG.warn("Could not get validated landing page URL, falling back to log out URL.", ex);
        }
        
        // As a fallback, send the user to the SSOi Log out page.
        return getValidatedUrlFromSsoi(servletRequest, SsoiHeader.SSOI_LOGGEDOUT_URL);
    }
    
    private static String getValidatedUrlFromSsoi(final HttpServletRequest servletRequest, final SsoiHeader header) {
        return ESAPIValidator.validateStringInput(
            servletRequest.getHeader(header.getHeaderName()), ESAPIValidationType.VA_HTTPS_URL);
    }

    /**
     * Prevent instantiation.
     */
    private PpsAuthentication() {
    };
}
