package gov.va.med.pharmacy.peps.presentation.common.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import gov.va.med.pharmacy.peps.common.exception.AuthenticationException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.vo.EditSiteListVo;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.presentation.common.controller.item.AbstractManageItemController;
import gov.va.med.pharmacy.peps.presentation.common.spring.annotation.RoleNeeded;
import gov.va.med.pharmacy.peps.service.common.session.SiteListService;

/**
 * Handles the requests from the siteList.jsp front page.
 * @author DNS   RUSTRA
 *
 */
@Controller("siteListsController")
public class SiteListsController extends AbstractManageItemController {

    private static final String EDIT_SITE_LIST_INSTANCE = "editSiteListInstance";

    private static final String LIST_TITLE = "listTitle";

    /**
     * 
     */
    @Autowired
    private SiteListService siteListService;

    /** InitBinder.
     * @param binder WebDataBinder instance
     */
    @InitBinder
    protected void initBinderDTO(WebDataBinder binder) {
        binder.setAllowedFields(
            "sitesExcludedList.vistaSiteVo_*",
            "sitesIncludedList.vistaSiteVo_*"
            );
    }

    /**
     * Add EditSiteList along with the lists of sites to include and exclude to the model.
     * This method guarantees that this information will be added to the model first, 
     * so that it is available when modifying the list of sites.
     * 
     * @param model Spring model object
     */    
    @ModelAttribute()
    public void populateModel(Model model) {
        model.addAttribute(LIST_TITLE, "Edit Site List");
        EditSiteListVo editSiteList = getEditSiteList();
        editSiteList.setSitesExcludedList(siteListService.getExcludedComplianceSites());
        editSiteList.setSitesIncludedList(siteListService.getIncludedComplianceSites());
        model.addAttribute(EDIT_SITE_LIST_INSTANCE, editSiteList);
    }

    /**
     * A get to display site lists.
     * 
     * @param model Model instance
     * @return String instance
     * @throws AuthenticationException exception
     */
    @RequestMapping(value = "/editSiteList.go", method = RequestMethod.GET)
    @RoleNeeded(roles = { Role.PSS_PPSN_SUPERVISOR })
    public String displaySiteLists(Model model) throws AuthenticationException {

        pageTrail.clearTrail();
        pageTrail.addPage("editSiteList", "Edit Site List", true);

        return "edit.site.list";
    }

    /**
     * Updates the site lists.
     * 
     * @param editSiteList EditSiteListVo instance
     * @return String instance
     * @throws ValidationException exception
     */
    @RequestMapping(value = "/editSiteList.go", method = RequestMethod.POST)
    @RoleNeeded(roles = { Role.PSS_PPSN_SUPERVISOR })
    public String submitSiteLists(@ModelAttribute(EDIT_SITE_LIST_INSTANCE) EditSiteListVo editSiteList)
        throws ValidationException {

        EditSiteListVo newStiteList = getEditSiteList();
        if (newStiteList != null) {
            getUser();
            // We set vaDataFields to null to avoid the framework
            // to look for this fields when getting the values from the request.
            newStiteList.setVaDataFields(null);
            bindAndUpdateSpecialHandling(newStiteList);

            siteListService.saveExcludedSites(newStiteList.getSitesExcludedList(), getUser());
            siteListService.saveIncludedSites(newStiteList.getSitesIncludedList(), getUser());
        }
        
        return REDIRECT + "editSiteList.go";
    }

    /**
     * Gets the current EditSiteListVo instance.
     * @return EditSiteListVo instance
     */
    private EditSiteListVo getEditSiteList() {
        return flowScope.get(EditSiteListVo.class);
    }


}
