/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.presentation.common.controller.item;


import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.exception.ValueObjectValidationException;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.ModificationSummaryVo;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.common.vo.validator.ErrorKey;
import gov.va.med.pharmacy.peps.common.vo.validator.Errors;
import gov.va.med.pharmacy.peps.common.vo.validator.ValidationError;
import gov.va.med.pharmacy.peps.presentation.common.controller.ControllerConstants;
import gov.va.med.pharmacy.peps.presentation.common.controller.PpsUrls;
import gov.va.med.pharmacy.peps.presentation.common.controller.bean.EditManagedItemBean;
import gov.va.med.pharmacy.peps.presentation.common.controller.bean.MultipleSelectItemBean;
import gov.va.med.pharmacy.peps.presentation.common.pagetrail.FlowScope;
import gov.va.med.pharmacy.peps.presentation.common.spring.annotation.RoleNeeded;
import gov.va.med.pharmacy.peps.presentation.common.utility.DomainUtility;
import gov.va.med.pharmacy.peps.presentation.common.utility.JspTagUtility;


/**
 * Controller for editing {@link ManagedItemVo}s.
 */
@Controller
@RequestMapping("{" + ControllerConstants.ENTITY_TYPE + "}/{" + ControllerConstants.ITEM_ID + "}")
public class ManageItemController extends AbstractManageItemController {

    private static final String MULTI_EDIT_BEAN = "multiEditBean";

    private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(ManageItemController.class);
    
    @Autowired
    private DomainUtility domainUtility;

    @Autowired
    private AddItemController addItemController;
    
    @Autowired
    private Errors errors;

    /**
     * Configures the binder for multiEditBean.
     * @param binder the binder to configure.
     */
    @InitBinder(MULTI_EDIT_BEAN)
    protected void initBinderMultiEditBean(WebDataBinder binder) {
        binder.setAllowedFields(
            "itemIds",
            "requestIds",
            "itemEntityTypes");
    }

    /**
     * Presents the edit page for the specified item.
     *
     * @param entityType The {@link EntityType} of the item to edit.
     * @param itemId The ID of the item to edit.
     * @param tab to display
     * @param model Model
     * @return String
     * @throws ItemNotFoundException exception 
     */
    @RequestMapping(value = PpsUrls.PAGE_EDIT, method = RequestMethod.GET)
    public String edit(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @RequestParam(value = ControllerConstants.TAB_KEY, required = false) String tab,
        Model model) throws ItemNotFoundException {


        updateMultiEditIndex(itemId);

        String tabKey;

        if (StringUtils.isEmpty(tab)) {
            tabKey = getTabKey(entityType);
        } else {
            tabKey = tab;
        }

        pageTrail.addPage("editItem", "Edit " + entityType.getName(), true);

        EditManagedItemBean editManageItemBean = getEditManagedItemBean();

        if (itemChanged(editManageItemBean, entityType, itemId)) {
            
            LOG.debug("Requested item changed to {} {}.", entityType, itemId);

            final ManagedItemVo item = getManagedItemService().retrieve(itemId, entityType, getUser());

            String redirectToRequest = redirectToRequest(entityType, itemId, tabKey);

            if (!redirectToRequest.isEmpty()) {
                return redirectToRequest;
            }

            final ManagedItemVo originalItem = item.copy();

            editManageItemBean = flowScope.getNew(EditManagedItemBean.class);

            editManageItemBean.setItem(item);
            editManageItemBean.setOriginalItem(originalItem);
        }

       
        // if this was a partial save, set it reset it
        editManageItemBean.getItem().setPartialSave(false);

        return presentEditView(model, tabKey, editManageItemBean);
    }

    /**
     * Post from edit screen. Saves any user inputed values if they are changing tabs
     *
     * @param entityType EntityType
     * @param itemId String
     * @param tab to display
     * @return String
     * @throws ValidationException ValidationException
     */
    @RequestMapping(value = PpsUrls.PAGE_EDIT, method = RequestMethod.POST)
    public String editPostTabChange(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @RequestParam(value = ControllerConstants.POST_TAB_KEY, required = false) String tab)
        throws ValidationException {

        EditManagedItemBean editManageItemBean = getEditManagedItemBean();
        ManagedItemVo item = editManageItemBean.getItem();

        // If the item isn't null, save any changes the user has made
        if (item != null) {
            bindAndUpdateSpecialHandling(item);
        }

        return ControllerConstants.REDIRECT_SLASH
            + PpsUrls.getItemEditUrl(entityType, itemId) + "?" + ControllerConstants.TAB_KEY + "=" + tab;
    }
    
    /**
     * Triggers a manual rematch for the given item.
     * 
     * @param entityType The {@link EntityType} of the item to edit.
     * @param itemId The ID of the item to edit.
     * @return the view to present
     */
    @RequestMapping(value = "/manualRematch.go", method = RequestMethod.POST)
    public String triggerManualRematch(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId) {

        final EditManagedItemBean editManagedItemBean = getEditManagedItemBean();
        
        // Redirect to the item edit page if the user is requesting a new item. It would probably be better to just
        // trigger a rematch for the given item here, but no time to code that.
        if (itemChanged(editManagedItemBean, entityType, itemId)) {
            return ControllerConstants.REDIRECT_SLASH + PpsUrls.getItemEditUrl(entityType, itemId);
        }
        
        // If the user has made any edits to the item, discard them: manual rematches are not submitted with item
        // changes.
        editManagedItemBean.resetItem();
        
        return ControllerConstants.REDIRECT_SLASH + PpsUrls.getItemUrl(entityType, itemId, PpsUrls.PAGE_REMATCH);
    }

    /**
     * Starts a multi-item-edit flow.
     *
     * @param multipleSelectItemBean specifies the items to edit
     * @return the name of the view to present to the user
     * @throws ValueObjectValidationException 
     * @throws ItemNotFoundException 
     */
    public String startMultiItemEditFlow(MultipleSelectItemBean multipleSelectItemBean) 
        throws ValueObjectValidationException, ItemNotFoundException {
        flowInputScope.put("multipleSelectItemBean", multipleSelectItemBean);
        multipleSelectItemBean.setCurrentIndex(0);
        
        //check the selected items to see if any are pending - if so, return an error message
        for (int x = 0; x < multipleSelectItemBean.getItemIds().length; x++) {
            ManagedItemVo item = getManagedItemService().retrieve(multipleSelectItemBean.getItemIds()[x], 
                multipleSelectItemBean.getItemEntityTypes()[x], getUser());
            RequestItemStatus itemStatus = item.getRequestItemStatus();
                
            if ((itemStatus.equals(RequestItemStatus.PENDING)) 
                || (requestService.retrieveNonCompletedRequest(item.getId(), item.getEntityType()) != null)) {
                errors.addError(new ValidationError(ErrorKey.PENDING_MULTIEDIT, item.getValue()));
            }                
        }
        
        if (errors.hasErrors()) {
            throw new ValueObjectValidationException(errors);
        }
        
        return ControllerConstants.REDIRECT_SLASH
            + PpsUrls.getItemEditUrl(multipleSelectItemBean.getItemEntityTypes()[0], multipleSelectItemBean.getItemIds()[0]);
    }

    /**
     * submitModifications.
     *
     * @param entityType EntityType
     * @param itemId String
     * @param tab String
     * @param requestedLocale the requested locale
     * @return String
     * @throws ValidationException ValidationException
     */
    @RequestMapping(value = "/submitModifications.go", method = RequestMethod.POST)
    public String submitModifications(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @RequestParam(value = ControllerConstants.TAB_KEY, required = false) String tab,
        final Locale requestedLocale)
        throws ValidationException {

        EditManagedItemBean editManageItemBean = getEditManagedItemBean();

        if (itemChanged(editManageItemBean, entityType, itemId)) {
            return ControllerConstants.REDIRECT_SLASH + PpsUrls.getItemEditUrl(entityType, itemId);
        }

        final ManagedItemVo item = editManageItemBean.getItem();

        // build the modified item
        bindAndUpdateSpecialHandling(item);

        item.validate(getUser(), environmentUtility.getEnvironment());

        // get the summary of changes between the original item and the modified one
        ModificationSummaryVo modificationSummary =
            getManagedItemService().submitModifications(editManageItemBean.getOriginalItem(), item, getUser());
        editManageItemBean.setDifferences(modificationSummary.getModDifferences());
        
         // Make the special handling update
        
        flashScope.put(ControllerConstants.WARNINGS, modificationSummary.getWarnings().getLocalizedErrors(requestedLocale));
        
        // Redirect to rematch if rematch is triggered.
        if (modificationSummary.shouldTriggerRematch()) {
            return ControllerConstants.REDIRECT_SLASH + PpsUrls.getItemUrl(entityType, itemId, PpsUrls.PAGE_REMATCH) + "?"
                + ControllerConstants.TAB_KEY + "=" + tab;
        }
        
        // redirect to the modification summary page
        return ControllerConstants.REDIRECT_SLASH + entityType.toString().toLowerCase() + "/" + itemId
            + "/modificationSummary.go?"
            + ControllerConstants.TAB_KEY + "=" + tab;
    }

    /**
     * Cancel edit and return to the last page in the previous flow or the search page.
     *
     * @param entityType the EntityType of the item to commit
     * @param itemId the id of the item to commit
     * @return the last page in the previous flow or the search page
     */
    @RequestMapping(value = "/editCancel.go", method = RequestMethod.POST)
    public String cancelEdit(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId) {

        flowScope.getNew(EditManagedItemBean.class);

        return REDIRECT + pageTrail.goToPreviousUrl("/searchItems.go");
    }

    /**
     * startPartialSave
     *
     * @param entityType EntityType
     * @param itemId String
     * @param tab String
     * @return String 
     * @throws ValidationException exception
     */
    @RequestMapping("/startPartialSave.go")
    public String startPartialSave(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @RequestParam(value = ControllerConstants.TAB_KEY, required = false) String tab) throws ValidationException {

        EditManagedItemBean editManageItemBean = getEditManagedItemBean();

        if (itemChanged(editManageItemBean, entityType, itemId)) {
            return ControllerConstants.REDIRECT_SLASH + PpsUrls.getItemEditUrl(entityType, itemId);
        }

        bindAndUpdateSpecialHandling(editManageItemBean.getItem());

        flowInputScope.put(editManageItemBean);

        // redirect to the modification summary page
        return REDIRECT + "/" + entityType.toString().toLowerCase() + "/" + itemId + "/editPartialSave.go?"
            + ControllerConstants.TAB_KEY + "=" + tab;

    }

    /**
     * 
     * Prepare to forward to the add managed domain screen from the edit screen.
     *
     * @param entityType The entityType being added.
     * @param formElementName formElementName
     * @param model model
     * @return add managed domain view
     * @throws ValidationException ValidationException
     */
    @RoleNeeded(roles = { Role.PSS_PPSN_MANAGER, Role.PSS_PPSN_SUPERVISOR })
    @RequestMapping(value = "edit/prepareAddManagedDomain.go", method = RequestMethod.POST)
    public String prepareForAddManagedDomain(
        @RequestParam(value = ControllerConstants.DOMAIN_ENTITY_TYPE) EntityType entityType,
        @RequestParam(value = ControllerConstants.FORM_ELEMENT_NAME, required = true) String formElementName,
        Model model) throws ValidationException {

        return prepForAddManagedDomain(entityType, formElementName);
    }

    /**
     * Helper for prepareForManagedDomain().
     *
     * @param entityType of domain
     * @param formElementName of domain
     * @return add managed domain view
     * @throws ValidationException ValidationException
     */
    public String prepForAddManagedDomain(EntityType entityType, String formElementName)
        throws ValidationException {
        EditManagedItemBean editBean = getEditManagedItemBean();
        
        Map<String, Object> keyMap = JspTagUtility.getParentAndChildFieldKeys(formElementName);
        FieldKey<?> fieldKey = (FieldKey<?>)keyMap.get(JspTagUtility.PARENT_FIELD_KEY);
        
        if (fieldKey == null) {
            fieldKey = (FieldKey<?>)keyMap.get(JspTagUtility.FIELD_KEY);
        }
        
        String key = fieldKey.fromDotsToCamelCase();
        
        flowScope.put(key + "_" + ControllerConstants.PLUS_BUTTON_INDEX, (Integer) keyMap.get(JspTagUtility.INDEX));

        bindAndUpdateSpecialHandling(editBean.getItem());

        flowInputScope.put(ControllerConstants.ITEM_KEY, editBean.getItem());
        flowInputScope.put(key + "_" + ControllerConstants.PLUS_BUTTON_INDEX, 
            flowScope.get(key + "_" + ControllerConstants.PLUS_BUTTON_INDEX));

        return addItemController.getAddManagedDomainRedirectUrl(entityType, formElementName);
    }

    /**
     * startMoveChildren
     *
     * @param entityType EntityType
     * @param itemId String
     * @param multiEditBean MultipleSelectItemBean
     * @return String
     * @throws ValidationException ValidationException
     */
    @RequestMapping(value = "/startMoveChildren.go", method = RequestMethod.POST)
    public String startMoveChildren(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @ModelAttribute(MULTI_EDIT_BEAN) MultipleSelectItemBean multiEditBean) throws ValidationException {

        flowInputScope.put(MULTI_EDIT_BEAN, multiEditBean);
        EditManagedItemBean editBean = getEditManagedItemBean();
        bindAndUpdateSpecialHandling(editBean.getItem());

        flowInputScope.put(editBean);

        return REDIRECT + "/searchNewParent.go?entityType="
            + multiEditBean.getItemEntityTypes()[0].getParent().toLowerCase();

    }

    /**
     * startCopyChildren
     *
     * @param entityType EntityType
     * @param itemId String
     * @param multiEditBean MultipleSelectItemBean
     * @return String
     * @throws ValidationException ValidationException
     */
    @RequestMapping(value = "/startCopyChildren.go", method = RequestMethod.POST)
    public String startCopyChildren(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @ModelAttribute(MULTI_EDIT_BEAN) MultipleSelectItemBean multiEditBean) throws ValidationException {
        
        //add the selected NDCs to the url
        flowInputScope.put(MULTI_SEL_ITEM_BEAN, multiEditBean);
        
//        String ndcString = "";
//        int ndcCount = multiEditBean.getItemIds().length;

//        for (int a=0; a < ndcCount; a++) {
//            ndcString = ndcString + "&ndc" + a + "=" + multiEditBean.getItemIds()[a].toString();
//        }
      
        //add the current product to flowScope
        EditManagedItemBean editBean = getEditManagedItemBean();
        bindAndUpdateSpecialHandling(editBean.getItem());

        return REDIRECT + "/product/" + itemId + "/openTemplate/add.go?entityType=" + editBean.getItem().getEntityType();
        
//            + multiEditBean.getItemEntityTypes()[0].getParent().toLowerCase() + ndcString + "&ndcCount=" + ndcCount;

    }


    /**
     * 
     * Rescinds the rejection on an item.
     *
     * @param entityType of item
     * @param itemId of item
     * @return the user to the previous URL, which is probably the search
     * @throws ValidationException 
     */
    @RequestMapping(value = "/rescindRejection.go", method = RequestMethod.POST)
    public String rescindRejection(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId) throws ValidationException {

        final EditManagedItemBean editManageItemBean = getEditManagedItemBean();

        if (itemChanged(editManageItemBean, entityType, itemId)) {
            return ControllerConstants.REDIRECT_SLASH + PpsUrls.getItemEditUrl(entityType, itemId);
        }

        getManagedItemService().commitRescindRejection(editManageItemBean.getItem(), getUser());

        return REDIRECT + pageTrail.goToPreviousFlowUrl();
    }

    /**
     * Returns the current tab key.
     *
     * @param entityType the EntityType
     * @return the String tab key
     */
    protected String getTabKey(EntityType entityType) {
        String tabKey = request.getParameter(ControllerConstants.TAB_KEY);

        if (StringUtils.isEmpty(tabKey)) {
            tabKey = ControllerConstants.DEFAULT_TAB.get(entityType);

            if (tabKey == null && entityType.isDomainType()) {
                tabKey = ControllerConstants.ALPHA_TAB;
            }
        }

        return tabKey;
    }

    /**
     * Get the domainUtility.
     * 
     * @return the domainUtility
     */
    @Override
    public DomainUtility getDomainUtility() {
        return domainUtility;
    }

    /**
     * Sets domainUtility field.
     * 
     * @param domainUtility the domainUtility to set
     */
    public void setDomainUtility(DomainUtility domainUtility) {
        this.domainUtility = domainUtility;
    }

    /**
     * Get the addItemController.
     * 
     * @return the addItemController
     */
    public AddItemController getAddItemController() {
        return addItemController;
    }

    /**
     * Sets addItemController field.
     * 
     * @param addItemController the addItemController to set
     */
    public void setAddItemController(AddItemController addItemController) {
        this.addItemController = addItemController;
    }

    /**
     * Returns the editManagedItemBean being used to edit the managedItem from the flowScope.
     * @return the editManagedItemBean; if none exists returns a new editManagedItemBean
     */
    private EditManagedItemBean getEditManagedItemBean() {
      return getEditManagedItemBean(flowScope);
    }
    
    /**
     * Returns the editManagedItemBean being used to edit the managedItem from the given flowScope. If none if found
     * will add a new editManagedItemBean to the flowScope.
     * @param flowScope the flowScope to look in
     * @return the existing or new editManagedItemBean
     */
    public static EditManagedItemBean getEditManagedItemBean(FlowScope flowScope) {
      return flowScope.get(EditManagedItemBean.class);
    }

    /**
     * Checks whether the requested item is different from the current item in the {@link EditManagedItemBean}. The
     * current item may be null, in which case this method will always return true.
     * 
     * @return true if the item is different, false otherwise.
     */
    private boolean itemChanged(EditManagedItemBean editManageItemBean, EntityType requestedEntityType, String requestedId) {
        return editManageItemBean.getItem() == null
            || !requestedId.equals(editManageItemBean.getItem().getId())
            || editManageItemBean.getItem().getEntityType() != requestedEntityType;
    }
}
