/**
 * Source file created in 2007 by Southwest Research Institute
 */

package gov.va.med.pharmacy.peps.presentation.common.controller.item;

import java.util.Collection;
import java.util.List;

import org.springframework.validation.DataBinder;

import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.presentation.common.utility.DifferenceUtility;


/**
 * Bean used for presenting item modifications to the user for confirmation.
 */
public class ModificationSummary {
    
    // NOTE WELL: If you add or modify any fields in this object, make sure to update configureBinder() below.
    
    private List<Modification> modifications;
    
    private boolean saveRematch;
    
    private String rematchReason;

    /**
     * getModifications
     *
     * @return List
     */
    public List<Modification> getModifications() {
        return modifications;
    }

    /**
     * setModifications
     *
     * @param modifications List
     */
    public void setModifications(List<Modification> modifications) {
        this.modifications = modifications;
    }

    public boolean isSaveRematch() {
        return saveRematch;
    }

    public void setSaveRematch(boolean saveRematch) {
        this.saveRematch = saveRematch;
    }
    
    /**
     * Returns true if a user-entered rematch reason will be saved with the changes; false otherwise.
     * @param rematch the rematch. May be null if there is no rematch to save.
     * @return see description
     */
    public boolean willSaveRematchReason(final RematchVo rematch) {
        return saveRematch
            // Note: duplicating business rules from RematchCapability here. Hopefully we'll always save rematch reasons
            // and this code will go away soon.
            && ((rematch == null) || (rematch.getMatchReason() == MatchReason.MANUAL_REMATCH));
    }
    
    /**
     * Returns the user-entered reason for the rematch. This cannot be stored in {@link #getModifications()} because
     * that list must match up with the list of {@link ModDifferenceVo}s.
     * @return possibly null
     */
    public String getRematchReason() {
        return rematchReason;
    }

    public void setRematchReason(String rematchReason) {
        this.rematchReason = rematchReason;
    }
    
    /**
     * Configures the given Spring {@link DataBinder} to allow binding to all appropriate fields in this object.
     * @param binder the binder to configure
     */
    public static void configureBinder(final DataBinder binder) {
        binder.setAllowedFields(
            // At time of writing, we allow binding to all properties of this class.
            "modifications",
            // allow binding to the properties of the modifications.
            "modifications[*",
            "saveRematch",
            "rematchReason");
    }
    
    /**
     * Updates the given list of {@link ModDifferenceVo}s with the user-supplied reasons and "Accept Change" options stored in
     * this object.
     * 
     * @param differences the differences to update. Iteration order of these differences must match up with the order of
     *            {@link #getModifications()}.
     */
    public void updateModDifferences(final Collection<ModDifferenceVo> differences) {
        int index = 0;

        for (ModDifferenceVo difference : differences) {
            difference.setModificationReason(getModifications().get(index).getReason());
            difference.setAcceptChange(getModifications().get(index).getAcceptChange());
            index++;
        }
    }

    /**
     * Returns whether this bean has modifications to commit. This may be true even if there are no ModDifferences in
     * the case of a "manual" rematch.
     * 
     * @param differences the differences to commit. This is not an ideal solution but we do not know if just having
     * modifications present is sufficient (due to the complexity of "editable differences").
     * @return true if there are modifications to commit, false otherwise.
     */
    public boolean hasModificationsToCommit(Collection<ModDifferenceVo> differences) {
        return DifferenceUtility.hasEditableDifference(differences) || isSaveRematch();
    }
}
