/**
 * Source file created in 2012 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.presentation.common.controller.item;


import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.OptimisticLockingException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo;
import gov.va.med.pharmacy.peps.common.vo.ProcessedItemVo;
import gov.va.med.pharmacy.peps.common.vo.RequestVo;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.presentation.common.controller.ControllerConstants;
import gov.va.med.pharmacy.peps.presentation.common.controller.PepsController;
import gov.va.med.pharmacy.peps.presentation.common.controller.PpsUrls;
import gov.va.med.pharmacy.peps.presentation.common.controller.bean.EditManagedItemBean;
import gov.va.med.pharmacy.peps.presentation.common.controller.bean.MultipleSelectItemBean;
import gov.va.med.pharmacy.peps.presentation.common.controller.rematch.RematchController;
import gov.va.med.pharmacy.peps.presentation.common.spring.annotation.RoleNeeded;
import gov.va.med.pharmacy.peps.service.common.capability.ItemModDifferences;
import gov.va.med.pharmacy.peps.service.common.capability.ModRematch;
import gov.va.med.pharmacy.peps.service.common.session.ManagedItemService;


/**
 * Spring Web MVC Controller for the Modification Summary pages.
 */
@Controller
@RoleNeeded(roles = { Role.PSS_PPSN_MANAGER, Role.PSS_PPSN_SUPERVISOR })
@RequestMapping("{" + ControllerConstants.ENTITY_TYPE + "}/{" + ControllerConstants.ITEM_ID + "}")
public class ModificationSummaryController extends PepsController {
    
    private static final String MODIFICATION_SUMMARY = "modificationSummary";

    private static final Logger LOG = LogManager.getLogger(ModificationSummaryController.class);

    private static final String NATIONAL_MODIFICATION_SUMMARY = "NATIONAL.modification.summary";
    
    private ManagedItemService managedItemService;

    /**
     * Configures the binder for the modification summary.
     * 
     * @param binder the binder to configure
     */
    @InitBinder(MODIFICATION_SUMMARY)
    protected void modificationSummaryBinder(WebDataBinder binder) {
        ModificationSummary.configureBinder(binder);
    }
    
    /**
     * Constructs an instance with the required dependencies.
     * 
     * @param pManagedItemService the {@link ManagedItemService} to use
     */
    @Autowired
    public ModificationSummaryController (ManagedItemService pManagedItemService) {
      this.managedItemService = pManagedItemService;
    }

    /**
     * Handles a request to apply changes to all selected items (part of the multi-edit workflow).
     *
     * @param entityType of item
     * @param itemId of item
     * @param model Spring Model
     * @param modificationSummary ModificationSummary
     * @return view
     * @throws ItemNotFoundException ItemNotFoundException
     */
    @RequestMapping(value = "/startApplyToAll.go", method = RequestMethod.POST)
    public String submitApplyToAll(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        Model model,
        @ModelAttribute(MODIFICATION_SUMMARY) ModificationSummary modificationSummary)
        throws ItemNotFoundException {

        EditManagedItemBean editManageItemBean = ManageItemController.getEditManagedItemBean(flowScope);
        modificationSummary.updateModDifferences(editManageItemBean.getDifferences());
        flowInputScope.put(editManageItemBean);

        return REDIRECT + "/" + entityType.toLowerCase(Locale.US) + "/" + itemId + "/applyToAll.go";
    }

    /**
     * Presents the Modification Summary screen to the user.
     *
     * @param entityType the EntityType of the item to commit
     * @param itemId the id of the item to commit
     * @param tab the tab of the edit pages
     * @param includeRematch true if the rematch in the current {@link EditManagedItemBean} should be included in the changes
     *            being submitted; false otherwise. (This is necessary because the bean is stored in the session and can get out
     *            of sync sometimes.)
     * @param model the Model
     * @throws ValidationException exception
     * @return the tile definition for the modification summary
     */
    @RequestMapping(value = "/modificationSummary.go", method = RequestMethod.GET)
    public String modificationSummary(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId, 
        @RequestParam(value = ControllerConstants.TAB_KEY, required = false) String tab,
        @RequestParam(value = RematchController.INCLUDE_REMATCH, required = false) boolean includeRematch,
        Model model) throws ValidationException {
        boolean showPsrName = false;
        pageTrail.addPage("editItemSummary", "Modification Summary");

        // the current item
        EditManagedItemBean editManageItemBean = ManageItemController.getEditManagedItemBean(flowScope);
        ManagedItemVo item = editManageItemBean.getItem();
        

        // validate we are editing the correct item id
        if (item == null || !itemId.equals(item.getId())) {
            return ControllerConstants.REDIRECT_SLASH + PpsUrls.getItemEditUrl(entityType, itemId);
        }

        // check if the psrName field should be displayed
        for (ModDifferenceVo vo : editManageItemBean.getDifferences()) {
            if (vo.getDifference().getFieldKey().isRequiresSecondApproval(entityType)) {
                showPsrName = true;
                break;
            }
        }
        
        ModificationSummary modificationSummary = new ModificationSummary();
        modificationSummary.setSaveRematch(includeRematch);
        
        model.addAttribute(EDIT_MANAGED_ITEM_BEAN_KEY, editManageItemBean);
        model.addAttribute(ControllerConstants.ITEM_KEY, item);
        model.addAttribute("modDifferences", editManageItemBean.getDifferences());
        model.addAttribute(ControllerConstants.TAB_KEY, tab);
        model.addAttribute(MODIFICATION_SUMMARY, modificationSummary);
        model.addAttribute("showPsrName", showPsrName);

        return NATIONAL_MODIFICATION_SUMMARY;
    }
    
    /**
     * 
     * Cancels the modification summary, redirects to the last URL in the page trail.
     * 
     * @return the last URL in the page trail
     */
    @RequestMapping(value = "/modificationSummaryCancel.go", method = RequestMethod.POST)
    public String cancelModificationSummary() {
        return REDIRECT + pageTrail.goToPreviousUrl();
    }

    /**
     * Commit the modifications.
     *
     * @param entityType the EntityType of the item to commit
     * @param itemId the id of the item to commit
     * @param tab the tab of the edit pages
     * @param modificationSummary the summary bean posted
     * @return the URL of the confirm modifications view
     * @throws ValidationException ValidationException 
     */
    @RequestMapping(value = "/commitModifications.go", method = RequestMethod.POST)
    public String commitModifications(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @RequestParam(value = "tab", required = false) String tab,
        @ModelAttribute(MODIFICATION_SUMMARY) ModificationSummary modificationSummary)
        throws ValidationException {

        @SuppressWarnings({ "unused" })
        Map<String, String[]> params = request.getParameterMap();
        
        final String psrName = request.getParameter(FieldKey.PSR_NAME.toAttributeName());

        // get the original item
        EditManagedItemBean editManagedItemBean = ManageItemController.getEditManagedItemBean(flowScope);
        ManagedItemVo originalItem = editManagedItemBean.getOriginalItem();

        modificationSummary.updateModDifferences(editManagedItemBean.getDifferences());
        ItemModDifferences itemModDifference = new ItemModDifferences(originalItem, editManagedItemBean.getDifferences());
        itemModDifference.setOptionalModRematch(makeModRematch(modificationSummary, editManagedItemBean));

        try {
            ProcessedItemVo processedItem = managedItemService.commitModifications(itemModDifference, getUser());
            flashScope.put(ControllerConstants.WARNINGS, processedItem.getWarnings().getLocalizedErrors(getLocale()));
        
            // if user enters a PSR Name, update the requestVo with the name
            if (!StringUtils.isEmpty(psrName)) {
                savePsrName(originalItem, psrName);
            }
        } catch (OptimisticLockingException e) {

            editManagedItemBean.setItem(managedItemService.retrieve(itemId, entityType, getUser()));
            String redirectToRequest = redirectToRequest(entityType, itemId, tab);

            List<String> errors = new ArrayList<String>();
            errors.add(e.getLocalizedMessage(getLocale()));
            flashScope.put(ERRORS, errors);

            if (!redirectToRequest.isEmpty()) {
                return redirectToRequest;
            }

            return REDIRECT + "/" + entityType.toString().toLowerCase() + "/" + itemId + "/edit.go";
        }

//        } catch (OptimisticLockingException e) {
//            flowInputScope.put("mergeVo",
//                managedItemService.computeMergeInformation(editManageItemBean.getItem(), differences, getUser()));
//            flowInputScope.put("editManagedItemBean", editManageItemBean);
//
//            return REDIRECT + "/" + entityType.toString().toLowerCase() + "/" + itemId + "/mergeModifications.go";
//        }

        return REDIRECT + "/" + entityType.toString().toLowerCase() + "/" + itemId + "/confirmModifications.go";
    }

    /**
     * Makes a {@link ModRematch}, if appropriate, from the user-submitted {@link ModificationSummary}.
     * @return never null
     */
    private Optional<ModRematch> makeModRematch(
        final ModificationSummary modificationSummary, final EditManagedItemBean editManagedItemBean) {
        // Whether we even create a ModRematch or not depends on whether the user asked to commit rematch changes.
        return modificationSummary.isSaveRematch()
            ? Optional.of(new ModRematch(
                Optional.ofNullable(editManagedItemBean.getRematch()), modificationSummary.getRematchReason()))
            : Optional.empty();
    }
    
    /**
     * Save the psrName to the respective RequestVo
     *
     * @param item the item being created
     * @param psrName the psrName to be saved
     * @throws ValidationException ValidationException
     */
    private void savePsrName(ManagedItemVo item, String psrName) throws ValidationException { //NOPMD
        LOG.debug("Saving PSR name as a separate step.");
        // Assumption: there is a RequestVo.
        RequestVo req = checkForRequest(item.getId(), item.getEntityType());
        item = managedItemService.retrieve(item.getId(), item.getEntityType(), getUser());
        req.setPsrName(psrName);
        req.setMarkedForPepsSecondReview(true);
        Collection<ModDifferenceVo> differences = new ArrayList<ModDifferenceVo>();
        managedItemService.commitRequest(item, req, differences, getUser(), false);
    }

    /**
     * Presents the "Confirm Modifications" page to the user. This is the information page the user sees after the
     * changes have been committed.
     *
     * @param itemId the id of the item to confirm
     * @param tab the current tab
     * @param model the Model
     * @return the tile definition for confirm modifications
     */
    @RequestMapping(value = "/confirmModifications.go", method = RequestMethod.GET)
    public String confirmModifications(@PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @RequestParam(value = ControllerConstants.TAB_KEY, required = false) String tab, Model model) {
        List<?> warnings = (List<?>) request.getAttribute(ControllerConstants.WARNINGS);

        model.addAttribute(ControllerConstants.WARNINGS);
        model.addAttribute("hasWarnings", !CollectionUtils.isEmpty(warnings));

        if (flowScope.containsKey(ControllerConstants.IS_REDIRECTED_FROM_MERGE_SUMMARY)) {
            pageTrail.goToPreviousFlowUrl("/searchItems.go");
        }

        MultipleSelectItemBean multiBean = (MultipleSelectItemBean) flowScope.get("multipleSelectItemBean");
        pageTrail.goToPreviousFlowUrl("/searchItems.go");

        flowInputScope.put("multipleSelectItemBean", multiBean);

        pageTrail.addPage("editItemConfirm", "Confirm Modifications", true);

        return "confirm.modifications";
    }

    /**
     * Redirects the browser to the appropriate next page after acknowledging modification commit.
     *
     * @return view name
     */
    @RequestMapping(value = "/acknowledgeModifications.go", method = RequestMethod.POST)
    public String acknowledgeModifications() {

        int index = getNextIndexOfMultiEdit();

        if (index == -1) {
            LOG.debug("Redirecting to previous flow after acknowledging modification commit.");
            return REDIRECT + pageTrail.goToPreviousFlowUrl("/searchItems.go");
        }

        MultipleSelectItemBean multipleSelectItemBean = (MultipleSelectItemBean) flowScope.get("multipleSelectItemBean");

        return REDIRECT + "/" + multipleSelectItemBean.getItemEntityTypes()[index].toLowerCase() + "/"
            + multipleSelectItemBean.getItemIds()[index] + "/edit.go";

    }

}
