/**
 * Source file created in 2012 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.presentation.common.controller.item;


import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.OptimisticLockingException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.exception.ValueObjectValidationException;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo;
import gov.va.med.pharmacy.peps.common.vo.ProcessedItemVo;
import gov.va.med.pharmacy.peps.common.vo.validator.ErrorKey;
import gov.va.med.pharmacy.peps.common.vo.validator.Errors;
import gov.va.med.pharmacy.peps.common.vo.validator.ValidationError;
import gov.va.med.pharmacy.peps.common.vo.validator.ValidationErrorLevel;
import gov.va.med.pharmacy.peps.presentation.common.controller.ControllerConstants;
import gov.va.med.pharmacy.peps.presentation.common.controller.PepsController;
import gov.va.med.pharmacy.peps.presentation.common.controller.bean.EditManagedItemBean;
import gov.va.med.pharmacy.peps.presentation.common.controller.bean.MultipleSelectItemBean;
import gov.va.med.pharmacy.peps.service.common.capability.ItemModDifferences;
import gov.va.med.pharmacy.peps.service.common.session.ManagedItemService;


/**
 * MultiModificationSummaryController's brief summary
 * 
 * Details of MultiModificationSummaryController's operations, special dependencies
 * or protocols developers shall know about when using the class.
 *
 */
@Controller
@RequestMapping("{" + ControllerConstants.ENTITY_TYPE + "}/{" + ControllerConstants.ITEM_ID + "}")
public class MultiModificationSummaryController extends PepsController {

    /** The tile view for multi-edit modification summary. */
    private static final String MULTI_EDIT_MOD_SUMMARY = "multi.edit.mod.summary";
    /** managedItemService */
    @Autowired
    private ManagedItemService managedItemService;

    /**
     * Configures the binder for the entity type. We do not allow binding to any properties of the entity type as it is
     * not needed. Spring likely does not bind to this anyways, but this makes fortify happy.
     * @param binder the binder to configure
     */
    @InitBinder(ControllerConstants.ENTITY_TYPE)
    protected void initBinderEntityType(WebDataBinder binder) {
        binder.setDisallowedFields("*");
    }

    /**
     * Applies multi-select edit changes to all selected items and provides a summary of each item's changes.
     *
     * @param entityType EntityType
     * @param itemId String
     * @param tab String
     * @param model Model
     * @return String
     * @throws ItemNotFoundException exception
     */
    @RequestMapping(value = "/applyToAll.go", method = RequestMethod.GET)
    public final String applyToAllModificationSummary(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @RequestParam(value = ControllerConstants.TAB_KEY, required = false) String tab,
        Model model) throws ItemNotFoundException {

        pageTrail.addPage("applyToAll", "Review All Changes");

        // the current item
        EditManagedItemBean editManageItemBean = flowScope.get(EditManagedItemBean.class);
        ManagedItemVo item = editManageItemBean.getItem();

        // validate we are editing the correct item id
        if (item == null || !itemId.equals(item.getId())) {
            return "redirect:/" + entityType.toString().toLowerCase() + "/" + itemId + "/edit.go";
        }

        // the current item differences
        Collection<ModDifferenceVo> differences = editManageItemBean.getDifferences();

        Collection<ManagedItemVo> targetItems = retrieveSelectedItems(entityType);

        // the current item differences
        @SuppressWarnings("unchecked") //casting from Object to Collection<ItemModDifferences>
        Collection<ItemModDifferences> itemModDifferences =
            (Collection<ItemModDifferences>) flashScope.get("itemModDifferences");

        if (itemModDifferences == null) {
            ItemModDifferences templateItemModDifference = new ItemModDifferences(editManageItemBean.getItem(), differences);
            itemModDifferences = managedItemService.submitAllModifications(templateItemModDifference, targetItems, getUser()); 
        }

        Errors errors = aggregateErrors(itemModDifferences);
        List<String> aggregatedErrorStrings = errors.getHardErrors().getLocalizedErrors(getLocale());
        if (flashScope.get(ControllerConstants.ERRORS) != null){
            aggregatedErrorStrings.addAll((List<String>)flashScope.get(ControllerConstants.ERRORS));
        }
        // Why do we put these in the flash scope? We're just going to display a page to the user now, no need to use it.
        flashScope.put(ControllerConstants.ERRORS, aggregatedErrorStrings);
        // ISSUE: We are overwriting anything that is already in the 'WARNINGS' attribute.
        flashScope.put(ControllerConstants.WARNINGS, errors.getWarnings().getLocalizedErrors(getLocale()));
        
        int fromIndex = 0;

        // propagate reasons from the original item to all other items
        for (ModDifferenceVo applyfromDifference : differences) {
            String applyFromReason = applyfromDifference.getModificationReason();

            if (StringUtils.isEmpty(applyFromReason)) {

                for (ItemModDifferences applyToItemDifference : itemModDifferences) {
                    int toIndex = 0;

                    for (ModDifferenceVo applyToDifference : applyToItemDifference.getDifferences()) {
                        if (toIndex++ == fromIndex) {
                            applyToDifference.setModificationReason(applyFromReason);
                            break;
                        }
                    }
                }
            }

            fromIndex++;
        }

        model.addAttribute(ControllerConstants.ITEM_KEY, item);
        model.addAttribute(ControllerConstants.TAB_KEY, tab);
        model.addAttribute("itemModDifferences", itemModDifferences);

        return MULTI_EDIT_MOD_SUMMARY;
    }

    /**
     * Aggregate all errors from all the given {@link ItemModDifferences} objects into one list.   If multiple items
     * have an {@link ErrorKey.COPAY_TIER_INVALID_MULTI_EDIT_SCENARIO} error, only one error will be placed into the
     * aggregated list for this type of error.
     *
     * @param collectionToAggregate the item mod differences
     * @return the errors
     */
    private Errors aggregateErrors(Collection<ItemModDifferences> collectionToAggregate) {
        Errors errors = new Errors();
        boolean invalidScenario = false;

        for (ItemModDifferences item : collectionToAggregate) {
            if (item.getErrors() != null && item.getErrors().hasErrors()) {
                for (ValidationError validationError : item.getErrors().getErrors()) {
                    if (ErrorKey.COPAY_TIER_INVALID_MULTI_EDIT_SCENARIO.equals(validationError.getErrorKey())) {
                        invalidScenario = true;
                    } else {
                        errors.addError(validationError);
                    }
                }
            }
        }

        if (invalidScenario) {
            errors
                .addError(new ValidationError(ErrorKey.COPAY_TIER_INVALID_MULTI_EDIT_SCENARIO, ValidationErrorLevel.ERROR));
        }
        return errors;
    }

    /**
     * Retrieve selected items.
     *
     * @param entityType the entity type
     * @return A collection of the retrieved items
     * @throws ItemNotFoundException the item not found exception
     */
    private Collection<ManagedItemVo> retrieveSelectedItems(EntityType entityType) throws ItemNotFoundException {
        // get all the items
        Collection<ManagedItemVo> items = new ArrayList<ManagedItemVo>();

        MultipleSelectItemBean multipleSelectItemBean = (MultipleSelectItemBean) flowScope.get("multipleSelectItemBean");

        if (multipleSelectItemBean != null) {
            for (String currentItemId : multipleSelectItemBean.getItemIds()) {
                items.add(managedItemService.retrieve(currentItemId, entityType, getUser()));
            }
        }
        return items;
    }



    /**
     * Cancel CommitMultipleModifications.
     *
     * @param entityType EntityType
     * @param itemId String
     * @param tab String
     * @return String
     * @throws ValidationException exception
     * @throws OptimisticLockingException exception
     */
    @RequestMapping(value = "/cancelCommitMultiModifications.go", method = RequestMethod.POST)
    public final String cancelCommitMultipleModifications(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @RequestParam(value = "tab", required = false) String tab) throws ValidationException, OptimisticLockingException {

        return REDIRECT + pageTrail.goToPreviousUrl();
    }

    /**
     * Applies multi-select edit changes to all selected items.  
     * If the changes require second approval, a request is created, otherwise the changes are applied.
     *
     * @param entityType EntityType
     * @param itemId String
     * @param tab String
     * @return String
     * @throws ValidationException exception
     * @throws OptimisticLockingException exception
     */
    @RequestMapping(value = "/commitMultiModifications.go", method = RequestMethod.POST)
    public final String commitMultipleModifications(
        @PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @RequestParam(value = "tab", required = false) String tab)
        throws ValidationException, OptimisticLockingException {

        Map<String, String[]> params = request.getParameterMap();

        // get the original item
        EditManagedItemBean editManageItemBean = flowScope.get(EditManagedItemBean.class);

        // the current item differences
        Collection<ModDifferenceVo> differences = editManageItemBean.getDifferences();

        Collection<ManagedItemVo> targetItems = retrieveSelectedItems(entityType);

        ItemModDifferences templateItemModDifference = new ItemModDifferences(editManageItemBean.getItem(), differences);
        Collection<ItemModDifferences>  itemModDifferences = 
            managedItemService.submitAllModifications(templateItemModDifference, targetItems, getUser()); 
        
        Errors errors = aggregateErrors(itemModDifferences);      

        for (ItemModDifferences itemDiff : itemModDifferences) {
            int index = 0;

            if (itemDiff.getErrors().hasHardErrors()){
                //skip to the next item
                continue;
            }
           
            for (ModDifferenceVo difference : itemDiff.getDifferences()) {
                boolean acceptChange =
                    Boolean.valueOf(params.get(itemDiff.getItem().getId() + "-" + index + "-acceptChange")[0]);
                String reason = params.get(itemDiff.getItem().getId() + "-" + index + "-reason")[0];

                difference.setModificationReason(reason);
                difference.setAcceptChange(acceptChange);
                index++;
            }

        }

        // incase there is an exception, send the diffs to the next request
        flashScope.put("itemModDifferences", itemModDifferences);

        Collection<ProcessedItemVo> processedItems = new ArrayList<ProcessedItemVo>();
        String redirect = "";
        boolean validationSucceeded = true;
        
        try {
            processedItems = 
                managedItemService.commitAllModifications(itemModDifferences, getUser());
        }
        catch(ValueObjectValidationException ex) {
            flashScope.put(ControllerConstants.ERRORS, ex.getErrors().getLocalizedErrors());
            flashScope.put(ControllerConstants.ITEM_KEY, editManageItemBean.getItem());
            redirect = MULTI_EDIT_MOD_SUMMARY;
            validationSucceeded = false;
        }
            
        if(validationSucceeded) {
            for (ProcessedItemVo processedItem : processedItems) {
                errors.addErrors(processedItem.getWarnings());
                //I'm guessing this might set the focus on the item in error?
                updateMultiEditIndex(processedItem.getItem().getId());
            }
            
            flashScope.put(ControllerConstants.ERRORS, errors.getHardErrors().getLocalizedErrors(getLocale()));
            flashScope.put(ControllerConstants.WARNINGS, errors.getWarnings().getLocalizedErrors(getLocale()));
            redirect = "redirect:/" + entityType.toString().toLowerCase() + "/" + itemId + "/confirmModifications.go";
        }
        
        return redirect;
    }

    
    /**
     * Gets the managed item service.
     *
     * @return the managedItemService
     */
    public final ManagedItemService getManagedItemService() {
        return managedItemService;
    }

    
    /**
     * Sets the managed item service.
     *
     * @param managedItemService the managedItemService to set
     */
    public final void setManagedItemService(ManagedItemService managedItemService) {
        this.managedItemService = managedItemService;
    }

}
