package gov.va.med.pharmacy.peps.presentation.common.controller.migration;

import static gov.va.med.pharmacy.peps.common.utility.IOUtility.safeClose;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.multipart.commons.CommonsMultipartFile;
import org.springframework.web.servlet.ModelAndView;

import gov.va.med.pharmacy.peps.common.vo.MigrationCopayTierVo;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.presentation.common.pagetrail.PageTrail;
import gov.va.med.pharmacy.peps.presentation.common.spring.annotation.RoleNeeded;
import gov.va.med.pharmacy.peps.presentation.common.spring.interceptor.PepsSpringInterceptor;
import gov.va.med.pharmacy.peps.service.common.migration.CopayTierImportReader;
import gov.va.med.pharmacy.peps.service.common.migration.MigrationStatus;
import gov.va.med.pharmacy.peps.service.common.session.CopayTierMigrationService;

/**
 * Controller for Copay Tier migration.
 */
@Controller
@RoleNeeded(roles = { Role.PSS_PPSN_MIGRATOR })
public class CopayTierMigrationController extends AbstractMigrationController {

  private static final Logger LOG = LogManager.getLogger(CopayTierMigrationController.class);

  /** Windows file separator. */
  public static final String WINDOWS_SEPARATOR = "\\";
  
  /**
   * The model key for the {@link MigrationStatus} object.
   */
  private static final String MODEL_MIGRATION_STATUS = "migrationStatus";

  private final PageTrail pageTrail;
  
  private final CopayTierMigrationService ctMigrationService;

  /**
   * Constructs an instance.
   * 
   * @param pPageTrail the application-wide {@link PageTrail} instance to use
   * @param pCopayTierMigrationService the {@link CopayTierMigrationService} to use
   */
  @Autowired
  public CopayTierMigrationController(
      final PageTrail pPageTrail,
      final CopayTierMigrationService pCopayTierMigrationService) {
    pageTrail = pPageTrail;
    ctMigrationService = pCopayTierMigrationService;
  }

  /**
   * Migrate Copay Tier Data home page (migrationCopayTier.jsp).
   * @param pRequest pRequest
   * @param pResponse pResponse
   * @return ModelAndView
   * @throws Exception Exceptions
   */
  @RequestMapping(value = "/migrationCopayTier.go", method = RequestMethod.GET)
  public final ModelAndView migrationCopayTier(
      HttpServletRequest pRequest,
      HttpServletResponse pResponse) throws Exception {

    Map<String, Object> model = new HashMap<String, Object>();
    LOG.debug("executing: migrationCopayTier ");
    pageTrail.clearTrail();
    pageTrail.addPage("migrationCopayTier", "Migrate Copay Tier Data", true);

    model.put(MODEL_MIGRATION_STATUS, ctMigrationService.getProgress());

    return new ModelAndView("migration-copay-tier", model);
  }
  
  /**
   * Import Copay Tier Data (migrationCopayTier.jsp).
   * @param pRequest pRequest
   * @param pResponse pResponse
   * @return ModelAndView
   * @throws Exception Exceptions
   */
  @RequestMapping(value = "/migrationCopayTierImport.go", method = RequestMethod.POST)
  public ModelAndView migrationCopayTierImport(
      HttpServletRequest pRequest,
      HttpServletResponse pResponse) throws Exception {
      Map<String, Object> model = new HashMap<String, Object>();
      
      LOG.debug("executing: migrationCopayTier ");
      
      // Note that there is no check here that a migration is not currently in process. All "locking" is done purely
      // through the user interface.

      ctMigrationService.updateProgress(0,0,MigrationStatus.INPROGRESS);
      FileItem inputContent = getFileItem(pRequest);
      String fileName = inputContent.getName();
      InputStream is = null;
      
      try {
          is = inputContent.getInputStream();
          
          if (fileName != null && is != null) {
              // PepsController has getUser(). Should this class inherit from PepsController?
              UserVo user = PepsSpringInterceptor.getUserContext(pRequest.getSession()).getUser();
              final CopayTierImportReader importReader = new CopayTierImportReader();
              List<MigrationCopayTierVo> list = importReader.getCopayTierImportList(is);
              // Only kick off the process if the upload actually had data.
              if (list.size() > 0) {
                  ctMigrationService.updateProgress(0, list.size(), MigrationStatus.INPROGRESS);
                  processCopayTierImport(list, user);
              } else {
                ctMigrationService.updateProgress(0, list.size(), MigrationStatus.DONE);
              }
          } 
      } finally {
          safeClose(is);
      }
          
      model.put(MODEL_MIGRATION_STATUS, ctMigrationService.getProgress());

      return new ModelAndView("migration-copay-tier", model);
  }
  
  /**
   * Download copay tier migration results (migrationCopayTier.jsp).
   * @param pRequest pRequest
   * @param pResponse pResponse
   * @return ModelAndView
   * @throws Exception Exception
   */
  @RequestMapping(value = "/migrationCopayTierDownload.go", method = RequestMethod.GET)
  public ModelAndView migrationCopayTierDownload(
      HttpServletRequest pRequest,
      HttpServletResponse pResponse) throws Exception {

      LOG.debug("executing: migrationCopayTierDownload ");
      
      downloadExcel(pResponse, ctMigrationService.getResultFilePath());

      return null;
  }
  
  /**
   * Get migration statuses - progress count and total count (migrationCopayTier.jsp).
   * @return MigrationStatus
   */
  @RequestMapping(value = "/migrationGetStatus.go", method = RequestMethod.GET)
  @ResponseBody
  public final MigrationStatus getStatus() {
      return ctMigrationService.getProgress();
  }
  
  /**
   * Process the copay tier import file asynchronously. 
   * @param list List<MigrationCopayTierVo>
   * @param user UserVo
   */
  private void processCopayTierImport(List<MigrationCopayTierVo> list, UserVo user) {
      
      new Thread(new Runnable() {  
          public void run(){
               ctMigrationService.processCopayTierImport(list,user);
          }
      }).start();
      
  }
  
  /**
   * Get import file name.
   * @param pRequest HttpServletRequest
   * @return fileName String
   */
  protected final FileItem getFileItem(HttpServletRequest pRequest) {
      MultipartHttpServletRequest mpRequest = (MultipartHttpServletRequest) pRequest;     
      FileItem fileItem = null;
      try {
          // Assumption: there is only one file in the request.
          for (Iterator<String> iterator = mpRequest.getFileNames(); iterator.hasNext();) {
              String filename = iterator.next();
              LOG.debug("file: " + filename);

              MultipartFile multipartFile = mpRequest.getFile(filename);
              fileItem = ((CommonsMultipartFile)multipartFile).getFileItem();
          }
      } catch (Exception ex) {
          LOG.error(ex);
      }
      
      return fileItem;
  }
    
}
