package gov.va.med.pharmacy.peps.presentation.common.controller.migration;

import static gov.va.med.pharmacy.peps.common.utility.IOUtility.safeClose;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.POIXMLException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidationType;
import gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator;
import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorImportRow;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.presentation.common.spring.annotation.RoleNeeded;
import gov.va.med.pharmacy.peps.service.common.migration.FormularyDesignatorImportReader;
import gov.va.med.pharmacy.peps.service.common.migration.MigrationStatus;
import gov.va.med.pharmacy.peps.service.common.session.FormularyDesignatorMigrationService;
import gov.va.med.pharmacy.peps.service.common.session.impl.FormularyDesignatorImportFormatException;
import gov.va.med.pharmacy.peps.service.common.session.impl.FormularyDesignatorMigrationInProgressException;

/**
 * Controller for Formulary Designator migration.
 */
@Controller
@RoleNeeded(roles = { Role.PSS_PPSN_MIGRATOR })
public class FormularyDesignatorMigrationController extends AbstractMigrationController {

    private static final Logger LOG = LogManager.getLogger(FormularyDesignatorMigrationController.class);
  
    /**
     * The model key for the {@link MigrationStatus} object.
     */
    private static final String MODEL_MIGRATION_STATUS = "migrationStatus";
  
    private final FormularyDesignatorMigrationService fdMigrationService;

    /**
     * Constructs an instance.
     * 
     * @param pFormularyDesignatorMigrationService the {@link FormularyDesignatorMigrationService} to use
     */
    @Autowired
    public FormularyDesignatorMigrationController(
            final FormularyDesignatorMigrationService pFormularyDesignatorMigrationService) {
        super();
        fdMigrationService = pFormularyDesignatorMigrationService;
    }

  /**
   * Migrate Formulary Designator Data home page (migrationFormularyDesignator.jsp).
   * @param pRequest pRequest
   * @param pResponse pResponse
   * @return ModelAndView
   * @throws Exception Exceptions
   */
    @RequestMapping(value = "/migrationFormularyDesignator.go", method = RequestMethod.GET)
    public final ModelAndView migrationFormularyDesignator(
        HttpServletRequest pRequest,
        HttpServletResponse pResponse) {

        Map<String, Object> model = new HashMap<String, Object>();
        LOG.debug("executing: migrationFormularyDesignator ");
        pageTrail.clearTrail();
        pageTrail.addPage("migrationFormularyDesignator", "Migrate Formulary Designator Data", true);

        model.put(MODEL_MIGRATION_STATUS, fdMigrationService.getStatus());

        return new ModelAndView("migration.formulary.designator", model);
    }
  
    /**
     * Import Formulary Designator Data (migrationFormularyDesignator.jsp).
     * 
     * @param pRequest pRequest
     * @param pResponse pResponse
     * @param file MultipartFile
     * @return ModelAndView
     * @throws Exception Exceptions
     */
    @RequestMapping(value = "/migrationFormularyDesignatorImport.go", method = RequestMethod.POST)
    public ModelAndView migrationFormularyDesignatorImport(HttpServletRequest pRequest, HttpServletResponse pResponse,
            @RequestParam("file") MultipartFile file) {
        Map<String, Object> model = new HashMap<String, Object>();
        InputStream is = null;
        List<String> errors = new ArrayList<String>();
        
        if (file.getSize() > 0) {
            try {
                is = file.getInputStream();
                FormularyDesignatorImportReader reader = 
                        new FormularyDesignatorImportReader();
                List<FormularyDesignatorImportRow> rows = reader.getRows(is);

                if (rows.size() > 0) {
                    startMigration(rows);
                } else {
                    errors.add("Received Formulary Designator import spreadsheet with no data rows.");
                }
            } catch (IOException e) {
                errors.add("Failed to read Formulary Designator import file.");
            } catch (POIXMLException e) {
                errors.add("Import file is not an Excel spreadsheet (.xlsx).");
            } catch (FormularyDesignatorMigrationInProgressException e) {
                errors.add("A Formulary Designator migration is already in progress.");
            } catch (FormularyDesignatorImportFormatException e) {
                errors.add("The import file data is formatted incorrectly: " 
                        + ESAPIValidator.validateStringInput(
                                e.getMessage(), 
                                ESAPIValidationType.SYSTEM_INFORMATION_LEAK_EXTERNAL));
            } finally {
                safeClose(is);
            }
        } else {
            errors.add("No import file submitted, or file size is zero.");
        }

        model.put(MODEL_MIGRATION_STATUS, fdMigrationService.getStatus());
        model.put("errors", errors);
        
        return new ModelAndView("migration.formulary.designator", model);
    }

    private void startMigration(List<FormularyDesignatorImportRow> rows) 
            throws FormularyDesignatorMigrationInProgressException {
        fdMigrationService.initialize(rows.size(), getUser());
        
        new Thread(new Runnable() {
            public void run() {
                List<FormularyDesignatorImportRow> processedRows = 
                        fdMigrationService.process(rows);
                fdMigrationService.writeResultFile(processedRows);
            }
        }).start();
    }

    /**
     * Get migration statuses - progress count and total count (migrationCopayTier.jsp).
     * 
     * @return MigrationStatus
     */
    @RequestMapping(value = "/formularyDesignatorMigrationStatus.go", method = RequestMethod.GET)
    @ResponseBody
    public final MigrationStatus getStatus() {
        return fdMigrationService.getStatus();
    }

    /**
     * Download formulary designator migration results (migrationFormularyDesignator.jsp).
     * 
     * @param pRequest pRequest
     * @param pResponse pResponse
     * @return ModelAndView
     * @throws IOException IOException
     */
    @RequestMapping(value = "/migrationFormularyDesignatorDownload.go", method = RequestMethod.GET)
    public ModelAndView download(HttpServletRequest pRequest, HttpServletResponse pResponse) throws IOException {
        LOG.debug("executing: migrationFormularyDesignatorDownload ");
        downloadExcel(pResponse, fdMigrationService.getResultFile());
        return null;
    }
}
