package gov.va.med.pharmacy.peps.presentation.common.controller.migration;

import static gov.va.med.pharmacy.peps.common.utility.IOUtility.safeClose;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.POIXMLException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidationType;
import gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator;
import gov.va.med.pharmacy.peps.common.vo.HazardousWasteImportRow;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.presentation.common.spring.annotation.RoleNeeded;
import gov.va.med.pharmacy.peps.service.common.migration.HazardousWasteImportReader;
import gov.va.med.pharmacy.peps.service.common.migration.MigrationStatus;
import gov.va.med.pharmacy.peps.service.common.session.impl.MigrationService;
import gov.va.med.pharmacy.peps.service.common.session.impl.MigrationImportFormatException;
import gov.va.med.pharmacy.peps.service.common.session.impl.MigrationInProgressException;

/**
 * Controller for Hazardous Waste migration.
 */
@Controller
@RoleNeeded(roles = { Role.PSS_PPSN_MIGRATOR })
public class HazardousWasteMigrationController extends AbstractMigrationController {

    private static final Logger LOG = LogManager.getLogger(HazardousWasteMigrationController.class);
  
    /**
     * The model key for the {@link MigrationStatus} object.
     */
    private static final String MODEL_MIGRATION_STATUS = "migrationStatus";
      
    private final MigrationService<HazardousWasteImportRow> hazardousWasteMigrationService;

    /**
     * Constructs an instance.
     * 
     * @param pHazardousWasteMigrationService the {@link MigrationService<HazardousWasteImportRow>} to use
     */
    @Autowired
    public HazardousWasteMigrationController(
            final MigrationService<HazardousWasteImportRow> pHazardousWasteMigrationService) {
        super();
        hazardousWasteMigrationService = pHazardousWasteMigrationService;
    }

  /**
   * Migrate Hazardous Waste Data home page (migrationHazardousWaste.jsp).
   * @param pRequest pRequest
   * @param pResponse pResponse
   * @return ModelAndView
   * @throws Exception Exceptions
   */
    @RequestMapping(value = "/migrationHazardousWaste.go", method = RequestMethod.GET)
    public final ModelAndView migrationHazardousWaste(
        HttpServletRequest pRequest,
        HttpServletResponse pResponse) {

        Map<String, Object> model = new HashMap<String, Object>();
        LOG.debug("executing: migrationHazardousWaste ");
        pageTrail.clearTrail();
        pageTrail.addPage("migrationHazardousWaste", "Migrate Hazardous Waste Data", true);

        model.put(MODEL_MIGRATION_STATUS, hazardousWasteMigrationService.getStatus());

        return new ModelAndView("migration.hazardous.waste", model);
    }
  
    /**
     * Import Hazardous Waste Data (migrationHazardousWaste.jsp).
     * 
     * @param pRequest pRequest
     * @param pResponse pResponse
     * @param file MultipartFile
     * @return ModelAndView
     * @throws Exception Exceptions
     */
    @RequestMapping(value = "/migrationHazardousWasteImport.go", method = RequestMethod.POST)
    public ModelAndView migrationHazardousWasteImport(HttpServletRequest pRequest, HttpServletResponse pResponse,
            @RequestParam("file") MultipartFile file) {
        Map<String, Object> model = new HashMap<String, Object>();
        InputStream is = null;
        List<String> errors = new ArrayList<String>();
        
        if (file.getSize() > 0) {
            try {
                is = file.getInputStream();
                HazardousWasteImportReader reader = 
                        new HazardousWasteImportReader();
                List<HazardousWasteImportRow> rows = reader.getRows(is);

                if (rows.size() > 0) {
                    startMigration(rows);
                } else {
                    errors.add("Received Hazardous Waste import spreadsheet with no data rows.");
                }
            } catch (IOException e) {
                errors.add("Failed to read Hazardous Waste import file.");
            } catch (POIXMLException e) {
                errors.add("Import file is not an Excel spreadsheet (.xlsx).");
            } catch (MigrationInProgressException e) {
                errors.add("A Hazardous Waste migration is already in progress.");
            } catch (MigrationImportFormatException e) {
                errors.add("The import file data is formatted incorrectly: " 
                        + ESAPIValidator.validateStringInput(
                                e.getMessage(), 
                                ESAPIValidationType.SYSTEM_INFORMATION_LEAK_EXTERNAL));
            } finally {
                safeClose(is);
            }
        } else {
            errors.add("No import file submitted, or file size is zero.");
        }

        model.put(MODEL_MIGRATION_STATUS, hazardousWasteMigrationService.getStatus());
        model.put("errors", errors);
        
        return new ModelAndView("migration.hazardous.waste", model);
    }

    private void startMigration(List<HazardousWasteImportRow> rows) 
            throws MigrationInProgressException {
        hazardousWasteMigrationService.initialize(rows.size(), getUser());
        
        new Thread(new Runnable() {
            public void run() {
                List<HazardousWasteImportRow> processedRows = 
                        hazardousWasteMigrationService.process(rows);
                hazardousWasteMigrationService.writeResultFile(processedRows, "Hazardous Waste Data");
            }
        }).start();
    }

    /**
     * Get migration statuses - progress count and total count.
     * 
     * @return MigrationStatus
     */
    @RequestMapping(value = "/hazardousWasteMigrationStatus.go", method = RequestMethod.GET)
    @ResponseBody
    public final MigrationStatus getStatus() {
        return hazardousWasteMigrationService.getStatus();
    }

    /**
     * Download hazardous waste migration results (migrationHazardousWaste.jsp).
     * 
     * @param pRequest pRequest
     * @param pResponse pResponse
     * @return ModelAndView
     * @throws IOException IOException
     */
    @RequestMapping(value = "/migrationHazardousWasteDownload.go", method = RequestMethod.GET)
    public ModelAndView download(HttpServletRequest pRequest, HttpServletResponse pResponse) throws IOException {
        LOG.debug("executing: migrationHazardousWasteDownload ");
        downloadExcel(pResponse, hazardousWasteMigrationService.getResultFile());
        return null;
    }
}
