package gov.va.med.pharmacy.peps.presentation.common.controller.rematch;

import java.util.Arrays;
import java.util.Collection;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Objects;
import java.util.Set;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.UnmatchRematchVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.presentation.common.controller.ControllerConstants;
import gov.va.med.pharmacy.peps.presentation.common.controller.PepsController;
import gov.va.med.pharmacy.peps.presentation.common.controller.PpsUrls;
import gov.va.med.pharmacy.peps.presentation.common.controller.bean.EditManagedItemBean;
import gov.va.med.pharmacy.peps.presentation.common.controller.item.ManageItemController;
import gov.va.med.pharmacy.peps.service.common.capability.RematchPreperationBean;
import gov.va.med.pharmacy.peps.service.common.session.ManagedItemService;
import gov.va.med.pharmacy.peps.service.common.session.RematchService;

/**
 * The controller used for creating and viewing rematches.
 * 
 * @since 3.0
 */
@Controller
@RequestMapping("product/{" + ControllerConstants.ITEM_ID + "}")
public class RematchController extends PepsController {

    /**
     * Message code for when a rematch for a given product already exists.
     */
    private static final String REMATCH_ALREADY_EXISTS_WARNING = "rematch.already.exists.warning";

    /**
     * The includeRematch request parameter.
     */
    public static final String INCLUDE_REMATCH = "includeRematch";

    /**
     * View used when creating or editing a rematch.
     */
    private static final String REMATCH_PRODUCT = "unmatch.rematch";

    private static final String EDIT_REMATCH_BEAN = "editRematchBean";

    private final ManagedItemService managedItemService;

    private final RematchService rematchService;

    /**
     * Constructor used to explicitly set services.
     * 
     * @param pManagedItemService the managedItemService
     * @param pRematchService the rematchService
     */
    @Autowired
    public RematchController(ManagedItemService pManagedItemService, RematchService pRematchService) {
        managedItemService = pManagedItemService;
        rematchService = pRematchService;
    }

    /**
     * Modifies the binder to prevent an incoming request from manipulating more properties of the command and form objects than
     * should be allowed.
     * 
     * @param binder the binder to be modified
     */
    @InitBinder
    protected void initBinder(WebDataBinder binder) {
        binder.setAllowedFields(
            "toSuggestedProductIdSet",
            "toAutoProductId");
    }

    /**
     * Presents the rematch view to the user.
     * 
     * @param itemId the id for the product
     * @param model the spring model
     * @param locale the current locale
     * @return the view
     * @throws ItemNotFoundException if productVo for given itemId can not be found
     */
    @RequestMapping(value = PpsUrls.PAGE_REMATCH, method = RequestMethod.GET)
    public String viewRematch(
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId, Model model, Locale locale)
        throws ItemNotFoundException {

        // Add page trail
        pageTrail.addPage("rematch", "Rematch");

        EditRematchBean editRematchBean = (EditRematchBean) flowScope.get(EDIT_REMATCH_BEAN);
        EditManagedItemBean editManagedItemBean = ManageItemController.getEditManagedItemBean(flowScope);

        // Create an editRematchBean if it has not been yet created and added to the flow
        if (editRematchBean == null) {
            // Create EditRematchBean
            editRematchBean = new EditRematchBean();
            // Populate EditRematchBean
            // Get the potential matches for the unmatched product with the given productId
            RematchPreperationBean prepareRematch = rematchService.prepareForRematch(Long.parseLong(itemId));
            editRematchBean.setPotentialMatches(prepareRematch.getPotentialRematchProducts());
            editRematchBean.setRematchForProductExists(prepareRematch.getRematchExistsForProduct());

            // Get the product and add it to the controller
            ProductVo product = retrieveProductForRematch(itemId);
            // Could modify code so that you can just directly add the productVo to the model. Currently sticking with
            // unmatchRematchVo for now.
            UnmatchRematchVo unmatchedProduct = new UnmatchRematchVo();
            unmatchedProduct.setProductName(product.getProductLongName());
            unmatchedProduct.setCmop(product.getCmopId());
            unmatchedProduct.setIen(Objects.toString(product.getIen()));
            unmatchedProduct.setId(product.getId());
            editRematchBean.setUnmatchedProduct(unmatchedProduct);

            // Keep existing re-matches on the flowScope while searching for more potential matches.
            flowScope.put(EDIT_REMATCH_BEAN, editRematchBean);
        }
        
        // Gives a warning if the user is about to overwrite the rematch.
        if (editRematchBean.getRematchForProductExists() 
            && determineMatchReason(editManagedItemBean).equals(MatchReason.MANUAL_REMATCH)) {
            model.addAttribute(
                ControllerConstants.WARNINGS, Arrays.asList(
                    messageSource.getMessage(REMATCH_ALREADY_EXISTS_WARNING, null, locale)));
        }
        
        // Add EditRematchBean to model
        model.addAttribute(EDIT_REMATCH_BEAN, editRematchBean);

        return REMATCH_PRODUCT;
    }

    /**
     * Creating a rematch for modification summary screen.
     * 
     * @param itemId the id for the product
     * @param tab edit product page tab
     * @param model the spring model
     * @param editRematchBean bean used to create and edit rematches
     * @return the view name
     * @throws ItemNotFoundException if either of the specified products could not be found
     * @throws ValidationException if the rematch could not be created for any other reason
     */
    @RequestMapping(value = "/createRematch.go", method = RequestMethod.POST)
    public String createRematch(
        @ModelAttribute(EDIT_REMATCH_BEAN) EditRematchBean editRematchBean,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        @RequestParam(value = ControllerConstants.TAB_KEY, required = false) String tab,
        Model model) throws ItemNotFoundException, ValidationException {

        String modSummaryView = ControllerConstants.REDIRECT_SLASH + "product/" + itemId + "/modificationSummary.go?"
            + ControllerConstants.TAB_KEY + "=" + tab;
        
        // Used for when you want to include a rematch in the modification summary.
        String includeRematchView = modSummaryView + "&" + INCLUDE_REMATCH + "=true";
        
        EditManagedItemBean editManagedItemBean = ManageItemController.getEditManagedItemBean(flowScope);
        
        // If the user submits without creating a rematch
        if (editRematchBean.getToProductIdSet() == null) {
            if (determineMatchReason(editManagedItemBean).equals(MatchReason.MANUAL_REMATCH)) {
                // Set the rematch to null in order to clear out the rematch in the editManagedItemBean.
                editManagedItemBean.setRematch(null);
                return includeRematchView;
            }
            return modSummaryView;
        }

        ProductVo fromProduct = retrieveProductForRematch(itemId);
        MatchReason matchReason = determineMatchReason(editManagedItemBean);
        RematchVo rematch = (editRematchBean.isAuto())
            ? RematchVo.automaticRematch(
                fromProduct,
                getAutomaticProductForRematch(editRematchBean),
                matchReason)
            : RematchVo.suggestedRematch(
                fromProduct,
                getSuggestedProductSetForRematch(editRematchBean),
                matchReason);

        rematch.validateMatchProducts();

        editManagedItemBean.setRematch(rematch);

        return includeRematchView;
    }

    private MatchReason determineMatchReason(EditManagedItemBean editManagedItemBean) {
        final Collection<ModDifferenceVo> modDifferences = editManagedItemBean.getDifferences();
        if (CollectionUtils.isEmpty(modDifferences)) {
            // If no differences, this must be a manual rematch (via the Rematch button). Otherwise the rematch screen
            // would not have been triggered.
            return MatchReason.MANUAL_REMATCH;
        } else {
            final Collection<Difference> differences = ModDifferenceVo.extractDifferences(modDifferences);
            final EnumSet<MatchReason> matchReasons = editManagedItemBean.getItem().determineRematchTriggers(differences);
            return matchReasons.iterator().next(); // the first returned reason;
        }
    }

    private ProductVo getAutomaticProductForRematch(EditRematchBean editRematchBean) throws ItemNotFoundException {
        return retrieveProductForRematch(editRematchBean.getToAutoProductId().toString());
    }

    private Set<ProductVo> getSuggestedProductSetForRematch(EditRematchBean editRematchBean) throws ItemNotFoundException {
        Set<ProductVo> toProductSet = new HashSet<ProductVo>();
        for (Long productId : editRematchBean.getToSuggestedProductIdSet()) {
            toProductSet.add(retrieveProductForRematch(productId.toString()));
        }
        return toProductSet;
    }

    private ProductVo retrieveProductForRematch(String itemId) throws ItemNotFoundException {
        return (ProductVo) managedItemService.retrieveMinimal(itemId, EntityType.PRODUCT);
    }

    /**
     * Cancel rematch and go back to previous page in trail.
     * 
     * @return the view name
     */
    @RequestMapping(value = "/cancelRematch.go", method = RequestMethod.POST)
    public String cancelRematch() {

        return REDIRECT + pageTrail.goToPreviousUrl();
    }

    /**
     * Add a new suggestion obtained by search to the rematch view to the user.
     * 
     * @param itemId the id for the product
     * @param model the spring model
     * @return the view
     * @throws ItemNotFoundException if productVo for given itemId can not be found
     */
    @RequestMapping(value = "/addRematchResultProductSearch.go", method = RequestMethod.GET)
    public String addRematchResultProductSearch(
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId,
        Model model)
        throws ItemNotFoundException {

        // Get the product and add it to the existing rematch list
        ProductVo product = retrieveProductForRematch(itemId);

        // Could modify code so that you can just directly add the productVo to the model.
        // Currently sticking with UnmatchRematchVo for now.
        UnmatchRematchVo potentialMatch = new UnmatchRematchVo();

        potentialMatch.setMatchedIen(String.valueOf(product.getNdfProductIen()));
        potentialMatch.setMatchedProductName(product.getVaProductName());
        potentialMatch.setMatchedCmop(product.getCmopId());
        potentialMatch.setMatchedProductId(Long.parseLong(product.getId()));
        potentialMatch.setItemStatus(product.getItemStatus());

        // Add product to existing re-matches, kept in flowScope while using search
        EditRematchBean editRematchBean = (EditRematchBean) flowScope.get(EDIT_REMATCH_BEAN);

        // Product information for this potential match should be from the product that initiated rematch
        potentialMatch.setProductName(editRematchBean.getUnmatchedProduct().getProductName());
        potentialMatch.setIen(Objects.toString(editRematchBean.getUnmatchedProduct().getIen()));
        potentialMatch.setId(editRematchBean.getUnmatchedProduct().getId());
        potentialMatch.setCmop(editRematchBean.getUnmatchedProduct().getCmop());

        // Check if product already exist in the list of matches, before adding it
        List<UnmatchRematchVo> potentialMatches = editRematchBean.getPotentialMatches();
        Long matchedProductId = potentialMatch.getMatchedProductId();
        if (!potentialMatches.stream().map(UnmatchRematchVo::getMatchedProductId).anyMatch(matchedProductId::equals)) {
            potentialMatches.add(potentialMatch);
            editRematchBean.setPotentialMatches(potentialMatches);
        }

        UnmatchRematchVo originalProduct = (UnmatchRematchVo) editRematchBean.getUnmatchedProduct();

        // After adding manual match to the list of rematches redirect to the the view to display them
        return ControllerConstants.REDIRECT_SLASH + PpsUrls.getItemUrl(EntityType.PRODUCT, originalProduct.getId(), PpsUrls.PAGE_REMATCH);
    }

}
