/**
 * Copyright 2008, Southwest Research Institute
 *
 * Provides cookie functions. Prefixes all cookies to be unique to the PEPS application.
 */

var PEPS_COOKIE_PREFIX = "gov.va.med.pharmacy.peps.";

/**
 * This function is used to set cookies 
 * 
 * @param name cookie name
 * @param value cookie value
 * @param expires (optional) Date cookie expires
 * @param path (optional) path cookie is valid for
 * @param domain (optional) domain cookie is valid for
 * @param secure (optional) set the cookie to use SSL 
 */
function setCookie(name, value, expires, path, domain, secure) {
  var cookieString = PEPS_COOKIE_PREFIX + name + "=" + escape(value) +
	  ((expires) ? "; expires=" + expires.toGMTString() : "") +
	  ((path) ? "; path=" + path : "") +
	  ((domain) ? "; domain=" + domain : "") + 
	  ((secure) ? "; secure" : "");
  document.cookie = cookieString;
}

function set_cookie ( cookie_name, cookie_value,
	    lifespan_in_days, valid_domain, secure )
	{
	    var domain_string = valid_domain ?
	                       ("; domain=" + valid_domain) : '',
	        safe_cookie_value = validate_cookie_value(cookie_value);

	    document.cookie =  PEPS_COOKIE_PREFIX + cookie_name +
	                       "=" + encodeURIComponent( safe_cookie_value ) +
	                       "; max-age=" + 60 * 60 *
	                       24 * lifespan_in_days +
	                       "; path=/" + domain_string +
	                       ((secure) ? "; secure" : "");
	}

/**
 * Remove CRLF characters from cookie_value to prevent Header Manipulation
 * vulnerability.
 * 
 * @param cookie_value to be validated
 * @returns a value that is safe to be used in a cookie
 */
function validate_cookie_value(cookie_value) {
	return cookie_value.toString().replace(/[\n\r]/g, ' ');
}

function get_cookie ( cookie_name )
{
    var cookie_string = document.cookie ;
    cookie_name = PEPS_COOKIE_PREFIX + cookie_name;
    if (cookie_string.length != 0) {
        var cookie_value = cookie_string.match (
                        '(^|;)[\s]*' +
                        cookie_name +
                        '=([^;]*)' );
        return decodeURIComponent ( cookie_value[2] ) ;
    }
    return '' ;
}


/**
 * This function is used to get cookies 
 * 
 * @param name cookie name
 */
function getCookie(name) {
	var prefix = PEPS_COOKIE_PREFIX + name + "=";
	var start = document.cookie.indexOf(prefix);
	
	if (start == -1) {
		return null;
	}

	var end = document.cookie.indexOf(";", start + prefix.length);

	if (end == -1) {
		end=document.cookie.length;
	}

	var value = document.cookie.substring(start + prefix.length, end);

	return unescape(value);
}

/**
 * This function is used to delete cookies 
 *
 * @param name cookie name
 * @param path (optional) path cookie is valid for
 * @param domain (optional) domain cookie is valid for
 */
function deleteCookie(name, path, domain) {
  if (getCookie(name)) {
    document.cookie = PEPS_COOKIE_PREFIX + name + "=" +
      ((path) ? "; path=" + path : "") +
      ((domain) ? "; domain=" + domain : "") +
      "; max-age=0";
  }
}

/**
 * Delete all PEPS cookies.
 */
function deleteAllCookies() {
	var cookies = window.document.cookie.split(";");
	
	for (var i = 0; i < cookies.length; i++) {
		var cookie = cookies[i];
		var eqPos = cookie.indexOf("=");
		var name = eqPos > -1 ? cookie.substr(0, eqPos) : cookie;
		var prefixIndex = name.indexOf(PEPS_COOKIE_PREFIX);
		
		if (prefixIndex != -1) {
			var shortName = name.replace(PEPS_COOKIE_PREFIX, "").trim();
			setCookie(shortName, "", new Date(1970, 0, 1), "/");
        }
	}
}

/**
 * Test if the user just came from the KAAJEE login screen. If so, delete all PEPS cookies.
 */
function deleteCookiesOnLogin() {
	var referrer = window.document.referrer;
	// Warning: users no longer hit this JSP upon login now that KAAJEE is gone. This is, however, the state of the code
	// in v1.3 so the v3.0 team is leaving it this way for now.
	var index = referrer.indexOf("login/login.jsp"); // partial URL for KAAJEE login
	
	if (index != -1) {
		deleteAllCookies();
	}
}
