/**
 * Apply the default copay tier value when adding a new copay tier
 * row. If the product is associated with any excluded drug classes,
 * then the copay tier is defaulted to '0' and the effective date is
 * defaulted to the first day of the second month from now. Otherwise,
 * copay tier and effective date are set to blank.
 * 
 * @param element the button that was clicked, either 'Add New Row' 
 * or 'Remove'.
 */
function setDefaultCopayTier(element, callBack) {
	//Contains the default values for copay tier and effective date.
	var copayTierAssoc, 
	
		//The copay tier table DOM element.
		tableElement,
		
		//The copay tier select DOM element.
		copayTierElement,
		
		//The copay tier effective date DOM element.
		effectiveDateElement,
		
		//The option DOM elements within the copay tier select DOM element.
		optionElements,
		
		//for loop index.
		i,
	
	    //Check for which calculation to use
	    calculateFunction;
	
	    if(typeof callBack == 'undefined') {
	    	calculateFunction = calculateDefaultCopayTierAssoc;
	    }else {
	    	calculateFunction = callBack;
	    }

	
	//Check whether the add new row button was clicked.
	if(element.id === "copay.tier.assocs_add") {
		copayTierAssoc = calculateFunction();
		tableElement = document.getElementById("copay.tier.assocs_table");
		copayTierElement = getComponent("copay.tier", tableElement.rows.length - 3);
		optionElements = copayTierElement.getElementsByTagName("option");
		
		//Select the proper default copay tier option in the select DOM element.
		for(i = 0; i < optionElements.length; i++) {
			if(optionElements[i].innerText === copayTierAssoc.copayTier) {
				optionElements[i].selected = true;
				break;
			}
		}
		
		//Set the default effective date.
		effectiveDateElement = getComponent("effective.date", tableElement.rows.length - 3);
		effectiveDateElement.value = copayTierAssoc.effectiveDate;
		manageCopayTierDateChange(effectiveDateElement);
	}
}

/**
 * Compute the correct copay tier and effective date. Returns
 * JSON object containing the proper default values.
 */
function calculateDefaultCopayTierAssoc() {
	var effectiveDate;
	
	if(hasExcludedDrugClass()) {
		effectiveDate = calculateCopayTierEffectiveDate(new Date());		
		
		return {
			copayTier: "0",
			effectiveDate: effectiveDate.toString("dd-MMM-yyyy")
		};
	}
	else {
		return {
			copayTier: "",
			effectiveDate: ""
		};
	}
}

/**
 * Compute the proper default effective date, given the current date.
 * 
 * @param currentDate Today's date.
 * @returns {Date} Effective date.
 */
function calculateCopayTierEffectiveDate(currentDate) {
	var effectiveDate = new Date(currentDate);
	effectiveDate.addMonths(2);
	effectiveDate.setDate(1);
	return effectiveDate;
}

/**
 * Determine whether the current product is associated with
 * an excluded drug class.
 * 
 * @returns {Boolean} True if and only if at least one of the
 * associated drug classes is excluded.
 */
function hasExcludedDrugClass() {
	//The drug classes table DOM element.
	var tableElement = document.getElementById("drug.classes_table"),
	
		//For loop index.
		i,
		
		//The VA drug class select DOM element.
		selectElement,
		
		//The id of the current VA drug class option DOM element.
		drugClassId,
		
		//Set to true if and only if an excluded VA drug class is found.
		result = false;
	
	for(i = 0; i < tableElement.rows.length - 1; i++) {
		selectElement = document.getElementById("drug.class_" + i);
		drugClassId = selectElement.options[selectElement.selectedIndex].value;
		
		if(isDrugClassExcluded[drugClassId]) {
			result = true;
		}
	}
	
	return result;
}

function manageCopayTierDateChange(element) {
	
	var tableElement,            // copay tier table
		effectiveDateElement,   // effective element
		endDateElement,        // end date element
		endDateValue,         //end date value
		effectiveDateValue,  //effective date value
		j,                  //index
		i,                //index
	    mSecondsInDay = 86400000,  // milli seconds in one day
	    assocIndex; //The row number of the element that received the change event.
	
	tableElement = document.getElementById("copay.tier.assocs_table");
	if(tableElement.rows.length > 1) {
		for(i = 1; i < tableElement.rows.length; i++) {
			j = i-1;
			endDateElement = document.getElementById("end.date_"+j);
			effectiveDateElement = document.getElementById("effective.date_"+ i);
			effectiveDateValue = null;
			endDateValue = null;
			if (effectiveDateElement!=null) {
				effectiveDateValue = effectiveDateElement.value;
			}
			if (endDateElement!=null) {
				endDateValue = endDateElement.value;
			}
			if(effectiveDateValue!=null && effectiveDateValue != "" && (endDateValue ==null || endDateValue=="")) {
				var str= effectiveDateValue.replace(/-/g,' ');
				var priorDate = new Date(Date.parse(str) - mSecondsInDay);
				endDateElement.value = priorDate.toString("dd-MMM-yyyy");
			}
		}
	}
	
	//Apply default effective date when copay tier dropdown changes.
	if (startsWith(element.id, "copay.tier_")) {	
		assocIndex = element.id.split("_")[1];
		effectiveDateElement = document.getElementById("effective.date_" + assocIndex);
		effectiveDateValue = effectiveDateElement.value;
		
		if(effectiveDateValue === null || effectiveDateValue === "" || effectiveDateValue === "undefined") {
			effectiveDateElement.value = calculateCopayTierEffectiveDate(new Date()).toString("dd-MMM-yyyy");
			manageCopayTierDateChange(effectiveDateElement);
		}
	}
}

/**
 * Compute the correct copay tier and effective date for the A-Z popup screen. Returns
 * object containing the default values. Mirrors the calculateDefaultCopayTierAssoc
 * but references the hasExcludedCurrentDrugClass function. Mirrors 
 * calculateDefaultCopayTierAssoc function.
 */
function calculatePopUpCopayTierAssoc() {
	var effectiveDate;

	if (hasExcludedCurrentDrugClass()) {
		effectiveDate = calculateCopayTierEffectiveDate(new Date());

		return {
			copayTier : "0",
			effectiveDate : effectiveDate.toString("dd-MMM-yyyy")
		};
	} else {
		return {
			copayTier : "",
			effectiveDate : ""
		};
	}
}

/**
 * Determine whether the current product is associated with an excluded drug
 * class. Drug classes are extracted from the request in currentDrugClasses.tag
 * rather than from the DOM.
 * 
 * @returns {Boolean} True if and only if at least one of the associated drug
 *          classes is excluded.
 */
function hasExcludedCurrentDrugClass() {
	// The drug classes table retrieved for the current request.

	// For loop index.
	i,

	// Set to true if and only if an excluded VA drug class is found.
	result = false;
	var drugClassId = null;

	for (i = 0; i < currDrugClasses.length; i++) {
		drugClassId = currDrugClasses[i];

		if (isDrugClassExcluded[drugClassId]) {
			result = true;
		}
	}

	return result;
}
